\\ Copyright 2015, 2017 Kevin Ryde
\\
\\ This file is free software; you can redistribute it and/or modify it
\\ under the terms of the GNU General Public License as published by the Free
\\ Software Foundation; either version 3, or (at your option) any later
\\ version.
\\
\\ This file is distributed in the hope that it will be useful, but
\\ WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
\\ or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
\\ for more details.
\\
\\ You should have received a copy of the GNU General Public License along
\\ with this file.  See the file COPYING.  If not, see
\\ <http://www.gnu.org/licenses/>.



\\ Mendes France APMEP Bulletin
\\ p/q = [0,a1,...,an]
\\ p/q + 1/q^2 = [0,a1,..., an+1, an-1, ...a1]

\\ Cohn
\\ pn/qn + (-1)^n/(x*qn^2) = [ a0, w_forward, x, -w_reversed ]
\\ [...,a,0,b,...] =[...,a+b,...]
\\ [...,a,-b,...] =[...,a-1,1,b-1,...]


check_count = 0;
check_exact(got,want,name="") =
{
  check_count++;
  if(got!= want,
     print("wrong "name);
     print("got  "got);
     print("want "want);
     quit(1),
     print("ok "check_count" "name));
}

join(v,sep=",") = if(#v==0,"", concat(vector(2*#v-1,i, if(i%2,Str(v[(i+1)/2]),sep))));
join([])=="" || error("join empty");
join([1])=="1" || error("join 1");
join([1,2])=="1,2" || error("join 1,2");
join([1,2]~)=="1,2" || error("join col");

to_binary(n) = subst(Pol(digits(n,2)),'x,10);    \\ n to decimal digits which are bits
from_binary(n) = from_binary_vector(digits(n));  \\ from decimal digits of n are bits
from_binary_vector(v) = subst(Pol(v),x,2);   \\ high to low

binary_string(n) =
{
 \\ print("n=",n);
  if(type(n)=="t_MAT",
     my(s=matsize(n));
     if(s[1]==1,n=n[1,],
        s[2]==1,n=n[,1],
        return(join(vector(s[1],i,binary_string(n[i,])),"\n"))));
  if(type(n)=="t_COL", n=Vec(n));
  if(type(n)=="t_VEC",
     \\ print("v=",join(apply(binary_string,n),", "));
     return(Str("[",join(apply(binary_string,n),", "),"]")));
  \\ print("n=",n);
  concat(apply((x)->Str(x),binary(n)));
}
binary_string(5)=="101" || error("binary 5");
binary_string([5])=="[101]" || error("binary [5]");
binary_string([5,9]~)=="[101, 1001]" || error("binary col");
binary_string([1;2])=="[1, 10]" || error("binary mat col");

vec_product(v) = prod(i=1,#v, v[i]);

/* pell(n) returns the n'th Pell number as per OEIS A000129.
   Starting from n=0 is 0, 1, 2, 5, 12, 29, 70, ...
   Negative n works too, reversing recurrence
   L[n] = 2*L[n-1] + L[n-2] */
pell(n)=([2,1;1,0]^n*[0;1])[1,1];

\\-----------------------------------------------------------------------------

turn(n) = if(BitAboveLowestOne(n), -1, 1);
BitAboveLowestOne(n) = bittest(n, valuation(n,2)+1);
BitAboveLowestZero(n) = n+=1; bittest(n, valuation(n,2)+1);
L=[1,0;1,1];
R=[1,1;0,1];
L*L*L*[1;1]

check_exact(L^2, [1,0; 2,1]);
check_exact(R^2, [1,2; 0,1]);

S=[0,1;1,0];
S*L*S == R || quit;
\\(R*L*L*R*L)
\\(R*L*L*R*L)^-1
(L*R*R*L*L*L*R)
(R*L*L*L*R*R*L)

\\ L,R = [a,b;c,d] run taken in reverse is [d,b;c,a]
\\ opposite diagonal transpose
opposite_transpose(m)= [m[2,2], m[1,2]; m[2,1], m[1,1]];
check_exact(opposite_transpose([a,b;c,d]), [d,b;c,a])
{
  print("opposite_transpose\n");
  for(i=16,31,
     my(v=binary(i)[^1]);
     v = apply((b)->if(b,L,R),v);
     my(m = vec_product(v),
        r = vec_product(Vecrev(v)),
        opp = opposite_transpose(m)==r);

     print(binary(i),"    ",m[1,],"  ",r[1,]);
     print("                   ",m[2,],"  ",r[2,],"  opp equal=",opp);
  );
}

[ a,b; c,d ] *R* [ d,b; c,a ]
z = L * [ a,b; c,d ] *R* [ d,b; c,a ] *[1;1]
z == [a^2 + 2*a*b + a*c + a*d + b*c; \
      a^2 + 2*a*b + a*c + a*d + b*c  \
                  + a*c + a*d + b*c + 2*c*d + c^2]

\\ both transpose
both_transpose(m)= [m[2,2], m[2,1]; m[1,2], m[1,1]];
check_exact(both_transpose([a,b;c,d]), [d,c; b,a])
{
  print("both_transposes\n");
  for(i=16,31,
     my(v=binary(i)[^1]);
     my(w = apply((b)->if(b,R,L),v));
        v = apply((b)->if(b,L,R),v);
     my(m = vec_product(v),
        f = vec_product(w),
        feq = both_transpose(m)==f);

     print(binary(i),"    ",m[1,],"  ",f[1,]);
     print("                   ",m[2,],"  ",f[2,],"  both_transpose equal=",feq);
  );
}

check_exact(opposite_transpose(both_transpose([a,b;c,d])), [a,c; b,d])
check_exact(both_transpose(opposite_transpose([a,b;c,d])), [a,c; b,d])
check_exact(opposite_transpose(both_transpose([a,b;c,d])), \
            both_transpose(opposite_transpose([a,b;c,d])));
check_exact(opposite_transpose(both_transpose([a,b;c,d])), \
            mattranspose([a,b;c,d]));

\\-----------------------------------------------------------------------------

print("both_transpose");
print([ a,b; c,d ] * both_transpose([ a,b; c,d ]) * [1;1]);
print("opposite_transpose");
print([ a,b; c,d ] * opposite_transpose([ a,b; c,d ]) * [1;1]);
print("both_transpose and opposite_transpose");
print([ a,b; c,d ] * opposite_transpose(both_transpose([ a,b; c,d ])) * [1;1]);

print();
print("R^2 with transpose");
\\ something wrong ...:
\\ m[1,1] = 1, 5, 29, 985, 1136689, 1513744654945
\\ m[2,2] = 1, 1, 5, 169, 195025, 259717522849
\\
\\ m[1,2] = 2, 2, 12, 408, 470832, 627013566048
\\ m[2,1] = 0, 2, 12, 408, 470832, 627013566048
\\          = A051009 = 2^n Pell number    n>=4 A098890(n-2)-A098890(n-3)
\\ A098890=A001652(n)*A046090(n)
\\        =A098602(n)+1  =
\\ A098602(n) = 2*r*(r+1)    a(n) = 35*a(n-1) - 35*a(n-2) + a(n-3)
\\ polroots(-1*x^3+35*x^2-35*x+1) = .0294, 1, 33.97
\\ factor(-1*x^3+35*x^2-35*x+1) = (x-1)*(x^2-34*x+1)
\\ n=1 to 15 is n=1 to 7 8 n=9 to 15
\\                  7of  1     7of
{
  my(m=matid(2));
  my(middle=R^2);
  my(l=List([]));
  for(i=1,8,
     my(v = L*m*[1;1]);
     \\ print(v);
      print(m);

     \\ listput(l,v[1,1]);
     listput(l,m[2,2]);
     m = m * middle * mattranspose(m));
  print(l);
}
print();

frac_by_sum(k) = sum(i=0,k, 1/2^2^i);
num_by_frac(k) = numerator(frac_by_sum(k));
den_by_frac(k) = denominator(frac_by_sum(k));
print("num  ",vector(6,k,my(k=k-1); num_by_frac(k)))
print("frac  ",frac_by_sum(8)*1.0)

check_exact(2^from_binary(111) + 2^from_binary(110) + 2^from_binary(100) + 2^from_binary(000), \
            num_by_frac(3))
check_exact(2^from_binary(1111) + 2^from_binary(1110) + 2^from_binary(1100) + 2^from_binary(1000) + 2^from_binary(0000), \
            num_by_frac(4))

num_by_iteration(k) = my(ret=1); for(i=0,k-1, ret = 2^2^i*ret + 1); ret;
check_exact(vector(10,k,my(k=k-1); num_by_iteration(k)), \
            vector(10,k,my(k=k-1); num_by_frac(k)), \
            "num_by_iteration()")

\\ 16+8+4+2 == 30
num_by_exponents(k) = 1 + sum(i=0,k-1, 2^(sum(j=0,i, 2^(k-1-j))));
print("num  ",vector(6,k,my(k=k-1); num_by_exponents(k)))
check_exact(vector(10,k,my(k=k-1); num_by_exponents(k)), \
            vector(10,k,my(k=k-1); num_by_frac(k)), \
            "num_by_exponents()")

den(k) = 2^2^k;
print("num/den ",[ num_by_frac(5);den_by_frac(5) ]);

m_r2(k) =
{
  my(m = matid(2));
  for(i=1,k, m = m * R^2 * mattranspose(m));
  m;
}
print("a(k) = ",vector(9,k,my(k=k-1); m_r2(k)[1,1]));
print("b(k) = ",vector(9,k,my(k=k-1); m_r2(k)[1,2]));
print("c(k) = ",vector(9,k,my(k=k-1); m_r2(k)[2,1]));
print("d(k) = ",vector(9,k,my(k=k-1); m_r2(k)[2,2]));

check_exact(vector(10,k,my(k=k-1); matdet(m_r2(k))), \
            vector(10,k,my(k=k-1); 1), \
            "m matdet")
check_exact(vector(10,k,my(k=k-1); m_r2(k)[1,1] * m_r2(k)[2,2] - m_r2(k)[1,2]*(m_r2(k)[1,2] - 2)), \
            vector(10,k,my(k=k-1); 1), \
            "m det")

Lm_r2(k) = L*m_r2(k);
print("La(k) = ",vector(7,k,my(k=k-1); Lm_r2(k)[1,1]));
print("Lb(k) = ",vector(7,k,my(k=k-1); Lm_r2(k)[1,2]));
print("Lc(k) = ",vector(7,k,my(k=k-1); Lm_r2(k)[2,1]));
print("Ld(k) = ",vector(7,k,my(k=k-1); Lm_r2(k)[2,2]));

\\ L*m*[1;1] = [ num(k); 2^2^k ]
\\ m = [ a(k), b(k); b(k)-2, d(k) ]
\\ L*m = [ a(k), b(k); a+c(k), b(k)+d(k) ]
\\     = [ a(k), b(k); a+b(k)-2, b(k)+d(k) ]
\\ a(k) + b(k) = num(k)
\\ a(k)+b(k)-2 + b(k)+d(k) = 2^2^k
\\ b(k)-2 + d(k) = 2^2^k - num(k)
\\ a(k)*d(k) - b(k)*(b(k)-2) = 1
\\
print("num/den ",[ num_by_frac(5);den_by_frac(5) ]);

check_exact(vector(10,k,my(k=k-0); den_by_frac(k)), \
            vector(10,k,my(k=k-0); Lm_r2(k)[2,1] + Lm_r2(k)[2,2]), \
            "Lc()+Ld()")
check_exact(vector(10,k,my(k=k-1); num_by_frac(k)), \
            vector(10,k,my(k=k-1); Lm_r2(k)[1,1] + Lm_r2(k)[1,2]), \
            "La()+Lb()")

\\ (num(k)-b(k)*(2^2^k-num(k)-b(k)+2)) - b(k)*(b(k)-2) = 1
check_exact(vector(10,k,my(k=k-0); (num_by_frac(k) - m_r2(k)[1,2])*(2^2^k - num_by_frac(k) - m_r2(k)[1,2] + 2) \
                                   - m_r2(k)[1,2]*(m_r2(k)[1,2] - 2)), \
            vector(10,k,my(k=k-0); 1), \
            "b(k) A")

\\ (num(k)-b(k)*(2^2^k-num(k)-b(k)+2)) - b(k)*(b(k)-2) = 1
\\ (N-B)*(K-N-B+2) - B*(B-2) == -N^2 + K*N + 2*N - K*B
\\ -N^2 + K*N + 2*N - K*B == 1
\\ K*B = -N^2 + K*N + 2*N - 1
\\ B = (-N^2 + K*N + 2*N - 1 )/K
check_exact(vector(10,k,my(k=k-0); m_r2(k)[1,2]), \
            vector(10,k,my(k=k-0); (-num_by_frac(k)^2 + 2^2^k*num_by_frac(k) + 2*num_by_frac(k) - 1) / 2^2^k), \
            "b(k) B")

\\ (-N^2 + K*N + 2*N - 1 )/K  == (-N^2 + K*N + 2*N - 1 )/K
\\ (-N^2 + K*N + 2*N - 1 )/K  == (-N^2 + 2*N - 1 )/K + N
\\ (-N^2 + K*N + 2*N - 1 )/K  == -(N^2 - 2*N + 1 )/K + N
\\ (-N^2 + K*N + 2*N - 1 )/K  == -(N-1)^2/K + N
\\ (-N^2 + K*N + 2*N - 1 )/K  == N - (N-1)^2/K
\\ B = N - (N-1)^2/K
check_exact(vector(10,k,my(k=k-0); num_by_frac(k) - (num_by_frac(k) - 1)^2/2^2^k), \
            vector(10,k,my(k=k-0); m_r2(k)[1,2]), \
            "b(k) C")

{
check_exact(2^from_binary(111) + 2^from_binary(110) + 2^from_binary(100) + 2^from_binary(000)
            - (2^from_binary(111) + 2^from_binary(110)  + 2^from_binary(100))^2/2^from_binary(1000),
            m_r2(3)[1,2],
            "b(3) exponents")
}
{
check_exact(2^from_binary(111) + 2^from_binary(110) + 2^from_binary(100) + 2^from_binary(000)
            - (2^from_binary(1110) + 2^from_binary(1101)  + 2^from_binary(1011)
               2^from_binary(1101) + 2^from_binary(1100)  + 2^from_binary(1010)
               2^from_binary(1011) + 2^from_binary(1010)  + 2^from_binary(1000))/2^from_binary(1000),
            m_r2(3)[1,2],
            "b(3) exponents")
}
{
check_exact(2^from_binary(111) + 2^from_binary(110) + 2^from_binary(100) + 2^from_binary(000)
            - (2^from_binary(1110) + 2*2^from_binary(1101) + 2*2^from_binary(1011)
                                   + 2^from_binary(1100)   + 2*2^from_binary(1010)
                                                           + 2^from_binary(1000))/2^from_binary(1000),
            m_r2(3)[1,2],
            "b(3) exponents")
}
{  \\ /2^from_binary(1000) removes that bit from each
check_exact(2^from_binary(111) + 2^from_binary(110) + 2^from_binary(100) + 2^from_binary(000)
            - (2^from_binary(0110) + 2*2^from_binary(0101) + 2*2^from_binary(0011)
                                   + 2^from_binary(0100)   + 2*2^from_binary(0010)
                                                           + 2^from_binary(0000)),
            m_r2(3)[1,2],
            "b(3) exponents")
}
{
check_exact(2^from_binary(111) + 2^from_binary(110) + 2^from_binary(100) + 2^from_binary(000)
            - (2^from_binary(110) + 2*2^from_binary(101) + 2*2^from_binary(011)
                                  + 2^from_binary(100)   + 2*2^from_binary(010)
                                                         + 2^from_binary(000)),
            m_r2(3)[1,2],
            "b(3) exponents")
}
{
check_exact(2^from_binary(111)
            - (                     2*2^from_binary(101) + 2*2^from_binary(011)
                                                         + 2*2^from_binary(010)
                                                                             ),
            m_r2(3)[1,2],
            "b(3) exponents")
}
{
check_exact(2^from_binary(1111)
            - (   2*2^from_binary(1101) + 2*2^from_binary(1011) + 2*2^from_binary(0111) \\
                                        + 2*2^from_binary(1010) + 2*2^from_binary(0110) \\ 1110 + 1100,1000
                                                                + 2*2^from_binary(0100) ), \\ 1100 + 1000
            m_r2(4)[1,2],
            "b(4) exponents")
}
{
check_exact(2^from_binary(11111)
    - ( 2*2^from_binary(11101) + 2*2^from_binary(11011) + 2*2^from_binary(10111) + 2*2^from_binary(01111)
                               + 2*2^from_binary(11010) + 2*2^from_binary(10110) + 2*2^from_binary(01110)
                                                        + 2*2^from_binary(10100) + 2*2^from_binary(01100)
                                                                                 + 2*2^from_binary(01000) ),
      m_r2(5)[1,2],
      "b(5) exponents")
}
vector(8,k, - (m_r2(k)[1,2] - 2^(2^k-1))/4)

\\ vector(8,k,my(k=k-1); 2^2^(k+1) - pell(2^k))

quit

\\ A = N - B = (N-1)^2/K
check_exact(vector(10,k,my(k=k-0); m_r2(k)[1,1]), \
            vector(10,k,my(k=k-0); (num_by_frac(k) - 1)^2/2^2^k), \
            "a(k)")

\\ B-2 + D = K - N
\\ D = K - N - B + 2
\\   = K - N + 2 - (N - (N-1)^2/K)
\\   = K - 2*(N-1) + (N-1)^2/K
\\   = K + (N-1)*((N-1)/K - 2)
check_exact(K - N + 2 - (N - (N-1)^2/K),   K + (N-1)*((N-1)/K - 2));
check_exact(K - N + 2 - (N - (N-1)^2/K),   (K^2 - 2*(N-1)*K + (N-1)^2)/K);
check_exact(K - N + 2 - (N - (N-1)^2/K),   (K - (N-1))^2/K);
check_exact(K - N + 2 - (N - (N-1)^2/K),   (K+1 - N)^2/K);
check_exact(K - N + 2 - (N - (N-1)^2/K),   (N-1-K)^2/K);
check_exact(vector(10,k,my(k=k-0); m_r2(k)[2,2]), \
            vector(10,k,my(k=k-0); 2^2^k - 2*(num_by_frac(k) - 1) + (num_by_frac(k) - 1)^2/2^2^k), \
            "d(k)")

num_minus_1_by_exponents(k) = sum(i=0,k-1, 2^(sum(j=0,i, 2^(k-1-j))));
check_exact(vector(10,k,my(k=k-0); num_minus_1_by_exponents(k)), \
            vector(10,k,my(k=k-0); num_by_frac(k) - 1), \
           "num_minus_1_by_exponents()")
check_exact(2^from_binary(111) + 2^from_binary(110) + 2^from_binary(100), \
            num_by_frac(3) - 1)

b_exponent_list(k) = vector(k,i,my(i=i-1); sum(j=0,i, 2^(k-1-j)));
num_minus_1_by_b_exponents(k) = vecsum(apply((n)->2^n, b_exponent_list(k)));
print("b_exponent_list     ",vecsort(b_exponent_list(5)))
print("b_exponent_list bin ",apply(to_binary,vecsort(b_exponent_list(5))))

check_exact(vector(4,k,my(k=k-0); num_minus_1_by_b_exponents(k)), \
            vector(4,k,my(k=k-0); num_by_frac(k) - 1), \
            "num_minus_1_by_b_exponents()")

vector_sum_pairs(v) = concat(vector(#v,i, apply((n)->n+v[i], v)));
print("b_exponent_list squared  ",vecsort(vector_sum_pairs(b_exponent_list(5))))
print("b_exponent_list sdiv     ",vecsort(apply((n)->n-2^5,vector_sum_pairs(b_exponent_list(5)))))
print("b_exponent_list sdiv bin ",apply(to_binary,vecsort(apply((n)->n-2^5,vector_sum_pairs(b_exponent_list(5))))))
num_minus_1_squared_by_b_exponents(k) = vecsum(apply((n)->2^n, vector_sum_pairs(b_exponent_list(k))));
check_exact(vector(4,k,my(k=k-0); num_minus_1_squared_by_b_exponents(k)), \
            vector(4,k,my(k=k-0); (num_by_frac(k) - 1)^2), \
            "num_minus_1_squared_by_b_exponents()")


vector(8,k, (num_by_frac(k) - 1)^2/2^2^k)
quit

{
  print("R^2 and transpose");
  my(m=[ a,b; c,d ] * R^2 * opposite_transpose(both_transpose([ a,b; c,d ])));
  print("m="m);
  print("v=",m*[1;1]);
}

{                                                                       a  b   c   d
  check_exact([ a,b; c,d ] * R * mattranspose([ a,b; c,d ]),    \\  a   A  A  C,B   B
              [a^2 + a*b + b^2,   a*c + a*d + b*d;              \\  b      A  C   C,B
               a*c + b*c + b*d,   c^2 + c*d + d^2]);            \\  c          D   D
}                                                               \\  d              D
{
  check_exact([ a,b; c,d ] * R^2 * mattranspose([ a,b; c,d ]),
              [(a+b)^2,               d*(a + b) + a*(c + d);
               c*(a + b) + b*(c + d),              (c+d)^2   ]);
}
{
  check_exact([ a,b; c,d ] * R^3 * mattranspose([ a,b; c,d ]),
              [a^2 + 3*a*b + b^2,   a*c + 3*a*d + b*d;
               a*c + 3*b*c + b*d,   c^2 + 3*c*d + d^2]);
}
{
  check_exact(L * [ a,b; c,d ] * R^2 * mattranspose([ a,b; c,d ]),
              [(a+b)^2,               d*(a + b) + a*(c + d);
               a^2 + 2*b*a + c*a + b^2 + (2*b*c + d*b),
               a*c + 2*a*d + b*d + c^2 + 2*c*d + d^2 ]);
}
{
  check_exact(L * [ a,b; c,d ] * R^2 * mattranspose([ a,b; c,d ]),
              [(a+b)^2,               d*(a + b) + a*(c + d);
               (a+b)^2 + c*(a+b) + b*(c+d),
               a*c + b*d + 2*a*d + (c+d)^2 ]);
}
{
  check_exact(L * [ a,b; c,d ] * R^2 * opposite_transpose(both_transpose([ a,b; c,d ])) * [1;1], \
              [ a^2 + a*b + a*c + a*d   + b^2 + a*b + b*d   + a*d;
               a^2 + 2*a*b + 2*a*c + 2*a*d + b^2 + 2*b*c + 2*b*d + c^2 + 2*d*c + d^2]);
}

\\ if b==c
{
  check_exact([ a,b; b,d ] * R^2 * mattranspose([ a,b; b,d ]), \
              [(a+b)^2,             a*b + 2*a*d + b*d;
               a*b + 2*b*b + b*d,          (b+d)^2 ]);
}

\\ det^2 since R^2==1
check_exact((a+b)^2 * (c+d)^2  -  (d*(a + b) + a*(c + d))*(c*(a + b) + b*(c + d)), \
            a^2*d^2 - 2*a*b*c*d + b^2*c^2);
check_exact((a+b)^2 * (c+d)^2  -  (d*(a + b) + a*(c + d))*(c*(a + b) + b*(c + d)), \
            (a*d - b*c)^2);



print("iterating with a given middle");
{
  my(middle);
  middle = [a,b;c,d];
  middle = R^2;
  my(m=matid(2));
  for(i=1,6,
     v = L*m*[1;1];
     print("v="v);
     print("m="m);
     m = m* middle * mattranspose(m));
}
quit

\\-----------------------------------------------------------------------------

m=(L*R*R*L*L*L*R)*(R*L*L*L*R*R*L)
L*[a,b;c,a]*L
R*[a,b;c,a]*R
L*m*L
R*m*R

LR(n) = if(n==0,L, turn(floor((n+1)/2))==1, R,L);  /* turns duplicated */
LR_name(n) = if(n==0,"L", turn(floor((n+1)/2))==1, "R","L");
vector(30,n,LR_name(n))
L * R*R * R*R * L*L * [1;1]
id=[1,0;0,1];

print("\n")
m = R*R * R*R * L*L
print(binary_string(m))
L * m
L * m * [1;1]

print("\n")
m =       R*R       * R*R       * L*L        *R*R       *  R*R      * L*L       * L*L
print(binary_string(m))
L * m
L * m * [1;1]

print("\n")
m=  R*R * R*R * L*L * R*R * R*R * L*L * L*L  *R*R*  R*R * R*R * L*L * L*L * R*R * L*L * L*L
print(binary_string(m))
L * m
L * m * [1;1]

print("\n")
m=  prod(i=1,62, LR(i));
print(binary_string(m))
L * m
v = L * m * [1;1]
print(binary_string(v))
print(binary_string(L*m))

\\-----------------------------------------------------------------------------

is_pow2(n) = n==2^(length(binary(n))-1);
is_pow2(3)==0 || quit;
is_pow2(4)==1 || quit;
is_pow2(5)==0 || quit;

{
  for(limit=0,66,
     my(v = prod(n=0,limit, LR(n)) * [1;1],
       p = v[1,1],
       q = v[2,1]);
     my(pow2 = if(is_pow2(q),"  ***",""));
     print("n=",limit,"  ",binary_string(v), "  ",p,"/",q,pow2));
}


LR(n) = if(turn(n)==1, R,L);
LR(n) = if(BitAboveLowestOne(n), L,R);  /* turns alone */
LR(n) = if(BitAboveLowestOne(n), R^2,L^2);  /* turns squared */
{
  for(limit=0,33,
     my(m = prod(n=1,limit, LR(n)));
     m = L*m;
     my(v = m * [1;1]);
     my(p = v[1,1],
        q = v[2,1]);
     my(pow2 = if(is_pow2(q),"  ***",""));
     print("n=",limit,"  ",binary_string(v), "  ",p,"/",q,pow2));
}
