\\ Copyright 2016, 2017, 2019 Kevin Ryde
\\
\\ This file is free software; you can redistribute it and/or modify it
\\ under the terms of the GNU General Public License as published by the Free
\\ Software Foundation; either version 3, or (at your option) any later
\\ version.
\\
\\ This file is distributed in the hope that it will be useful, but
\\ WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
\\ or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
\\ for more details.
\\
\\ You should have received a copy of the GNU General Public License along
\\ with this file.  See the file COPYING.  If not, see
\\ <http://www.gnu.org/licenses/>.


default(strictargs,1);
default(recover,0);
read("dragon-defines.gp");

check_count = 0;
check_exact(got,want,name="") =
{
  check_count++;
  if(got!= want,
     print("wrong "name);
     print("got  "got);
     print("want "want);
     quit(1),
     print("ok "check_count" "name));
}

differ(x,y) =
{
  \\ if(type(x)!=type(y),
  \\    return(Strprintf("different types %s %s",type(x),type(y))));
  if(type(x)=="t_COMPLEX" || type(y)=="t_COMPLEX",
     my(d=differ(real(x),real(y)));
     if(d != 0, return(Strprintf("real %s",d)));
     d=differ(imag(x),imag(y));
     if(d != 0, return(Strprintf("imag %s",d)));
     return(0));

  if(type(x)=="t_VEC" || type(x)=="t_COL",
     if(length(x)!=length(y),
        return(Strprintf("different lengths %s %s",length(x),length(y))));
     for(i=1,#x,
        my(d=differ(x[i],y[i]));
        if(d != 0, return(Strprintf("at index %d %s",i,d))));
     return(0));

  if(! nearly_equal(x,y),
     return(Strprintf("different values\ngot  %s\nwant %s",x,y)));
  return(0);
}

check_equal(got,want,name="") =
{
  check_count++;
  my(d = differ(got,want));
  if(d != 0,
     print("wrong ",name,"\n",d); quit(1),
     print("ok ",check_count," ",name));
}

\\-----------------------------------------------------------------------------

delta(k) = norm(PlusPoint(2^k-1) - PlusPoint(2^k));

delta_formula(k) = \
  2*2^k + vector_modulo([-2,-2,-2,0, 2,2,2,0],k)*2^ceil(k/2) + 1;
check_exact(vector(100,k,k--; delta_formula(k)), \
            vector(100,k,k--; delta(k)));

distance_by_n(k) = sum(n=1,2^k-1,  \
                       abs(PlusPoint(n-1) - PlusPoint(n)));
for(k=0,5, print(distance_by_n(k)));

distance(k) = sum(j=0,k-1,  2^j * sqrt(delta_formula(k-1-j)));
check_equal(vector(12,k,k--; distance(k)), \
            vector(12,k,k--; distance_by_n(k)));

distance_mean(k) = distance(k) / (2^k-1);
my(k=5); print(distance_mean(k),"     ",distance(k)/2^k);
print();
print(distance_mean(200));
\\ print(distance_mean(1000));
\\ print(distance_mean(5000));
\\print(distance_mean(10000));

\\ limit 1.932993914006609383641219803
\\ limit 1.93299391400660938364121
\\ cf manhattan 2.243 or maxreim 1.803

\\ norm((dx+dy*I))
\\ norm((dx+dy*I)*b)
distance(6)*2
distance(7)

distance_mean_rearranged(k) =
{
  2^k/(2^k-1) * sum(j=0,k-1,  2^j/2^k * sqrt(delta_formula(k-1-j)));
}
check_equal(vector(12,k, distance_mean_rearranged(k)), \
            vector(12,k, distance_mean(k)));

distance_mean_rearranged(k) =
{
  sum(j=0,k-1, (2^(j-k) + (2^j - 2^(j-k)*(2^k-1))/(2^k-1)) * sqrt(delta_formula(k-1-j)));
}
check_equal(vector(12,k, distance_mean_rearranged(k)), \
            vector(12,k, distance_mean(k)));

distance_mean_rearranged(k) =
{
  sum(j=0,k-1, (2^(j-k) + 1/(2^(2*k-j) - 2^(k-j))) * sqrt(delta_formula(k-1-j)));
}
check_equal(vector(12,k, distance_mean_rearranged(k)), \
            vector(12,k, distance_mean(k)));

distance_mean_rearranged(k) =
{
  PlusDist_by_recurrence(k)/(2^k-1);
}
check_equal(vector(12,k, distance_mean_rearranged(k)), \
            vector(12,k, distance_mean(k)));

distance_mean_rearranged(k) =
{
  2^k/(2^k-1) * sum(j=1,k, abs(1/b^j + I/2^j));
}
check_equal(vector(12,k, distance_mean_rearranged(k)), \
            vector(12,k, distance_mean(k)));

distance_mean_rearranged(k) =
{
  2^k/(2^k-1) * sum(j=1,k, abs(1/b^j - 1/(I*2^j)));
}
check_equal(vector(12,k, distance_mean_rearranged(k)), \
            vector(12,k, distance_mean(k)));

distance_mean_rearranged(k) =
{
  2^k/(2^k-1) * sum(j=1,k, abs(1/sqrt(2)^j - 1/(I*2^j*w8_inexact^-j)));
}
check_equal(vector(12,k, distance_mean_rearranged(k)), \
            vector(12,k, distance_mean(k)));

distance_mean_rearranged(k) =
{
  2^k/(2^k-1) * sum(j=1,k, abs(1/sqrt(2)^j - 1/(2^j*conj(w8_inexact)^(j-2))));
}
check_equal(vector(12,k, distance_mean_rearranged(k)), \
            vector(12,k, distance_mean(k)));

distance_mean_rearranged(k) =
{
  2^k/(2^k-1) * sum(j=1,k, abs(1/sqrt(2)^j - w8_inexact^(j-2)/2^j));
}
check_equal(vector(12,k, distance_mean_rearranged(k)), \
            vector(12,k, distance_mean(k)));

distance_mean_rearranged(k) =
{
  2^k/(2^k-1) * sum(j=1,k, abs(sqrt(2)^j/2^j - w8_inexact^(j-2)/2^j));
}
check_equal(vector(12,k, distance_mean_rearranged(k)), \
            vector(12,k, distance_mean(k)));

distance_mean_rearranged(k) =
{
  2^k/(2^k-1) * sum(j=1,k, abs( (sqrt(2)^j - w8_inexact^(j-2))/2^j ));
}
check_equal(vector(12,k, distance_mean_rearranged(k)), \
            vector(12,k, distance_mean(k)));

distance_mean_rearranged(k) =
{
  2^k/(2^k-1) * sum(j=1,k, abs(
    (sqrt(2)^j - w8_inexact^(j-2))*(sqrt(2)^j - conj(w8_inexact^(j-2)))
     / 2^j / conj(sqrt(2)^j - w8_inexact^(j-2))
  ));
}
check_equal(vector(12,k, distance_mean_rearranged(k)), \
            vector(12,k, distance_mean(k)));

distance_mean_rearranged(k) =
{
  2^k/(2^k-1) * sum(j=1,k, abs(
    (2^j - w8_inexact^(j-2)*sqrt(2)^j - conj(w8_inexact^(j-2))*sqrt(2)^j + 1)
     / 2^j / conj(sqrt(2)^j - w8_inexact^(j-2))
  ));
}
check_equal(vector(12,k, distance_mean_rearranged(k)), \
            vector(12,k, distance_mean(k)));

\\ numerator terms in j mod 8 if try to go to denominator
distance_mean_rearranged(k) =
{
  2^k/(2^k-1) * sum(j=1,k, abs(
    (2^j - w8_inexact^(j-2)*sqrt(2)^j - conj(w8_inexact^(j-2))*sqrt(2)^j + 1)
    / 2^j / conj(sqrt(2)^j - w8_inexact^(j-2))
  ));
}
check_equal(vector(12,k, distance_mean_rearranged(k)), \
            vector(12,k, distance_mean(k)));

distance_mean_rearranged(k) =
{
  2^k/(2^k-1) * sum(j=1,k, abs( 1/sqrt(2)^j + I/(conj(b)*sqrt(2))^j ));
}
check_equal(vector(12,k, distance_mean_rearranged(k)), \
            vector(12,k, distance_mean(k)));

distance_mean_rearranged(k) =
{
  2^k/(2^k-1) * sum(j=1,k, abs( 1/sqrt(2)^j + I/(2*conj(w8_inexact))^j ));
}
check_equal(vector(12,k, distance_mean_rearranged(k)), \
            vector(12,k, distance_mean(k)));

distance_mean_rearranged(k) =
{
  2^k/(2^k-1) * sum(j=1,k, abs( 1/sqrt(2)^j + I/(2*conj(w8_inexact))^j ));
}
check_equal(vector(20,k, distance_mean_rearranged(k)), \
            vector(20,k, distance_mean(k)));

distance_mean_rearranged(k) =
{
  2^k/(2^k-1) * sum(j=1,k, abs( 1/sqrt(2)^j + I*(w8_inexact/2)^j ));
}
check_equal(vector(20,k, distance_mean_rearranged(k)), \
            vector(20,k, distance_mean(k)));

distance_mean_rearranged(k) =
{
  2^k/(2^k-1) * sum(j=1,k, abs( 1/sqrt(2)^j - I/(2*w8_inexact)^j ));
}
check_equal(vector(20,k, distance_mean_rearranged(k)), \
            vector(20,k, distance_mean(k)));

vector(6,j, 1/(1/b^j + I/2^j)/b^j)

\\-----------------------------------------------------------------------------
print("end");