\\ Copyright 2016, 2017, 2019, 2020 Kevin Ryde
\\
\\ This file is free software; you can redistribute it and/or modify it
\\ under the terms of the GNU General Public License as published by the Free
\\ Software Foundation; either version 3, or (at your option) any later
\\ version.
\\
\\ This file is distributed in the hope that it will be useful, but
\\ WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
\\ or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
\\ for more details.
\\
\\ You should have received a copy of the GNU General Public License along
\\ with this file.  See the file COPYING.  If not, see
\\ <http://www.gnu.org/licenses/>.

default(recover,0);
read("../tools/geom.gp");
read("../tools/fracper.gp");
read("../tools/vecper.gp");

check_exact(got,want,name="") =
{
  check_count++;
  if(got!= want,
     print("wrong "name);
     print("got  "got);
     print("want "want);
     quit(1),
     print("ok "check_count" "name));
}

\\-----------------------------------------------------------------------------

sqrt3i = quadgen(-12);
check_exact(sqrt3i^2, -3);
w6 = 1/2 + 1/2*sqrt3i;
w3 = w6^2;
check_exact(w6^3, -1);
check_exact(w6^6, 1);
check_exact(w3^3, 1);
check_exact(imag(w3),1/2);
sqrt3 = geom_sqrt_as_quad(3);
sqrt7 = geom_sqrt_as_quad(7);
sqrt21 = geom_sqrt_as_quad(21);
check_exact(sqrt21^2, 21);
vector_differences(v) = if(#v==0,[], vector(#v-1,i,v[i+1]-v[i]));

\\-----------------------------------------------------------------------------
\\       .     .
\\      / \   / \
\\     /   \ /   \
\\    .     .     .
\\    |     |     |
\\    |     |     |
\\    4*****5*****6
\\   /*\   / \   /*\
\\  / * \ /   \ / * \
\\ .   * .     .   * .
\\ |   * |     |    *|
\\ |    *|     |    *|
\\ .     3*****2     7...
\\  \   / \   /*\   /
\\   \ /   \ / * \ /                 4-->5-->6             6<---7
\\    .     . *   .                  ^       ^             ^
\\    |     | *   |                   \       \           /
\\    |     |*    |                    3-->2   7     0   5<--4
\\    0*****1     .                       /           \       \
\\     \   / \   /                       v             v       v
\\      \ /   \ /                    0-->1             1<--2<--3
\\       .     .


\\ 13,31,37 /42

check_exact(vecper_frac_digits(13/42, 7), [[2], [1]]);
check_exact(vecper_frac_digits(31/42, 7), [[5], [1]]);
check_exact(vecper_frac_digits(37/42, 7), [[6], [1]]);

table =
{[ \\ z, new_rev, add_rot6
  [ \\ pos  rev rot
   [0,       1,  0],  \\ 0
   [1,       2,  1],  \\ 1
   [1+w6,    2,  3],  \\ 2
   [w6,      1,  2],  \\ 3
   [-1+2*w6, 1,  0],  \\ 4
   [2*w6,    1,  0],  \\ 5
   [1+2*w6,  2,  5]   \\ 6
  ],
  [ \\ pos rev rot
   [0,      1,  5],  \\ 0
   [1-w6,   2,  0],  \\ 1
   [2-w6,   2,  0],  \\ 2
   [3-w6,   2,  2],  \\ 3
   [2,      1,  3],  \\ 4
   [1,      1,  1],  \\ 5
   [1+w6,   2,  0]   \\ 6
  ]
]};

b = 2 + w6;

n_to_z(n) =
{
  my(debug=0);
  my(v=digits(n,7), z=0,rev=1,rot=Mod(0,6));
  if(debug, print("n_to_z() n="n" digits="v));
  for(i=1,#v,
     my(elem=table[rev][v[i]+1]);
     z *= b;
     if(debug, print(" i="i" digit="v[i]" rev="rev" rot="rot"  elem="elem));
     z += elem[1]*w6^lift(rot);
     rev = elem[2];
     rot += elem[3];
  );
  if(debug, print(" end rev="rev" rot="rot));
  z;
}
check_exact(n_to_z(0), 0,       "n=0");
check_exact(n_to_z(1), 1,       "n=1");
check_exact(n_to_z(2), 1+w6,    "n=2");
check_exact(n_to_z(3), w6,      "n=3");
check_exact(n_to_z(4), -1+2*w6, "n=4");
check_exact(n_to_z(5), 2*w6,    "n=5");
check_exact(n_to_z(6), 1+2*w6,  "n=6");
check_exact(n_to_z(7), b,       "n=7");
check_exact(n_to_z(8), b+1,     "n=8");
check_exact(n_to_z(9), b+1+w6,  "n=9");
check_exact(n_to_z(14), 1+4*w6, "n=14");
check_exact(n_to_z(49), b^2,    "n=49");
\\quit(0);

\\ {
\\   my(k=3);
\\   my(v=vector(7^k+1,n,n--; n_to_z(n)));
\\   my(xv=apply(real,v));
\\   my(yv=apply(imag,v)*sqrt(3));
\\   plothraw(xv,yv, 1); \\ flag=1 join points
\\ }

fracper_to_flowsnake(v) =
{
  my(debug=0);
  if(debug, print("fracper_to_flowsnake() v="v));

  my(p=v[#v]);   \\ periodic part
  v=v[^#v];      \\ fixed part

  \\ if(#v[#v],
  \\    for(i=1,6, v=fracper_unroll(v));
  \\    if(debug,print("unroll to ",v));
  \\   );
  my(rev=1,rot=Mod(0,6));
  for(i=1,#v,        \\ fixed part
     my(elem=table[rev][v[i]+1]);
     v[i] = elem[1]*w6^lift(rot);
     rev = elem[2];
     rot += elem[3]);

  my(p_revrot=List([]),
     p_out=List([]));
  while(1,
        if(debug,print(" periodic "#p_revrot" at rev="rev" rot="rot
                       "   #p_out="#p_out));
        my(out=vector(#p));
        for(i=1,#p,
           my(elem=table[rev][p[i]+1]);
           if(debug, print("  i="i" digit="p[i]
                           " rev="rev" rot="rot"  elem="elem));
           out[i] = elem[1]*w6^lift(rot);
           rev = elem[2];
           rot += elem[3]);

        my(revrot=[rev,rot]);
        for(i=1,#p_out,
           if(p_revrot[i] == revrot,
              if(debug,print(" periodic repeat at i="i" "revrot));
              listput(p_out,out);
              p_out=Vec(p_out);
              if(debug,print("  p_out="p_out));

              \\ p_out 1 to i inclusive as fixed part
              \\ p_out i+1 to end and this out as periodic part

              v = concat(v,concat(p_out[1..i]));
              if(debug,print("  extend fixed to v="v));

              p_out=if(i<#p_out,concat(p_out[i+1..#p_out]),[]);
              if(debug,print("  periodic concat "p_out));
              v=concat(v,[p_out]);
              if(debug,print("  return "v));
              return(v)));
       listput(p_revrot,revrot);
       listput(p_out,out));
}
fracper_to_flowsnake(frac_to_fracper(13/42, 7))

frac_to_z(f) =
{
  my(v=frac_to_fracper(f,7));
  v=fracper_to_flowsnake(v);
  fracper_to_frac(v,b);
}

2/7 + 2/7^2 + 2/7^3 + 2/7^4 + 2/7^5 + 2/7^6 + 2/7^7.0
digits(floor(1/3*7^20),7)
frac_to_z(1/2)

\\ Brad Klee examples in A261120
{
  check_exact(vector(7,num,num--; frac_to_z(num/6)),
              [ 0,
               1/2 - sqrt3i/6,      \\ lower
               4/7 + 2*sqrt3i/7,
               1/6 + sqrt3i/6,
               1/2 + sqrt3i/2,      \\ top
               1   + sqrt3i/3,      \\ far
               1]);
}
check_exact(frac_to_z(3/8), 1/2 + 1/6*sqrt3i); \\ middle of hexagon

\\ corners, forward
check_exact(frac_to_z(1/6),  1/2 - 1/6*sqrt3i);  \\ lower corner
check_exact(frac_to_z(4/6),   w6);               \\ top corner
check_exact(frac_to_z(5/6),   1  + 1/3*sqrt3i);  \\ far corner
check_exact(frac_to_z(23/42),      1/3*sqrt3i);  \\ near corner

\\        
\\           .  4/6
\\          / \
\\         /   \
\\  23/42 .     . 5/6
\\        |     |
\\        |     |
\\      0 ******* 1
\\         \   /
\\          \ /
\\      1/6  .
\\        
check_exact(23/42, 3/6 + 2/42);
check_exact(23/42 > 3/6, 1);
check_exact(23/42 < 4/6, 1);

print("5,11,17");
check_exact( 5/6, 0+5/6);       \\ forward far corner at +5/6
check_exact(11/6, 1+5/6);       \\ reverse lower corner at +5/6
check_exact(17/6, 2+5/6);       \\ reverse lower corner at +5/6
frac_to_z( 5/42)
frac_to_z(11/42)
frac_to_z(17/42)

print("13,31,37");
check_exact(13/6, 2+1/6);       \\ reverse near corner at +1/6
check_exact(31/6, 5+1/6);       \\ forward lower corner at +1/6
check_exact(37/6, 6+1/6);       \\ reverse near corner at +1/6
frac_to_z(13/42)
frac_to_z(31/42)
frac_to_z(37/42)

print("16,19,25");
check_exact(103/42, 2 + (2+5/7)/6);  \\ reverse far corner at +(2+5/7)/6
check_exact(19/6, 3+1/6);          \\ forward lower corner at +1/6
check_exact(25/6, 4+1/6);          \\ forward lower corner at +1/6
frac_to_z(103/7/42)
frac_to_z(19/42)
frac_to_z(25/42)
\\ for(num=1,42*7-1,print(num," ",frac_to_z(num/42/7)));

print("103,133,175");
\\quit

\\ for(num=1,41,print(num" "frac_to_z(num/42)));

{
  \\ triples   7,15,          255,511
  \\ not     3,     31,63,127,       2047
  my(k=2);
  my(den=6*7^k);
  print("den "den);
  my(m=Map());
  for(num=1,den-1,
     my(z=frac_to_z(num/den));
     my(l=if(mapisdefined(m,z),
             l=mapget(m,z),
             List([])));
     listput(l,num);
     \\ print(num/den"  "z" to "Vec(l));
     mapput(m,z,l));
  m=Mat(m);
  \\print(m);
  for(i=1,matsize(m)[1],
     my(z=m[i,1],l=m[i,2]);
     if(#l==3,
        print(z"  "Vec(l)"   "Vec(l)/den));
  );
  my(v=m[,2]);
  \\ print("column v="v);
  v=select(x->#x==3,v);
  \\print("column len3 v="v);
  print("count ",#v);
  \\ v=apply(x->numerator(x[1]/den),v);
  v=apply(x->x[1],v);
  v=Set(v);
  print(v);
  print(vector_differences(v));
}
\\ A261120  2, 17, 134, 989, 7082, 50057

check_exact((5/7 + 4/21*sqrt3i) * b,   1 + w6 + 1/3*sqrt3i);
check_exact((5/14 + 11/42*sqrt3i) * b,     w6 + 1/3*sqrt3i);


\\     *
\\    / \         top at w6, 60deg
\\   /   \        hex side (1/2)/s=cos(30)
\\  .     .                s = (1/2)/(sqrt3/2) = 1/sqrt3
\\  |     |                                    = sqrt3/3
\\  |     |
\\  0*****1       lower at l/(1/2)=tan(30)
\\   \   /                 l = 1/2*tan(3) = 1/6*sqrt3
\\    \ /
\\     .


\\-----------------------------------------------------------------------------
\\ lambda

\\ exp(-I*acos(5*sqrt(7)/14))
\\ exp(-I*acos(5*sqrt(7)/14)) / sqrt(7)
\\ 5/14 + 1/14*sqrt3i
\\ (5/14 + 1/14*sqrt3i)*b^2
\\ quit

\\ l*w6 - 2*l^2/(1-l) = I/sqrt3
\\ l*(1-l)*w6 - 2*l^2 = I/sqrt3*(1-l)
\\ l*(1-l)*w6 - 2*l^2 - I/sqrt3*(1-l) = 0
\\ l*w6 - l^2*w6 - 2*l^2 - I/sqrt3 + l*I/sqrt3 = 0
\\ (-2-w6)*l^2 + (w6 + I/sqrt3)*l - I/sqrt3 = 0
\\ 
\\ (-2-w6)*l^2 + (w6 + sqrt3i/3)*l - sqrt3i/3
\\ 5*sqrt(7)/14
\\ acos(5*sqrt(7)/14)
\\
\\ 1.0/b
\\ exp(-I*acos(5*sqrt(7)/14)) / sqrt(7)


\\-----------------------------------------------------------------------------
\\ spiral trajectory towards centre

nums={[
    3, 13, 115, 125, 19, 141, 1011, 1021, 7171, 7181, 1027, 7197, 50403,      
    50413, 352915, 352925, 50419, 352941, 2470611, 2470621, 17294371,         
    17294381, 2470627, 17294397, 121060803, 121060813, 847425715,             
    847425725, 121060819, 847425741 
]};

dens={[
    56, 56, 392, 392, 56, 392, 2744, 2744, 19208, 19208, 2744, 19208,         
    134456, 134456, 941192, 941192, 134456, 941192, 6588344, 6588344,         
    46118408, 46118408, 6588344, 46118408, 322828856, 322828856,              
    2259801992, 2259801992, 322828856, 2259801992
]};

gf_terms(g,n) = \
  my(x = variable(g), \
     zeros = min(n,valuation(g,x)), \
     v = Vec(g + O(x^n))); \
  if(zeros>=0, concat(vector(zeros,i,0), v), \
               v[-zeros+1 .. #v]);

gdens(x) = 56*(1+x+7*x^2+7*x^3+x^4+7*x^5) / ((1-7*x^3)*(1+7*x^3));
gnums(x) =
{         (3 +13*x +115*x^2 +125*x^3 +19*x^4 +141*x^5 +861*x^6    
                +371*x^7 +1421*x^8 +931*x^9 +77*x^10 +147*x^11)
  / ((1 -x)*(1+x)*(1 -x +x^2)*(1 +x +x^2)*(1 -7*x^3)*(1 +7*x^3));
}
check_exact(gf_terms(gnums(x),#nums), nums);
check_exact(gf_terms(gdens(x),#dens), dens);

nums=gf_terms(gnums(x),200);
dens=gf_terms(gdens(x),200);

\\ extra +1/sqrt21
\\ nums=concat(0,nums);
\\ dens=concat(1,dens);

spiral_fracs=vector(#nums,i, nums[i]/dens[i]);

frac_core(f) = core(f*denominator(f)^2);  \\ square-free part as integer
spiral_zs = apply(frac_to_z,spiral_fracs);
print("zs ",spiral_zs[1..5]*1.0);
spiral_z_steps = vector_differences(spiral_zs);
print("first step ",spiral_z_steps[1]," norm ",norm(spiral_z_steps[1]));
spiral_step_norms = norm(spiral_z_steps);
\\ apply(issquare, spiral_step_norms)
apply(frac_core, spiral_step_norms)

spiral_steps = apply(geom_abs,spiral_z_steps);
s=vecsum(spiral_steps);
print ("forward length ",s);
print ("forward length ",s*1.0);
\\ 1.40015175052915991648695
print ("forward length ", 5/6 + 3/14*sqrt(7));

check_exact(norm(frac_to_z(3/56)), 1/21)
check_exact(geom_abs(frac_to_z(3/56)), 1/sqrt21)

\\       .     .
\\      / \   / \
\\     /   \ /   \
\\    .     .     .
\\    |  _  |  _  |
\\    |     |     |
\\    4*****5*****6
\\   /*\   / \   /*\
\\  / * \ /   \ / * \
\\ .   * .     .   * .
\\ |  _* |     |  _ *|
\\ |    *|     |    *|
\\ .     3*****2     7...
\\  \   / \   /*\   /
\\   \ /   \ / * \ /                 4-->5-->6             6<---7
\\    .     . *   .                  ^       ^             ^
\\    |  _  |* _  |                   \       \           /
\\    |     |*    |                    3-->2   7     0   5<--4
\\    0*****1     .                       /           \       \
\\     \   / \   /                       v             v       v
\\      \ /   \ /                    0-->1             1<--2<--3
\\       .     .

\\ b + b*w6

forward_spiral_zs = spiral_zs;

check_exact(norm    (1 - frac_to_z(1-3/56)), 1/21)
check_exact(geom_abs(1 - frac_to_z(1-3/56)), 1/sqrt21)

v = [ 1-3/8/7, 5/7+3/8/7, 4/7+3/8/7, 3/7+3/8/7 ];
spiral_fracs = concat(v, apply(f->3/7-f/7, spiral_fracs));

spiral_zs = apply(frac_to_z,spiral_fracs);
spiral_z_steps = vector_differences(spiral_zs);
spiral_step_norms = norm(spiral_z_steps);
spiral_steps = apply(geom_abs,spiral_z_steps);
print("rev steps ",spiral_steps[1..5]);
apply(frac_core, spiral_step_norms)
s=vecsum(spiral_steps);
print("rev length ",s*1.0);
bestappr(real(s)*1.0,200)
bestappr(imag(s)*1.0,200)
print("rev length ",1/2 + 23/42*sqrt(7));

print("total ",5/6 + 3/14*sqrt(7)  + 1/2 + 23/42*sqrt(7));
check_exact(5/6 + 3/14*sqrt7   + 1/2 + 23/42*sqrt7, \
            4/3 + 16/21*sqrt7);

print("forward ",forward_spiral_zs[1..4]*1.0);
print("reverse ",spiral_zs[1..4]*1.0);
v=concat([0,forward_spiral_zs,Vecrev(spiral_zs),[1]]);  \\ forward

print("total inc ends ",geom_length_of_path(v*1.0));
print("total inc ends ", 4/3 + 16/21*sqrt(7) + 2/sqrt(21));
print("total inc ends ", 4/3 + 16/3/sqrt(7) + 2/sqrt(21));

{
  \\ with hexagon around
  my(v=concat([0, 1/2-1/6*sqrt3i,
               1, 1+(1/2-1/6*sqrt3i)*w3,
               w6, (1/2+1/6*sqrt3i)*w6], v));
  if(0,
     plothraw(apply(real,v),apply(imag,v), 1); \\ flag=1 join points
    );
}

{
  print(v[#v]);
  my(v = concat([1/2-1/6*sqrt3i],v));
  a = geom_area_of_polygon(Vecrev(v));  \\ anti-clockwise
  print("area ",a/sqrt3*1.0);
  print("  appr ",bestappr(simplify(a/sqrt3),5000));
  print("  11/48 ", 11/48.0);
}

{
  my(v = concat(v,[ 1+(1/2-1/6*sqrt3i)*w3,
                    w6, (1/2+1/6*sqrt3i)*w6 ]));
  a = geom_area_of_polygon(v);
  print("top area ",a/sqrt3*1.0);
  print("  appr ",bestappr(simplify(a/sqrt3),5000));
  print("  13/48 ", 13/48.0);
}

{
  my(v=v);
  v = v[2..#v];
  v = v[1..#v-4];
  a = geom_area_of_polygon(v);
  print("sans outside ",a/sqrt3*1.0);
  print("  appr ",bestappr(simplify(a/sqrt3),5000));
}

{
  my(v=v);
  v = v[3..#v];
  v = v[1..#v-1];
  a = geom_area_of_polygon(Vecrev(v));
  print("top sans outside ",a/sqrt3*1.0);
  print("  appr ",bestappr(simplify(a/sqrt3),5000));
}
{
  my(v=v);
  v = v[6..#v];
  v = v[1..#v-4];
  a = geom_area_of_polygon(Vecrev(v));
  print("top sans outside ",a/sqrt3*1.0);
  print("  appr ",bestappr(simplify(a/sqrt3),5000));
}

{
  check_exact(geom_area_of_polygon([0, 1/2-1/6*sqrt3i,
                                    1, 1+(1/2-1/6*sqrt3i)*w3,
                                    w6, (1/2+1/6*sqrt3i)*w6]),
              1/2*sqrt3);
}

check_exact(11/48 + 13/48,  1/2);

33/48
15/48

\\ {
\\   my(v=[0, 1/2-1/6*sqrt3i,
\\                                   1, 1+(1/2-1/6*sqrt3i)*w3,
\\                                   w6, (1/2+1/6*sqrt3i)*w6]);
\\   plothraw(apply(real,v),apply(imag,v), 1); \\ flag=1 join points
\\ }

frac_to_z(3/56)
frac_to_z(13/56)
frac_to_z(13/56) - frac_to_z(3/56)
norm(w3)
norm(sqrt3i)
norm(frac_to_z(13/56) - frac_to_z(3/56))
(5/14)^2 + (1/14*sqrt3)^2
