#!/usr/bin/perl -w

# Copyright 2016, 2017 Kevin Ryde
#
# This file is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the Free
# Software Foundation; either version 3, or (at your option) any later
# version.
#
# This file is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
# or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
# for more details.
#
# You should have received a copy of the GNU General Public License along
# with this file.  If not, see <http://www.gnu.org/licenses/>.

use 5.004;
use strict;
use Math::PlanePath::DragonCurve 117; # v.117 for level_to_n_range()

use FindBin;
use lib "$FindBin::Bin/../tools";
use MyDragon ':all';
use MyLineSegments;
use MyPrintwrap ':all';


my $level = 13;

print "  % generated by tools/$FindBin::Script\n";
print "\\begin{scope}\n";

my $path = Math::PlanePath::DragonCurve->new;

my ($n_lo, $n_hi) = $path->level_to_n_range($level);
my ($x_hi,$y_hi) = $path->n_to_xy($n_hi);

 $n_lo = BlobN($level)-3;
$n_hi = BlobN($level+1);

$Printwrap = 2;
printwrap("\\node at (0,0) [my dot,name=start];\n");
# printwrap("\\node at ($x_hi,$y_hi) [my dot,name=end];\n");
my $vec = '';
my $prev_Dpred = 0;
my $Dpred_n;
my $max = 0;
foreach my $n (0 .. $n_hi-1) {
  my ($x,$y) = $path->n_to_xy($n);
  my @n_list = $path->xy_to_n_list($x,$y);
  @n_list = grep {$_ <= $n_hi} @n_list;
  my $Dpred = (@n_list == 2);

  if ($Dpred && ! $prev_Dpred) {
    # start of run
    $Dpred_n = $n;
  } elsif ($prev_Dpred && ! $Dpred) {
    # end of run
    my $len = $n - $Dpred_n - 1;
    if ($len > $max) {
      $max = $len;
      foreach my $n ($Dpred_n .. $n-2) {
        vec($vec,$n,1) = 1;
      }
    }
  }
  $prev_Dpred = $Dpred;
}

my $line = '';
printwrap("\\draw[$line]");
my $join = '';
foreach my $n ($n_lo .. $n_hi-1) {
  my ($x,$y) = $path->n_to_xy($n);

  my ($sdx,$sdy) = $path->n_to_dxdy($n-1);
  my ($tdx,$tdy) = $path->n_to_dxdy($n);
  my $sx = $x . chamfer(-$sdx);
  my $sy = $y . chamfer(-$sdy);
  my $tx = $x . chamfer($tdx);
  my $ty = $y . chamfer($tdy);

  my $want_line = vec($vec,$n,1) ? 'very thick' : '';
  if ($line ne $want_line) {
    $line = $want_line;
    printwrap("$join($sx,$sy);\n");
    printwrap("\\draw[$line]");
    $join = '';
  }
  printwrap("$join($sx,$sy)");
  printwrap("--($tx,$ty)");
  $join = '--';
}
printwrap(";\n");

print "\\end{scope}\n";
print "  % end generated\n";
exit 0;
