#!/usr/bin/perl -w

# Copyright 2014, 2015, 2016, 2017, 2018, 2019 Kevin Ryde
#
# This file is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the Free
# Software Foundation; either version 3, or (at your option) any later
# version.
#
# This file is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
# or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
# for more details.
#
# You should have received a copy of the GNU General Public License along
# with this file.  If not, see <http://www.gnu.org/licenses/>.


use 5.004;
use strict;
use Graph::Maker::TwindragonAreaTree;
use Math::PlanePath::ComplexPlus;
use Math::PlanePath::DragonCurve 117; # v.117 for level_to_n_range()

use FindBin;
use lib $FindBin::Bin;
use lib "$FindBin::Bin/../tools";
use MyLineSegments;
use MyPrintwrap ':all';


my $level = 7;  # of twindragon

print "  % generated by tools/$FindBin::Script\n";
print "  \\newcommand{\\MyLevel}{$level}\n";

sub DOneN {
  my ($m) = @_;
  if ($m==0) { return 0; }
  if ($m % 3 == 0) { return 1/3*(2**($m+2) - ($m%2==0 ? 1 : 2)); }
  if ($m % 3 == 0) { return 1/3*(2**($m+2) + ($m%2==0 ? 2 : -5)); }
  {                  return 1/3*(2**($m+1) - ($m%2==0 ? 2 : 1)); }
  die;
}

my $complex_plus = Math::PlanePath::ComplexPlus->new;
my ($n_lo, $n_hi) = $complex_plus->level_to_n_range($level);

# twindragon area tree
my $segments = MyLineSegments->new;
my $graph = Graph::Maker->new ('twindragon_area_tree',
                               level => $level,
                               undirected => 1);
foreach my $edge ($graph->edges) {
  $segments->add(map {$complex_plus->n_to_xy($_)} @$edge);
}

$Printwrap = 2;
printwrap("\\draw[my tree line] ");
$Printwrap += 2;
$segments->printwrap;
printwrap(";\n");

$Printwrap = 2;
$Printwrap = 4;
foreach my $m (0 .. 2*$level) {
  my $n = DOneN($m);
  next if $n > $n_hi;
  my ($x,$y) = $complex_plus->n_to_xy($n);
  printwrap(" \\node (p$m) at ($x,$y) [my dot];");
}
printwrap_newline();

# $Printwrap = 2;
# {
#   my ($x,$y) = $path->n_to_xy($n_lo);
#   printwrap("\\node at ($x,$y) [my dot,name=start];\n");
# }
# {
#   my ($x,$y) = $path->n_to_xy($n_hi);
#   printwrap("\\node at ($x,$y) [my dot,name=end];\n");
# }
# {
#   my ($x,$y) = $path->n_to_xy($n_hi*3/4);
#   printwrap("\\coordinate (middle) at ($x,$y);\n");
# }

print "  % end generated\n";
exit 0;
