\\ Copyright 2015, 2017 Kevin Ryde
\\
\\ This file is free software; you can redistribute it and/or modify it
\\ under the terms of the GNU General Public License as published by the Free
\\ Software Foundation; either version 3, or (at your option) any later
\\ version.
\\
\\ This file is distributed in the hope that it will be useful, but
\\ WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
\\ or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
\\ for more details.
\\
\\ You should have received a copy of the GNU General Public License along
\\ with this file.  See the file COPYING.  If not, see
\\ <http://www.gnu.org/licenses/>.


default(recover,0);
read("dragon-defines.gp");

\\-----------------------------------------------------------------------------

check_count = 0;
check_exact(got,want,name="") =
{
  check_count++;
  if(got!= want,
     print("wrong "name);
     print("got  "got);
     print("want "want);
     quit(1),
     print("ok "check_count" "name));
}

is_pow2(n) = n==2^(length(binary(n))-1);

\\-----------------------------------------------------------------------------

\\ floor(k/2)
\\ x = A112447 a(2n) = A001045(n+2); a(2n+1) = A001045(n+1).

\\ ceil(k/2)
\\ x,y not in OEIS


rotate_factor(k) = I^ceil(k/2);
fz_to_x(k,z) = abs(real(z*rotate_factor(k)));
fz_to_y(k,z) = abs(imag(z*rotate_factor(k)));

show_some(f) =
{
  for(k=6,26,
     my(z = f(k));
     my(re = abs(real(z)));
     my(im = abs(imag(z)));
     my(n = norm(z));
     my(x = fz_to_x(k,z));
     my(y = fz_to_y(k,z));
     my(have_3 = if(x%3==0 || im%3==0, "3", "*"));
     my(have_4 = if(y%4==0 || im%4==0, "4", "*"));
     my(have_pow2 = if(is_pow2(re) || is_pow2(im), "pow2", "*"));
     my(bits = hammingweight(y));
     print1("k="k" norm="n"   ",x," ",y,"  ",have_3,have_4," ",have_pow2);
     if(have_pow2,
        print1("  x="x" ["to_binary(x)"]"));
     print();
  );
}

P5_to_P6(k) = P5(k) - P6(k);
P5_to_P6_norm(k) = norm(P5_to_P6(k));

P9_to_P10(k) = P9(k) - P10(k);
P9_to_P10_norm(k) = norm(P9_to_P10(k));

\\ never a square
\\ for(k=0,100000,if(issquare(P5_to_P6_norm(k)),print("k="k" P5-P6 norm="P5_to_P6_norm(k))));

show_some(P9_to_P10);

\\-----------------------------------------------------------------------------

{
  print1("x  ");
  for(k=6,20,
     my(z = P9_to_P10(k));
     my(x = fz_to_x(k,z));
     my(y = fz_to_y(k,z));
     print1(x,","));
  print("...");
}

{
  print1("y  ");
  for(k=6,20,
     my(z = P9_to_P10(k));
     my(x = fz_to_x(k,z));
     my(y = fz_to_y(k,z));
     print1(y,","));
  print("...");
}
\\ y = A112447

{
  print1("[");
  for(k=6,20,
     my(z = P5_to_P6(k));
     my(n = norm(z));
     print1(n,","));
  print("]");
}
\\ y = A112447

{
  print();
  print("P8-P3 less P6-P5");
  for(k=6,20,
     my(f = I^-ceil(k/2));
     print(f*(P8(k)-P3(k) - (P6(k)-P5(k))));
  );
}


\\-----------------------------------------------------------------------------

P5_to_P6_norm(k) = 
{
  13/18 * 2^k
  + 1/9 * vector_modulo([11, 4, -11, -4],k) * 2^floor(k/2)
  + 1/9 * vector_modulo([5, 1],k);
}
check_exact(vector(100,k,k+=5; P5_to_P6_norm(k)), \
            vector(100,k,k+=5; norm(P5_to_P6(k))))

P9_to_P10_norm(k) = 
{
  13/36 * 2^k
  + 1/9 * vector_modulo([-2, 11, 2, -11],k) * 2^floor(k/2)
  + 1/9 * vector_modulo([1, 5],k);
}
check_exact(vector(100,k,k+=5; P9_to_P10_norm(k)), \
            vector(100,k,k+=5; norm(P9_to_P10(k))))

\\-----------------------------------------------------------------------------
\\ boundary

polygon_increments(v) = vector(#v,i, v[if(i==#v,1,i+1)] - v[i]);

polygon_length(v) = vecsum(apply(abs,polygon_increments(v)));
check_exact(polygon_length([0,1]), 2);
check_exact(polygon_length([0,1,1+I,I]), 4);

core_of_frac(x) = core(numerator(x) * denominator(x));

{
  my(v = hull_fractal);
  my(one = 0);
  my(two = 0);
  my(thirteen = 0);
  my(twentysix = 0);
  v = polygon_increments(v);
  for(i=1,#v,
     my(z = v[i]);
     my(n = norm(z));
     my(s = core_of_frac(n));
     my(f = n / s);
     issquare(f,&f) || error();
     print(f" * sqrt("s")   "abs(z));

     if(s==1, one += f,
        s==2, two += f,
        s==13, thirteen += f,
        s==26, twentysix += f,
        error("sqrt of ",s));
  );

  print(one" + "two"*sqrt(2) + "thirteen"*sqrt(13) + "twentysix"*sqrt(26)");
  my(l = one + two*sqrt(2) + thirteen*sqrt(13) + twentysix*sqrt(26));
  print(l);
  print(polygon_length(hull_fractal));

  print();
  print(abs(Benedek_Panzone_P9 - Benedek_Panzone_P10));
  print(abs(Benedek_Panzone_P8 - Benedek_Panzone_P9));
  print(abs(Benedek_Panzone_P7 - Benedek_Panzone_P8));
  print(abs(Benedek_Panzone_P6 - Benedek_Panzone_P7));
  print(abs(Benedek_Panzone_P5 - Benedek_Panzone_P6));
  print(abs(Benedek_Panzone_P4 - Benedek_Panzone_P5));
  print(abs(Benedek_Panzone_P3 - Benedek_Panzone_P4));
  print(abs(Benedek_Panzone_P2 - Benedek_Panzone_P3));
  print(abs(Benedek_Panzone_P1 - Benedek_Panzone_P2));
  print(abs(Benedek_Panzone_P10 - Benedek_Panzone_P1));

  print(abs(Benedek_Panzone_P9 - Benedek_Panzone_P10)
      + abs(Benedek_Panzone_P8 - Benedek_Panzone_P9)
      + abs(Benedek_Panzone_P7 - Benedek_Panzone_P8)
      + abs(Benedek_Panzone_P6 - Benedek_Panzone_P7)
      + abs(Benedek_Panzone_P5 - Benedek_Panzone_P6)
      + abs(Benedek_Panzone_P4 - Benedek_Panzone_P5)
      + abs(Benedek_Panzone_P3 - Benedek_Panzone_P4)
      + abs(Benedek_Panzone_P2 - Benedek_Panzone_P3)
      + abs(Benedek_Panzone_P1 - Benedek_Panzone_P2)
      + abs(Benedek_Panzone_P10 - Benedek_Panzone_P1));
}
  
\\-----------------------------------------------------------------------------
z_is_straight(z) = real(z)==0 || imag(z)==0;
HB_ones_ref(k) = \
  vecsum(apply(abs,select(z_is_straight,polygon_increments(hull(k)))));

HB_ones(k) =
{
  1/6 * vector_modulo([9, 10],k) * 2^floor(k/2)
  + 2/3 * vector_modulo([0, -1, 0, 1],k);
}
check_exact(vector(100,k,k+=5; HB_ones(k)), \
            vector(100,k,k+=5; HB_ones_ref(k)))

\\-----------------------------------------------------------------------------
z_is_diagonal(z) = abs(real(z)) == abs(imag(z));
HB_twos_ref(k) =
{
  my(v = hull(k));
  v = polygon_increments(v);
  v = select(z_is_diagonal,v);
  v = apply(z->my(n = norm(z));
           n /= 2;
           issquare(n,&n) || error("n="n);
           n,
           v);
  vecsum(v);
}

HB_twos(k) =
{
  1/6 * vector_modulo([5, 9],k) * 2^floor(k/2)
  - 1/3 * vector_modulo([7, 6, 5, 6],k);
}
check_exact(vector(100,k,k+=5; HB_twos(k)), \
            vector(100,k,k+=5; HB_twos_ref(k)))

{
  for(k=6,20,
     my(t = HB_twos(k+1));
     my(o = HB_ones(k));
     print(t"  "o"  "t-o)
     );
}

\\-----------------------------------------------------------------------------

HB_vector(k) =
{
  [ HB_ones(k),
   HB_twos(k)*sqrt2,
   sqrt_as_quad(P5_to_P6_norm(k)),
   sqrt_as_quad(P9_to_P10_norm(k)) ];
}

HB_inexact(k) = vecsum(apply(x->x*1.0, HB_vector(k)));

{
  print();
  print("HB");
  for(k=6,20,
     my(h = HB_inexact(k));
     print(h"   ",h/sqrt(2)^k);
  );
}
my(k=100); print(HB_inexact(k)/sqrt(2)^k);

\\-----------------------------------------------------------------------------
system("sleep 1");