#!/usr/bin/perl -w

# Copyright 2014, 2015, 2016, 2017 Kevin Ryde
#
# This file is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the Free
# Software Foundation; either version 3, or (at your option) any later
# version.
#
# This file is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
# or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
# for more details.
#
# You should have received a copy of the GNU General Public License along
# with this file.  If not, see <http://www.gnu.org/licenses/>.


use 5.004;
use strict;
use FindBin;
use File::chdir;
use File::stat;
use File::Temp;
use IPC::Run;
use List::Util 'max','min';
use Math::PlanePath::DragonCurve 117; # v.117 for level_to_n_range()


my $level = 16;
my $chamfer = .2;
my $AlphaBits = 4;   # 1,2,4

my $version = 0;
foreach my $arg (@ARGV) {
  if ($arg =~ /^VERSION=(.*)/) {
    $version = $1;
  }
}

my $tempdir = "/tmp";
local $CWD = $tempdir;

open my $tex_fh, '>', 'image.tex' or die;

# scale=.129 for k=12

print $tex_fh <<'HERE';
\documentclass[a4paper]{article}
\usepackage[a4paper,margin=1cm]{geometry}
\pagestyle{empty}
\usepackage{tikz}
\tikzset{>=latex}       % arrowhead style
\begin{document}
\begin{tikzpicture}[scale=.072,rotate=90]
  \filldraw (0,0) circle(.1);
  \filldraw (16,0) circle(.1);
  \node at (3,1.5) [above=1ex,font=\scriptsize] (start-text) {start};
  \node at (18,-2) [below=1ex,font=\scriptsize] (end-text) {end};

  % \draw[->,dashed,shorten >=.5em]
  %   (start-text.-90) .. controls +(-90:3) and +(-45:3) .. (0,0);
  % \draw[->,dashed,shorten >=.5em]
  %   (end-text.180) .. controls +(180:2) and +(-130:4) .. (16,0);

  \draw[very thin]
HERE

my $path = Math::PlanePath::DragonCurve->new;
my ($n_lo, $n_hi) = $path->level_to_n_range($level);
my $join = '';
my $split_count = 0;
foreach my $n ($n_lo .. $n_hi) {
  my ($x,$y) = $path->n_to_xy($n);

  if ($n != $n_lo && $n != $n_hi) {
    my ($dx,$dy) = $path->n_to_dxdy($n);
    my ($prev_dx,$prev_dy) = $path->n_to_dxdy($n-1);
    my $sx = $x - $chamfer*$prev_dx;
    my $sy = $y - $chamfer*$prev_dy;
    my $tx = $x + $chamfer*$dx;
    my $ty = $y + $chamfer*$dy;
    print $tex_fh "  $join($sx,$sy)\n";
    if ($split_count++ > 16) {
      $split_count = 0;
      my $mx = $x + $dx/2;
      my $my = $y + $dy/2;
      print $tex_fh "  --($mx,$my); \\draw ($mx,$my)";
    }
    print $tex_fh "  --($tx,$ty)\n";
  } else {
    print $tex_fh "  $join($x,$y)\n";
  }
  $join = '--';
}
print $tex_fh ";\n";


# {
#   # dots at all ven points
#   foreach my $n ($n_lo .. $n_hi) {
#     my ($x,$y) = $path->n_to_xy($n);
#
#     if ($n % 2 == 0) {
#       print $tex_fh "  \\filldraw ($x,$y)circle(.1);\n";
#     }
#     $join = '--';
#   }
# }


print $tex_fh <<'HERE';
\end{tikzpicture}
\end{document}
HERE
close $tex_fh or die;
# IPC::Run::run(['cat','image.tex']);

IPC::Run::run(['latexmk','-pdf','image.tex'], '<',File::Spec->devnull)
  or die "latexmk";

IPC::Run::run(['pdfcrop',
               '--verbose',
               '--margins','10',
               'image.pdf',
               'image-cropped.pdf'],
              '<', File::Spec->devnull)
  or die;

exit 0;
