\\ Copyright 2017 Kevin Ryde
\\
\\ This file is free software; you can redistribute it and/or modify it
\\ under the terms of the GNU General Public License as published by the Free
\\ Software Foundation; either version 3, or (at your option) any later
\\ version.
\\
\\ This file is distributed in the hope that it will be useful, but
\\ WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
\\ or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
\\ for more details.
\\
\\ You should have received a copy of the GNU General Public License along
\\ with this file.  See the file COPYING.  If not, see
\\ <http://www.gnu.org/licenses/>.


read("pol-pfrac.gp");
read("dragon-defines.gp");

default(recover,0);
check_count = 0;
check_exact(got,want,name="") =
{
  check_count++;
  if(got!= want,
     print("wrong "name);
     print("got  "got);
     print("want "want);
     quit(1),
     print("ok "check_count" "name));
}


\\-----------------------------------------------------------------------------


BoundaryPathLen_ref(k) =
{
  my(v=BlobList(k));
  TotalNonBlob(k)
  + sum(i=1,#v,my(j=v[i]); min(BlobL(j),BlobR(j)));
}
print("BoundaryPathLen ",  vector(16,k,my(k=k-1); BoundaryPathLen_ref(k)));

BoundaryPathLen(k) = \
  my(v=BlobList(k)); TotalNonBlob(k) + sum(i=1,#v,my(j=v[i]); BlobL(j));
check_exact(vector(64,k,my(k=k-1); BoundaryPathLen(k)), \
            vector(64,k,my(k=k-1); BoundaryPathLen_ref(k)));

BoundaryPathLen(k) =
{
  if(k==0, return(1));
  1/4*(dJA(k+6) - (-1)^k) + k - 1;
}
check_exact(vector(16,k,my(k=k-1); BoundaryPathLen(k) - BoundaryPathLen_ref(k)), \
            vector(16,k,my(k=k-1); 0));

\\ 2*JA(k) == dJA(k+2) - 1
BoundaryPathLen(k) =
{
  if(k==0, return(1));
  1/2*(JA(k+4) + (k%2)) + k - 1;
}
check_exact(vector(16,k,my(k=k-1); BoundaryPathLen(k) - BoundaryPathLen_ref(k)), \
            vector(16,k,my(k=k-1); 0));


\\-----------------------------------------------------------------------------

BlobL_cumulative(k) = sum(j=0,k,BlobL(j));
print("BlobL_cumulative       ",  vector(16,k,my(k=k-1); BlobL_cumulative(k)));

gBlobL_cumulative(x) = 1/(1-x)*gBlobL(x);
check_exact(gf_terms(gBlobL_cumulative(x),64), \
            vector(64,k,my(k=k-1); BlobL_cumulative(k)));

BlobL_cumulative_ksub2(k) = BlobL_cumulative(k-2);
print("BlobL_cumulative_ksub2 ",  vector(16,k,my(k=k-1); BlobL_cumulative_ksub2(k)));

gBlobL_cumulative_ksub2(x) = gBlobL_cumulative(x)*x^2;
check_exact(gf_terms(gBlobL_cumulative_ksub2(x),64), \
            vector(64,k,my(k=k-1); BlobL_cumulative_ksub2(k)));

gBoundaryPathLen_ref(x) = \
  gBlobL_cumulative(x) + gBlobL_cumulative_ksub2(x) + gTotalNonBlob(x);
check_exact(gf_terms(gBoundaryPathLen_ref(x),64), \
            vector(64,k,my(k=k-1); BoundaryPathLen(k)));


gBoundaryPathLen(k) = \
  gBlobL_cumulative(x) + gBlobL_cumulative_ksub2(x) + gTotalNonBlob(x);
check_exact(gBoundaryPathLen(x), gBoundaryPathLen_ref(x));

gBoundaryPathLen(k) =
{
  14 + 9*x + 5*x^2 + 3*x^3 + x^4
  - 19/(1-x)
  + 6/(1-x)^2
  + 1/(1-x)* x^4 * (3*x + 3*x^2 + 5*x^3 + 3*x^4 + 4*x^5 + 2*x^6) / (1 - x - 2*x^3 - x^4 + x^5 + 2*x^7)
  + 1/(1-x)* x^6 * (3*x + 3*x^2 + 5*x^3 + 3*x^4 + 4*x^5 + 2*x^6) / (1 - x - 2*x^3 - x^4 + x^5 + 2*x^7)
  ;
}
check_exact(gBoundaryPathLen(x), gBoundaryPathLen_ref(x));

gBoundaryPathLen(k) =
{
  1
  -      2/(1 - x)
  +      1/(1 - x)^2
  - 1/4* 1/(1 + x)
  + 1/4* (5 + 2*x + 6*x^2)/(1 - x - 2*x^3)
  ;
}
check_exact(gBoundaryPathLen(x), gBoundaryPathLen_ref(x));

{
check_exact(x^4 + x^3 + 3*x^2 + 7/2*x + 5
            + 6/(x - 1)
            + 1/(x^2 - 2*x + 1)
            + -1/(4*x + 4)
            + -1/(8*x^3 + 4*x - 4)
            ,
            x^4*gBoundaryPathLen_ref(x));
}
{
check_exact(1
            + 1/x^6 * (x^2*(5 + 7/2*x + 3*x^2 + x^3)
                       + x^2*6/(x - 1)
                       + x^2*1/(1-x)^2
                       - x^2*1/4 * 1/(1+x)
                       + 1/4 * gdJA(x)
                       )
            ,
            gBoundaryPathLen_ref(x));
}
{
check_exact(1
            + 1/x^6 * 1/4 * (gdJA(x) - x^2 - x^3 - x^4 - 3*x^5)
            + 1/x^6 * (2*x^5 + 3*x^4 + 4*x^3 + 5*x^2
                       + x^2 * 6/(x - 1)
                       + x^2 * 1/(1-x)^2
                       - x^6*1/4 * 1/(1+x)
                       )
            ,
            gBoundaryPathLen_ref(x));
}
{
check_exact(1
            + 1/4 * 1/x^6*(gdJA(x) - x^2 - x^3 - x^4 - 3*x^5)
            + 1/(1-x)^2
            - 2/(1 - x)
            - 1/4 * 1/(1+x)
            ,
            gBoundaryPathLen_ref(x));
}


\\-----------------------------------------------------------------------------

BlobL_to_end_ref(k) =
{
  if (k < 4, return(0));
  BlobL(k) + sum(j=4,k-2, BlobL(j)+3) + if(k==4,5,7);
}
print("BlobL_to_end_ref ",  vector(16,k,my(k=k-1); BlobL_to_end_ref(k)));

BlobB_half(k) = BlobB(k)/2;
print("BlobB_half       ",  vector(16,k,my(k=k-1); BlobB_half(k)));

diff(k) = BlobL_to_end_ref(k) - 1/2*BlobB(k);
\\ vector(500,k,my(k=k-1); diff(k));

gBlobL_to_end(x) = gBlobL(x) + 1/(1-x)*gBlobL(x)*x^2 \
                  + 3/(1-x)^2 - 11/(1-x) \
                  + 8 + 5*x + 2*x^2 - x^3 + x^4;
\\ gf_terms(gBlobL_to_end(x),20) - vector(20,k,k--; BlobL_to_end_ref(k))
check_exact(gf_terms(gBlobL_to_end(x),100), \
            vector(100,k,k--; BlobL_to_end_ref(k)));

gdiff(x) = gBlobL_to_end(x) - 1/2*gBlobB(x);
check_exact(gf_terms(gdiff(x),100), \
            vector(100,k,k--; diff(k)));
pol_ascending_print(pol_partial_fractions(gdiff(x)))

check_exact(-1*gdJA(x)/x^2, (-1)/(1 - x - 2*x^3))

diff_by_JA(k) = {
  1/8*(
       -dJA(k+2) - 2*dJA(k+1) + 10*dJA(k)
       + vector_modulo([-3,3],k)
       + vector_modulo([4,-4,-4,4],k)
       + 4*(k+1)
       + 4
  );
}
vector(20,k,k--; diff(k)) - vector(20,k,k--; diff_by_JA(k))
\\gf_terms((-1 - 2*x + 10*x^2)/(1 - x - 2*x^3),15)

diff_by_JA(k) = {
  if(k<5,return(diff(k)));
  1/8*(
       13*dJA(k-3) + 10*dJA(k-4) - 2*dJA(k-5)
       + vector_modulo([9, 7, 1, 15],k)
       + 4*k
  );
}
vector(20,k,k--; diff(k)) - vector(20,k,k--; diff_by_JA(k))



\\-----------------------------------------------------------------------------

\\ BoundaryPathLenR(k) = \
\\   my(v=BlobList(k)); TotalNonBlob(k) + sum(i=1,#v,my(j=v[i]); BlobR(j));
\\ print("BoundaryPathLenR ",  vector(56,k,my(k=k-1); BoundaryPathLenR(k)));
\\ 
\\      \\ + -1
\\      \\ + (-1)/(1 - 2*x + x^2)
\\      \\ + (1/4)/(1 + x)
\\      \\ + (11/4 + 3/2*x + 5/2*x^2)/(1 - x - 2*x^3)
\\ 
\\ BoundaryPathLenRLdiff(k) = \
\\   my(v=BlobList(k)); TotalNonBlob(k) + sum(i=1,#v,my(j=v[i]); BlobR(j) - BlobL(j));
\\ print("BoundaryPathLenRLdiff ",  vector(56,k,my(k=k-1); BoundaryPathLenRLdiff(k)));
\\ \\ 2*gJA(x)*x^-3
\\      \\ + 12 + 9*x + 5*x^2 + 3*x^3 + x^4
\\      \\ + (-17)/(1 - x)
\\      \\ + (4)/(1 - 2*x + x^2)
\\      \\ + (1/2)/(1 + x)
\\      \\ + (3/2 + x + x^2)/(1 - x - 2*x^3)
