\\ Copyright 2014, 2015, 2017 Kevin Ryde
\\
\\ This file is free software; you can redistribute it and/or modify it
\\ under the terms of the GNU General Public License as published by the Free
\\ Software Foundation; either version 3, or (at your option) any later
\\ version.
\\
\\ This file is distributed in the hope that it will be useful, but
\\ WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
\\ or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
\\ for more details.
\\
\\ You should have received a copy of the GNU General Public License along
\\ with this file.  See the file COPYING.  If not, see
\\ <http://www.gnu.org/licenses/>.


read("../tools/memoize.gp");
read("pol-pfrac.gp");
read("dragon-defines.gp");

default(recover,0);
check_count = 0;
check_exact(got,want,name="") =
{
  check_count++;
  if(got!= want,
     print("wrong "name);
     print("got  "got);
     print("want "want);
     quit(1),
     print("ok "check_count" "name));
}

print("HQL         ",vector(16,k,k--; HQL(k)));
print("HQ2L        ",vector(16,k,k--; HQ2L(k)));
print("HQ4L        ",vector(16,k,k--; HQ4L(k)));
print("HQR         ",vector(16,k,k--; HQR(k)));
print("HQ2R        ",vector(16,k,k--; HQ2R(k)));
print("HQ4R        ",vector(16,k,k--; HQ4R(k)));
print();

print("            ",vector(16,k,k--; k));
print("BlobL       ",vector(16,k,k--; BlobL(k)));
print("BlobHQL     ",vector(16,k,k--; BlobHQL(k)));
print("BlobHQ2L    ",vector(16,k,k--; BlobHQ2L(k)));
print("BlobHQ4L    ",vector(16,k,k--; BlobHQ4L(k)));
print("BlobHQAL    ",vector(16,k,k--; BlobHQAL(k)));
print("BlobLnohang ",vector(16,k,k--; BlobLnohang(k)));
print();

print("BlobR       ",vector(16,k,k--; BlobR(k)));
print("BlobHQR     ",vector(16,k,k--; BlobHQR(k)));
print("BlobHQ2R    ",vector(16,k,k--; BlobHQ2R(k)));
print("BlobHQ4R    ",vector(16,k,k--; BlobHQ4R(k)));
print("BlobHQAR    ",vector(16,k,k--; BlobHQAR(k)));
print("BlobRnohang ",vector(16,k,k--; BlobRnohang(k)));

diff(k) = BlobRnohang(k) - BlobLnohang(k);
gdiff(k) = gBlobRnohang(k) - gBlobLnohang(k);
print("diff        ",vector(16,k,k--; diff(k)))

\\-----------------------------------------------------------------------------

BoundaryPathLen_ref(k) =
{
  my(v=BlobList(k));
  NonBlobSegments(k)
  + sum(i=1,#v,my(j=v[i]); min(BlobLnohang(j),BlobRnohang(j)));
}
print("BoundaryPathLen        ", vector(16,k,my(k=k-1); BoundaryPathLen_ref(k)));

BlobLnohang_cumulative(k) = sum(j=0,k,BlobLnohang(j));
print("BlobLnohang_cumulative ", vector(16,k,my(k=k-1); BlobLnohang_cumulative(k)));

gBlobLnohang_cumulative(x) = 1/(1-x)*gBlobLnohang(x);
check_exact(gf_terms(gBlobLnohang_cumulative(x),64), \
            vector(64,k,my(k=k-1); BlobLnohang_cumulative(k)));

  gBlobLnohang_cumulative(x) + gBlobLnohang_cumulative(x) + gNonBlobSegments(x);

gBoundaryPathLen_ref(x) = \
  gBlobLnohang_cumulative(x) + x^2*gBlobLnohang_cumulative(x) \
  + gNonBlobSegments(x);
check_exact(gf_terms(gBoundaryPathLen_ref(x),10), \
            vector(10,k,my(k=k-1); BoundaryPathLen_ref(k)));

BoundaryPathLen(k) = \
  my(v=BlobList(k)); NonBlobSegments(k) + sum(i=1,#v,my(j=v[i]); BlobLnohang(j));
check_exact(vector(100,k,my(k=k-1); BoundaryPathLen(k)), \
            vector(100,k,my(k=k-1); BoundaryPathLen_ref(k)));

gBoundaryPathLen(x) = \
  10 + 5*x + 4*x^2 + 2*x^3 \
  - 13 * 1/(1 - x) \
  + 4 * 1/(1-x)^2 \
  - 1/2 * 1/(1 + x) \
  + 1/2 * (1 + 2*x)/(1 - x - 2*x^3);
check_exact(gBoundaryPathLen(x), gBoundaryPathLen_ref(x))

gBoundaryPathLen(x) = \
  1 + 3*x^2 + 2*x^3 \
  - 1 * x^3/(1 - x) \
  + 4 * x^3/(1-x)^2 \
  - 1/2 * 1/(1 + x) \
  + 1/2*x^-2*gdJA(x) + 1*x^-1*gdJA(x);
print("gBoundaryPathLen(x)");
pol_ascending_print(pol_partial_fractions(gBoundaryPathLen_ref(x)))
pol_ascending_print(pol_partial_fractions(gBoundaryPathLen(x)))
check_exact(gBoundaryPathLen(x), gBoundaryPathLen_ref(x))
check_exact(gf_terms(gBoundaryPathLen(x),100), \
            vector(100,k,my(k=k-1); BoundaryPathLen_ref(k)));

BoundaryPathLen(k) =
{
  if(k==0, return(1));
  if(k==1, return(2));
  if(k==2, return(4));
  if(k==3, return(8));
  1/2*dJA(k+2) + dJA(k+1) + -1/2*(-1)^k + 4*k - 9
 ;
}
check_exact(vector(16,k,my(k=k-1); BoundaryPathLen(k) - BoundaryPathLen_ref(k)), \
            vector(16,k,my(k=k-1); 0));

\\ JA(k) = JA(k-1) + 2*JA(k-3) + 1
\\ 2*JA(k) == dJA(k+2) - 1
BoundaryPathLen(k) =
{
  if(k==0, return(1));
  if(k==1, return(2));
  if(k==2, return(4));
  if(k==3, return(8));
  JA(k) + 2*JA(k-1) + 4*k - vector_modulo([8,7],k);
}
check_exact(vector(100,k,my(k=k-1); BoundaryPathLen(k) - BoundaryPathLen_ref(k)), \
            vector(100,k,my(k=k-1); 0));

gBoundaryPathLen(x) = \
  1 + 2*x + 4*x^2 + 2*x^3 \
  + 1/2 * x^3/(1 - x) \
  + 4 * x^3/(1-x)^2 \
  + 1/2 * x^3/(1 + x) \
  + gJA(x) + 2*x*gJA(x);
check_exact(gBoundaryPathLen(x), gBoundaryPathLen_ref(x))
check_exact(gf_terms(gBoundaryPathLen(x),100), \
            vector(100,k,my(k=k-1); BoundaryPathLen_ref(k)));


\\-----------------------------------------------------------------------------


\\-----------------------------------------------------------------------------

BlobLnohang_to_end_ref(k) =
{
  if (k < 4, return(0));
  BlobLnohang(k) + sum(j=4,k-2, BlobLnohang(j)+3) + if(k==4,5,7);
}
print("BlobLnohang_to_end_ref ",  vector(16,k,my(k=k-1); BlobLnohang_to_end_ref(k)));

BlobBnohang_half(k) = BlobBnohang(k)/2;
print("BlobBnohang_half       ",  vector(16,k,my(k=k-1); BlobBnohang_half(k)));

diff(k) = BlobLnohang_to_end_ref(k) - 1/2*BlobBnohang(k);
print("diff                   ",  vector(16,k,my(k=k-1); diff(k)));

gBlobLnohang_to_end(x) = gBlobLnohang(x) + 1/(1-x)*gBlobLnohang(x)*x^2 \
                  + 3/(1-x)^2 - 11/(1-x) \
                  + 8 + 5*x + 2*x^2 - x^3 + x^4;
\\ gf_terms(gBlobLnohang_to_end(x),20) - vector(20,k,k--; BlobLnohang_to_end_ref(k))
check_exact(gf_terms(gBlobLnohang_to_end(x),100), \
            vector(100,k,k--; BlobLnohang_to_end_ref(k)), \
            "gBlobLnohang_to_end");

\\ pol_ascending_print(pol_partial_fractions(gBlobLnohang_to_end_ref(x)))
\\ pol_ascending_print(pol_partial_fractions(gBlobLnohang_to_end(x)))
gdiff(x) = gBlobLnohang_to_end(x) - 1/2*gBlobBnohang(x);
check_exact(gf_terms(gdiff(x),100), \
            vector(100,k,k--; diff(k)), \
            "gdiff(x)");
print("gdiff(x)");
pol_ascending_print(pol_partial_fractions(gdiff(x)))

check_exact(-1*gdJA(x)/x^2, (-1)/(1 - x - 2*x^3))

diff_by_JA(k) = {
  if(k<4,return(0));
  if(k==4,return(5));

    5/4*dJA(k+2) - 1/2*dJA(k+1) - 5/2*dJA(k)
  + 2*k - 3 + (-3/4)*(-1)^k
  + vector_modulo([1/2,1/2, -1/2,-1/2],k)
  ;
}
check_exact(vector(100,k,my(k=k-1); diff_by_JA(k)), \
            vector(100,k,my(k=k-1); diff(k)));

\\ 2*JA(k) == dJA(k+2) - 1
diff_by_JA(k) = {
  if(k<4,return(0));
  if(k==4,return(5));
 
    5/2*JA(k) - JA(k-1) - 5*JA(k-2)
  + 2*k 
  + vector_modulo([-5, -7/2, -6, -9/2],k)
  ;
}
check_exact(vector(100,k,my(k=k-1); diff_by_JA(k)), \
            vector(100,k,my(k=k-1); diff(k)));

\\ 2*JA(k) == dJA(k+2) - 1
\\ JA(k) = JA(k-1) + 2*JA(k-3) + 1
\\ JA(k) - JA(k-1) - 1 = 2*JA(k-3)
diff_by_JA(k) = {
  if(k<4,return(0));
  if(k==4,return(5));
 
  - 5/2*JA(k+1) + 5*JA(k) - JA(k-1)
  + 2*k 
  + vector_modulo([-5/2, -1, -7/2, -2],k)
  ;
}
vector(20,k,k--; diff(k)) - vector(20,k,k--; diff_by_JA(k))
check_exact(vector(100,k,my(k=k-1); diff_by_JA(k)), \
            vector(100,k,my(k=k-1); diff(k)));

diff_by_JA(k) = {
  if(k<4,return(0));
  if(k==4,return(5));
  if(k==5,return(7));
  if(k==6,return(8));
  if(k==7,return(16));
 
  1/2*JA(k-6) + 9*JA(k-7) - 5*JA(k-8)
  + 2*k  
  + vector_modulo([-1, 1/2, -2, -1/2],k)
  ;
}
vector(20,k,k--; diff(k)) - vector(20,k,k--; diff_by_JA(k))
check_exact(vector(20,k,my(k=k-1); diff_by_JA(k)), \
            vector(20,k,my(k=k-1); diff(k)));

\\ Diameter grows as sqrt(2)^k = 1.4, BoundaryPathLen grows as r^k = 1.69
\\ for(k=0,60, print(BoundaryPathLen(k)/Diameter(k)*1.0))