\\ Copyright 2014, 2017 Kevin Ryde
\\
\\ This file is free software; you can redistribute it and/or modify it
\\ under the terms of the GNU General Public License as published by the Free
\\ Software Foundation; either version 3, or (at your option) any later
\\ version.
\\
\\ This file is distributed in the hope that it will be useful, but
\\ WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
\\ or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
\\ for more details.
\\
\\ You should have received a copy of the GNU General Public License along
\\ with this file.  See the file COPYING.  If not, see
\\ <http://www.gnu.org/licenses/>.


\\ Experiments deriving the convex hull area.
\\

read("../tools/geom.gp");

hull_area_samples = [ 1,3,7,19,43,91,187,395,811,1643,3307,6699 ];
hull_area_samp(k) = hull_area_samples[k+1];

b = 1+I;
p_vector = [ -1, -b, -b^2, -b^3-3, 1, b, b^2, b^3+3 ];
p(m) = m%=8; p_vector[m+1];

P1(k)  = - (b^(k+3) + p(k+3))/3;
P2(k)  = - (b^(k+2) + p(k+2))/3;
P3(k)  = - (b^(k+1) + p(k+1))/3;
P4(k)  = - (b^(k+0) + p(k+0))/3;

check() = my(bad=0); \
          forstep(k=3,length(hull_area_samples)-1,4, \
              my(want=hull_area_samp(k)); \
              my(got=hull_area(k)); \
              if(got != want, \
                 bad+=1;print("k=",k," want ",want," got ",got); \
                 if(bad>10,break))); \
          if(bad!=0,error("bad")); \
          print("ok");

hull_area(k) = hull_area_samp(k);
check();

hull_vertices(k) =
{
  Vecrev([P1(k+1),P2(k+1),P3(k+1),P4(k+1),
          b^(k+1) - P1(k+1),
          b^(k+1) - P2(k+1),
          b^(k+1) - P3(k+1),
          b^(k+1) - P4(k+1)]);
}
hull_vertices(0)
my(v=hull_vertices(0)); vector(length(v),i,geom_area_of_triangle(v[i],v[(i%length(v))+1]))

print("by geom_area_of_polygon()");
hull_area(k) = geom_area_of_polygon(hull_vertices(k));
check();

hull_area(k) =
{
  -(geom_area_of_triangle(P1(k+1),   P2(k+1))
    + geom_area_of_triangle(P2(k+1), P3(k+1))
    + geom_area_of_triangle(P3(k+1), P4(k+1))
    + geom_area_of_triangle(P4(k+1),           b^(k+1) - P1(k+1))
    + geom_area_of_triangle(b^(k+1) - P1(k+1), b^(k+1) - P2(k+1))
    + geom_area_of_triangle(b^(k+1) - P2(k+1), b^(k+1) - P3(k+1))
    + geom_area_of_triangle(b^(k+1) - P3(k+1), b^(k+1) - P4(k+1))
    + geom_area_of_triangle(b^(k+1) - P4(k+1), P1(k+1))
   );
check();

half_factor_vector = [-8, -12, -10, -12];
half_factor(k) = k%=4; half_factor_vector[k+1];
hull_area(k) = 1/3 * (10*2^k + half_factor(k)*2^floor(k/2) + 1);
check();
