\\ Copyright 2016, 2017, 2018 Kevin Ryde
\\
\\ This file is free software; you can redistribute it and/or modify it
\\ under the terms of the GNU General Public License as published by the Free
\\ Software Foundation; either version 3, or (at your option) any later
\\ version.
\\
\\ This file is distributed in the hope that it will be useful, but
\\ WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
\\ or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
\\ for more details.
\\
\\ You should have received a copy of the GNU General Public License along
\\ with this file.  See the file COPYING.  If not, see
\\ <http://www.gnu.org/licenses/>.


default(recover,0);
read("dragon-defines.gp");

check_count = 0;
check_exact(got,want,name="") =
{
  check_count++;
  if(got!= want,
     print("wrong "name);
     print("got  "got);
     print("want "want);
     quit(1),
     print("ok "check_count" "name));
}

arg_0_to_2pi(z) = my(a=arg(z)); if(a<0,a+=2*Pi); a;
lex_by_abs(a,b) = \
  if(abs(a) == abs(b), lex(arg_0_to_2pi(a),arg_0_to_2pi(b)), lex(abs(a),abs(b)));
lex_by_abs_descending(a,b) = - lex_by_abs(a,b);


\\-----------------------------------------------------------------------------

n_other_high_common(n) =
{
  my(other=n_to_other(n));
  if(other == -1, return(""));
  n = binary(n);
  other = binary(other);
  if(#n != #other, return(0));
  for(i=1,#n, if(n[i] != other[i], return(i-1)));
  #n;
}

n_to_other(n) =
{
  my(z = dragon_point(n),
     an1 = dragon_unpoint(z,1),
     an2 = dragon_unpoint(z,-1));
  if(! ((an1[1] == 0 && an1[2] == n)
        || (an2[1] == 0 && an2[2] == n)),
     error("oops, ",an1," ",an2));
  if(an1[1] == 0 && an1[2] != n,  an1[2],
     an2[1] == 0 && an2[2] != n,  an2[2],
     -1);
}
vector(16+1,n,n--; n_to_other(n))
{
  for(n=0,50,
     my(other = n_to_other(n));
     if(other > n, printf("%3d %3d %10s %10s     %s\n",
                          n, other, to_binary(n), to_binary(other),
                          n_other_high_common(n)));
  );
}

\\-----------------------------------------------------------------------------

binary_to_b_powers(v) =
{
  my(s=1);
  for(i=2,#v, v[i]*=s; if(v[i]&&!v[i-1], s = -s));
  v;
}

\\ 7,11 15,19 14,22
v = concat([0,0],binary(7));
v = binary_to_b_powers(v)
subst(Pol(v),'x,b)

v = concat([0,0,0],binary(11));
v = binary_to_b_powers(v)
subst(Pol(v),'x,b)

{
  pos = #v;
  while(pos>1 && v[pos]==0, pos--);
  s = v[pos];
  if(v[pos] && !v[pos-1], s = -s);
  s = -s;
  delta = 0;
  while(pos > 1,
        print();
        print("at pos="pos" "v" delta "delta"  s="s);

        print("  n bits  "v[pos-1]," "v[pos]);
        z = v[pos-1]*b + v[pos];
        z_plus_delta = z + delta;
        print("  z = ",z,"   z+delta = "z_plus_delta"  mod b^2 = "z_mod_bsquared_bits(z_plus_delta));
        if(z_mod_bsquared_bits(z_plus_delta) == 1,
           s=-s;
           print("  s change to ",s));

        new_bit = z_mod_b(z_plus_delta);
        new_bit_signed = new_bit * s;
        more_delta = v[pos] - new_bit_signed;
        print("  z mod b = "new_bit"  new_bit="new_bit_signed"  more delta "more_delta);
        delta += more_delta;
        v[pos] = new_bit_signed;

        pos--;
        delta /= b;
       );
}
print();
print("end "v" delta "delta" s="s"  n="from_binary_vector(abs(v)));
subst(Pol(v),'x,b)
print();

m = Map();

n_to_other_by_delta(n) =
{
  my(v = concat([0,0],binary(n)));
  v = binary_to_b_powers(v);
  my(pos = #v);
  while(pos>1 && v[pos]==0, pos--);
  my(s = v[pos]);
  if(v[pos] && !v[pos-1], s = -s);
  s = -s;
  my(delta = 0);
  while(pos > 1,
        mapput(m,Strprintf("%s s=%d",delta,s),1);
        \\ print("at pos="pos" "v" delta "delta"  s="s);

        \\ print("  n bits  "v[pos-1]," "v[pos]);
        z = v[pos-1]*b + v[pos];
        \\ print("  z = ",z,"   z+delta = "z+delta);
        z += delta;
        if(z_mod_bsquared_bits(z) == 1, s=-s);

        new_bit = z_mod_b(z) * s;
        \\ print("  new_bit "new_bit" more delta "v[pos] - new_bit);
        delta += v[pos] - new_bit;
        v[pos] = new_bit;

        pos--;
        delta /= b;
       );
  \\ print("end "v" delta "delta" s="s"  n="from_binary_vector(abs(v)));
  mapput(m,Strprintf("%s s=%d",delta,s),1);
  if(delta, -1, from_binary_vector(abs(v)));
}

{
  my(count=0);
  for(n=1,1000,
     my(want = n_to_other(n));
     \\ if(want == -1, next());
     my(got  = n_to_other_by_delta(n));
     if(got != want,
        print("n="n);
        print("want="want);
        print("got ="got);
        quit(1);
        );
     count++;
  );
  print(count" ok");
}

print("delta seen");
m = Mat(m);
v = m[,1];
v = vecsort(v);
{
  for(i=1,#v,
     print(v[i]);
  );
}
\\ v = vecsort(v,lex_by_abs);
\\ apply(x->printf("%6s   %d\n", x, z_mod_bsquared_bits(x)),v);

