\\ Copyright 2016, 2017, 2018 Kevin Ryde
\\
\\ This file is free software; you can redistribute it and/or modify it
\\ under the terms of the GNU General Public License as published by the Free
\\ Software Foundation; either version 3, or (at your option) any later
\\ version.
\\
\\ This file is distributed in the hope that it will be useful, but
\\ WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
\\ or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
\\ for more details.
\\
\\ You should have received a copy of the GNU General Public License along
\\ with this file.  See the file COPYING.  If not, see
\\ <http://www.gnu.org/licenses/>.


default(recover,0);
read("dragon-defines.gp");

check_count = 0;
check_exact(got,want,name="") =
{
  check_count++;
  if(got!= want,
     print("wrong "name);
     print("got  "got);
     print("want "want);
     quit(1),
     print("ok "check_count" "name));
}

arg_0_to_2pi(z) = my(a=arg(z)); if(a<0,a+=2*Pi); a;
lex_by_abs(a,b) = \
  if(abs(a) == abs(b), lex(arg_0_to_2pi(a),arg_0_to_2pi(b)), lex(abs(a),abs(b)));
lex_by_abs_descending(a,b) = - lex_by_abs(a,b);


\\-----------------------------------------------------------------------------

n_other_high_common(n) =
{
  my(other=n_to_other(n));
  if(other == -1, return(""));
  n = binary(n);
  other = binary(other);
  if(#n != #other, return(0));
  for(i=1,#n, if(n[i] != other[i], return(i-1)));
  #n;
}

n_to_other(n) =
{
  my(z = dragon_point(n),
     an1 = dragon_unpoint(z,1),
     an2 = dragon_unpoint(z,-1));
  if(! ((an1[1] == 0 && an1[2] == n)
        || (an2[1] == 0 && an2[2] == n)),
     error("oops, ",an1," ",an2));
  if(an1[1] == 0 && an1[2] != n,  an1[2],
     an2[1] == 0 && an2[2] != n,  an2[2],
     -1);
}
vector(16+1,n,n--; n_to_other(n))
{
  for(n=0,50,
     my(other = n_to_other(n));
     if(other > n, printf("%3d %3d %10s %10s     %s\n",
                          n, other, to_binary(n), to_binary(other),
                          n_other_high_common(n)));
  );
}

\\-----------------------------------------------------------------------------

binary_to_b_powers(v) =
{
  my(s=1);
  for(i=2,#v, v[i]*=s; if(v[i]&&!v[i-1], s = -s));
  v;
}

\\ 7,11 15,19
v = concat([0,0],binary(11));
v = binary_to_b_powers(v);
printf("%40s\n", v);
subst(Pol(v),'x,b)

v = concat([0,0,0,0,0,0],binary(7));
v = binary_to_b_powers(v);
printf("%40s\n", v);
subst(Pol(v),'x,b)

{
  v = concat([0,0,0],binary(11));
  vabs = vector(#v);
  pos = #v;
  old_s = 1;
  new_s = -1;
  delta = 0;
  while(pos > 1,
        print();
        if(v[pos] && !v[pos-1],
           old_s = -old_s;
           print("  old_s change to ",old_s));
        old_bit_signed = v[pos] * old_s;
        print("at pos="pos" "v" delta "delta"  old_s="old_s"  new_s="new_s);

        print("  old n bits  "v[pos-1]," "old_bit_signed);
        my(z = v[pos-1]*b + old_bit_signed);
        my(z_plus_delta = z+delta);
        print("  z = ",z,"   z+delta = "z_plus_delta"  mod b^2 = "z_mod_bsquared_bits(z_plus_delta));
        if(z_mod_bsquared_bits(z_plus_delta) == 1,
           new_s = -new_s;
           print("  new_s change to ",new_s));

        new_bit = z_mod_b(z_plus_delta);
        new_bit_signed = new_bit * new_s;
        more_delta = old_bit_signed - new_bit_signed;
        print("  new_bit="new_bit" "new_bit_signed"  more delta "more_delta);
        delta += more_delta;
        vabs[pos] = new_bit;
        v[pos] = new_bit_signed;

        pos--;
        delta /= b;
       );
}
print();
print("end "v" delta "delta" old_s="old_s" new_s="new_s"   n="from_binary_vector(vabs));
print("  v    "v);
print("  vabs "vabs);
subst(Pol(v),'x,b)
print();

delta_seen = Map();
more_delta_seen = Map();

n_to_other_by_bits(n) =
{
  my(v = concat([0,0],binary(n)));
  my(pos = #v);
  while(pos >= 1 && v[pos]==0, pos--); \\ skip low 0s
  if(pos < 1, return(-1));
  if(pos < 1, return(from_binary_vector(v)));

  pos--;  \\ skip lowest 1 bit
  if(pos < 1, return(-1));
  if(pos < 1, return(from_binary_vector(v)));

  my(middle = v[pos]);
  while(1,
        pos--;
        if(pos < 1, return(-1));
        if(pos < 1, break());

        my(bit = v[pos]);
        v[pos]=1-v[pos];  \\ flip
        pos--;
        while(pos >= 1 && v[pos]==bit, v[pos]=1-v[pos]; pos--);  \\ flip run
        if(pos < 1, return(-1));
        if(pos < 1, break());

        v[pos]=1-v[pos];  \\ flip one more
        pos--;
        if(pos < 1, return(-1));
        if(pos < 1, break());

        if(v[pos] != middle,
           break());  \\ rest unchanged
       );
  from_binary_vector(v);
}

{
  my(count=0);
  for(n=1,1000,
     my(want = n_to_other(n));
     \\ if(want == -1, next());
     my(got  = n_to_other_by_bits(n));
     if(got != want,
        print("n="n);
        print("want="want);
        print("got ="got);
        quit(1);
        );
     count++;
  );
  print(count" ok");
}
