// Copyright 2015, 2016 Kevin Ryde
//
// This file is free software; you can redistribute it and/or modify it
// under the terms of the GNU General Public License as published by the Free
// Software Foundation; either version 3, or (at your option) any later
// version.
//
// This file is distributed in the hope that it will be useful, but
// WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
// or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
// for more details.
//
// You should have received a copy of the GNU General Public License along
// with this file.  See the file COPYING.  If not, see
// <http://www.gnu.org/licenses/>.


// Draw the dragon curve of given "level".
//
// dragon() draws along the X axis [0,0] to [1,0].  The recursion
// draws sub-curves from those ends inward to the right-side middle at
// [.5, -.5].
//
//     0,0          1,0
//        \        /
//         \      /
//          .5,-.5
//
// Line segments are drawn in 3D as elongated cubes.
//
// Line segments could be drawn instead as flat 2D polygons and
// extruded up into Z.  In fact doing so runs faster under a full
// "openscad --render" (as opposed to the preview quality).
//
// Drawing each line segment individually results in ends overlapping.
// Taking a union() doesn't seem to make much difference to speed or
// appearance.  It'd be possible to calculate exact polygons or
// polyhedrons with holes instead of overlapping.  There's enough
// mathematics in openscad (just) to do that, but making such a grid
// would probably be easier done by an external generator program.


level = 8;
linewidth = .1;  // fraction of segment length
sqrt2 = pow(2, .5);

// Draw a dragon curve "level" going from [0,0] to [1,0]
module dragon(level) {
    if (level <= 0) {
        translate([.5,0]) cube([1+linewidth,linewidth,linewidth],center=true);
    } else {
        rotate(-45) scale(1/sqrt2) dragon(level-1);
        translate([1,0]) rotate(-135) scale(1/sqrt2) dragon(level-1);
    }
}

scale(40) {  // scale to nicely visible in the default GUI
    sphere(1.5*linewidth / pow(2,level/2));  // mark the start of the curve
    dragon(level);
}

// Local variables:
// compile-command: "openscad --imgsize=1000,800 -o /tmp/dragon.png dragon.scad && xzgv /tmp/dragon.png"
// End:
