#!/usr/bin/perl -w

# Copyright 2019 Kevin Ryde
#
# This file is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the Free
# Software Foundation; either version 3, or (at your option) any later
# version.
#
# This file is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
# or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
# for more details.
#
# You should have received a copy of the GNU General Public License along
# with this file.  If not, see <http://www.gnu.org/licenses/>.


use 5.004;
use strict;
use List::Util 'min';
use Math::PlanePath::DragonCurve 117; # v.117 for level_to_n_range()

use FindBin;
use lib $FindBin::Bin;
use MyDragon '@triple_7ths_state_table','@triple_7ths_state_to_conf';
use MyLineSegments;
use MyPrintwrap ':all';

# uncomment this to run the ### lines
# use Smart::Comments;

print "% generated by tools/$FindBin::Script\n";

MyDragon::triple_7ths_surround_init();

{
  printwrap_indent("% GP-DEFINE  ");
  printwrap("sevenths_state_table = {[\n");
  $Printwrap += 2;
  foreach my $state (0 .. $#triple_7ths_state_table) {
    printwrap(' ' . join(',',
                         map {$_+1}    # for 1-based
                         @{$triple_7ths_state_table[$state]})
              . ($state == $#triple_7ths_state_table ? '' : ';'));
  }
  $Printwrap -= 2;
  printwrap(" ]};\n");
}

{
  printwrap("sevenths_state_triples_matrix = {[\n");
  $Printwrap += 2;
  foreach my $state (0 .. $#triple_7ths_state_table) {
    my $conf = $triple_7ths_state_to_conf[$state];
    my @row = (undef, ((0) x 6));
    foreach my $triple ($conf->triple_7ths_list) {
      $row[$triple]++;
    }
    printwrap(join(',',@row[1..$#row])
              . ($state == $#triple_7ths_state_table ? '' : ';'));
  }
  $Printwrap -= 2;
  printwrap(" ]};\n");
}

{
  printwrap("sevenths_state_diffs_matrix = {[\n");
  $Printwrap += 2;
  my $max_diff = 9;
  foreach my $state (0 .. $#triple_7ths_state_table) {
    ### $state
    my $conf = $triple_7ths_state_to_conf[$state];
    my @row = (undef, ((0) x $max_diff));
    foreach my $diff ($conf->triple_7ths_diffs,
                      $conf->triple_7ths_diff_to_next) {
      ### $diff
      $diff <= $max_diff or die "oops";
      $row[$diff]++;
    }
    ### row: @row[1..$#row]
    printwrap(join(',',@row[1..$#row])
              . ($state == $#triple_7ths_state_table ? '' : ';'));
  }
  $Printwrap -= 2;
  printwrap(" ]};\n");
}

print "% end generated\n";
exit 0;
