#!/usr/bin/perl -w

# Copyright 2016, 2017, 2019 Kevin Ryde
#
# This file is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the Free
# Software Foundation; either version 3, or (at your option) any later
# version.
#
# This file is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
# or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
# for more details.
#
# You should have received a copy of the GNU General Public License along
# with this file.  If not, see <http://www.gnu.org/licenses/>.


use 5.004;
use strict;
use List::Util 'min';
use Math::PlanePath::DragonCurve 117; # v.117 for level_to_n_range()

use FindBin;
use lib $FindBin::Bin;
use MyDragon '$triple_7ths_union_segs',
  'xy_to_node_name', '@dir4_to_dx','@dir4_to_dy';
use MyLineSegments;
use MyPrintwrap ':all';

print "  % generated by tools/$FindBin::Script\n";

{
  my $str = $triple_7ths_union_segs->print_to_str;
  $str =~ s/^/  %    /mg;
  print $str;
}

my %seen;
foreach my $seg (sort $triple_7ths_union_segs->segs_list) {
  my ($x,$y,$dir) = split /,/, $seg;
  foreach (0, 1) {
    my $name = xy_to_node_name($x,$y);
    unless ($seen{$name}++) {
      my $blank = ($x==0 && $y==0) || ($x==1 && $y==0) ? '' : ' blank';
      print "  \\node ($name) at ($x,$y) [my$blank dot];\n";
    }
    $x += $dir4_to_dx[$dir];
    $y += $dir4_to_dy[$dir];
  }
}

foreach my $seg (sort $triple_7ths_union_segs->segs_list) {
  my ($x,$y,$dir) = split /,/, $seg;
  my $from = xy_to_node_name($x,$y);
  my $to   = xy_to_node_name($x+$dir4_to_dx[$dir], $y+$dir4_to_dy[$dir]);
  my $style = ($seg eq '0,0,0' ? 'my m arrow' : 'my arrow');
  print "  \\draw[$style] ($from)--($to);\n";
}

print "  % end generated\n";
exit 0;
