#!/usr/bin/perl -w

# Copyright 2014, 2015, 2016, 2017, 2019 Kevin Ryde
#
# This file is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the Free
# Software Foundation; either version 3, or (at your option) any later
# version.
#
# This file is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
# or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
# for more details.
#
# You should have received a copy of the GNU General Public License along
# with this file.  If not, see <http://www.gnu.org/licenses/>.


use 5.004;
use strict;
use Set::IntSpan::Fast;
use Math::PlanePath::DragonCurve;
use Math::PlanePath::Base::Digits 'round_down_pow';

use FindBin;
use lib $FindBin::Bin;
use MyDragon 'xy_rotate_plus90';


my $type = 'SpredInf';
if (@ARGV) { $type = $ARGV[0]; }

my $pred_count = 16;
my $at_count = 15;

my $path = Math::PlanePath::DragonCurve->new;

my $indent = "         ";

sub SpredInf {
  my ($n) = @_;
  my ($x,$y) = $path->n_to_xy($n);
  my @n_list = $path->xy_to_n_list($x,$y);
  return @n_list == 1;
}
sub DpredInf {
  my ($n) = @_;
  my ($x,$y) = $path->n_to_xy($n);
  my @n_list = $path->xy_to_n_list($x,$y);
  return @n_list == 2;
}

sub set_count_runs {
  my ($set) = @_;
  my $count = 0;
  my $iter = $set->iterate_runs();
  while ($iter->()) { $count++; }
  return $count;
}
sub set_pari_string {
  my ($set) = @_;
  my @runs;
  my $iter = $set->iterate_runs();
  while (my ($from,$to) = $iter->()) {
    if ($from == $to) {
      push @runs, "[$from]";
    } else {
      push @runs, "[$from..$to]";
    }
  }
  return 'concat([' . join(',',@runs) . '])';
}

print "% generated by tools/$FindBin::Script $type\n";

#------------------------------------------------------------------------------
# Spred

if ($type eq 'SpredInf') {
  my $set = Set::IntSpan::Fast->new;
  my $count = 0;
  for (my $n = 0; ; $n++) {
    if (SpredInf($n)) {
      $set->add($n);
      $count++;
    } else {
      last if set_count_runs($set) >= 6;
    }
  }
  my $iter = $set->iterate_runs();
  while (my ($from,$to) = $iter->()) {
    print $indent;
    if ($from+2 >= $to) {
      print join(',',$from..$to),", \\;\n";
    } else {
      print "$from \\MyTightDots $to, \\;\n";
    }
  }
  print $indent, "\\dotsc\n";
  print "% GP-DEFINE  SpredInf_at_samples = ",set_pari_string($set),";\n";
}

if ($type eq 'DpredInf') {
  my $set = Set::IntSpan::Fast->new;
  my $count = 0;
  for (my $n = 0; ; $n++) {
    if (DpredInf($n)) {
      $set->add($n);
      $count++;
    } else {
      last if set_count_runs($set) >= 9;
    }
  }
  my $iter = $set->iterate_runs();
  while (my ($from,$to) = $iter->()) {
    print $indent;
    if ($from+2 >= $to) {
      print join(',',$from..$to),", \\;\n";
    } else {
      print "$from \\MyTightDots$to, \\;\n";
    }
  }
  print $indent, "\\dotsc\n";
  print "% GP-DEFINE  DpredInf_at_samples = ",set_pari_string($set),";\n";
}

print "% end generated\n";
exit 0;
