#!/usr/bin/perl -w

# Copyright 2014, 2015, 2016, 2017, 2018, 2019 Kevin Ryde
#
# This file is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the Free
# Software Foundation; either version 3, or (at your option) any later
# version.
#
# This file is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
# or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
# for more details.
#
# You should have received a copy of the GNU General Public License along
# with this file.  If not, see <http://www.gnu.org/licenses/>.


use 5.004;
use strict;
use Math::PlanePath::ComplexPlus 117; # v.117 for level_to_n_range()

use FindBin;
use lib $FindBin::Bin;
use MyDragon 'TAVertexToUpper','TAVertexToLower',
  'TAVertexToLeft','TAVertexToRight';
use MyPrintwrap ':all';


my $level = 4;

print "  % generated by tools/$FindBin::Script\n";
print "  \\newcommand{\\MyLevel}{$level}\n";

foreach my $radix (10,2) {
  print "  \\begin{scope}[my pos $radix]\n";

  my $path = Math::PlanePath::ComplexPlus->new;
  my ($n_lo, $n_hi) = $path->level_to_n_range($level);

  $Printwrap = 4;
  foreach my $n ($n_lo .. $n_hi) {
    my ($x,$y) = $path->n_to_xy($n);
    my $nr = ($radix==10 ? $n : sprintf '%0*b', $level, $n);
    printwrap(" \\node (p$n) at ($x,$y) {\$$nr\$};");
  }
  printwrap_newline();

  my $num_edges = 0;
  $Printwrap = 4;
  printwrap("\\draw");
  $Printwrap += 2;
  foreach my $from ($n_lo .. $n_hi) {
    foreach my $to (TAVertexToLeft($from),
                    TAVertexToRight($from),
                    TAVertexToLower($from),
                    TAVertexToUpper($from)) {
      if (defined $to
          && $to > $from
          && $to <= $n_hi) {
        printwrap(" (p$from)--(p$to)");
        $num_edges++;
      }
    }
  }
  printwrap(";\n");
  $num_edges == 2**$level - 1
    or die "oops, num_edges=$num_edges";

  print "  \\end{scope}\n";
}

print "  % end generated\n";
exit 0;
