#!/usr/bin/perl -w

# Copyright 2015, 2016, 2017 Kevin Ryde
#
# This file is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the Free
# Software Foundation; either version 3, or (at your option) any later
# version.
#
# This file is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
# or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
# for more details.
#
# You should have received a copy of the GNU General Public License along
# with this file.  If not, see <http://www.gnu.org/licenses/>.


use 5.004;
use strict;
use Graph::Maker::TwindragonAreaTree;

use FindBin;
use lib $FindBin::Bin;
use MyDragon 'TA_complex_c';
use MyGraphs 'Graph_terminal_Wiener_index',
  'Graph_terminal_Wiener_part_at_vertex';


my $max_level = 6;
if (@ARGV) { $max_level = shift @ARGV; }

print "% generated by tools/$FindBin::Script $max_level\n";

my @TAtwS;
my @TAtwC;
my @TATW;

foreach my $k (0 .. $max_level) {
  my $graph = Graph::Maker->new ('twindragon_area_tree',
                                 level => $k,
                                 undirected => 1);
  my $Sv = '0';
  my $Cv = TA_complex_c($k);
  $graph->has_vertex($Sv) or die;
  $graph->has_vertex($Cv) or die;

  push @TAtwC, Graph_terminal_Wiener_part_at_vertex($graph, $Cv);
  push @TAtwS, Graph_terminal_Wiener_part_at_vertex($graph, $Sv);
  push @TATW, Graph_terminal_Wiener_index($graph);
}

{
  print "% GP-DEFINE  gen_TAtwS_samples = [";
  my $join = '';
  foreach my $value (@TAtwS) {
    print $join,$value;
    $join = ',';
  }
  print "];\n";
}
{
  print "% GP-DEFINE  gen_TAtwC_samples = [";
  my $join = '';
  foreach my $value (@TAtwC) {
    print $join,$value;
    $join = ',';
  }
  print "];\n";
}
{
  print "% GP-DEFINE  gen_TATW_samples = [";
  my $join = '';
  foreach my $value (@TATW) {
    print $join,$value;
    $join = ',';
  }
  print "];\n";
}

print "% end generated\n";
exit 0;
