#!/usr/bin/perl -w

# Copyright 2014, 2015, 2016, 2017, 2019 Kevin Ryde
#
# This file is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the Free
# Software Foundation; either version 3, or (at your option) any later
# version.
#
# This file is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
# or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
# for more details.
#
# You should have received a copy of the GNU General Public License along
# with this file.  If not, see <http://www.gnu.org/licenses/>.


use 5.004;
use strict;
use Math::PlanePath::DragonCurve 117; # v.117 for level_to_n_range()

use FindBin;
use lib $FindBin::Bin;
use MyDragon ':all';
use MyLineSegments;
use MyPrintwrap ':all';


my $level = 7;

print "  % generated by tools/$FindBin::Script\n";
print "  \\newcommand{\\MyLevel}{$level}\n";
my $path = Math::PlanePath::DragonCurve->new;

my $n_lo = BlobN($level);
my $n_hi = BlobN($level+1) - 3;
my ($x_lo,$y_lo) = $path->n_to_xy($n_lo);
my ($x_hi,$y_hi) = $path->n_to_xy($n_hi);

$Printwrap = 2;

my $segments = MyLineSegments->new;
foreach my $n ($n_lo .. $n_hi-1) {
  $segments->add($path->n_to_xy($n),
                 $path->n_to_xy($n+1));
}

my %count = (horizontal => 0,
             vertical   => 0);
foreach my $dir ('horizontal','vertical') {

  print "  \\begin{scope}[shift=(pos-$dir)]\n";
  print "    \\node (start-$dir) at ($x_lo,$y_lo) [my dot];\n";
  print "    \\node (end-$dir) at ($x_hi,$y_hi) [my dot];\n";
  $Printwrap = 4;
  printwrap("\\draw");
  $Printwrap++;
  foreach my $line ($segments->lines) {
    my ($x1,$y1, $x2,$y2) = @$line;

    if ($x1 == $x2) {
      next unless $dir eq 'vertical';
    } else {
      next unless $dir eq 'horizontal';
    }
    $count{$dir}++;

    printwrap(" ($x1,$y1)--($x2,$y2)");
  }
  printwrap(";\n");
  print "  \\end{scope}\n";
}

my $horizontals = $count{horizontal};
my $verticals   = $count{vertical};
print "  \\newcommand{\\MyHorizontal}{$horizontals}\n";
print "  \\newcommand{\\MyVertical}{$verticals}\n";
my $lines = $horizontals + $verticals;
print "% GP-Test-Last  BlobHorizontals($level)==$horizontals\n";
print "% GP-Test-Last  BlobVerticals($level)==$verticals\n";
print "% GP-Test-Last  BlobLines($level)==$lines\n";

print "  % end generated\n";
exit 0;
