#!/usr/bin/perl -w

# Copyright 2016, 2017, 2018 Kevin Ryde
#
# This file is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the Free
# Software Foundation; either version 3, or (at your option) any later
# version.
#
# This file is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
# or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
# for more details.
#
# You should have received a copy of the GNU General Public License along
# with this file.  If not, see <http://www.gnu.org/licenses/>.

use 5.004;
use strict;
use List::Util 'min','max';
use Math::PlanePath::DragonCurve 117; # v.117 for level_to_n_range()

use FindBin;
use lib $FindBin::Bin;
use MyDragon 'chamfer','BlobN';
use MyPrintwrap ':all';


my $level = 8;
if (@ARGV) { $level = shift @ARGV; }

print "  % generated by tools/$FindBin::Script\n";
print "  \\newcommand{\\MyLevel}{$level}\n";

my $path = Math::PlanePath::DragonCurve->new;
my ($n_lo,$n_hi) = $path->level_to_n_range($level);

my $blob_n_start = BlobN($level);
my $blob_n_end = BlobN($level+1)-3;

my $ab_length = $n_hi / 16;
my $a_start = 6*$ab_length;
my $a_end   = $a_start + $ab_length;
my $b_start = 12*$ab_length;
my $b_end   = $b_start + $ab_length;

{
  my ($x,$y) = $path->n_to_xy($a_start);
  print "  \\coordinate (start-a) at ($x,$y);\n";
}
{
  my ($x,$y) = $path->n_to_xy($a_end);
  print "  \\coordinate (end-a) at ($x,$y);\n";
}
{
  my ($x,$y) = $path->n_to_xy($b_start);
  print "  \\coordinate (start-b) at ($x,$y);\n";
}
{
  my ($x,$y) = $path->n_to_xy($b_end);
  print "  \\coordinate (end-b) at ($x,$y);\n";
}
{
  my ($x,$y) = $path->n_to_xy($n_hi);
  print "  \\coordinate (end) at ($x,$y);\n";
}

my $type = 0;
my @type_to_style = ('[my grey]', '', '[my thick]');
$Printwrap = 2;
printwrap("\\draw$type_to_style[$type] (0,0)");
$Printwrap += 2;
my $join = '--';
foreach my $n ($n_lo+1 .. $n_hi) {
  my ($x,$y) = $path->n_to_xy($n);

  my $new_type;
  if (($n >= $a_start && $n < $a_end) || ($n >= $b_start && $n < $b_end)) {
    $new_type = 2;  # thick
  } elsif ($n < $blob_n_start || $n >= $blob_n_end) {
    $new_type = 0;  # grey
  } else {
    $new_type = 1;  # thin
  }

  if ($n == $n_hi) {
    printwrap("--($x,$y);\n");
  } else {
    my ($sdx,$sdy) = $path->n_to_dxdy($n-1);
    my ($tdx,$tdy) = $path->n_to_dxdy($n);
    my $sx = $x . chamfer(-$sdx);
    my $sy = $y . chamfer(-$sdy);
    my $tx = $x . chamfer($tdx);
    my $ty = $y . chamfer($tdy);

    if ($new_type < $type) {
      printwrap("$join($sx,$sy);\n");
      $Printwrap = 2;
      printwrap("\\draw$type_to_style[$new_type] ");
      $Printwrap += 2;
      $join = '';
    }
    printwrap("$join($sx,$sy)");
    $join = '--';

    if ($new_type > $type) {
      printwrap("$join($tx,$ty);\n");
      $Printwrap = 2;
      printwrap("\\draw$type_to_style[$new_type] ");
      $Printwrap += 2;
      $join = '';
    }
    printwrap("$join($tx,$ty)");
    $join = '--';
  }
  $type = $new_type;
}

print "  % end generated\n";
exit 0;
