#!/usr/bin/perl -w

# Copyright 2019 Kevin Ryde
#
# This file is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the Free
# Software Foundation; either version 3, or (at your option) any later
# version.
#
# This file is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
# or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
# for more details.
#
# You should have received a copy of the GNU General Public License along
# with this file.  If not, see <http://www.gnu.org/licenses/>.


use 5.004;
use strict;
use List::Util 'min';
use Math::BaseCnv;
use Math::BigInt;

use FindBin;
use lib $FindBin::Bin;
use MyDragon;
use MyFLAT;
$|=1;


print "% generated by tools/$FindBin::Script\n";
my $max_len = Math::BigInt->new(32);

{
  MyDragon::fBpred_hullstouch_init();
  MyFLAT::FLAT_print_gp_inline_table($MyDragon::fNonBpred_hullstouch_flat,
                                     "fBpred_hullstouch_state_table");
  foreach my $f ($MyDragon::fNonRpred_hullstouch_flat,
                 $MyDragon::fNonLpred_hullstouch_flat,
                 $MyDragon::fNonBpred_hullstouch_flat) {
    my $name = $f->{'name'};
    $name =~ s/ /_/;
    MyFLAT::FLAT_print_gp_inline_accepting($f, "${name}_accepting");
  }

  foreach my $f ($MyDragon::fNonRpred_hullstouch_flat,
                 $MyDragon::fNonLpred_hullstouch_flat,
                 $MyDragon::fNonBpred_hullstouch_flat) {
    join(',',$f->get_starting) eq '0' or die "oops";
    my $name = $f->{'name'};
    $name =~ s/ /_/;
    print "% GP-DEFINE  ${name}_counts = [",
      join(',',MyFLAT::FLAT_count_contains($f,$max_len)),"];\n";
  }

  foreach my $f ($MyDragon::fNonRpred_hullstouch_flat,
                 $MyDragon::fNonLpred_hullstouch_flat,
                 $MyDragon::fNonBpred_hullstouch_flat) {
    join(',',$f->get_starting) eq '0' or die "oops";
    my $name = $f->{'name'};
    $name =~ s/ /_/;
    $f = $f->MyFLAT::minimize;
    print "% GP-DEFINE  ${name}_minimized_size = ",$f->num_states,";\n";
  }
}

foreach my $elem (
                  # [$MyDragon::fNonRpred_hullstouch_flat, 7],
                  [$MyDragon::fNonBpred_hullstouch_flat, 8],
                 ) {
  my ($f,$max_depth) = @$elem;
  my $name = $f->{'name'};
  my @want_count = MyFLAT::FLAT_count_contains($f,$max_depth);
  $name =~ s/ /_/;
  print "% GP-DEFINE  ${name}_contains = {[\n";
  foreach my $k (0 .. $max_depth) {
    my @strs;
    foreach my $n (0 .. 2**$k-1) {
      my $str = Math::BaseCnv::cnv($n,10,2);
      $str = sprintf '%0*s', $k, $str;
      if ($f->contains($str)) {
        push @strs, $str;
      }
    }
    scalar(@strs) == $want_count[$k] or die;
    print "% GP-DEFINE    [";
    foreach my $i (0 .. $#strs) {
      print '[',join(',', split //, $strs[$i]), ']',
        $i==$#strs ? '' : ',';
    }
    print "]", $k==$max_depth ? '' : ',', "\n";
  }
  print "% GP-DEFINE  ]};\n";
}

print "%\n";
{
  MyDragon::fBpred_surround_init();
  MyFLAT::FLAT_print_gp_inline_table($MyDragon::fNonBpred_surround_flat,
                                     "fBpred_surround_state_table");
  foreach my $f ($MyDragon::fNonRpred_surround_flat,
                 $MyDragon::fNonLpred_surround_flat,
                 $MyDragon::fNonBpred_surround_flat) {
    my $name = $f->{'name'};
    $name =~ s/ /_/;
    MyFLAT::FLAT_print_gp_inline_accepting($f, "${name}_accepting");
  }

  foreach my $f ($MyDragon::fNonRpred_surround_flat,
                 $MyDragon::fNonLpred_surround_flat,
                 $MyDragon::fNonBpred_surround_flat) {
    join(',',$f->get_starting) eq '0' or die "oops";
    my $name = $f->{'name'};
    $name =~ s/ /_/;
    print "% GP-DEFINE  ${name}_counts = [",
      join(',',MyFLAT::FLAT_count_contains($f,$max_len)),"];\n";
  }
}

print "% end generated\n";
exit 0;
