#!/usr/bin/perl -w

# Copyright 2015, 2016, 2017, 2019 Kevin Ryde
#
# This file is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the Free
# Software Foundation; either version 3, or (at your option) any later
# version.
#
# This file is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
# or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
# for more details.
#
# You should have received a copy of the GNU General Public License along
# with this file.  If not, see <http://www.gnu.org/licenses/>.

use 5.004;
use strict;
use Math::PlanePath::DragonCurve 117; # v.117 for level_to_n_range()

use FindBin;
use lib $FindBin::Bin;
use MyDragon 'BlobN','xy_rotate_minus90','xy_rotate_plus90';
use MyLineSegments;
use MyPrintwrap ':all';


my $k = 8;

print "  % generated by tools/$FindBin::Script\n";
my $path = Math::PlanePath::DragonCurve->new;

my ($n_lo, $n_hi) = $path->level_to_n_range($k);

sub xyxy_is_traversed {
  my ($x1,$y1, $x2,$y2) = @_;
  my $n = $path->xyxy_to_n_either($x1,$y1, $x2,$y2);
  return (defined $n && $n < $n_hi);
}

sub n_is_right_boundary {
  my ($n) = @_;
  my ($x1,$y1) = $path->n_to_xy($n);
  my ($x2,$y2) = $path->n_to_xy($n+1);
  my ($dx,$dy) = $path->n_to_dxdy($n);
  ($dx,$dy) = xy_rotate_minus90($dx,$dy);  # to right side
  my $x1s = $x1 + $dx;
  my $y1s = $y1 + $dy;
  my $x2s = $x2 + $dx;
  my $y2s = $y2 + $dy;
  return (! xyxy_is_traversed($x1,$y1, $x1s,$y1s)
          || ! xyxy_is_traversed($x2,$y2, $x2s,$y2s)
          || ! xyxy_is_traversed($x1s,$y1s, $x2s,$y2s));
}

sub n_is_left_boundary {
  my ($n) = @_;
  my ($x1,$y1) = $path->n_to_xy($n);
  my ($x2,$y2) = $path->n_to_xy($n+1);
  my ($dx,$dy) = $path->n_to_dxdy($n);
  ($dx,$dy) = xy_rotate_plus90($dx,$dy);  # to left side
  my $x1s = $x1 + $dx;
  my $y1s = $y1 + $dy;
  my $x2s = $x2 + $dx;
  my $y2s = $y2 + $dy;
  return (! xyxy_is_traversed($x1,$y1, $x1s,$y1s)
          || ! xyxy_is_traversed($x2,$y2, $x2s,$y2s)
          || ! xyxy_is_traversed($x1s,$y1s, $x2s,$y2s));
}

sub n_is_boundary {
  my ($n) = @_;
  return n_is_left_boundary($n) || n_is_right_boundary($n);
}

sub n_is_hanging {
  my ($n) = @_;
  my ($x,$y) = $path->n_to_xy($n);
  foreach my $i (1 .. 20) {
    my $n2 = $n + $i;
    last if $n2 >= $n_hi;
    last if ! n_is_boundary ($n2);
    my ($x2,$y2) = $path->n_to_xy($n+$i);
    if ($x == $x2 && $y == $y2) {
      return $n2;
    }
  }
  return 0;
}

my ($x_hi,$y_hi) = $path->n_to_xy($n_hi);
$Printwrap = 2;
printwrap("\\node (start) at (0,0) [my dot];\n");
printwrap("\\node (end) at ($x_hi,$y_hi) [my dot];\n");

my $dragon_segments = MyLineSegments->new;
my $path_segments = MyLineSegments->new;
foreach my $n ($n_lo .. $n_hi-1) {
  if (n_is_boundary($n)) {
    $dragon_segments->add($path->n_to_xy($n),
                          $path->n_to_xy($n+1));
  }
}

my $BoundaryPathLen = 0;

my $switch_n = BlobN($k+1);
for (my $n = $n_lo; $n < $n_hi; $n++) {
  if ($n < $switch_n ? n_is_left_boundary($n) : n_is_right_boundary($n)) {
    $n = n_is_hanging($n) || $n;
    my ($x,$y) = $path->n_to_xy($n);
    my ($x2,$y2) = $path->n_to_xy($n+1);
    $dragon_segments->remove($x,$y, $x2,$y2);
    $path_segments->add($x,$y, $x2,$y2);
    $BoundaryPathLen++;
  }
}

foreach my $elem ([$dragon_segments,'my dragon'],
                  [$path_segments,'my path']) {
  my ($segments, $colour) = @$elem;
  $Printwrap = 2;
  printwrap("\\draw[$colour] ");
  $Printwrap += 2;
  $segments->printwrap;
  printwrap(";\n");
}

print "  % n=$n_lo to $n_hi  switch at $switch_n\n";
print "  \\newcommand{\\MyLevel}{$k}\n";
print "  \\newcommand{\\MyBoundaryPathLen}{$BoundaryPathLen}\n";
print "% GP-Test-Last  BoundaryPathLen($k) == $BoundaryPathLen\n";
print "  % end generated\n";
exit 0;
