#!/usr/bin/perl -w

# Copyright 2016, 2017 Kevin Ryde
#
# This file is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the Free
# Software Foundation; either version 3, or (at your option) any later
# version.
#
# This file is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
# or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
# for more details.
#
# You should have received a copy of the GNU General Public License along
# with this file.  If not, see <http://www.gnu.org/licenses/>.


use 5.004;
use strict;
use Math::PlanePath::DragonCurve 117; # v.117 for level_to_n_range()

use FindBin;
use lib $FindBin::Bin;
use MyDragon 'dir','turn','@dir4_to_dx','@dir4_to_dy',
  'xy_to_node_name','chamfer';
use MyPrintwrap ':all';


my $level = 3;

print "  % generated by tools/$FindBin::Script\n";
print "  \\newcommand{\\MyLevel}{$level}\n";

my $path = Math::PlanePath::DragonCurve->new;
my ($n_lo, $n_hi) = $path->level_to_n_range($level+2);
my ($ex,$ey) = $path->n_to_xy($n_hi);

$Printwrap = 2;
printwrap("\\draw[my L line]");
$Printwrap += 2;
my %seen;
foreach my $rev (0, 1) {
  for (my $n = $n_lo; $n < $n_hi; $n += 2) {
    my ($x1,$y1) = $path->n_to_xy($n);
    my ($x2,$y2) = $path->n_to_xy($n + 2);
    if ($rev) {
      $x1 = $ex - $x1;
      $y1 = $ey - $y1;
      $x2 = $ex - $x2;
      $y2 = $ey - $y2;
    }
    my $dx = $x2 - $x1;
    my $dy = $y2 - $y1;

    my $sx = $x1 . chamfer($dx);
    my $sy = $y1 . chamfer($dy);
    my $tx = $x2 . chamfer(-$dx);
    my $ty = $y2 . chamfer(-$dy);

    printwrap("($sx,$sy)--");
    printwrap("($tx,$ty)--");
  }
}
printwrap("cycle;\n");

$Printwrap = 2;
my @edges;
my $prev_name;
foreach my $rev (0, 1) {
  foreach my $n ($n_lo+1 .. $n_hi) {
    my ($x,$y) = $path->n_to_xy($n);
    if ($rev) {
      $x = $ex - $x;
      $y = $ey - $y;
    }
    if (turn($n) == -1 && ! $seen{"$x,$y"}) {
      my $name = xy_to_node_name($x,$y);
      printwrap("\\node ($name) at ($x,$y) [my dot];");
      if (defined $prev_name) {
        push @edges, " ($name)--($prev_name)";
      }
      $prev_name = $name;
    } else {
      undef $prev_name;
    }
  }
}

$Printwrap = 2;
printwrap("\\draw[my R line]");
$Printwrap += 2;
foreach my $edge (@edges) { printwrap($edge); }
printwrap(";\n");

print "  % end generated\n";
exit 0;
