#!/usr/bin/perl -w

# Copyright 2014, 2015, 2016, 2017, 2019 Kevin Ryde
#
# This file is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the Free
# Software Foundation; either version 3, or (at your option) any later
# version.
#
# This file is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
# or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
# for more details.
#
# You should have received a copy of the GNU General Public License along
# with this file.  If not, see <http://www.gnu.org/licenses/>.


use 5.004;
use strict;
use List::Util 'min';
use Math::PlanePath::DragonCurve 117; # v.117 for level_to_n_range()

use FindBin;
use lib $FindBin::Bin;
use MyDragon 'xy_rotate_plus90','xy_rotate_minus90','transform_to_horizontal',
  'BlobStart_xy','BlobEnd_xy';
use MyLineSegments;
use MyPrintwrap ':all';


my $level = 9;  # of twindragon

print "  % generated by tools/$FindBin::Script\n";
print "  \\newcommand{\\MyLevel}{$level}\n";
my $path = Math::PlanePath::DragonCurve->new;

my ($n_lo, $n_hi) = $path->level_to_n_range($level+1);
my ($x_hi,$y_hi) = $path->n_to_xy($n_hi);

sub transform {
  my ($x,$y) = @_;
  return transform_to_horizontal($x,$y, $level+1);
}

my $segments = MyLineSegments->new;
my $y_min = 0;

foreach my $n ($n_lo .. $n_hi-1) {
  my ($x,$y) = $path->n_to_xy($n);
  my ($x2,$y2) = $path->n_to_xy($n+1);

  {
    my ($x,$y) = transform($x,$y);
    my ($x2,$y2) = transform($x2,$y2);
    $segments->add($x,$y, $x2,$y2);
    $y_min = min($y_min, $y);
  }

  ($x,$y) = ($x_hi-$x, $y_hi-$y);
  ($x2,$y2) = ($x_hi-$x2, $y_hi-$y2);

  {
    my ($x,$y) = transform($x,$y);
    my ($x2,$y2) = transform($x2,$y2);
    $segments->add($x,$y, $x2,$y2);
    $y_min = min($y_min, $y);
  }
}

$Printwrap = 2;
printwrap("\\draw[my dragon] ");
$Printwrap += 2;
$segments->printwrap;
my $draw_count = $segments->lines_joined_count_points;
printwrap(";");
printwrap(" % using $draw_count points\n");

sub blob_crossing_line {
  my ($sx,$sy, $tx,$ty) = @_;
  ($sx,$sy) = transform($sx,$sy);
  ($tx,$ty) = transform($tx,$ty);
  my $len = ($sx-$tx)**2 + ($sy-$ty)**2;
  my $style = ($len <= 2 ? "my zero line" : "my line");
  printwrap(" \\draw[$style] ($sx,$sy)--($tx,$ty);");
}

{
  $Printwrap = 2;
  foreach my $k (4 .. $level+1) {
    my ($sx,$sy) = BlobStart_xy($k);
    my ($tx,$ty) = BlobEnd_xy($k);
    blob_crossing_line ($sx,$sy, $tx,$ty);

    ($sx,$sy) = ($x_hi-$sx,$y_hi-$sy);
    ($tx,$ty) = ($x_hi-$tx,$y_hi-$ty);
    blob_crossing_line ($sx,$sy, $tx,$ty);
  }
  foreach my $k (4 .. $level-1) {
    my ($sx,$sy) = BlobStart_xy($k);
    my ($tx,$ty) = BlobEnd_xy($k);
    ($sx,$sy) = xy_rotate_minus90($sx,$sy);
    ($tx,$ty) = xy_rotate_minus90($tx,$ty);
    $sx += $x_hi;
    $sy += $y_hi;
    $tx += $x_hi;
    $ty += $y_hi;
    blob_crossing_line ($sx,$sy, $tx,$ty);

    ($sx,$sy) = ($x_hi-$sx,$y_hi-$sy);
    ($tx,$ty) = ($x_hi-$tx,$y_hi-$ty);
    blob_crossing_line ($sx,$sy, $tx,$ty);
  }
  printwrap_newline();
}

{
  my ($x,$y) = transform(0,0);
  print "  \\node (start) at ($x,$y) [my dot];\n";
}
{
  my ($x,$y) = transform($x_hi/2,$y_hi/2);
  print "  \\coordinate (middle) at ($x,$y);\n";
}
{
  my ($x,$y) = transform($x_hi,$y_hi);
  print "  \\node (end) at ($x,$y) [my dot];\n";
}

{
  my $k = $level - 1;
  my ($sx,$sy) = BlobStart_xy($k);
  ($sx,$sy) = xy_rotate_plus90($sx,$sy);
  my ($tx,$ty) = ($x_hi-$sx,$y_hi-$sy);

  ($sx,$sy) = transform($sx,$sy);
  ($tx,$ty) = transform($tx,$ty);
  print "  \\coordinate (middle-start) at  ($sx,$sy);\n";
  print "  \\coordinate (middle-end) at  ($tx,$ty);\n";
}

print "  \\coordinate (bottom) at (0,$y_min);\n";
print "  % end generated\n";
exit 0;
















__END__

# Old code with chamfering
# {
#   {
#     my ($dx,$dy) = (1,0);
#     ($dx,$dy) = transform($dx,$dy);
#     my $x = '0'.chamfer($dx,"\\MyEnd");
#     my $y = '0'.chamfer($dy,"\\MyEnd");
#     print "  \\node (start1) at ($x,$y) [my dot];\n";
#   }
#   {
#     my ($x,$y) = transform($x_hi/2,$y_hi/2);
#     print "  \\coordinate (middle) at ($x,$y);\n";
#   }
#   {
#     my ($x,$y) = ($x_hi,$y_hi);
#     my ($dx,$dy) = (0,-1);
#     ($x,$y) = transform($x,$y);
#     ($dx,$dy) = transform($dx,$dy);
#     $x .= chamfer($dx,"\\MyEnd");
#     $y .= chamfer($dy,"\\MyEnd");
#     print "  \\node (end1) at ($x,$y) [my dot];\n";
#   }
#   $Printwrap = 2;
#   printwrap("\\draw (start1)");
#   $Printwrap += 2;
#   foreach my $n ($n_lo+1 .. $n_hi-1) {
#     my ($x,$y) = $path->n_to_xy($n);
# 
#     if ($xyxy_is_twindragon_touch->($x,$y)) {
#       my ($sdx,$sdy) = $path->n_to_dxdy($n-1);
#       my ($tdx,$tdy) = $path->n_to_dxdy($n);
#       ($x,$y) = transform($x,$y);
#       ($sdx,$sdy) = transform($sdx,$sdy);
#       ($tdx,$tdy) = transform($tdx,$tdy);
#       my $sx = $x . chamfer(-$sdx);
#       my $sy = $y . chamfer(-$sdy);
#       my $tx = $x . chamfer($tdx);
#       my $ty = $y . chamfer($tdy);
#       printwrap("--($sx,$sy)");
#       printwrap("--($tx,$ty)");
#     } else {
#       ($x,$y) = transform($x,$y);
#       printwrap("--($x,$y)");
#     }
#     $y_min = min($y_min, $y);
#   }
#   printwrap("--(end1);\n");
# }
# 
# {
#   {
#     my ($x,$y) = ($x_hi,$y_hi);
#     my ($dx,$dy) = (-1, 0);
#     ($x,$y) = transform($x,$y);
#     ($dx,$dy) = transform($dx,$dy);
#     $x .= chamfer($dx,"\\MyEnd");
#     $y .= chamfer($dy,"\\MyEnd");
#     print "  \\node (start2) at ($x,$y) [my dot];\n";
#   }
#   {
#     my ($dx,$dy) = (0, 1);
#     ($dx,$dy) = transform($dx,$dy);
#     my $x = '0'.chamfer($dx,"\\MyEnd");
#     my $y = '0'.chamfer($dy,"\\MyEnd");
#     print "  \\node (end2) at ($x,$y) [my dot];\n";
#   }
#   $Printwrap = 2;
#   printwrap("\\draw (start2)");
#   $Printwrap += 2;
#   foreach my $n ($n_lo+1 .. $n_hi-1) {
#     my ($x,$y) = $path->n_to_xy($n);
#     ($x,$y) = ($x_hi-$x, $y_hi-$y);
# 
#     if ($xyxy_is_twindragon_touch->($x,$y)) {
#       my ($sdx,$sdy) = $path->n_to_dxdy($n-1);
#       my ($tdx,$tdy) = $path->n_to_dxdy($n);
#       ($x,$y) = transform($x,$y);
#       ($sdx,$sdy) = transform($sdx,$sdy);
#       ($tdx,$tdy) = transform($tdx,$tdy);
# 
#       ($sdx,$sdy) = (-$sdx,-$sdy);
#       ($tdx,$tdy) = (-$tdx,-$tdy);
#       my $sx = $x . chamfer(-$sdx);
#       my $sy = $y . chamfer(-$sdy);
#       my $tx = $x . chamfer($tdx);
#       my $ty = $y . chamfer($tdy);
#       printwrap("--($sx,$sy)");
#       printwrap("--($tx,$ty)");
# 
#     } else {
#       ($x,$y) = transform($x,$y);
#       printwrap("--($x,$y)");
#     }
#     $y_min = min($y_min, $y);
#   }
#   printwrap("--(end2);\n");
# }
# print "  \\end{scope}\n";

