#!/usr/bin/perl -w

# Copyright 2014, 2015, 2016, 2017 Kevin Ryde
#
# This file is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the Free
# Software Foundation; either version 3, or (at your option) any later
# version.
#
# This file is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
# or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
# for more details.
#
# You should have received a copy of the GNU General Public License along
# with this file.  If not, see <http://www.gnu.org/licenses/>.


use 5.004;
use strict;
use Math::PlanePath::DragonCurve 117; # v.117 for level_to_n_range()
use Math::PlanePath::ComplexPlus 117; # v.117 for level_to_n_range()

use FindBin;
use lib $FindBin::Bin;
use MyDragon 'chamfer','xy_rotate_minus90';
use MyLineSegments;
use MyPrintwrap ':all';


my $level = 5;
if (@ARGV) { $level = shift @ARGV; }

print "  % generated by tools/$FindBin::Script\n";
print "  \\newcommand{\\MyLevel}{$level}\n";

my $segments = MyLineSegments->new;
{
  my $path = Math::PlanePath::ComplexPlus->new;
  my ($n_lo, $n_hi) = $path->level_to_n_range($level);

  foreach my $n ($n_lo .. $n_hi) {
    my ($x,$y) = $path->n_to_xy($n);
    $segments->add_cycle (2*$x-1, 2*$y-1,   # SW
                          2*$x+1, 2*$y-1,   # SE
                          2*$x+1, 2*$y+1,   # NE
                          2*$x-1, 2*$y+1);  # NW
  }
}

my @nodes;
my @dragon_parts;
{
  my $shift = sub {
    my ($x,$y) = @_;
    return ($x,$y+1);
  };
  my $transform = sub {
    my ($x,$y) = @_;
    ($x,$y) = xy_rotate_minus90 ($x,$y);
    $x *= 2;
    $y *= 2;
    return ($x,$y);
  };
  my $path = Math::PlanePath::DragonCurve->new;
  my ($n_lo, $n_hi) = $path->level_to_n_range($level);
  my ($x_hi,$y_hi) = $path->n_to_xy($n_hi);

  {
    my ($x,$y) = $shift->(0,0);
    push @nodes, "  \\node (start) at ($x,$y) [my dot];\n";
  }
  {
    my ($x,$y) = $transform->($x_hi,$y_hi);
    ($x,$y) = $shift->($x,$y);
    push @nodes, "  \\node (end) at ($x,$y) [my dot];\n";
  }

  my $join = '';
  foreach my $rot (0,1) {
    push @dragon_parts, $rot==0 ? "(start)" : "(end)";

    foreach my $n ($n_lo .. $n_hi-1) {
      my ($x1,$y1) = $path->n_to_xy($n);
      my ($x2,$y2) = $path->n_to_xy($n+1);
      my ($dx,$dy) = $path->n_to_dxdy($n);
      if ($rot) {
        $x1 = $x_hi - $x1;
        $y1 = $y_hi - $y1;
        $x2 = $x_hi - $x2;
        $y2 = $y_hi - $y2;
        $dx = -$dx;
        $dy = -$dy;
      }
      ($x1,$y1) = $transform->($x1,$y1);
      ($x2,$y2) = $transform->($x2,$y2);
      ($dx,$dy) = $transform->($dx,$dy);
      ($x1,$y1) = $shift->($x1,$y1);
      ($x2,$y2) = $shift->($x2,$y2);

      $segments->remove($x1,$y1, $x2,$y2);

      $x1 .= chamfer($dx);
      $y1 .= chamfer($dy);
      $x2 .= chamfer(-$dx);
      $y2 .= chamfer(-$dy);
      if ($n > $n_lo) {
        push @dragon_parts, "--($x1,$y1)";
        $join = '--';
      }
      if ($n == $n_hi-1) {
        push @dragon_parts, "--" . ( $rot==0 ? "(end)" : "(start);\n" );
      } elsif ($n == 0 && $rot == 0) {
        push @dragon_parts, "--($x2,$y2);", " \\draw[my dragon line] ($x2,$y2)";
      } else {
        push @dragon_parts, "--($x2,$y2)";
      }
    }
  }
}

$Printwrap = 2;
printwrap("\\draw[my base line] ");
$Printwrap += 2;
$segments->printwrap;
printwrap(";\n");

print @nodes;
$Printwrap = 2;
printwrap("\\draw[my first dragon line] ");
$Printwrap += 2;
foreach my $str (@dragon_parts) {
  printwrap($str);
}


print "  % end generated\n";
exit 0;
