#!/usr/bin/perl -w

# Copyright 2014, 2015, 2016, 2017, 2019 Kevin Ryde
#
# This file is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the Free
# Software Foundation; either version 3, or (at your option) any later
# version.
#
# This file is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
# or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
# for more details.
#
# You should have received a copy of the GNU General Public License along
# with this file.  If not, see <http://www.gnu.org/licenses/>.


use 5.004;
use strict;
use Math::PlanePath::DragonCurve 117; # v.117 for level_to_n_range()

use FindBin;
use lib $FindBin::Bin;
use MyLineSegments;
use MyDragon 'transform_to_horizontal','Rnext';
use MyPrintwrap ':all';

# uncomment this to run the ### lines
# use Smart::Comments;


# $level is of two dragons back-to-back, so twindragon k = $level-1.
# Must odd level here so lines between boundary squares are horizontal when
# rotated to endpoints horizontal.
my $level = 15;
if (@ARGV) { $level = shift @ARGV; }

print "  % generated by tools/$FindBin::Script $level\n";
print "  % A high-resolution approximation to the twindragon,\n";
print "  % going from (0,0) to (1,0).  Can fill or draw.\n";
print "  % Lines are drawn connecting the middles of boundary squares.\n";
print "  my twindragon fractal pic/.pic={\n";
print "  \\begin{scope}[scale=1/2^",($level+1)/2,"]\n";

my $path = Math::PlanePath::DragonCurve->new;
my ($n_lo, $n_hi) = $path->level_to_n_range($level);
my $scale = 2**(($level+1)/2);

sub transform {
  my ($x,$y) = @_;
  return transform_to_horizontal($x,$y, $level);
}

sub n_to_cxcy {
  my ($n) = @_;
  my ($x1,$y1) = $path->n_to_xy($n);
  my ($x2,$y2) = $path->n_to_xy($n+1);
  my $dx = $x2 - $x1;
  my $dy = $y2 - $y1;

  my $cx = ($x1+$x2 + $dy)/2;   # right side
  my $cy = ($y1+$y2 - $dx)/2;
  return transform($cx,$cy);
}

my $segments = MyLineSegments->new;
my ($x_hi,$y_hi) = transform($path->n_to_xy($n_hi));

my $TBQ = 0;
{
  my ($prev_cx,$prev_cy) = n_to_cxcy($n_hi - 1);
  ($prev_cx,$prev_cy) = ($x_hi - $prev_cx, $y_hi - $prev_cy);
  ### initial prev: "$prev_cx,$prev_cy"
  for (my $n = 0; $n < $n_hi; $n = Rnext($n)) {
    my ($cx,$cy) = n_to_cxcy($n,1);
    ### cxcy: "$cx,$cy"

    $TBQ += 2*($cx != $prev_cx || $cy != $prev_cy);
    $segments->add($prev_cx,$prev_cy, $cx,$cy);
    $segments->add($x_hi-$prev_cx,$y_hi-$prev_cy, $x_hi-$cx,$y_hi-$cy);
    $prev_cx = $cx;
    $prev_cy = $cy;
  }
}

$Printwrap = 2;
printwrap("\\path[pic actions] ");
$Printwrap += 2;
$segments->printwrap;
printwrap(";\n");
my $draw_count = $segments->lines_joined_count_points;

print "  \\end{scope}\n";
print "  % drawn with $draw_count points\n";
print "% GP-Test-Last  TBQ($level-1) == $TBQ\n";
print "  },\n";
print "  % end generated\n";
exit 0;





__END__

# Previous code drawing boundary segments.

# # prefer an odd level here so squares are horizontal when curve rotated to
# # endpoints horizontal
# my $level = 13;
# if (@ARGV) { $level = shift @ARGV; }
# 
# print "% generated by tools/$FindBin::Script\n";
# print "\\tikzset{\n";
# print "  my twindragon fractal pic/.pic={\n";
# 
# my $path = Math::PlanePath::DragonCurve->new;
# my ($n_lo, $n_hi) = $path->level_to_n_range($level+1);
# my ($x_hi,$y_hi) = $path->n_to_xy($n_hi);
# my $scale = 2**(($level+1)/2);
# 
# sub transform {
#   my ($x,$y) = @_;
#   return transform_to_horizontal($x,$y, $level+1);
# }
# 
# sub path_n_is_right_boundary {
#   my ($path, $n, $n_limit) = @_;
#   ### path_n_is_right_boundary(): $n
#   my ($x1,$y1) = $path->n_to_xy($n);
#   my ($x2,$y2) = $path->n_to_xy($n+1);
#   my ($dx,$dy) = $path->n_to_dxdy($n);
#   ($dx,$dy) = xy_rotate_minus90($dx,$dy);  # to right side
#   my $x1s = $x1 + $dx;
#   my $y1s = $y1 + $dy;
#   my $x2s = $x2 + $dx;
#   my $y2s = $y2 + $dy;
#   return (! path_xyxy_is_traversed_either($path, $x1,$y1, $x1s,$y1s, $n_limit)
#           || ! path_xyxy_is_traversed_either($path, $x2,$y2, $x2s,$y2s, $n_limit)
#           || ! path_xyxy_is_traversed_either($path, $x1s,$y1s, $x2s,$y2s, $n_limit));
# }
# sub path_n_is_left_boundary {
#   my ($path, $n, $n_limit) = @_;
#   ### path_n_is_left_boundary(): $n
#   my ($x1,$y1) = $path->n_to_xy($n);
#   my ($x2,$y2) = $path->n_to_xy($n+1);
#   my ($dx,$dy) = $path->n_to_dxdy($n);
#   ($dx,$dy) = xy_rotate_plus90($dx,$dy);  # to side
#   my $x1s = $x1 + $dx;
#   my $y1s = $y1 + $dy;
#   my $x2s = $x2 + $dx;
#   my $y2s = $y2 + $dy;
#   return (! path_xyxy_is_traversed_either($path, $x1,$y1, $x1s,$y1s, $n_limit)
#           || ! path_xyxy_is_traversed_either($path, $x2,$y2, $x2s,$y2s, $n_limit)
#           || ! path_xyxy_is_traversed_either($path, $x1s,$y1s, $x2s,$y2s, $n_limit));
# }
# sub path_xyxy_is_traversed_either {
#   my ($path, $x1,$y1, $x2,$y2, $n_limit) = @_;
#   my $n = $path->xyxy_to_n_either($x1,$y1, $x2,$y2);
#   return defined $n && $n < $n_limit;
# }
# 
# 
# $Printwrap = 2;
# printwrap("\\begin{scope}[scale=1/$scale]\n");
# my $count_segments = 0;
# my $segments = MyLineSegments->new;
# foreach my $n ($n_lo .. $n_hi-1) {
#   next unless path_n_is_right_boundary($path,$n, $n_hi);
#   my ($x1,$y1) = $path->n_to_xy($n);
#   my ($x2,$y2) = $path->n_to_xy($n+1);
#   $segments->add($x1,$y1, $x2,$y2);
# 
#   $x1 = $x_hi - $x1;
#   $y1 = $y_hi - $y1;
#   $x2 = $x_hi - $x2;
#   $y2 = $y_hi - $y2;
#   $segments->add($x1,$y1, $x2,$y2);
# 
#   $count_segments += 2;
# }
# 
# {
#   $Printwrap = 2;
#   printwrap("\\path[pic actions] ");
#   $Printwrap += 2;
#   $segments->printwrap (transform => \&transform);
#   printwrap(";\n");
# }
# 
# $Printwrap = 2;
# printwrap("\\end{scope}\n");
# 
# my $draw_count = $segments->lines_joined_count_points;
# 
# print "  % total boundary segments $count_segments drawn with $draw_count points\n";
# print "% GP-Test-Last  TB($level) == $count_segments\n";
# print "  },\n";
# print "}%\n";
# print "% end generated\n";
# exit 0;
