#!/usr/bin/perl -w

# Copyright 2016, 2017, 2019 Kevin Ryde
#
# This file is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the Free
# Software Foundation; either version 3, or (at your option) any later
# version.
#
# This file is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
# or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
# for more details.
#
# You should have received a copy of the GNU General Public License along
# with this file.  If not, see <http://www.gnu.org/licenses/>.

use 5.004;
use strict;
use Math::PlanePath::DragonCurve 117; # v.117 for level_to_n_range()

use FindBin;
use lib $FindBin::Bin;
use MyDragon 'transform_to_horizontal';
use MyLineSegments;
use MyXYConvexHull;
use MyPrintwrap ':all';


my $level = 8;
if (@ARGV) { $level = shift @ARGV; }

print "  % generated by tools/$FindBin::Script\n";
print "  \\newcommand{\\MyLevel}{$level}\n";

my $path = Math::PlanePath::DragonCurve->new;
my ($n_lo, $n_hi) = $path->level_to_n_range($level);

sub transform {
  my ($x,$y) = @_;
  ($x,$y) = transform_to_horizontal($x,$y, $level);
  return ($x, $y);
}

my $dragon_segments = MyLineSegments->new;
foreach my $n ($n_lo .. $n_hi-1) {
  $dragon_segments->add($path->n_to_xy($n),
                        $path->n_to_xy($n + 1));
}

my @points;
my %points;
print "  \\node (start) at (0,0) [my dot];\n";
$points{'0,0'}++;
{
  my ($x,$y) = $path->n_to_xy($n_hi);
  ($x,$y) = transform($x,$y);
  $points{"$x,$y"}++;
  print "  \\node (end) at ($x,$y) [my dot];\n";
}
{
  $Printwrap = 2;
  printwrap("\\path foreach \\p in {");
  $Printwrap += 2;
  foreach my $n ($n_lo .. $n_hi) {
    my ($x,$y) = $path->n_to_xy($n);
    ($x,$y) = transform($x,$y);
    next if $points{"$x,$y"}++;
    printwrap("($x,$y),");
    push @points, [$x,$y];
  }
  printwrap("}");
  printwrap(" { \\p node[my dot] };\n");
}

my $hull = MyXYConvexHull->new_from_points (@points);
my $hull_vertices = $hull->vertices;
my $hull_segments = MyLineSegments->new;
foreach my $i (0 .. $#$hull_vertices) {
  $hull_segments->add(@{$hull_vertices->[$i]}, @{$hull_vertices->[$i-1]});
}
$dragon_segments->remove_segments($hull_segments);
my $XYhullB = $hull_segments->length;

$Printwrap = 2;
printwrap("\\draw[my dragon line] ");
$Printwrap = 4;
$dragon_segments->printwrap (transform => \&transform);
printwrap(";\n");

$Printwrap = 2;
printwrap("\\draw[my hull line] ");
$Printwrap = 4;
$hull_segments->printwrap;
printwrap(";\n");

my $num_points = ($level&1 ? $hull->num_points_even : $hull->num_points);
my $P = scalar keys %points;
my $XYhullA = $hull->area;
my $tex_area = $XYhullA;
my $gp_area = $XYhullA;
if ($level&1) {
  if ($XYhullA&1) {
    $tex_area = "\tfrac{$XYhullA}{2}";
    $gp_area = "$XYhullA/2";
  } else {
    $tex_area /= 2;
    $gp_area /= 2;
  }
}
print "  \\newcommand{\\MyPoints}{$num_points}\n";
print "  \\newcommand{\\MyArea}{$tex_area}\n";
print "% GP-Test  P($level) == $P\n";
print "% GP-Test-Last  XYpoints($level) == $num_points\n";
print "% GP-Test-Last  XYhullA($level) == $gp_area\n";
print "% GP-Test-Last  XYhullB($level) == $XYhullB\n";
print "  % end generated\n";
exit 0;







# dots at otherwise unvisited points inside the hull
#
# if (my ($x1,$y1, $x2,$y2) = $h->bbox) {
#   $Printwrap = 2;
#   printwrap("\\path foreach \\p in {");
#   $Printwrap += 2;
#   foreach my $y ($y1 .. $y2) {
#     my ($x1,$x2) = $h->y_to_xrange($y);
#     foreach my $x ($x1 .. $x2) {
#       next if $points{"$x,$y"};
#       printwrap("($x,$y),");
#     }
#   }
#   printwrap("}");
#   printwrap(" { \\p node[my extra dot] };\n");
# }


# my $hull_vertices = $h->vertices;
# $Printwrap = 2;
# printwrap("\\draw[my hull line] ");
# $Printwrap = 4;
# my $join = '';
# foreach my $p (@$hull_vertices) {
#   my ($x,$y) = @$p;
#   printwrap("$join($x,$y)");
#   $join = '--';
# }
# printwrap("--cycle;\n");
