#!/usr/bin/perl -w

# Copyright 2014, 2015, 2016, 2017 Kevin Ryde
#
# This file is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the Free
# Software Foundation; either version 3, or (at your option) any later
# version.
#
# This file is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
# or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
# for more details.
#
# You should have received a copy of the GNU General Public License along
# with this file.  If not, see <http://www.gnu.org/licenses/>.


use 5.010;
use strict;
use FindBin;
use Math::PlanePath::DragonCurve 118; # v.118 for xyxy_to_n_either()

use FindBin;
BEGIN { unshift @INC, $FindBin::Bin }
use MyDragon 'transform_to_horizontal','GS_xy';
use MyLineSegments;
use MyPrintwrap ':all';


print "  % generated by tools/$FindBin::Script\n";
my $level = 14;

my $path = Math::PlanePath::DragonCurve->new;
my ($n_lo, $n_hi) = $path->level_to_n_range($level);
my $scale = 2**($level/2);
print "  \\begin{scope}[scale=1/$scale]\n";

sub transform {
  my ($x,$y) = @_;
  return transform_to_horizontal($x,$y, $level);
}

my $segments = MyLineSegments->new;
foreach my $n ($n_lo .. $n_hi-1) {
  my ($x,$y) = $path->n_to_xy($n);
  my ($dx,$dy) = $path->n_to_dxdy($n);
  ($x,$y) = transform($x,$y);
  ($dx,$dy) = transform($dx,$dy);
  $segments->add($x,$y, $x+$dx,$y+$dy);
}

$Printwrap = 4;
printwrap("\\draw[my segment colour] ");
$Printwrap += 2;
$segments->printwrap;
printwrap(";");
printwrap(" % drawn with " . ($segments->lines_joined_count_points) . " points\n");

{
  my ($x,$y) = GS_xy($level);
  print "% GP-Test  GS($level) == $x+I*$y\n";
  ($x,$y) = transform($x,$y);
  $Printwrap = 4;
  printwrap("\\coordinate (centroid) at ($x,$y);\n");
}

print "  \\end{scope}\n";

print "  % end generated\n";
exit 0;
