#!/usr/bin/perl -w

# Copyright 2014, 2015, 2016 Kevin Ryde
#
# This file is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the Free
# Software Foundation; either version 3, or (at your option) any later
# version.
#
# This file is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
# or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
# for more details.
#
# You should have received a copy of the GNU General Public License along
# with this file.  If not, see <http://www.gnu.org/licenses/>.


use 5.004;
use strict;
use FindBin;
use Math::PlanePath::DragonCurve 117; # v.117 for level_to_n_range()

use FindBin;
BEGIN { unshift @INC, $FindBin::Bin }
use MyDragon 'PlusOffsetH_xy','PlusOffsetV_xy','PlusToDragon',
  'xy_rotate_plus90','rect_corners_sort';
use MyLineSegments;
use MyPrintwrap ':all';

# uncomment this to run the ### lines
# use Smart::Comments;


my $level = 7;  # twindragon level

print "  % generated by tools/$FindBin::Script\n";
print "  \\newcommand{\\MyLevel}{$level}\n";
my $path = Math::PlanePath::DragonCurve->new;

my ($n_lo, $n_hi) = $path->level_to_n_range($level+1);
my ($x_hi,$y_hi) = $path->n_to_xy($n_hi);

$Printwrap = 2;
printwrap("\\node (start) at (0,0) [my dot];\n");
printwrap("\\node (end) at ($x_hi,$y_hi) [my dot];\n");

my ($hx,$hy) = PlusOffsetH_xy($level);
my ($hdragon_x,$hdragon_y) = PlusToDragon($hx,$hy);
print "% GP-Test  PlusOffsetH($level) == $hx+I*$hy\n";
print "% GP-Test  PlusToDragon($hx+I*$hy) == $hdragon_x+I*$hdragon_y\n";

my ($vx,$vy) = PlusOffsetV_xy($level);
my ($vdragon_x,$vdragon_y) = PlusToDragon($vx,$vy);
print "% GP-Test  PlusOffsetV($level) == $vx+I*$vy\n";
print "% GP-Test  PlusToDragon($vx+I*$vy) == $vdragon_x+I*$vdragon_y\n";

my %squares;
my $segments = MyLineSegments->new;
foreach my $n ($n_lo .. $n_hi-1) {
  foreach my $copy (0, 1) {
    my ($x1,$y1) = $path->n_to_xy($n);
    my ($x2,$y2) = $path->n_to_xy($n+1);
    if ($copy) {
      $x1 = $x_hi - $x1;
      $y1 = $y_hi - $y1;
      $x2 = $x_hi - $x2;
      $y2 = $y_hi - $y2;
    }
    $segments->add($x1,$y1, $x2,$y2);

    my $dx = $x2 - $x1;
    my $dy = $y2 - $y1;
    ($dx,$dy) = xy_rotate_plus90($dx,$dy);  # to left side
    $x2 += $dx;
    $y2 += $dy;  # opposite corner
    ($x1,$y1, $x2,$y2) = rect_corners_sort ($x1,$y1, $x2,$y2);

    ### x1y1 lower left: "$x1,$y1"
    if ($x1==$hdragon_x || $y1==$hdragon_y
        || $x1==$vdragon_x || $y1==$vdragon_y) {
      $squares{"$x1,$y1"} = 1;
    }
  }
}

{
  $Printwrap = 2;
  printwrap("\\fill[my square colour] foreach \\p in {");
  $Printwrap = 4;
  foreach my $xy (sort keys %squares) {
    printwrap("($xy),");
  }
  printwrap("}\n");
  printwrap("{ \\p +(\\MyMargin,\\MyMargin) rectangle +(1-\\MyMargin,1-\\MyMargin) };\n");
}

$Printwrap = 2;
printwrap("\\draw ");
$Printwrap += 2;
$segments->printwrap;
printwrap(";\n");

$Printwrap = 2;
printwrap("\\node (PlusOffsetH) at ($hdragon_x+1/2,$hdragon_y+1/2) [my dot];\n");
printwrap("\\node (PlusOffsetV) at ($vdragon_x+1/2,$vdragon_y+1/2) [my dot];\n");

print "  % end generated\n";
exit 0;
