#!/usr/bin/perl -w

# Copyright 2016, 2017 Kevin Ryde
#
# This file is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the Free
# Software Foundation; either version 3, or (at your option) any later
# version.
#
# This file is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
# or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
# for more details.
#
# You should have received a copy of the GNU General Public License along
# with this file.  If not, see <http://www.gnu.org/licenses/>.

use 5.004;
use strict;
use FindBin;
use Math::PlanePath::DragonCurve 117; # v.117 for level_to_n_range()

use FindBin;
BEGIN { unshift @INC, $FindBin::Bin }
use MyDragon 'transform_to_horizontal','xy_to_node_name',
  'P1_xy', 'P2_xy', 'P3_xy', 'P4_xy', 'P5_xy', 'P6_xy',
  'P7_xy', 'P8_xy', 'P9_xy', 'P10_xy','p_xy';
use MyLineSegments;
use MyXYConvexHull;
use MyPrintwrap ':all';


my $level = 6;
if (@ARGV) { $level = shift @ARGV; }

print "  % generated by tools/$FindBin::Script\n";
print "  \\newcommand{\\MyLevel}{$level}\n";

my $path = Math::PlanePath::DragonCurve->new;
my ($n_lo, $n_hi) = $path->level_to_n_range($level);

sub transform {
  my ($x,$y) = @_;
  ($x,$y) = transform_to_horizontal($x,$y, $level);
  return ($x, $y);
}

my @points;
my %point_to_name;
sub node_point {
  my ($x,$y, $name) = @_;
  ($x,$y) = transform($x,$y);
  $name //= xy_to_node_name($x,$y);
  return if exists $point_to_name{"$x,$y"};
  $point_to_name{"$x,$y"} = $name;
  push @points, [$x,$y];
  printwrap(" \\node ($name) at ($x,$y) [my dot];");
}
$Printwrap = 2;
node_point(0,0, 'start');
node_point($path->n_to_xy($n_hi), 'end');
node_point(P1_xy($level), 'P1');
node_point(P2_xy($level), 'P2');
node_point(P3_xy($level), 'P3');
node_point(P4_xy($level), 'P4');
node_point(P5_xy($level), 'P5');
node_point(P6_xy($level), 'P6');
node_point(P7_xy($level), 'P7');
node_point(P8_xy($level), 'P8');
node_point(P9_xy($level), 'P9');
node_point(P10_xy($level), 'P10');
foreach my $n ($n_lo .. $n_hi) {
  my ($x,$y) = $path->n_to_xy($n);
  node_point($x,$y);
}
printwrap_newline();

my $dragon_segments = MyLineSegments->new;
print "  \\begin{scope}[my dragon line]\n";
foreach my $n ($n_lo .. $n_hi-1) {
  my ($x1,$y1) = $path->n_to_xy($n);
  my ($x2,$y2) = $path->n_to_xy($n+1);
  if ($n&1) { ($x1,$y1, $x2,$y2) = ($x2,$y2, $x1,$y1); }
  ($x1,$y1) = transform($x1,$y1);
  ($x2,$y2) = transform($x2,$y2);
  my $name1 = $point_to_name{"$x1,$y1"} // die;
  my $name2 = $point_to_name{"$x2,$y2"} // die "no name for $x2,$y2";
  printwrap(" \\draw($name1)--($name2);");

  unless ($level & 1) {
    $dragon_segments->add($x1,$y1, $x2,$y2);
  }
}
printwrap_newline();
print "  \\end{scope}\n";

my $hull = MyXYConvexHull->new_from_points (@points);
my $hull_points = $hull->vertices;
my $hull_segments = MyLineSegments->new;
foreach my $i (0 .. $#$hull_points) {
  $hull_segments->add(@{$hull_points->[$i]}, @{$hull_points->[$i-1]});
}
$hull_segments->remove_segments($dragon_segments);
$Printwrap = 2;
printwrap("\\draw[my hull line] ");
$Printwrap = 4;
$hull_segments->printwrap (stringize => sub {
                             my ($x,$y) = @_;
                             return '('.$point_to_name{"$x,$y"}.')';
                           });
printwrap(";\n");

# $Printwrap = 2;
# printwrap("\\draw[my hull line] ");
# $Printwrap = 4;
# my $join = '';
# foreach my $p (@$hull_points) {
#   my ($x,$y) = @$p;
#   printwrap("$join($x,$y)");
#   $join = '--';
# }
# printwrap("--cycle;\n");

# dots at otherwise unvisited points inside the hull
#
# if (my ($x1,$y1, $x2,$y2) = $h->bbox) {
#   $Printwrap = 2;
#   printwrap("\\path foreach \\p in {");
#   $Printwrap += 2;
#   foreach my $y ($y1 .. $y2) {
#     my ($x1,$x2) = $h->y_to_xrange($y);
#     foreach my $x ($x1 .. $x2) {
#       next if $points{"$x,$y"};
#       printwrap("($x,$y),");
#     }
#   }
#   printwrap("} ");
#   printwrap("{ \\p node[my extra dot] };\n");
# }

my $num_points = ($level&1 ? $hull->num_points_even : $hull->num_points);
my $P = scalar keys %point_to_name;
my $area = $hull->area;
my $tex_area = $area;
my $gp_area = $area;
if ($level&1) {
  if ($area&1) {
    $tex_area = "\tfrac{$area}{2}";
    $gp_area = "$area/2";
  } else {
    $tex_area /= 2;
    $gp_area /= 2;
  }
}
print "  \\newcommand{\\MyPoints}{$num_points}\n";
print "  \\newcommand{\\MyArea}{$tex_area}\n";
print "% GP-Test  P($level) == $P\n";
print "% GP-Test-Last  XYpoints($level) = $num_points\n";
print "% GP-Test-Last  XYhullA($level) = $gp_area\n";
print "  % end generated\n";
exit 0;
