#!/usr/bin/perl -w

# Copyright 2014, 2015, 2016, 2017, 2018, 2019, 2020, 2021 Kevin Ryde
#
# This file is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the Free
# Software Foundation; either version 3, or (at your option) any later
# version.
#
# This file is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
# or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
# for more details.
#
# You should have received a copy of the GNU General Public License along
# with this file.  If not, see <http://www.gnu.org/licenses/>.

use 5.010;
use strict;
use Carp 'croak';
use File::Spec;
use List::Util 'max','min','sum','pairmap';
use Math::BaseCnv 'cnv';
use Math::Complex 'pi';
use Math::Libm 'hypot';
use Math::Trig 'deg2rad', 'rad2deg';
use POSIX 'floor';

use Math::PlanePath::DragonCurve 118; # v.118 for xyxy_to_n_either()
use Math::PlanePath::Base::Digits
  'bit_split_lowtohigh', 'digit_split_lowtohigh',
  'digit_join_lowtohigh', 'round_down_pow';
use Math::PlanePath::Base::Generic
  'round_nearest';

# use Math::PlanePath;
# *_divrem_mutate = \&Math::PlanePath::_divrem_mutate;
# unshift @INC, '/usr/share/perl5';

use FindBin;
use lib::abs "$FindBin::Bin/../tools";
use MyDragon ':all';
our @ipc;
$|=1;

# uncomment this to run the ### lines
use Smart::Comments;

{
  # Fifths Triples per A260482
  # s1,s5
  # s1,s6
  # s1,s8
  # s8

  foreach my $elem (
                    # [1, $MyDragon::triple_5ths_segs[1]],
                    # [2, $MyDragon::triple_5ths_segs[2]],
                    [3, $MyDragon::triple_5ths_segs[3]],
                    # [4, $MyDragon::triple_5ths_segs[4]],
                   ) {
    my ($frac, $want_segs) = @$elem;
    fBpred_surround_init();
    my $f = $fBpred_surround_flat->clone;
    $f->unset_accepting($f->get_states);
    foreach my $state (0 .. $#fBpred_surround_state_to_conf) {
      my $conf = $fBpred_surround_state_to_conf[$state];
      if ($want_segs->is_subset($conf)) {
        print "accepting $state\n";
        $f->set_accepting($state);
      }
    }
    $f = $f->MyFLAT::minimize(verbose=>1);
    $f = $f->MyFLAT::digits_multiply (mul=>5, add=>1);
    $f = $f->MyFLAT::minimize(verbose=>1);
    $f->MyFLAT::view;

    # my $xf = fVisits_adjacent_DFA($x,0);
    # my $yf = fVisits_adjacent_DFA($y,0);
    # my $both = $xf->intersect($yf)->MyFLAT::minimize(verbose=>1);
    # $both->MyFLAT::view;
  }
  exit 0;
}
{
  my $n = cnv(110010,2,10);
  ### $n
  my $conf = MyDragonConf->initial
    (side => 0,
     # within => $MyDragon::triple_15ths_and_next_within,
    );
  foreach my $bit (
                   # 1,1,0,0,1,0
                   0,1,1,1,0
                  ) { $conf = $conf->expand($bit) ; }
  $conf->print;
  exit 0;
}

__END__
{
  require MyATT;
  require FLAT;
  require MyFLAT;
  my $att = MyATT->read_file('/tmp/foo.att');
  $att->as_FLAT(transducer=>1)->MyFLAT::view;
  exit 0;
}
{
  # BitAboveLowestOne_HtoL_flat()
  my $f = BitAboveLowestOne_HtoL_flat(1);
  # $f->MyFLAT::view;
  # BitAboveLowestOne_HtoL_flat(1)
  #   ->MyFLAT::minimize(verbose=>1)
  #   ->MyFLAT::view;
  $f->MyFLAT::FLAT_print_tikz;
  exit 0;
}

{
  # dNextL_flat
  dNextL_LtoH_flat(1)->MyFLAT::view;
  dNextL_LtoH_flat(1)
    ->MyFLAT::minimize(verbose=>1)
    ->MyFLAT::view;

  exit 0;
}


{
  # McWhorter, Tazelaar
  my ($D,$L,@cells,@I,@division);

  {
    # dragon curd
    $D = 4;
    $L = 2;
    @cells = (0);
    @I = (0,1,2,3,4,5,6,7);
    @division = ([0,5],
                 [2,5],
                 [2,7],
                 [0,7],
                 [4,1],
                 [6,1],
                 [6,3],
                 [4,3]);
  }


  {
    # dragon
    $D = 4;
    $L = 2;
    @cells = (0);
    @I = (0,1,2,3);
    @division = ([0,1],
                 [2,1],
                 [2,3],
                 [0,3]);
  }
  {
    # terdragon
    $D = 3;
    $L = 3;
    @cells = (0);
    @I = (0,1,2);
    @division = ([0,1,0],
                 [1,2,1],
                 [2,0,2]);
  }


  {
    # dragon boundary -- triple expansion
    $D = 4;
    $L = 2;
    @cells = (0,1,2,5,3,4);
    @I = (0,1,2,3,
          0,2);
    @division = ([ 1,0,1 ],  # 0 a -> c,a,c
                 [ 2  ],  # 1 c -> b
                 [ 5  ],  # 2 b -> a[other]
                 [ 4  ],  # 3 c -> b[other]
                 [ 0  ],  # 4 b -> a
                 [ 3,5,3 ],  # 5 a -> c,a,c[other]
                );

    #         a 5    b 2
    #       <------<------
    #      |              ^
    #  c 3 |              | 1 c
    #      v              |
    #       ------>|----->
    #         b 4    a 0
    #
    #                a 5    b 2
    #              <------<------        total 10
    #         c 3 |              ^
    #             |              | 1 c
    #        a 5  v              |
    #       <------       |----->
    #      |             ^  0 a
    #  c 3 |             |
    #      v             | 1 c  NEW
    #       ------>----->
    #         b 8   a 0
  }
  {
    # dragon interior
    $D = 4;
    $L = 2;
    @cells = (1);
    @I = (0,1,2,3);
    @division = ([5,1],
                 [2,1],
                 [7,3],
                 [0,3],
                 [0,4],
                 [5,6],
                 [2,6],
                 [7,4]);
  }

  {
    # Dragon Boundary
    $D = 4;
    $L = 2;
    @cells = (0,5,6,11);
    @I = (0,1,2,3,
          0,1,2,3,
          0,1,2,3,);
    @division = ([ 0,5],     # 0 = a2
                 [ 6,9],
                 [ 7,2],
                 [11,4],
                 [ 8  ],     # 4 = a3
                 [ 6  ],
                 [10  ],
                 [ 4  ],
                 [ 0,1],     # 8 = a1
                 [10,9],
                 [ 3,2],
                 [11,8]);

    #
    #      a 6
    #     <------    initial
    #    ^       ^
    # 11 |       | 5
    #    |       |
    #     ------>
    #      a 0
    #
    #            a     b
    #           10     6
    #        <------<------
    #       |              ^
    #  c 11 |              | 5 c
    #       v              |
    #        ------> O---->
    #           8      0
    #           b      a
    #
    #                  a     b
    #                 10     6
    #              <------<------        total 10
    #   NEW   c 3 |              ^
    #             |              | 5 c
    #   NEW  a 2  v              |
    #       <------       O----->
    #      |             ^  0 a
    # c 11 |             |
    #      v             | 1 c  NEW
    #       ------>----->
    #         b 8   a 0
    #
    #                  a     b
    #                 10     6
    #              <------<------        total 10
    #         c 3 |              ^
    #             |              | 5 c
    #        a 2  v              |
    #       <------       |----->
    #      |             ^   0 a
    # c 11 |             | 9 c
    #      v             |   6 b
    #       ------>       <-----
    #        b 4   |             ^
    #          c 7 |             | 5 c
    #        a 2   v             |
    #       <------        ----->
    #      |              ^  0 a
    # c 11 |              |
    #      v              | 1 c
    #       ------> ----->
    #         b 8   a 0
    #
    # my(a=1,b=1,c=1); 2*a + 2*b*2 + 2*c*2
    #
    # common prefix length states
    # 3,4,7,12,21,37,62,104,177,300,509,865,1466,2484
    # a+2c+b
    # 4,7,13,21,35,61,103,173,295,501,847,1437,2439
    #
    # common prefix length directions
    # 3,5,8,13,22,37,62,105,178,301,510,865,1466,2485,4214,7145,12114,20541,34830,59057,100138,169797,287910,488185
    # g = 1/2*(1)/(1 - x) +  1/2*(5 + 4*x + 6*x^2)/(1 - x - 2*x^3)
    # g == (1/2)/(1 - x) + 1/4*gdJA(x)/x^6 + 1/4*gdJA(x)/x^5 + 1/2*gdJA(x)/x^4 - (6*x^3 + 4*x^2 + 2*x + 1)/(4*x^4)
    # g == (1/2)/(1 - x) + gdJA(x)/x^4 + 1/2*gdJA(x)/x^3 + gdJA(x)/x^2 - (3*x + 2)/(2*x^2)
    # g == (1/2)/(1 - x) + gdJA(x)/x^4 + 1/2*gdJA(x)/x^3 + gdJA(x)/x^2 - (3*x + 2)/(2*x^2)
    # gRQ(x) + gLQ(x) + gLQ(x) - x*gJA(x)
    # vector(14,k,RQ(k) + LQ(k) + RQ(k) - JA(k))
  }

  my $step = 15;
  my $k = 10;
  my @prev_cells;
  my @common_length;
  foreach (1 .. $k) {
    @prev_cells = @cells;
    if (@cells < 32) {
      print join(', ',@cells),"\n";
    }
    @cells = map { @{$division[$_]} } @cells;

    my @dir = map {$I[$_] // ()} @cells;
    my @prev_dir = map {$I[$_] // ()} @prev_cells;
    push @common_length, aref_common_prefix_length(\@dir,\@prev_dir);
  }
  if (@cells < 32) {
    print join(', ',@cells),"\n";
  }
  print "cells length ",scalar(@cells),"\n";
  print "common prefix lengths ",join(',',@common_length),"\n";

  {
    require Math::OEIS::Grep;
    Math::OEIS::Grep->search(array => \@common_length, verbose=>1,
                             name => 'common length');

    my @values = map {$I[$_] // ()} @cells;
    @values = splice @values, 10, 60;
    # @values = map {$D-1 - $_} @values;   # opposite curl
    Math::OEIS::Grep->search(array => \@values, verbose=>1, name => 'direction');
    @values = map {($values[$_] - $values[$_+1]) % $D} 0 .. $#values-1;
    Math::OEIS::Grep->search(array => \@values, verbose=>1, name => 'turn');

    @values = map {$_ > $D/2 ? $_-$D : $_} @values;  # -1,0,1,2
    Math::OEIS::Grep->search(array => \@values, verbose=>1, name => 'turn +/-');

    @values = map {$_ > 0 ? 1 : 0} @values;  # +/-
    Math::OEIS::Grep->search(array => \@values, verbose=>1, name => 'turn sign');
  }

  require Language::Logo;
  my $lo = Logo->new(update => 2, port => 8200 + (time % 100),
                     width => 1200, height => 900);
  my $draw;
  # $lo->command("seth 135; backward 200; seth 90");
  $lo->command("pendown; hideturtle");
  $lo->command("forward 2;
                left 90; forward 2;
                left 90; forward 4;
                left 90; forward 4;
                left 90; forward 4;
                left 90; forward 2;
                right 90; backward 2");
  my $angle = 0;
  foreach my $cell (@cells) {
    my $a = $I[$cell] // next;
    $angle = -$a * 360/$D + 90;
    $lo->command("seth $angle");
    if ($a >= $D) {
      $lo->command("penup; forward $step; pendown");
    } else {
      $lo->command("forward $step");
    }
  }
  $lo->disconnect("Finished...");
  exit 0;

  sub aref_common_prefix_length {
    my ($a1, $a2) = @_;
    my $end = min($#$a1, $#$a2);
    for (my $i = 0; $i <= $end; $i++) {
      if ($a1->[$i] != $a2->[$i]) {
        return $i;
      }
    }
    return $end+1;
  }
}
{
  # LenDir4 pred

  require MyFLAT;
  foreach my $len (4 .. 10) {
    my $f = LenDir4_flat($len, le=>0);
    # $f->MyFLAT::view;
    $f = $f->MyFLAT::reverse->MyFLAT::minimize;
    $f->MyFLAT::view;
  }
  exit;

  sub LenDir4_flat {
    my ($len, %options) = @_;
    require FLAT;
    if ($len < 4) {
      return FLAT::Regex->new('#')->as_dfa;
    }
    my $ret;
    foreach my $d (0 .. 3) {
      my $f = dir4_flat($d);
      foreach (2 .. $len) {
        $f = $f->MyFLAT::digits_increment(add => -1)
          ->MyFLAT::optional_leading_0s
          ->union(dir4_flat($d))
          ->MyFLAT::minimize (verbose => 1);
      }
      $ret //= $f;
      $ret = $ret->intersect($f)
        ->MyFLAT::minimize;
    }
    if ($options{'le'}) {
      return $ret->MyFLAT::set_name("LenDir4 len <= $len");
    } else {
      return $ret->difference(LenDir4_flat($len-1))
        ->MyFLAT::minimize
        ->MyFLAT::set_name('LenDir4 len=$len');
    }
  }
  use Memoize;
  BEGIN { memoize('LenDir4_flat', LIST_CACHE=>'MERGE') }
}
{
  # dTurnLeft even n dNextL tranmuted low 0s to 1s
  # left at bit above lowest 1 is 0
  # d=1 even left  01 0* 00   -> 01 1* 11
  # d=2 even left  00    10   -> 00    11
  # d=3 even left  10    10   -> 10    11
  #     

  foreach my $n (0 .. 1024) {
    next unless TurnLpred($n);
    next unless $n%2==0;
    my $n2 = cnv($n,10,2);
    my $v2 = $n2;
    $v2 =~ s{0+$}{'1' x length($&)}e or die;
    my $d = dNextL($n);
    my $is1 = ($v2 =~ /(^|0)1 1* 11$/x ? 1 : 0);
    my $is1_bad = ($is1==($d==1) ? '' : ' is1bad');
    my $is2 = ($v2 =~ /(^|^0|00) 11$/x ? 1 : 0);
    my $is2_bad = ($is2==($d==2) ? '' : ' is2bad');
    my $is3 = ($v2 =~ /10 11$/x ? 1 : 0);
    my $is3_bad = ($is3==($d==3) ? '' : ' is3bad');
    $is1 + $is2 + $is3 == 1 or print "n=$n oops $is1 $is2 $is3\n";
    $d != 4 or die "d=4 at n=$n";
    print "$n2 $v2 d=$d $is1_bad$is2_bad$is3_bad\n";
  }
  exit 0;
}



{
  # dNextR state machine
  require MyFLAT;
  my $d = 1;
  TurnRpred_flat()->contains('110') or die;
  my $f = TurnRpred_flat()->MyFLAT::digits_increment(add => -$d)
    ->MyFLAT::minimize(verbose=>1);  # left at n+d
  foreach my $i (1 .. $d-1) {        # right at n+1 .. n+d-1
    $f = $f->intersect(TurnLpred_flat()->MyFLAT::digits_increment(add => -$i))
      ->MyFLAT::minimize(verbose=>1)
    }
  $f = $f->MyFLAT::optional_leading_0s
    ->MyFLAT::set_name("dNextR d=$d")
    ->MyFLAT::minimize(verbose=>1);
  # MyFLAT::FLAT_show_breadth($f, 7);
  # $f->MyFLAT::view;

  $f = $f->MyFLAT::reverse
    ->MyFLAT::minimize(verbose=>1);
  $f->MyFLAT::view;

  exit 0;
}
{
  # dNextL of n-1 state machine
  my $d = 3;
  require MyFLAT;
  my $T = TurnLpred_flat()->MyFLAT::digits_increment(add => -1)
    ->MyFLAT::optional_leading_0s;
  my $f = dNextL_flat($d)->MyFLAT::digits_increment(add => -1)
    ->MyFLAT::optional_leading_0s
    # ->intersect($T)
    ->MyFLAT::set_name("dNextL of n-1 d=$d")
    ->MyFLAT::minimize(verbose=>1)
    ;
   $f->MyFLAT::view;
  # $f->MyFLAT::FLAT_print_tikz;

  print TurnLpred(0b1010101 + 1),"\n";
  print dNextL(0b1010101 + 1),"\n";
  print $f->contains('1010101'),"\n";
  exit 0;
}
{
  # dNextL state machine
  require MyFLAT;
  my $d = 1;
  my $f = TurnLpred_flat()->MyFLAT::digits_increment(add => -$d)
    ->MyFLAT::minimize(verbose=>1);  # left at n+d
  foreach my $i (1 .. $d-1) {        # right at n+1 .. n+d-1
    $f = $f->intersect(TurnRpred_flat()->MyFLAT::digits_increment(add => -$i))
      ->MyFLAT::minimize(verbose=>1)
    }
  $f->MyFLAT::set_name("dNextL d=$d");
  $f = $f->MyFLAT::optional_leading_0s
    ->MyFLAT::minimize(verbose=>1);
  # MyFLAT::FLAT_show_breadth($f, 7);
  # $f->MyFLAT::view;

  foreach my $n (0 .. 255) {
    my $n2 = cnv($n,10,2);
    if (!!$f->contains($n2) != (dNextL($n)==$d)) {
      print "$n2\n";
      die;
    }
  }

  $f = $f->MyFLAT::reverse
    ->MyFLAT::minimize(verbose=>1);
  $f->MyFLAT::view;
  dNextL_LtoH_flat(1)->MyFLAT::view;
  dNextL_LtoH_flat(1)
    ->MyFLAT::minimize(verbose=>1)
    ->MyFLAT::view;

  exit 0;
}
{
  # dTurnLeft state machine of n-1
  foreach my $d (1 .. 4) {
    require MyFLAT;
    my $f = dTurnLeft_flat($d);
    $f = $f->intersect(TurnLpred_flat());
    $f = $f->MyFLAT::digits_increment(add => -1);
    $f = $f->MyFLAT::optional_leading_0s;
    $f = $f->MyFLAT::minimize(verbose=>1)
      ->MyFLAT::set_name("d=$d");
    # MyFLAT::FLAT_show_breadth($f, 7);
    $f->MyFLAT::view;
  }
  exit 0;
}
{
  # dTurnLeft state machine
  require MyFLAT;
  my $want = 3;
  my $TurnLpred = MyDragon::TurnLpred_flat();
  my $TurnLpred_not = $TurnLpred->complement;
  $TurnLpred = FLAT::Regex->new('0*')->as_dfa->MyFLAT::concat($TurnLpred);
  my $f = $TurnLpred;
  $f = $f->intersect($TurnLpred->MyFLAT::digits_increment(add => -$want))
    ->MyFLAT::minimize(verbose=>1);
  foreach my $i (1 .. $want-1) {
    $f = $f->intersect($TurnLpred_not->MyFLAT::digits_increment(add => -$i))
      ->MyFLAT::minimize(verbose=>1);
  }
  MyFLAT::FLAT_show_breadth($f, 7);
  $f->MyFLAT::view;
  exit 0;
}


{
  # w[k] in binary, similar to Wolfram's page 100
  require GD;
  require Image::Base::GD;

  my $width = 1300;
  my $height = 900;
  my $gd = GD::Image->new ($width,$height, 1); # truecolour 24-bit
  my $image = Image::Base::GD->new (-gd => $gd);
  $image->rectangle(0,0, $width-1,$height-1, 'black');

  my $w = to_bigint(1);
  my $mask = (to_bigint(1) << $height) - 1;
  my $count = 0;
  foreach my $x (0 .. $width-1) {
    if ($x < 10) { print "$w\n"; }
    my $bits = $w & $mask;
    my $y = $height-1;
    while ($bits && $y >= 0) {
      if ($bits & 1) {
        ### draw: "$x,$y  bits=$bits"
        $image->xy($x,$y, 'white');
        $count++;
      }
      $bits >>= 1;
      $y--;
    }
    $w += ($w>>1) + ($w&1);
  }
  print "w=$w, total 1s $count\n";
  my $filename = "/tmp/wbits.png";
  $image->save($filename);
  require IPC::Run;
  push @ipc, IPC::Run::start(['xzgv','-f',$filename],'&');
  exit 0;
}
{
  # A164395 how many binary no substring 0001 or 0101
  require FLAT;
  require MyFLAT;
  my $f = FLAT::Regex->new('(0|1)* (0001|0101) (0|1)*')
    ->as_dfa
    # ->complement
    ->MyFLAT::minimize(verbose=>1)
    ->MyFLAT::set_name('no substrings');
  $f->MyFLAT::view;
  $f = $f->MyFLAT::minimize(verbose=>1);

  $f->MyFLAT::reverse
    ->MyFLAT::minimize(verbose=>1)
    ->MyFLAT::set_name('no substrings LtoH')
    ->MyFLAT::view;

  MyDragon::Dpred_k_flat()
      ->MyFLAT::reverse
      ->MyFLAT::minimize(verbose=>1)
      ->MyFLAT::set_name('Dpred_k LtoH')
      ->MyFLAT::view;
  exit 0;
}
{
  require FLAT;
  require MyFLAT;
  my $f = FLAT::DFA->new;
  $f->add_states(3);
  $f->set_starting(0);
  $f->set_accepting(2);
  $f->add_transition(0,0, 0);
  $f->add_transition(0,1, 1);
  $f->add_transition(1,0, 0);
  $f->add_transition(1,1, 1);
  # $f->MyFLAT::view;
  my $re = MyFLAT::DFA_to_Regex($f);
  ### $re
  my $fre = FLAT::Regex->new($re);
  ### eq: $fre->as_dfa->equals($f)
  exit 0;
}
{
  # wparity strings state machine

  my $n = 4;
  require FLAT;
  require MyFLAT;
  my $f = FLAT::DFA->new;
  my $mask = (1<<$n)-1;
  $f->add_states(2**$n);
  foreach my $i (0 .. $mask) {
    foreach my $bit (0,1) {
      my $to = $i + ($bit << $n);
      $to += ($to>>1) + ($to&1);
      $to &= $mask;
      $f->add_transition($i,$to, $bit);
    }
  }
  $f->MyFLAT::view;
  exit 0;
}
{
  # Lines(k) count
  # 1,2,4,8,14,24,42,72,122,208,354,600,1018,1728,2930,4968
  # A164395 Number of binary strings of length n with no substrings equal to 0001 or 0101
  # A164395 14,24,42,72,122,208,354,600,1018,1728,2930,4968,8426,14288,24226,41080,69658,118112,200274,339592,575818,976368,1655554,2807192,4759930,8071040,13685426,23205288,39347370,66718224,113128802,191823544,
  # -2*x^4*(-7+2*x-4*x^2+8*x^3)/ ( (x-1)*(2*x^3+x-1) )
  # = num single points 0 to 2^k-1 inclusive
  #
  # unfold, join points lines extend
  # SEG(k) = if(k==0,1, 2*SEG(k-1) - 2*JA(k-1))
  # vector(10,k,SEG(k)) == [2, 4, 8, 14, 24, 42, 72, 122, 208, 354]
  #
  # every line segment has a single at each end, and all ends are re-used,
  # so S(k)-1
  # vector(10,k,S(k)-1) == [2, 4, 8, 14, 24, 42, 72, 122, 208, 354]
  #
  # half horizontal, half vertical

  require MyLineSegments;
  my $path = Math::PlanePath::DragonCurve->new;
  my @values;
  foreach my $k (0 .. 12) {
    my ($n_lo, $n_hi) = $path->level_to_n_range($k);
    my $segments = MyLineSegments->new;
    foreach my $n ($n_lo .. $n_hi-1) {
      $segments->add($path->n_to_xy($n),
                     $path->n_to_xy($n + 1));
    }
    my @lines = $segments->lines;
    @lines = grep {$_->[0] == $_->[2]} @lines;
    my $count = scalar(@lines);
    push @values, $count;
    print "k=$k  $count\n";
  }
  print join(',',@values),"\n";
  shift @values;
  shift @values;
  shift @values;
  shift @values;
  require Math::OEIS::Grep;
  Math::OEIS::Grep->search(array => \@values, verbose=>1);
  exit 0;
}

{
  # Rpred Not 7ths (not right)
  my $f = MyDragon::NonRpred_flat();
  my $sevenths = FLAT::Regex->new('(0|1)* (001)*')
    ->as_dfa
    ->MyFLAT::minimize(verbose=>1);
  my $non_sevenths = $sevenths->complement;
  $f = $f->intersect($sevenths);
  MyDragon::NonRpred_flat();
  $f = $f->MyFLAT::minimize(verbose=>1);
  $f->MyFLAT::view;
  exit 0;
}

{
  # fVisits NFA
  my $f = MyDragon::fVisits_adjacent_NFA();
  $f->MyFLAT::FLAT_print_tikz;

  fBpred_flat()->MyFLAT::view;
  exit 0;
}
{
  # fVisits adjacent states

  foreach my $s (1 .. 11) {
    printf "s=%-2d  %6s %6s\n", $s,
      join(',',MyDragon::fVisits_expand_bit($s, 0)),
      join(',',MyDragon::fVisits_expand_bit($s, 1));
  }

  my $f = MyDragon::fVisits_adjacent_NFA();
  $f = MyDragon::fVisits_adjacent_DFA(1,0);
  $f = $f->complement;
  $f = MyFLAT::eventually_accepting($f);
  $f = $f->MyFLAT::minimize(verbose=>1);
  print $f->num_states,"\n";
  $f->MyFLAT::view;

  exit 0;
}
{
  &fBpred_hullstouch_init (verbose => 2);
  exit 0;
}


{
  foreach my $s (1..4) {
    my $f = Rsides_flat($s);
    $f = $f->MyFLAT::minimize(verbose=>1);
    $f->MyFLAT::view;
  }
  exit 0;
}


{
  # single-visited on centreline

  # singles, unduplicated
  # 1,2,4,8,14,26,44,80,134
  # vector_modulo([5/3, 3],k) * 3^floor(k/2) - 1
  #
  # doubles (both visits)
  # 0,0,0,0,2,6,20,48,122,
  # 2^k - vector_modulo([5/3, 3],k) * 3^floor(k/2) + 1
  #
  # total visits
  # 1,2,4,8,16,32,64,128,
  # 2^k

  my $f = CentrelinePred_flat('point');
  $f = $f->intersect(Spred_k_flat());
  # $f = $f->intersect(Dpred_k_flat());
  $f = $f->MyFLAT::reverse;
  $f = $f->MyFLAT::minimize(verbose=>1);
  $f->MyFLAT::view;
  my @counts = MyFLAT::FLAT_count_contains ($f, to_bigint(20));
  # aref_delete_evens(\@counts);
  print join(',',@counts),"\n";
  require Math::OEIS::Grep;
  Math::OEIS::Grep->search(array => \@counts, verbose=>1);
  exit 0;

  sub aref_delete_evens {
    my ($aref) = @_;
    for (my $i = $#$aref & ~1; $i >= 0; $i -= 2) {
      splice @$aref, $i, 1;
    }
  }
}
{
  # N on centreline by bits

  #                    point at start   point at arrow
  #                    r3   r2    r1     e3  e2  e1
  #      ^   ^   ^       \   |   /         \  |  /
  #        \ | /           v v v            v v v
  #   -------*-------h-------*----------------*--
  #        / | \           / | \
  #      v   v   v       v   v   v
  #    s1   s2   s3    a1   a2   a3
  #   point at start   point at arrow

  $|=1;
  my %table = (h    => {0=>'s3', 1=>'a1'},
               s3   => {0=>'s2', 1=>'non'},
               s2   => {0=>'s1', 1=>'non'},
               s1   => {0=>'h',  1=>'e2'},

               hr   => {0=>'s1',  1=>'a3'},
               a1   => {0=>'r2',  1=>'hr'},
               a2   => {0=>'non', 1=>'a1'},
               a3   => {0=>'non', 1=>'a2'},
               r2   => {0=>'non', 1=>'a3'},

               e2   => {0=>'s3',  1=>'non'},

               non  => {0=>'non', 1=>'non' },
              );
  my %accepting = (
                   h  => 'centreline',
                   hr => 'centreline',
                   s1 => 'centreline',
                   s2 => 'centreline',
                   s3 => 'centreline',
                   e2 => 'centreline',
                   non => 'non',
                  );
  require MyFSM;
  my $fsm = MyFSM->new(table => \%table,
                       initial => 'h',
                       accepting => \%accepting,
                       title => 'CentrelinePred by bits',
                       flow => 'east',
                      );
  # $fsm = $fsm->digit_delta;
  $fsm->simplify (verbose => 1);

  #    # $fsm = $fsm->reverse;
  #  $fsm->simplify (verbose => 1);
  #  $fsm->renumber_states;

  # $fsm->view;
  # $fsm->print_tikz;
  # CentrelinePred_flat('segment')->MyFLAT::view;

  CentrelinePred_flat('segment')
    ->MyFLAT::reverse
    ->MyFLAT::minimize
    ->MyFLAT::view;


  my $path = Math::PlanePath::DragonCurve->new;
  foreach my $k (0 .. 16) {
    my ($n_lo, $n_hi) = $path->level_to_n_range($k);
    foreach my $n ($n_lo .. $n_hi-1) {
      my ($x,$y) = $path->n_to_xy($n);
      ($x,$y) = transform_to_horizontal($x,$y, $k);
      my $want = ($y==0) ? 1 : 0;
      my $got = CentrelinePred_by_bits($n,$k) ? 1 : 0;
      if ($got != $want) {
        my $n2 = cnv($n,10,2);
        print "k=$k  n=$n2  got=$got want=$want\n";
      }
    }
  }
  exit 0;

  sub CentrelinePred_by_bits {
    my ($n, $k) = @_;
    ### CentrelinePred(): "n=$n  k=$k"
    my $state = 'h';
    my @digits = bit_split_lowtohigh($n);
    $#digits = $k-1;
    ### digits: reverse @digits
    foreach my $digit (reverse @digits) {
      ### at: "state=$state"
      $state = $table{$state}->{$digit||0} // die;
      $state || return 0;
    }
    ### final: $state
    return ($accepting{$state}||'') eq 'centreline';
  }
}

{
  # Epred 9 consecutive
  my $f = Epred_flat();
  for (1 .. 7) {
    $f = Epred_flat()->intersect($f->MyFLAT::digits_increment(add => -1))
      ->MyFLAT::minimize(verbose=>1);
  }
  $f->MyFLAT::set_name('EpredNine');

  # $f = $f->intersect(TurnRpred_flat());
  # $f = $f->intersect( EpredR_flat()
  #                     ->MyFLAT::concat(FLAT::Regex->new('(0|1)*')->as_dfa) );
  # $f = $f->intersect( FLAT::Regex->new('(0|1)* 0111 (01|10)')->as_dfa );

  # $f = $f->MyFLAT::digits_increment(add => 1);
  # $f = $f->MyFLAT::minimize(verbose=>1);
  # $f->MyFLAT::view;

  $f = $f->MyFLAT::reverse->MyFLAT::set_name('EpredNine LtoH');
  $f = $f->MyFLAT::minimize(verbose=>1);
  $f->MyFLAT::view;

  # my $t = FLAT::Regex->new('101')->as_dfa
  #   ->MyFLAT::concat(EpredL_LtoH_flat())
  #   ->MyFLAT::minimize(verbose=>1)
  #   ->MyFLAT::set_name('concat');
  #
  # $t->MyFLAT::view;
  # $f->MyFLAT::view;
  # MyFLAT::FLAT_check_is_equal($t, $f);
  exit 0;
}

{
  # EpredR view
  EpredR_LtoH_flat->MyFLAT::view;
  exit 0;
}

{
  # EpredL three consecutive
  my $f = EpredL_flat();
  for (1 .. 2) {
    $f = EpredL_flat()->intersect($f->MyFLAT::digits_increment(add => -1))
      ->MyFLAT::minimize(verbose=>1);
  }
  $f->MyFLAT::set_name('EpredL3');

  # $f = $f->MyFLAT::digits_increment(add => 1);
  # $f = $f->MyFLAT::minimize(verbose=>1);
  # $f->MyFLAT::view;

  $f = $f->MyFLAT::reverse->MyFLAT::set_name('EpredL3 LtoH');
  $f = $f->MyFLAT::minimize(verbose=>1);
  # $f->MyFLAT::view;

  my $t = FLAT::Regex->new('101')->as_dfa
    ->MyFLAT::concat(EpredL_LtoH_flat())
    ->MyFLAT::minimize(verbose=>1)
    ->MyFLAT::set_name('concat');

  $t->MyFLAT::view;
  $f->MyFLAT::view;
  MyFLAT::FLAT_check_is_equal($t, $f);
  exit 0;
}

{
  require FLAT::Regex;
  require MyFLAT;
  my $f = FLAT::Regex->new('0*(111*0)00*')->as_dfa;
  $f->MyFLAT::view;
  $f = $f->MyFLAT::blocks(2);
  $f->MyFLAT::view;
  exit 0;
}

{
  # dir flat reversal

  require MyFLAT;
  foreach my $want_dir4 (0 .. 3) {
    my $HtoL = dir4_flat($want_dir4);
    # $HtoL->MyFLAT::view;
    my $LtoH = $HtoL->MyFLAT::reverse;
    $LtoH = $LtoH->MyFLAT::minimize(verbose=>1);
    $LtoH = $LtoH->MyFLAT::separate_sinks
      ->MyFLAT::set_name($HtoL->{'name'} . ' LtoH');
    $LtoH->MyFLAT::view;
  }
  #  MyFLAT::FLAT_print_tikz($LtoH);
  exit 0;
}
{
  # TA domination number
  # recurrence_guess([1,1,2,4,7,14,27,53,105,209,417]);
  # recurrence_guess([14,27,53,105,209,417]);
  # 2/5 * 2^n
  # gTAdomnum(x) = 2/5/(1 - 2*x) + 1/5*(3 + x - x^2)/(1 - x^2 - x^3)
  # polroots(1 - x^2 - x^3)

  require MyGraphs;
  require Graph::Maker::TwindragonAreaTree;
  my @values;
  foreach my $k (0 .. 12) {
    print "k=$k\n";
    my $graph = Graph::Maker->new('twindragon_area_tree',
                                  level => $k,
                                  undirected => 1);
    my $domnum = MyGraphs::Graph_tree_domnum($graph);

    print "  domnum $domnum\n";
    push @values, $domnum;
  }
  require Math::OEIS::Grep;
   Math::OEIS::Grep->search(array => \@values, verbose=>1);
  exit 0;
}

{
  # TA independent sets

  #      0000 0001 0010 0011 0100 0101 0110 0111 1000 1001 1010 1011 1100 1101 1110 1111
  #    2    1    0    0    0    0    0    0    0    0    0    0    0    0    0    0    1
  #    3    1    0    0    1    0    0    0    0    0    0    0    0    1    0    0    0
  #    8    1    1    1    0    1    1    0    0    1    1    1    0    0    0    0    0
  #   60    8    5    6    3    6    4    4    2    5    3    4    2    3    2    2    1
  # 3024  476  276  286  162  286  166  171   97  276  160  166   94  162   94   97   55

  #            0000   0001   0010   0011   0100   0101   0110   0111   1000   1001   1010   1011   1100   1101   1110   1111
  # 7872192 1277112 734328 736156 423164 736156 423284 424320 243912 734328 422232 423284 243316 423164 243316 243912 140208
  # to_binary(1277112) == 100110111110010111000
  # to_binary(140208)  == 100010001110110000

  require Graph::Maker::TwindragonAreaTree;
  my @all_count_indsets;
  my @all_count_by_type;
  foreach my $k (0 .. 5) {
    print "k=$k\n";
    my $graph = Graph::Maker->new('twindragon_area_tree',
                                  level => $k,
                                  undirected => 1);
    my @vertices = sort {$a<=>$b} $graph->vertices;
    my $num_vertices = scalar(@vertices);
    ### graph: "$graph"
    ### vertices: join(' ',@vertices)

    my @has_edge;
    foreach my $i (0 .. $#vertices) {
      foreach my $j (0 .. $#vertices) {
        $has_edge[$i][$j] = $graph->has_edge($vertices[$i],$vertices[$j]);
      }
    }

    my $count_indsets = 0;
    my @count_by_type;
    my @set = (-1);
    my $pos = 0;
    my $s = 0;
    my $c = ($k==0 ? 0 : 2**($k-1)-1);
    my $a = ($k==0 ? 0 : 2**($k-1)  );
    my $e = 2**$k-1;
  POS: for (;;) {
      my $this;
      if (($this = ++$set[$pos]) > 1) {
        # backtrack
        if (--$pos < 0) {
          last;
        }
        next;
      }
      if ($this) {
        foreach my $i (0 .. $pos-1) {
          if ($set[$i] && $has_edge[$i][$pos]) {
            ### have edge: "$vertices[$i] to $vertices[$pos]"
            next POS;
          }
        }
      }
      if ($pos == $#vertices) {
        ### set: join('',@set)
        $count_indsets++;

        my $type = ($set[$s] << 3) + ($set[$c] << 2)
          + ($set[$a] << 1) + ($set[$e] << 0);
        $count_by_type[$type]++;

        next;
      }
      $set[++$pos] = -1;
    }
    print "  indsets $count_indsets\n";
    $all_count_indsets[$k] = $count_indsets;
    $all_count_by_type[$k] = \@count_by_type;
  }

  print "    ";
  foreach my $type (0 .. 15) {
    printf " %04b", $type;
  }
  print "\n";
  foreach my $k (0 .. $#all_count_indsets) {
    printf "%4d", $all_count_indsets[$k] || 0;
    foreach my $type (0 .. 15) {
      printf " %4d", $all_count_by_type[$k][$type]||0;
    }
    print "\n";
  }
  exit 0;
}


{
  # blob ascii drawing

  # k=4
  #   *  *    S,E used only
  #   |  |
  #   *  E
  #
  # k=5  points=8      k=5  points=8    k=5  points=8
  #   *  *  S            *--*  S          *--*  S
  #   |  |  |
  #   *  *  S  *         *--*  S--*       *  *--S  *
  #            |                                   |
  #      *--*  *            *  *--*          *--*  *
  #
  #         E                  E                E
  #   S by bridge       S not bridge
  #
  # k=6  points=16           k=6  points=16
  #            S  *--*                  S  *--*
  #            |
  #   *  *--*  S  *--*         *  *--*  S--*  *
  #   |                        |              |
  #   *  *--*  *--*  E         *  *--*  *--*  E
  #
  #      *--*  *--*               *--*  *--*
  #
  # k=7  points=33               k=7  points=33
  #               *--*                         *--*
  #
  #            *--*  *                      *--*  *
  #                  |                            |
  #         *--*  *  *                   *--*  *  *
  #               |                            |
  #      S--S  *  *  *--E             S  S  *  *  *  E
  #            |                         |  |     |
  #         *  *  *--*  *                *  *  *  *  *
  #         |                                  |
  #   *  *  *  *--*  *--*          *--*  *  *  *  *--*
  #   |  |                               |
  #   *  *  *--*  *                *--*  *  *--*
  #
  #      *--*  *--*                   *--*  *--*

  # k=8  points=67
  #         *--*
  #
  #      *--*  *     E
  #            |
  #      *  *  *  *  *  *
  #      |  |
  #      *  *  *  *  *  *--*
  #
  #      *  *  *  *  *  *  *--*
  #      |
  #      *  *  *  *  *  *  *--*
  #         |     |     |
  #   S--S  *  *  *  *  *
  #            |     |              ===
  #      *--*  *  *  *  *  *         7    even
  #               |     |  |
  #               *  *  *  *  *      5
  #                  |        |
  #         *--*  *  *  *--*  *      7
  #               |
  #         *--*  *  *--*            5
  #
  #            *--*  *--*           4


  my $path = Math::PlanePath::DragonCurve->new;
  foreach my $k (4 .. 10) {
    my $P = BlobP($k);
    print "k=$k  points=$P\n";
    my $lo_extra = 1;
    my $hi_extra = 1;
    my $n_lo = BlobN($k)        - $lo_extra;
    my $n_hi = BlobN($k+1) - 3  + $hi_extra;

    my @points = map {$path->n_to_xy($_)} $n_lo .. $n_hi;
    my @x = pairmap {print $a; do{$a};} @points;
    my @y = pairmap {print $b; do{$b};} @points;
    my $x_max = max(@x);
    my $x_min = min(@x);
    my $y_max = max(@y);
    my $y_min = min(@y);

    my @m;
    foreach my $i (0 .. $#x) {
      $m[$x[$i]-$x_min][$y[$i]-$y_min] ||= ($i<=$lo_extra ? 'S' : $i>=$#x-$hi_extra ? 'E' : '*');
    }
    foreach my $y (reverse $y_min .. $y_max) {
      foreach my $x ($x_min .. $x_max) {
        print '  ', $m[$x-$x_min][$y-$y_min] || ' ';
      }
      print "\n\n";
    }
  }
  require Math::OEIS::Grep;
  #  Math::OEIS::Grep->search(array => \@values, verbose=>1);
  exit 0;
}


#------------------------------------------------------------------------------
#

{
  # postscript boundary of blob

  my $mm_to_bp = 72/25.4;
  my $bp_to_mm = 1/$mm_to_bp;

  # A4 = 595, 841
  my $width  = 211 * $mm_to_bp;   # mm to points
  my $height = 297 * $mm_to_bp;
  my $border = .1;
  my $chamfer = .2;
  my $horizontal_offset = 0;

  my $level = 8;
  if (@ARGV) { $level = shift @ARGV; }
  my $path = Math::PlanePath::DragonCurve->new;

  my $n_lo = BlobN($level) - 3;
  my $n_hi = BlobN($level+1);
  my ($x_lo,$y_lo) = $path->n_to_xy($n_lo);
  my ($x_hi,$y_hi) = $path->n_to_xy($n_hi);
  ### range: "$x_lo,$y_lo to $x_hi,$y_hi"

  my @points = ([$x_lo,$y_lo]);
  {
    my ($x,$y) = ($x_lo,$y_lo);
    my ($dx,$dy) = $path->n_to_dxdy($n_lo);
    my $n = $n_lo;
    do {
      ### at: "$x,$y  dxdy=$dx,$dy"
      $x += $dx;
      $y += $dy;
      push @points, [$x,$y];

      my @n_list = $path->xy_to_n_list($x,$y) or die;
      
      if (@n_list==2 && ($n_list[0] < $n || turn($n_list[0])==1)) {
        ### double left, shift away: join(', ',@n_list)
        shift @n_list;
      }
      $n = $n_list[0];
      ($dx,$dy) = $path->n_to_dxdy($n_list[0]);

    } until ($x==$x_hi && $y==$y_hi);
    @points = reverse @points;
  }
  {
    my ($x,$y) = ($x_lo,$y_lo);
    my ($dx,$dy) = $path->n_to_dxdy($n_lo);
    my $n = $n_lo;
    do {
      ### at: "$x,$y  dxdy=$dx,$dy"
      $x += $dx;
      $y += $dy;
      push @points, [$x,$y];

      my @n_list = $path->xy_to_n_list($x,$y) or die;
      
      if (@n_list==2 && ($n_list[0] < $n || turn($n_list[0])==-1)) {
        ### double left, shift away: join(', ',@n_list)
        shift @n_list;
      }
      $n = $n_list[0];
      ($dx,$dy) = $path->n_to_dxdy($n_list[0]);

    } until ($x==$x_hi && $y==$y_hi);
  }

  @points = map {[ transform_to_horizontal(@$_, $level - $horizontal_offset) ]}
    @points;

  my $x_min = min(map {$_->[0]} @points);
  my $x_max = max(map {$_->[0]} @points);
  my $y_min = min(map {$_->[1]} @points);
  my $y_max = max(map {$_->[1]} @points);

  my $x_size = $x_max - $x_min;
  my $y_size = $y_max - $y_min;

  # (1/3 + 1 + 1/6) / 2 == 3/4

  my $length = 2**($level/2);

  my $scale = min($width  / ($x_size||$y_size),
                  $height / ($y_size||$x_size));
  $scale *= (1-$border);
  my $segment_mm = $scale * $bp_to_mm;
  my $length_scaled = $length * $scale;
  my $length_mm  = $length_scaled * $bp_to_mm;
  print "segment $scale bp   $segment_mm mm\n";
  print "X $x_min to $x_max is $x_size   Y $y_min to $y_max is $y_size\n"; # length $length_mm mm\n";

  my $line_width = $scale*.1;

  my $x_middle = ($x_min + $x_max)/2;
  my $y_middle = ($y_min + $y_max)/2;
  print "curve middle $x_middle, $y_middle\n";

  my $mx = $width /2;
  my $my = $height/2;
  print "paper middle $mx $my bp\n";

  my $ox = $mx - $x_middle * $scale;
  my $oy = $my - $y_middle * $scale;
  print "paper origin $ox $oy bp\n";

  my $transform = sub {
    my ($x,$y) = @_;
    $x *= $scale;
    $y *= $scale;
    $x += $ox;
    $y += $oy;
    ### point: "$x, $y"
    return ($x,$y);
  };
  {
    my ($x,$y) = $transform->($x_middle,$y_middle);
    print "xy middle $x_middle, $y_middle  transformed $x, $y\n";
  }

  my $ps_filename = '/tmp/x.ps';
  my $pdf_filename = '/tmp/x.pdf';

  {
    print "output\n";
    open my $fh, '>', $ps_filename or die;

    print $fh <<"HERE";
%!PS-Adobe-3.0
%%Title: Dragon $level
%%LanguageLevel: 1
%%Creator: dragon.pl
%%DocumentMedia: A4 595.27559 841.88976 0 ( ) ( )
%%Orientation: Portrait
%%Pages: 1
%%EndComments
%%BeginProlog
%%EndProlog
%%BeginSetup
<< /PageSize [ 595.27559 841.88976 ] /ImagingBBox null >> setpagedevice
%%EndSetup
%%Page: 1 1
%%BeginPageSetup
/pagelevel save def
%%EndPageSetup
$line_width setlinewidth
newpath
HERE

    {
      my ($x,$y) = @{$points[0]};
      ($x,$y) = $transform->($x,$y);
      print $fh "$x $y moveto\n";
    }
    foreach my $i (1 .. $#points) {
      my ($x,$y) = @{$points[$i]};
      ($x,$y) = $transform->($x,$y);
      print $fh "$x $y lineto\n";
    }

    print $fh <<"HERE";
stroke
%%PageTrailer
pagelevel restore
showpage
%%EOF
HERE
  }

  if (0) {
    require PostScript::Simple;
    my $ps = PostScript::Simple->new(papersize => "A4",
                                     eps => 0,
                                     units => "bp");
    $ps->newpage;
    $ps->setlinewidth($line_width);

    {
      my ($x1,$y1) = @{$points[0]};
      my ($x2,$y2) = @{$points[1]};
      ($x1,$y1) = $transform->($x1,$y1);
      ($x2,$y2) = $transform->($x2,$y2);
      $ps->line($x1,$y1, $x2,$y2);
    }
    foreach my $i (2 .. $#points) {
      my ($x,$y) = @{$points[$i]};
      ($x,$y) = $transform->($x,$y);
      $ps->linextend($x,$y);
    }
    $ps->output($ps_filename);
  }

  require IPC::Run;
  print "ps2pdf\n";
  IPC::Run::run(['ps2pdf12',$ps_filename,$pdf_filename]);
  print "qpdf\n";
  IPC::Run::run(['qpdf','--stream-data=uncompress',$pdf_filename,'/tmp/uncompressed.pdf.txt']);

  # push @ipc, IPC::Run::start(['gv','-presentation','-f',$ps_filename],'&');
  # ,'-xrm','xpdf.initialZoom: 100'

  print "xpdf\n";
  push @ipc, IPC::Run::start(['xpdf',
                              '-z','50',
                              $pdf_filename],
                             '2>',File::Spec->devnull, '&');
  exit 0;
}

{
  # postscript chamfered

  my $mm_to_bp = 72/25.4;
  my $bp_to_mm = 1/$mm_to_bp;

  # A4 = 595, 841
  my $width  = 211 * $mm_to_bp;   # mm to points
  my $height = 297 * $mm_to_bp;
  my $border = .1;
  my $chamfer = .2;
  my $horizontal_offset = 0;

  my $level = 16;
  if (@ARGV) { $level = shift @ARGV; }
  my $path = Math::PlanePath::DragonCurve->new;


  my ($n_lo, $n_hi);
  if (0) {
    # whole curve
    ($n_lo, $n_hi) = $path->level_to_n_range($level);
    $horizontal_offset = 2;
  } else {
    # blob
    $n_lo = BlobN($level) - 3;
    $n_hi = BlobN($level+1);
  }

  # require Geometry::AffineTransform;
  # my $affine = Geometry::AffineTransform->new;
  # $affine->translate(1/3, 2/3);
  # ($x,$y) = $affine->transform($x,$y);

  my @points;
  {
    my ($x,$y) = $path->n_to_xy($n_lo);
    push @points, [$x,$y];
  }
  foreach my $n ($n_lo+1 .. $n_hi-1) {
    foreach my $c (-$chamfer, $chamfer) {
      my ($x,$y) = $path->n_to_xy($n + $c);
      push @points, [$x,$y];
    }
  }
  {
    my ($x,$y) = $path->n_to_xy($n_hi);
    push @points, [$x,$y];
  }
  @points = map {[ transform_to_horizontal(@$_, $level - $horizontal_offset) ]}
    @points;

  my $x_min = min(map {$_->[0]} @points);
  my $x_max = max(map {$_->[0]} @points);
  my $y_min = min(map {$_->[1]} @points);
  my $y_max = max(map {$_->[1]} @points);

  my $x_size = $x_max - $x_min;
  my $y_size = $y_max - $y_min;

  # (1/3 + 1 + 1/6) / 2 == 3/4

  my $length = 2**($level/2);

  my $scale = min($width  / ($x_size||$y_size),
                  $height / ($y_size||$x_size));
  $scale *= (1-$border);
  my $segment_mm = $scale * $bp_to_mm;
  my $length_scaled = $length * $scale;
  my $length_mm  = $length_scaled * $bp_to_mm;
  print "segment $scale bp   $segment_mm mm\n";
  print "X $x_min to $x_max is $x_size   Y $y_min to $y_max is $y_size\n"; # length $length_mm mm\n";

  my $line_width = $scale*.1;

  my $x_middle = ($x_min + $x_max)/2;
  my $y_middle = ($y_min + $y_max)/2;
  print "curve middle $x_middle, $y_middle\n";

  my $mx = $width /2;
  my $my = $height/2;
  print "paper middle $mx $my bp\n";

  my $ox = $mx - $x_middle * $scale;
  my $oy = $my - $y_middle * $scale;
  print "paper origin $ox $oy bp\n";

  my $transform = sub {
    my ($x,$y) = @_;
    $x *= $scale;
    $y *= $scale;
    $x += $ox;
    $y += $oy;
    ### point: "$x, $y"
    return ($x,$y);
  };
  {
    my ($x,$y) = $transform->($x_middle,$y_middle);
    print "xy middle $x_middle, $y_middle  transformed $x, $y\n";
  }

  my $ps_filename = '/tmp/x.ps';
  my $pdf_filename = '/tmp/x.pdf';

  {
    print "output\n";
    open my $fh, '>', $ps_filename or die;

    print $fh <<"HERE";
%!PS-Adobe-3.0
%%Title: Dragon $level
%%LanguageLevel: 1
%%Creator: dragon.pl
%%DocumentMedia: A4 595.27559 841.88976 0 ( ) ( )
%%Orientation: Portrait
%%Pages: 1
%%EndComments
%%BeginProlog
%%EndProlog
%%BeginSetup
<< /PageSize [ 595.27559 841.88976 ] /ImagingBBox null >> setpagedevice
%%EndSetup
%%Page: 1 1
%%BeginPageSetup
/pagelevel save def
%%EndPageSetup
$line_width setlinewidth
newpath
HERE

    {
      my ($x,$y) = @{$points[0]};
      ($x,$y) = $transform->($x,$y);
      print $fh "$x $y moveto\n";
    }
    foreach my $i (1 .. $#points) {
      my ($x,$y) = @{$points[$i]};
      ($x,$y) = $transform->($x,$y);
      print $fh "$x $y lineto\n";
    }

    print $fh <<"HERE";
stroke
%%PageTrailer
pagelevel restore
showpage
%%EOF
HERE
  }

  if (0) {
    require PostScript::Simple;
    my $ps = PostScript::Simple->new(papersize => "A4",
                                     eps => 0,
                                     units => "bp");
    $ps->newpage;
    $ps->setlinewidth($line_width);

    {
      my ($x1,$y1) = @{$points[0]};
      my ($x2,$y2) = @{$points[1]};
      ($x1,$y1) = $transform->($x1,$y1);
      ($x2,$y2) = $transform->($x2,$y2);
      $ps->line($x1,$y1, $x2,$y2);
    }
    foreach my $i (2 .. $#points) {
      my ($x,$y) = @{$points[$i]};
      ($x,$y) = $transform->($x,$y);
      $ps->linextend($x,$y);
    }
    $ps->output($ps_filename);
  }

  require IPC::Run;
  print "ps2pdf\n";
  IPC::Run::run(['ps2pdf12',$ps_filename,$pdf_filename]);
  print "qpdf\n";
  IPC::Run::run(['qpdf','--stream-data=uncompress',$pdf_filename,'/tmp/uncompressed.pdf.txt']);

  # push @ipc, IPC::Run::start(['gv','-presentation','-f',$ps_filename],'&');
  print "xpdf\n";
  push @ipc, IPC::Run::start(['xpdf',$pdf_filename],
                             '2>',File::Spec->devnull, '&');
  # ,'-xrm','xpdf.initialZoom: 100'
  exit 0;
}


{
  # no, points go by locations not distances

  sub Graph_vertex_collapse {
    my ($graph,$v) = @_;
    my @neighbours;
    foreach my $i (0 .. $#neighbours) {
      foreach my $j ($i+1 .. $#neighbours) {
        $graph->add_edge($neighbours[$i],$neighbours[$j]);
      }
    }
  }

  # return a list of the neighbours of $v which are $distance away from $v
  sub Graph_tree_neighbours_of_distance {
    my ($graph,$v,$distance) = @_;
    ### Graph_tree_neighbours_of_distance: $v
    my %seen;
    my @ret = ($v);
    foreach (1 .. $distance) {
      @seen{@ret} = ();
      @ret = grep {! exists $seen{$_}} map {$graph->neighbours($_)} @ret;
    }
    ### @ret
    return @ret;
  }

  sub Graph_delete_all_edges {
    my ($graph) = @_;
    $graph->delete_edges(map {@$_} $graph->edges);
  }

  sub Graph_collapse {
    my ($graph, $parity) = @_;
    my @keep_vertices = (grep {$graph->vertex_degree($_)==3}
                         $graph->vertices);
    # $_%2==$parity &&
    my @new_edges;
    foreach my $from (@keep_vertices) {
      foreach my $to (Graph_tree_neighbours_of_distance($graph,$from,4)) {
        push @new_edges, $from,$to;
      }
    }
    ### @new_edges
    Graph_delete_all_edges($graph);
    $graph->add_edges(@new_edges);

    my %keep_vertices;
    @keep_vertices{@keep_vertices} = ();
    $graph->delete_vertices(grep {! exists $keep_vertices{$_}} $graph->vertices);

    if (defined (my $name = $graph->get_graph_attribute('name'))) {
      $graph->set_graph_attribute(name => "$name, Collapsed");
    }
  }

  foreach my $k (5) {
    require Graph::Maker::TwindragonAreaTree;
    my $graph = Graph::Maker->new('twindragon_area_tree',
                                  level => $k,
                                  undirected => 1);
    ### $k
    require MyGraphs;
    MyGraphs::Graph_view($graph);
    Graph_collapse($graph,1);
    MyGraphs::Graph_view($graph);
    # my $graph_kminus4 = Graph::Maker->new('twindragon_area_tree',
    #                                       level => $k-4,
    #                                       undirected => 1);
  }
  exit 0;
}


{
  # TAparentDir flat reversal

  # TAparentDir0
  #   odd even even odd
  # 0 111 0000 1111 000
  #       \-------/
  #                10 at even pos
  #  01 at odd pos

  require MyFLAT;
  foreach my $HtoL (TAparentDir0_flat(),
                    TAparentDir1_flat(),
                    TAparentDir2_flat(),
                    TAparentDir3_flat()) {
    my $HtoL = $HtoL->MyFLAT::minimize(verbose=>1);
    # $HtoL->MyFLAT::view;
    my $LtoH = $HtoL->MyFLAT::reverse;
    $LtoH = $LtoH->MyFLAT::minimize(verbose=>1);
    $LtoH = $LtoH->MyFLAT::separate_sinks
      ->MyFLAT::set_name($HtoL->{'name'} . ' LtoH');
    $LtoH->MyFLAT::view;
  }
  #  MyFLAT::FLAT_print_tikz($LtoH);

  exit 0;
}


{
  # Rpred / Lpred on OEIS sample values
  # sequences which as binary fractions are on the fractal boundary
  #
  # 1, 1,0, 1,0,0,0, 1,0,0,0, 0,0,0,0, 1,0,0,0, 0,0,0,0, 0,0,0,0, 0,0,0,0, 1
  # A036987 Kempner-Mahler sum 1/2^(2^k)), transcendental per Kempner

  # A005713  Define strings S(0)=0, S(1)=11, S(n) = S(n-1)S(n-2); iterate.
  #  1101111011011110111101101111011011110111101101111011110110111
  #    Lpred  fib related
  # A084119 fib characteristic
  # A230603  Generalized Fibonacci word.  Binary complement of A221150.
  #  1101110110111011101101110110111011101101110111011011101101110
  #  Lpred
  #  0.866145577032390194074032
  #  fracper_to_frac([ 1, 1, 0, 1, 1, 1, 0, 1, 1, 0, 1, 1, 1, 0, 1, 1, 1, 0, 1, 1, 0, 1, 1, 1,    0, 1, 1, 0, 1, 1, 1, 0, 1, 1, 1, 0, 1, 1, 0, 1, 1, 1, 0, 1, 1, 1, 0, 1,    1, 0, 1, 1, 1, 0, 1, 1, 0, 1, 1, 1, 0, 1, 1, 1, 0, 1, 1, 0, 1, 1, 1, 0,    1, 1, 0, 1, 1, 1, 0, 1, 1, []],2)*1.0

  # A189628  Fixed point of the morphism 0->001, 1->010.
  #  0010010100010010100010100010010010100010010100010100010010010
  #  Rpred
  # A189664  Fixed point of the morphism 0->010, 1->001.
  #  0100010100100100010100010100100010100100010100100100010100010
  #  Rpred
  # A189668  Fixed point of the morphism 0->010, 1->100.
  #  0101000101000100100101000101000100100101000100101000100101000
  #  Rpred
  # A189816  a(3k-2)=0, a(3k-1)=1-a(k), a(3k)=1-a(k); k>=1, a(0)=0.
  #  0110000000110110110110110110110000000110000000110000000110000
  #  Lpred
  # A189820  a(3k-2)=a(k), a(3k-1)=a(k), a(3k)=1; k>=1, a(1)=0.
  #  0010011110010011111111111110010011110010011111111111111111111
  #  Rpred

  require Encode::Locale;
  binmode(STDOUT, ':encoding(console_out)');
  require Math::OEIS::Stripped;
  my $stripped = Math::OEIS::Stripped->new;
  require Math::OEIS::Names;
  my $names = Math::OEIS::Names->new;

  my $fh = $stripped->fh;
  my $count = 0;
  my $count_Rpred = 0;
  my $count_Lpred = 0;
  while (defined(my $line = readline $fh)) {
    $line =~ /^(A\d+)\s+,?(([01],)+)\s*$/ or next;
    my $anum = $1;
    my @values = split /,/, $2;
    $count++;

    my $Rpred = Rpred_of_bits(\@values);
    my $Lpred = Lpred_k_of_bits(\@values);
    if ($Rpred || $Lpred) {
      my $name = $names->anum_to_name($anum);
      print "$anum  $name\n";
      print " ",@values[0..min($#values,60)],"\n";
      print " ",($Rpred ? "Rpred" : ""),($Lpred ? "  Lpred" : ""),"\n";
      print "\n";
    }
    $count_Rpred += $Rpred;
    $count_Lpred += $Lpred;
  }
  print "count binaries $count Rpred $count_Rpred Lpred $count_Lpred\n";
  exit 0;
}
{
  # blob graph eccentricity of blob start for tikz

  require Graph;
  require MyGraphs;
  my $path = Math::PlanePath::DragonCurve->new;
  foreach my $k (4 .. 11) {
    print "  \\begin{scope}[shift=(pos-$k)]\n";

    my $n_lo = BlobN($k);
    my $n_hi = BlobN($k+1) - 3;

    print "    \\draw[my gray] ";
    my $join = '';
    foreach my $n ($n_lo .. $n_hi) {
      my ($x,$y) = $path->n_to_xy($n);
      print "$join($x,$y)";
      $join = '--';
    }
    print ";\n";

    my ($x_lo,$y_lo) = $path->n_to_xy($n_lo);
    print "    \\node at ($x_lo,$y_lo) [my dot,name=start];\n";

    my ($x_hi,$y_hi) = $path->n_to_xy($n_hi);
    print "    \\node at ($x_hi,$y_hi) [my dot,name=end];\n";

    my $graph = blob_Graph($k);
    my $apsp = $graph->APSP_Floyd_Warshall;
    # my @path = MyGraphs::Graph_eccentricity_path($graph, Graph_xy_to_vertex($x_lo,$y_lo));
    my @path = MyGraphs::Graph_eccentricity_path($graph, Graph_xy_to_vertex($x_hi,$y_hi));
    print "    \\draw[ultra thick] ";
    $join = '';
    foreach my $v (@path) {
      my ($x,$y) = Graph_vertex_to_xy($v);
      print "$join($x,$y)";
      $join = '--';
    }
    print ";\n";

    print "  \\end{scope}\n";
  }
  exit 0;
}
{
  # right boundary squares centroid -- by sum of LQ
  # NOT RIGHT (?)

  my @values;
  foreach my $k (0 .. 9) {
    my $x_total = .5;
    my $y_total = -.5;
    my $count = 1;
    foreach my $i (0 .. $k-1) {
      my ($ex,$ey) = bk($i+1);
      my ($x,$y) = GLQ_xy($i);
      ($x,$y) = xy_rotate_minus90($x,$y);
      $x = $ex + $x;
      $y = $ex + $y;
      my $lq = LQ($i);
      $x_total += $x * $lq;
      $y_total += $y * $lq;
      $count += $lq;
    }
    $count == RQ($k) or die "count=$count want RQ($k)=",RQ($k);

    my $gx = $x_total / $count;
    my $gy = $y_total / $count;
    $x_total *= 2;
    $y_total *= 2;
    print "k=$k  $count   $x_total,$y_total   $gx,$gy\n";
    push @values, $y_total;
  }

  require Math::OEIS::Grep;
  Math::OEIS::Grep->search(array => \@values, verbose=>1);
  exit 0;
}



{
  # N on centre line without positions

  # state 6 non
  # not in OEIS: 0, 1, 10, 52, 232, 976, 4000, 16192, 65152, 261376, 1047040
  # vector(10,h, 4^h - 3*2^(h-1))
  #
  # other states 3*2^(h-1)   is segments with one end or other on centreline


  my @state_table;
  my @accepting;
  my $not = 6;
  $state_table[$not] = [$not,$not,$not,$not];

  #  @ 0----------->1
  #    0     1 <--- .        s0
  #    |     ^
  #    v     |
  #     <----
  $state_table[0] = [2, $not, 4, 1];
  push @accepting, 0;

  #            ---->
  #           |     ^
  #           v     |
  #    0 --->       .        s1
  # @  0 <--------- 1
  $state_table[1] = [0, 5, $not, 3];
  push @accepting, 1;

  #    1
  #    ^ |
  #    | v                 s2
  #    | <---
  #    |     |
  #    |     v
  #  @ 0 --->
  $state_table[2] = [0, 5, $not, $not];
  push @accepting, 2;

  #    0
  #    ^ |                 s3
  #    | v
  #    | <---
  #    |     |
  #    |     v
  #  @ 1 --->
  $state_table[3] = [$not, $not, 4, 1];

  #  @ 1
  #    ^ |                 s4
  #    | v
  #    | <---
  #    |     |
  #    |     v
  #    0 --->
  $state_table[4] = [ $not, $not, $not, 3];

  #  @ 0
  #    ^ |                 s5
  #    | v
  #    | <---
  #    |     |
  #    |     v
  #    1 --->
  $state_table[5] = [ 2, $not, $not, $not];
  push @accepting, 5;

  require MyFLAT;
  my $f = MyFLAT::aref_to_FLAT_DFA(\@state_table,
                                   name => 'N centreline',
                                   accepting_list => \@accepting,
                                  );
  {
    # MyFLAT::view($f);
    # MyFLAT::view($f->MyFLAT::reverse->MyFLAT::minimize);
    MyFLAT::FLAT_print_tikz($f->MyFLAT::reverse->MyFLAT::minimize);
    exit;
    # MyFLAT::FLAT_print_tikz($f);
  }

  my $path = Math::PlanePath::DragonCurve->new;
  sub nk_is_on_centreline {
    my ($n, $k) = @_;
    my ($x,$y) = $path->n_to_xy($n);
    ($x,$y) = transform_to_horizontal($x,$y, $k);
    return $y==0;
  }

  {
    my @counts = MyFLAT::FLAT_count_contains($f, to_bigint(10));
    print join(',',@counts),"\n";
    require Math::OEIS::Grep;
    Math::OEIS::Grep->search(array => \@counts, verbose=>1);
  }

  foreach my $i (0..10) {
    my $k = 2*$i;
    print "i=$i k=$k\n";
    my ($n_lo, $n_hi) = $path->level_to_n_range($k);
    my $count = 0;
    foreach my $n ($n_lo .. $n_hi-1) {
      my $n4 = sprintf '%0*s', $i, cnv($n,10,4);
      my $is_f = $f->contains($n4) ? 1 : 0;
      my $is_p = nk_is_on_centreline($n,$k) ? 1 : 0;
      if ($is_f != $is_p) {
        print " n=$n n4=$n4  flat=$is_f path=$is_p\n";
        MyFLAT::FLAT_show_transitions($f, $n4);
      }
      $count += $is_p;
    }
    print "  count $count\n";
  }
  exit 0;
}


{
  # twindragon XY convex hull by expansion, aligned diagonally
  #
  # area
  # v=[1,2,6,17,38,84,172,374,756,1556,3120,6376,12768,25732,51480,103532,207096,415012,830056,1662452,3324968]

  require MyXYConvexHull;
  my @values;
  #       k=0          k=2                k=3  
  #                                            
  #     S---*           S   *           *---S   *---*
  #     |   |          / \ / \          |   |   |   |    
  #     *---E         *   *   *         *---*---*---*---*
  #                    \ / \ /              |   |   |   | 
  #                     *   E               e---*   E---*
  #  
  #     area=1         area=2            area=6

  # GP-Test  geom_area_of_polygon([0,1-I,2,1+I]) == 2

  my $even = [ 1, MyXYConvexHull->new_from_points([0,0],[1,0],[0,-1],[1,-1])];
  my $odd  = [ 2, MyXYConvexHull->new_from_points([0,0],[2,0],
                                                  [-1,-1],[1,-1],[3,-1],
                                                  [0,-2],[2,-2]) ];
  my $k_limit = 20;
 OUTER: for (my $k = 0; ; ) {
    foreach my $e ($even, $odd) {
      my ($len,$h) = @$e;
      print "k=$k\n";
      my $num_points = (!($k&1) ? $h->num_points_even : $h->num_points);
      my $area = $h->area / ($k&1 ? 2 : 1);
      my @num_sides = $h->num_sides4;
      my $num_sides = sum(@num_sides);
      print " sides ",join(',',@num_sides)," = $num_sides";
      print " points $num_points";
      print " area $area";
      print "\n";

      if ($k < 10) {
        $h->print;
      }
      push @values, $area;
      # push @values, $num_points;

      last OUTER if $k >= $k_limit;
      $k++;

      my $r  = $h->rotate_minus90;
      my $s0 = $r;
      my $s1 = $r->translate($len,-$len);
      my $s2 = $r->translate(2*$len,0);
      my $s3 = $r->translate(3*$len,-$len);
      $h = $s0->union($s1,$s2,$s3);
      $len *= 2;
      $e = [$len,$h];
    }
  }
  print join(',',@values),"\n";
  require Math::OEIS::Grep;
  Math::OEIS::Grep->search(array => \@values, verbose=>1);
  exit 0;
}
{
  # dragon XY convex hull by expansion, aligned diagonally
  #
  #
  #    k=0          k=1          k=2                 k=3  
  #                                  S           *---S       
  #    S           S                /            |           
  #     \          |               *   *         *---*   *---*
  #      E         *---E            \ / \            |   |   |
  #                                  *   E           *---*   E
  #    A=0          A=0             A=1/2          A=3         
  #
  # num points
  # v=
  # v[#v]/2^(#v-1)*1.0
  # bestappr(%,100)
  # vector(#v,i,my(k=i-1); v[i]-54/53*2^k)
  #
  # area
  # v=[0,0,1/2,3,13/2,17,95/2,100,427/2,435,934,1878,3829,7674,15623,31266,62856,125744,252625,505290,1011934]
  # vector(#v,i,my(k=i-1); v[i]-HA(k))

  require MyXYConvexHull;
  my @values;
  my $diagonal = 1;

  my @even_points = ([0,0],         [1,-1]);
  my @odd_points  = ([0,0], [0,-1], [1,-1]);
  my $even = [ 1, MyXYConvexHull->new_from_points (@even_points) ];
  my $odd  = [ 1, MyXYConvexHull->new_from_points (@odd_points)  ];

  my $k_limit = 20;
 OUTER: for (my $k = 0; ; ) {
    foreach my $e ($even, $odd) {
      my ($len, $h) = @$e;
      print "k=$k\n";
      my $num_points = ($k&1 ? $h->num_points : $h->num_points_even);
      my $area = $h->area / ($k&1 ? 1 : 2);
      my @num_sides = $h->num_sides4;
      my $num_sides = sum(@num_sides);
      print " sides ",join(',',@num_sides)," = $num_sides";
      print " points $num_points";
      print " area $area";
      print "\n";

      if ($k < 10) {
        $h->print;
      }
      push @values, max($area,0);
      # push @values, $num_points;

      last OUTER if $k >= $k_limit;
      $k++;

      #  S           S
      #   \         / 
      #    v       v  
      #     *     *     * 
      #            ^   ^ ^
      #             \ /   \
      #              *     E

      my $r   = $h->rotate_minus90;
      my $rev = $r->rotate_minus90;
      my $l   = $rev->rotate_minus90;
      my $s0 = $r;
      my $s1 = $rev->translate (0,      -2*$len);
      my $s2 = $l->translate   (0,      -2*$len);
      my $s3 = $rev->translate (2*$len, -2*$len);
      $h = $s0->union($s1,$s2,$s3);
      $len *= 2;
      $e = [$len,$h];
    }
  }
  print join(',',map{$_==int($_) ? $_ : ($_*2)."/2"}@values),"\n";
  require Math::OEIS::Grep;
  Math::OEIS::Grep->search(array => \@values, verbose=>1);
  exit 0;
}

{
  # XY convex hull by points
  #
  # even
  # recurrence_guess([4,6,10,16,24,36,52,76,108,156,220])
  # initial exceptions [4]
  # xy_num_sides_even(k) = vector_modulo([7,10],k) * 2^floor(k/2) - 4
  #
  # all
  # recurrence_guess([4,6,6,8,10,14,16,22,24,32,36,48,52,68,76,100,108,140,156,204,220])
  # initial exceptions [4, 6, 6, 8]
  # xy_num_sides(k) = vector_modulo([7,9,10,13],n) * 2^floor(n/4) - 4
  #
  # points aligned to curve start
  # recurrence_guess([2,3,5,9,17,31,67,128,262,493,1048,1994,4174,7908,16706,31734,66766,126682,267126,507166,1068302,2027686])
  # points aligned to end
  # v=[2,3, 5,14, 17,34, 67,132, 262,526, 1048,2090, 4174,8358, 16706, 33390,66766, 133574,267126, 534166,1068302, 2136774];
  # recurrence_guess(v)
  # h(k) = 9114/8945*2^k;
  # vector(#v,i,my(k=i-1); v[i]-h(k))*25
  #
  #          *
  #         / \
  #    S . * . E
  #   /     \
  #  * . * . *
  #   \ / \ /
  #    *   *
  #
  # diagonal area
  # v=[-0+1/2,0,0+1/2,3,6+1/2,17,47+1/2,100,213+1/2,435,934,1878,3829,7674,15623,31266,62856]

  require MyXYConvexHull;
  my $path = Math::PlanePath::DragonCurve->new;
  my @values;
  my $diagonal = 1;
  for (my $k = 0; $k <= 16; $k+=1) {
    print "k=$k\n";
    my ($n_lo,$n_hi) = $path->level_to_n_range($k);
    my @points = map {[$path->n_to_xy($_)]} $n_lo .. $n_hi;
    @points = map {[transform_to_horizontal(@$_, $k + $diagonal)]} @points;
    my $h = MyXYConvexHull->new_from_points (@points);
    my ($xmin,$ymin, $xmax,$ymax) = $h->bbox;
    my $num_rows = $ymax - $ymin + 1;
    my $num_points = ($k&1 ? $h->num_points_even : $h->num_points);
    my $area = $h->area / ($k&1 ? 1 : 2);
    my @num_sides = $h->num_sides4;
    my $num_sides = sum(@num_sides);
    print " sides ",join(',',@num_sides)," = $num_sides";
    print " points $num_points";
    print " area $area";
    print "\n";
    if ($k < 10) {
      $h->print;
    }
    push @values, $area;
    # push @values, $num_points;
  }
  print join(',',@values),"\n";
  require Math::OEIS::Grep;
  Math::OEIS::Grep->search(array => \@values, verbose=>1);
  exit 0;
}

{
  # dragon XY convex hull by expansion, aligned to endpoint
  #
  #    k=1         k=2          k=3    *
  #                                   / \
  #   S   E      S   *---E       S   *   E
  #    \ /       |   |          /     \
  #     *        *---*         *   *   *
  #                             \ / \ /
  #                              *   *
  #
  # num points
  # v=[2,3,5,9,17,34,67,132,262,526,1048,2090,4174,8358,16706,33390,66766,133574,267126,534166,1068302,2136774,4273502,8546662,17093262,34187206]
  # v[#v]/2^(#v-1)*1.0
  # bestappr(%,100)
  # vector(#v,i,my(k=i-1); v[i]-54/53*2^k)
  #
  # area
  # v=[0,0,1,2+1/2,8,22,48,108+1/2,223,480+1/2,969,2001+1/2,4015,8185+1/2,16387,33051+1/2,66127,132907+1/2,265847,532847+1/2,1065743,2134159+1/2,4268383,8541463+1/2,17083023]
  # vector(#v,i,my(k=i-1); v[i]-HA(k))

  require MyXYConvexHull;
  my @values;

  my $even = [ 1, MyXYConvexHull->new_from_points ([0,0], [1,0]) ];
  my $odd  = [ 2, MyXYConvexHull->new_from_points ([0,0], [1,-1], [2,0]) ];

  my $k_limit = 10;
 OUTER: for (my $k = 0; ; ) {
    foreach my $e ($even, $odd) {
      my ($len,$h) = @$e;
      print "k=$k\n";
      my $num_points = ($k&1 ? $h->num_points_even : $h->num_points);
      my $area = $h->area / ($k&1 ? 2 : 1);
      my @num_sides = $h->num_sides4;
      my $num_sides = sum(@num_sides);
      print " sides ",join(',',@num_sides)," = $num_sides";
      print " points $num_points";
      print " area $area";
      print "\n";

      if ($k < 10) {
        $h->print;
      }
      push @values, $area;
      # push @values, $num_points;

      last OUTER if $k >= $k_limit;
      $k++;

      #  *      * <--- *
      #  |      ^
      #  v      |
      #  * <--- *

      my $r   = $h->rotate_minus90;
      my $rev = $r->rotate_minus90;
      my $l   = $rev->rotate_minus90;
      my $s0 = $r;
      my $s1 = $rev->translate($len,-$len);
      my $s2 = $l->translate($len,-$len);
      my $s3 = $rev->translate(2*$len,0);
      $h = $s0->union($s1,$s2,$s3);
      $len *= 2;
      $e = [$len,$h];
    }
  }
  print join(',',map{$_==int($_)?$_:int($_)."+".2*($_-int($_))."/2"}@values),"\n";
  require Math::OEIS::Grep;
  Math::OEIS::Grep->search(array => \@values, verbose=>1);
  exit 0;
}
{
  # twindragon XY convex hull by expansion, aligned to endpoint
  #
  # num points
  # v=[4,7,13,25,51,99,195,387,779,1551,3091,6175,12371,24727,49411,98807,197699,395367,790563,1581095,3162531]
  #
  # area
  # v=[0,2,5,14,36,76,166,340,724,1456,2984,5984,12164,24344,49004,98040,196900,393832,788980,1578024,3159396]

  require MyXYConvexHull;
  my @values;
  #    k=0         k=1          k=2  s   s               *---s   *---s    
  #                                 / \ / \        k=3   |   |   |   |    
  #     *            *---*         *   *   *             *---*---*---*---*
  #    / \           |   |          \ / \ /                  |   |   |   |
  #   S   E      S---*---E       S   es  E       *---S   *--es---*   E---*
  #    \ /       |   |          / \ / \          |   |   |   |    
  #     *        *---*         *   *   *         *---*---*---*---*
  #                             \ / \ /              |   |   |   | 
  #                              e   e               e---*   e---*

  # GP-Test  geom_area_of_polygon([0,1-I,2,1+I]) == 2

  my $even = [ 2, MyXYConvexHull->new_from_points([0,0], [1,-1], [1,1], [2,0])];
  my $odd  = [ 2, MyXYConvexHull->new_from_points([0,0],[0,-1], [1,-1],[1,1],
                                                  [2,1],[2,0]) ];

  my $k_limit = 40;
 OUTER: for (my $k = 0; ; $k+=2) {
    foreach my $e ($even, $odd) {
      my ($len,$h) = @$e;
      print "k=$k\n";
      my $num_points = (!($k&1) ? $h->num_points_even : $h->num_points);
      my $area = $h->area / ($k&1 ? 1 : 2);
      my @num_sides = $h->num_sides4;
      my $num_sides = sum(@num_sides);
      print " sides ",join(',',@num_sides)," = $num_sides";
      print " points $num_points";
      print " area $area";
      print "\n";

      if ($k < 10) {
        $h->print;
      }
      push @values, $area;
      # push @values, $num_points;

      last OUTER if $k >= $k_limit;
      $k++;

      my $r  = $h->rotate_minus90;
      my $s0 = $r;
      my $s1 = $r->translate($len,0);
      my $s2 = $s1->translate(0,$len);
      my $s3 = $s2->translate($len,0);
      $h = $s0->union($s1,$s2,$s3);
      $len *= 2;
      $e = [$len,$h];
    }
  }
  print join(',',@values),"\n";
  require Math::OEIS::Grep;
  Math::OEIS::Grep->search(array => \@values, verbose=>1);
  exit 0;
}

{
  # twindragon XY convex hull by expansion, aligned to curve start
  #
  # num points
  # 4,7,13,25,53,99,203,387,819,1551,3247,6175,13023,24727,51991,98807,208119,395367,832103,1581095,3329063

  require MyXYConvexHull;
  my @values;

  # twindragon
  my $h = MyXYConvexHull->new_from_points ([0,0], [1,0], [0,1], [1,1]);
  my $ex = 0;
  my $ey = 1;

  my $k_limit = 20;
  for (my $k = 0; ; $k++) {
    print "k=$k\n";
    my $num_points = $h->num_points;
    my @num_sides = $h->num_sides4;
    my $num_sides = sum(@num_sides);
    print " sides ",join(',',@num_sides)," = $num_sides";
    print " points $num_points";
    print "\n";
    if ($k < 10) {
      $h->print;
    }
    # push @values, $num_sides;
    push @values, $num_points;

    last if $k >= $k_limit;

    ($ex,$ey) = mul($ex,$ey);
    $h = $h->union($h->translate($ex,$ey));
  }
  print join(',',@values),"\n";
  require Math::OEIS::Grep;
  Math::OEIS::Grep->search(array => \@values, verbose=>1);
  exit 0;
}


{
  # twindraon XY convex hull, horizontal, by points

  # sides count 12,8,20,12,28,20,44,28,60,44,92
  # [3, 2, 5, 3, 7, 5, 11, 7, 15, 11, 23]
  #  4*(vector_modulo([4,3,6,4],k) * 2^floor(k/4) - 1)

  # points with transform
  # 5,7,21,25,95,99,379,387,1543,1551,6159
  # even
  # v=[5,21,95,379,1543,6159,24711,98775,395335,1581031]

  # odd
  # v=[7,25,99,387,1551,6175,24727,98807,395367]
  # for(i=1,#v,my(k=2*(i-1)+1); print(THP(k)-v[i]))
  # [0,2,8,40,156,652,2580,10420,41540]

  # vector(10,k,k--;k=2*k+1; (T_P1o(k)-T_P4(k))*(-I)^((k+1)/2))
  # my(k=7); (T_P1o(k)-T_P4(k))*(-I)^((k+1)/2)

  # vector(10,k,k--;k=2*k+1; real((T_P1o(k)-T_P4(k))*(-I)^((k+1)/2)))
  # floor 2/3 * 2^k

  #  *
  #   *
  #    *
  #     *

  #  *
  #   *
  #   *@
  #   *@@
  #   ****
  #       *
  #        *

  #  *
  #   *
  #    *
  #     *
  #     *@
  #     *@@
  #     *@@@
  #     *@@@@
  #     ******
  #           *
  #            *
  #             *

  #  2,  4, 10,   20,   42
  # 10 100 1010 10100, 101010

  # k=0     *       k=1    * *
  #        S E             S * E
  #         *                * *
  #
  #                 k=4    * * * *
  #                        S * * * E
  #                          * * * *
  # THA(1)==3
  # THP(1)==7
  # THP(1)

  require MyXYConvexHull;
  my $path = Math::PlanePath::DragonCurve->new;
  my @values;
  for (my $k = 1; $k <= 17; $k+=2) {
    print "k=$k\n";
    my ($n_lo,$n_hi) = $path->level_to_n_range($k+1);
    my @points = map {[$path->n_to_xy($_)]} $n_lo .. $n_hi;
    my ($ex,$ey) = @{$points[-1]};
    push @points, map {[$ex-$_->[0], $ey-$_->[1]]} @points;
    @points = map {[transform_to_horizontal(@$_, $k+1)]} @points;
    # print points_str(\@points),"\n";
    my $h = MyXYConvexHull->new_from_points (@points);
    my $num_points = $h->num_points;
    my @num_sides = $h->num_sides4;
    my $num_sides = sum(@num_sides);
    print " sides ",join(',',@num_sides)," = $num_sides";
    print " points $num_points";
    print "\n";
    if ($k < 9) {
      $h->print;
    }
    # push @values, $num_sides;
    push @values, $num_points;
  }
  print join(',',@values),"\n";
  require Math::OEIS::Grep;
  Math::OEIS::Grep->search(array => \@values, verbose=>1);
  exit 0;
}

{
  # XY convex hull by expansion
  #

  sub xyhull_expand {
    my ($h, $ex,$ey) = @_;
    ($ex,$ey) = mul($ex,$ey);
    my $other = $h->rotate_minus90($h);
    $other = $other->translate($ex,$ey);
    $h = $h->union($other);
    return ($h, $ex,$ey);
  }

  require MyXYConvexHull;
  my @values;
  my $h = MyXYConvexHull->new_from_points ([0,0], [1,0]);
  my $ex = 1;
  my $ey = 0;

  # twindragon
  $h = MyXYConvexHull->new_from_points ([0,0], [1,0], [0,1], [1,1]);
  $ex = 0;
  $ey = 1;

  foreach my $k (0 .. 10) {
    print "k=$k\n";
    my $num_points = $h->num_points;
    my @num_sides = $h->num_sides4;
    my $num_sides = sum(@num_sides);
    print " sides ",join(',',@num_sides)," = $num_sides";
    print " points $num_points";
    print "\n";
    if ($k < 10) {
      $h->print;
    }
    # push @values, $num_sides;
    push @values, $num_points;

    ($h,$ex,$ey) = xyhull_expand($h,$ex,$ey);
  }
  print join(',',@values),"\n";
  require Math::OEIS::Grep;
  Math::OEIS::Grep->search(array => \@values, verbose=>1);
  exit 0;
}

{
  # N on centre line with positions

  my @state_table;

  #  0----------->1
  #  0     1 <--- .                  s0
  #  |     ^
  #  v     |
  #   <----
  $state_table[0] = [];

  #  0 . . . . . 1----------->
  #              0     1 <--- .      s1
  #              |     ^
  #              v     |
  #               <----
  $state_table[1] = [  ];

  #   ----------->0 . . . . . 1
  #   0     1 <--- .                 s2
  #   |     ^
  #   v     |
  #    <----
  $state_table[2] = [];

  #          ---->
  #         |     ^
  #         v     |
  #  0 --->       .                  s3
  #  0 <--------- 1
  $state_table[0] = [];

  #                       ---->
  #                      |     ^
  #                      v     |
  #               0 --->       .                  s3
  #  0 . . . . .  1 <---------
  $state_table[0] = [1,];

  #  0 . . . . . 1----------->
  #              0     1 <--- .      s1
  #              |     ^
  #              v     |
  #               <----
  $state_table[1] = [];

  #   ----------->0 . . . . . 1
  #   0     1 <--- .                 s2
  #   |     ^
  #   v     |
  #    <----
  $state_table[2] = [];

  #
  #  <------------     1
  #
  #  @    <---
  #      ^     |
  #      |     |
  #       ---> |
  #          ^ |
  #          | v
  #          .

  exit 0;
}

{
  # twindragon turn tree
  #   k=0    empty
  #   k=1..3 path
  #   k=4    https://hog.grinvin.org/ViewGraphInfo.action?id=26970
  require Graph;
  my $path = Math::PlanePath::DragonCurve->new;

  require MyGraphs;
  my @graphs;
  foreach my $k (0 .. 4) {
    my $graph = twindragon_turn_tree_Graph($k);
    MyGraphs::Graph_view($graph);
    MyGraphs::Graph_xy_print($graph);
    push @graphs, $graph;
  }
  MyGraphs::hog_searches_html(@graphs);
  exit 0;
}



{
  # left boundary by bit pattern

  my $bad = 0;
  my $path = Math::PlanePath::DragonCurve->new;
  foreach my $n (0 .. 2**14) {
    my $is_bits = is_left_boundary_by_high_bits($n);

    my $n_limit;
    {
      my ($pow,$exp) = round_down_pow($n,2);
      $n_limit = 2*$pow;
    }
    my $is_path = path_n_is_left_boundary($path,$n, $n_limit) ? 1 : 0;

    if ($is_bits != $is_path) {
      my $n2 = cnv($n,10,2);
      $n_limit = (defined $n_limit ? " [< $n_limit]" : '');
      print "oops n=$n [$n2]  is_path=$is_path$n_limit is_bits=$is_bits\n";
      die if $bad++ > 10;
    }
  }
  exit 0;

  sub is_left_boundary_by_high_bits {
    my ($n) = @_;
    ### is_left_boundary_by_high_bits(): $n

    {
      # works, $n within level
      my $state = left_boundary_reverse_state_LtoH($n);
      # print "state=m$state\n";
      return ($state == 10 || $state == 8 ? 0 : 1);
    }
    {
      # works, $n within level
      my $n2 = cnv($n,10,2);
      ### $n2
      $n2 =~ /.*(000|010|001|111)/ or return 1;
      my $pos = $-[1];
      $n2 = substr($n2,0,$pos+1);
      ### $n2
      # print "$n2\n";
      return ($n2 =~ /(^|0)(01+|10+)(1(01+|10+))*$/ ? 0 : 1);
    }
    {
      # works
      my $state = left_boundary_state_HtoL($n);
      return ($state == 10 ? 0 : 1);
    }

    {
      # wrong
      my $n2 = '0'.cnv($n,10,2);
      ### $n2
      return ($n2 =~ /^(1+0|0+1)*((1010|1000|1001)|0+1(11|000|001|010))/ ? 0 : 1);
    }
  }

  sub left_boundary_reverse_state_LtoH {
    my ($n) = @_;
    my $state = 0;
    foreach my $bit (bit_split_lowtohigh($n)) {
      $state = $Lpred_LtoH_state_aref->[$state]->[$bit];
    }
    return $state;
  }
}

{
  # Bpred by MyFSM
  require MyFSM;
  my $Rpred_fsm = MyFSM->new
    (table_aref => \@Rpred_state_table,
     initial    => 0,
     accepting  => { $#Rpred_state_table => 'non' },
     title      => 'NonRpred',
     flow       => 'east',
    );
  my $Lpred_fsm = MyFSM->new
    (table_aref => $Lpred_state_aref,
     initial    => 4,       # LpredInf
     accepting  => { $#$Lpred_state_aref => 'non' },
     title      => 'NonLpred',
     flow       => 'east',
    );
  # $Rpred_fsm->view;
  # $Lpred_fsm->view;
  my $fsm = $Rpred_fsm->union($Lpred_fsm, type => 'both');
  $fsm->delete_unreached_states;
  $fsm->simplify (verbose => 1);
  $fsm = $fsm->digit_delta;  $fsm->simplify (verbose => 1);
  $fsm->renumber_states;
   $fsm->view;
  # $fsm->show;
  exit 0;

  my $f = $fsm->as_FLAT;
  # $f = $f->trim_sinks;
  $f = $f->as_min_dfa;
  MyFLAT::view($f);
  exit 0;
}

{
  # f crossing lines all around, internals to enclosed

  my %table = (
               54=>{0=>86,1=>168},
               56=>{0=>90,1=>130},
               72=>{0=>115,1=>180},
               86=>{0=>131,1=>121},
               88=>{0=>134,1=>138},
               90=>{0=>90,1=>90},
               92=>{0=>140,1=>163},
               105=>{0=>158,1=>54},
               109=>{0=>161,1=>54},
               113=>{0=>162,1=>90},
               115=>{0=>90,1=>130},
               121=>{0=>90,1=>90},
               122=>{0=>90,1=>113},
               130=>{0=>134,1=>138},
               131=>{0=>115,1=>136},
               134=>{0=>90,1=>90},
               136=>{0=>90,1=>113},
               138=>{0=>90,1=>90},
               140=>{0=>131,1=>88},
               151=>{0=>90,1=>90},
               158=>{0=>131,1=>88},
               161=>{0=>90,1=>130},
               162=>{0=>131,1=>121},
               163=>{0=>134,1=>168},
               168=>{0=>162,1=>90},
               169=>{0=>90,1=>130},
               174=>{0=>131,1=>88},
               177=>{0=>131,1=>121},
               178=>{0=>115,1=>163},
               180=>{0=>162,1=>113},
              );
  require MyFSM;
  my $fsm = MyFSM->new(table => \%table,
                       title => 'internals to enc',
                       flow => 'east',
                      );
  my $easy = $fsm->as_graph_easy;
  require Graph::Convert;
  my $graph = Graph::Convert->as_graph($easy);
  print $graph->is_cyclic;
   $fsm->view;
  exit 0;
}

{
  # f NonRpred, by crossing lines all around

  my %table = (
               1=>{0=>3,1=>4},
               2=>{0=>5,1=>6},
               3=>{0=>7,1=>8},
               4=>{0=>9,1=>10},
               5=>{0=>11,1=>12},
               6=>{0=>13,1=>14},
               7=>{0=>7,1=>8},
               8=>{0=>15,1=>16},
               9=>{0=>17,1=>18},
               10=>{0=>19,1=>14},
               11=>{0=>7,1=>20},
               12=>{0=>21,1=>22},
               13=>{0=>23,1=>24},
               14=>{0=>13,1=>14},
               15=>{0=>17,1=>25},
               16=>{0=>26,1=>27},
               17=>{0=>28,1=>29},
               18=>{0=>30,1=>31},
               19=>{0=>32,1=>24},
               20=>{0=>15,1=>33},
               21=>{0=>34,1=>35},
               22=>{0=>36,1=>37},
               23=>{0=>38,1=>39},
               24=>{0=>40,1=>22},
               25=>{0=>41,1=>42},
               26=>{0=>43,1=>44},
               27=>{0=>45,1=>46},
               28=>{0=>28,1=>47},
               29=>{0=>48,1=>49},
               30=>{0=>50,1=>51},
               31=>{0=>52,1=>53},
               32=>{0=>54,1=>55},
               33=>{0=>56,1=>57},
               34=>{0=>58,1=>59},
               35=>{0=>60,1=>61},
               36=>{0=>62,1=>63},
               37=>{0=>64,1=>37},
               38=>{0=>65,1=>66},
               39=>{0=>67,1=>68},
               40=>{0=>69,1=>70},
               41=>{0=>50,1=>51},
               42=>{0=>52,1=>53},
               43=>{0=>54,1=>55},
               44=>{0=>71,1=>72},
               45=>{0=>73,1=>74},
               46=>{0=>75,1=>46},
               47=>{0=>48,1=>76},
               48=>{0=>77,1=>78},
               49=>{0=>79,1=>80},
               50=>{0=>58,1=>59},
               51=>{0=>81,1=>82},
               52=>{0=>83,1=>84},
               53=>{0=>36,1=>37},
               54=>{0=>85,1=>86},
               55=>{0=>67,1=>68},
               56=>{0=>43,1=>44},
               57=>{0=>87,1=>88},
               58=>{0=>28,1=>29},
               59=>{0=>89,1=>90},
               60=>{0=>91,1=>92},
               61=>{0=>52,1=>53},
               62=>{0=>93,1=>94},
               63=>{0=>95,1=>96},
               64=>{0=>97,1=>63},
               65=>{0=>65,1=>98},
               66=>{0=>99,1=>100},
               67=>{0=>101,1=>102},
               68=>{0=>56,1=>57},
               69=>{0=>58,1=>59},
               70=>{0=>60,1=>61},
               71=>{0=>103,1=>104},
               72=>{0=>105,1=>106},
               73=>{0=>107,1=>108},
               74=>{0=>109,1=>110},
               75=>{0=>111,1=>112},
               76=>{0=>113,1=>88},
               77=>{0=>28,1=>29},
               78=>{0=>114,1=>115},
               79=>{0=>111,1=>116},
               80=>{0=>117,1=>88},
               81=>{0=>118,1=>119},
               82=>{0=>120,1=>121},
               83=>{0=>122,1=>123},
               84=>{0=>124,1=>125},
               85=>{0=>65,1=>126},
               86=>{0=>127,1=>100},
               87=>{0=>73,1=>128},
               88=>{0=>129,1=>46},
               89=>{0=>130,1=>131},
               90=>{0=>132,1=>133},
               91=>{0=>134,1=>135},
               92=>{0=>136,1=>137},
               93=>{0=>85,1=>138},
               94=>{0=>139,1=>140},
               95=>{0=>141,1=>142},
               96=>{0=>36,1=>37},
               97=>{0=>85,1=>143},
               98=>{0=>144,1=>140},
               99=>{0=>145,1=>146},
               100=>{0=>147,1=>148},
               101=>{0=>149,1=>150},
               102=>{0=>151,1=>152},
               103=>{0=>58,1=>153},
               104=>{0=>154,1=>155},
               105=>{0=>156,1=>157},
               106=>{0=>158,1=>133},
               107=>{0=>159,1=>160},
               108=>{0=>161,1=>162},
               109=>{0=>163,1=>164},
               110=>{0=>165,1=>121},
               111=>{0=>149,1=>150},
               112=>{0=>109,1=>110},
               113=>{0=>111,1=>116},
               114=>{0=>130,1=>131},
               115=>{0=>120,1=>121},
               116=>{0=>109,1=>166},
               117=>{0=>167,1=>128},
               118=>{0=>134,1=>168},
               119=>{0=>122,1=>169},
               120=>{0=>170,1=>171},
               121=>{0=>172,1=>173},
               122=>{0=>122,1=>133},
               123=>{0=>174,1=>175},
               124=>{0=>176,1=>177},
               125=>{0=>178,1=>53},
               126=>{0=>99,1=>179},
               127=>{0=>180,1=>146},
               128=>{0=>109,1=>166},
               129=>{0=>167,1=>74},
               130=>{0=>58,1=>153},
               131=>{0=>181,1=>182},
               132=>{0=>170,1=>171},
               133=>{0=>122,1=>133},
               134=>{0=>159,1=>183},
               135=>{0=>174,1=>175},
               136=>{0=>184,1=>133},
               137=>{0=>165,1=>121},
               138=>{0=>127,1=>179},
               139=>{0=>180,1=>146},
               140=>{0=>105,1=>106},
               141=>{0=>134,1=>135},
               142=>{0=>124,1=>125},
               143=>{0=>139,1=>140},
               144=>{0=>145,1=>146},
               145=>{0=>134,1=>168},
               146=>{0=>185,1=>186},
               147=>{0=>156,1=>157},
               148=>{0=>187,1=>173},
               149=>{0=>122,1=>188},
               150=>{0=>161,1=>162},
               151=>{0=>189,1=>190},
               152=>{0=>178,1=>53},
               153=>{0=>114,1=>191},
               154=>{0=>192,1=>92},
               155=>{0=>52,1=>53},
               156=>{0=>93,1=>157},
               157=>{0=>127,1=>179},
               158=>{0=>184,1=>133},
               159=>{0=>122,1=>123},
               160=>{0=>158,1=>169},
               161=>{0=>167,1=>128},
               162=>{0=>161,1=>80},
               163=>{0=>58,1=>153},
               164=>{0=>181,1=>173},
               165=>{0=>122,1=>133},
               166=>{0=>122,1=>133},
               167=>{0=>149,1=>150},
               168=>{0=>122,1=>133},
               169=>{0=>165,1=>121},
               170=>{0=>122,1=>133},
               171=>{0=>122,1=>133},
               172=>{0=>122,1=>188},
               173=>{0=>132,1=>133},
               174=>{0=>122,1=>133},
               175=>{0=>122,1=>133},
               176=>{0=>134,1=>135},
               177=>{0=>136,1=>175},
               178=>{0=>193,1=>142},
               179=>{0=>105,1=>106},
               180=>{0=>122,1=>133},
               181=>{0=>170,1=>119},
               182=>{0=>120,1=>121},
               183=>{0=>158,1=>133},
               184=>{0=>134,1=>168},
               185=>{0=>159,1=>177},
               186=>{0=>178,1=>53},
               187=>{0=>184,1=>188},
               188=>{0=>122,1=>169},
               189=>{0=>58,1=>59},
               190=>{0=>81,1=>194},
               191=>{0=>132,1=>133},
               192=>{0=>122,1=>123},
               193=>{0=>122,1=>123},
               194=>{0=>132,1=>133},
              );

  my %accepting_Rpred = (
                         1=>'Rpred',
                         2=>'Rpred',
                         3=>'Rpred',
                         4=>'Rpred',
                         5=>'Rpred',
                         6=>'Rpred',
                         7=>'Rpred',
                         8=>'Rpred',
                         9=>'Rpred',
                         10=>'Rpred',
                         11=>'Rpred',
                         12=>'Rpred',
                         13=>'Rpred',
                         14=>'Rpred',
                         15=>'Rpred',
                         16=>'Rpred',
                         17=>'Rpred',
                         18=>'Rpred',
                         19=>'Rpred',
                         20=>'Rpred',
                         21=>'Rpred',
                         22=>'Rpred',
                         23=>'Rpred',
                         24=>'Rpred',
                         25=>'Rpred',
                         26=>'Rpred',
                         27=>'Rpred',
                         28=>'Rpred',
                         29=>'Rpred',
                         30=>'Rpred',
                         31=>'Rpred',
                         32=>'Rpred',
                         33=>'Rpred',
                         34=>'Rpred',
                         35=>'Rpred',
                         38=>'Rpred',
                         39=>'Rpred',
                         40=>'Rpred',
                         41=>'Rpred',
                         42=>'Rpred',
                         43=>'Rpred',
                         44=>'Rpred',
                         45=>'Rpred',
                         46=>'Rpred',
                         47=>'Rpred',
                         48=>'Rpred',
                         49=>'Rpred',
                         50=>'Rpred',
                         51=>'Rpred',
                         52=>'Rpred',
                         54=>'Rpred',
                         55=>'Rpred',
                         56=>'Rpred',
                         57=>'Rpred',
                         58=>'Rpred',
                         59=>'Rpred',
                         60=>'Rpred',
                         61=>'Rpred',
                         66=>'Rpred',
                         67=>'Rpred',
                         68=>'Rpred',
                         69=>'Rpred',
                         70=>'Rpred',
                         71=>'Rpred',
                         72=>'Rpred',
                         73=>'Rpred',
                         74=>'Rpred',
                         75=>'Rpred',
                         76=>'Rpred',
                         77=>'Rpred',
                         78=>'Rpred',
                         79=>'Rpred',
                         80=>'Rpred',
                         81=>'Rpred',
                         82=>'Rpred',
                         83=>'Rpred',
                         84=>'Rpred',
                         86=>'Rpred',
                         87=>'Rpred',
                         88=>'Rpred',
                         89=>'Rpred',
                         90=>'Rpred',
                         91=>'Rpred',
                         92=>'Rpred',
                         100=>'Rpred',
                         101=>'Rpred',
                         102=>'Rpred',
                         103=>'Rpred',
                         104=>'Rpred',
                         107=>'Rpred',
                         108=>'Rpred',
                         109=>'Rpred',
                         110=>'Rpred',
                         111=>'Rpred',
                         112=>'Rpred',
                         113=>'Rpred',
                         114=>'Rpred',
                         115=>'Rpred',
                         116=>'Rpred',
                         117=>'Rpred',
                         118=>'Rpred',
                         119=>'Rpred',
                         120=>'Rpred',
                         121=>'Rpred',
                         128=>'Rpred',
                         129=>'Rpred',
                         130=>'Rpred',
                         131=>'Rpred',
                         132=>'Rpred',
                         137=>'Rpred',
                         147=>'Rpred',
                         148=>'Rpred',
                         149=>'Rpred',
                         150=>'Rpred',
                         151=>'Rpred',
                         152=>'Rpred',
                         153=>'Rpred',
                         154=>'Rpred',
                         155=>'Rpred',
                         160=>'Rpred',
                         161=>'Rpred',
                         162=>'Rpred',
                         163=>'Rpred',
                         164=>'Rpred',
                         165=>'Rpred',
                         166=>'Rpred',
                         167=>'Rpred',
                         169=>'Rpred',
                         170=>'Rpred',
                         171=>'Rpred',
                         172=>'Rpred',
                         173=>'Rpred',
                         181=>'Rpred',
                         182=>'Rpred',
                         187=>'Rpred',
                         188=>'Rpred',
                         189=>'Rpred',
                         190=>'Rpred',
                         191=>'Rpred',
                         192=>'Rpred',
                         194=>'Rpred',
                        );
  my %accepting_Rnon = (
                        36=>'Rnon',
                        37=>'Rnon',
                        53=>'Rnon',
                        62=>'Rnon',
                        63=>'Rnon',
                        64=>'Rnon',
                        65=>'Rnon',
                        85=>'Rnon',
                        93=>'Rnon',
                        94=>'Rnon',
                        95=>'Rnon',
                        96=>'Rnon',
                        97=>'Rnon',
                        98=>'Rnon',
                        99=>'Rnon',
                        105=>'Rnon',
                        106=>'Rnon',
                        122=>'Rnon',
                        123=>'Rnon',
                        124=>'Rnon',
                        125=>'Rnon',
                        126=>'Rnon',
                        127=>'Rnon',
                        133=>'Rnon',
                        134=>'Rnon',
                        135=>'Rnon',
                        136=>'Rnon',
                        138=>'Rnon',
                        139=>'Rnon',
                        140=>'Rnon',
                        141=>'Rnon',
                        142=>'Rnon',
                        143=>'Rnon',
                        144=>'Rnon',
                        145=>'Rnon',
                        146=>'Rnon',
                        156=>'Rnon',
                        157=>'Rnon',
                        158=>'Rnon',
                        159=>'Rnon',
                        168=>'Rnon',
                        174=>'Rnon',
                        175=>'Rnon',
                        176=>'Rnon',
                        177=>'Rnon',
                        178=>'Rnon',
                        179=>'Rnon',
                        180=>'Rnon',
                        183=>'Rnon',
                        184=>'Rnon',
                        185=>'Rnon',
                        186=>'Rnon',
                        193=>'Rnon',
                       );
  my %accepting_Lnon = (
                        28=>'Lnon',
                        29=>'Lnon',
                        46=>'Lnon',
                        47=>'Lnon',
                        48=>'Lnon',
                        49=>'Lnon',
                        58=>'Lnon',
                        74=>'Lnon',
                        75=>'Lnon',
                        76=>'Lnon',
                        77=>'Lnon',
                        78=>'Lnon',
                        79=>'Lnon',
                        80=>'Lnon',
                        88=>'Lnon',
                        109=>'Lnon',
                        110=>'Lnon',
                        111=>'Lnon',
                        112=>'Lnon',
                        113=>'Lnon',
                        114=>'Lnon',
                        115=>'Lnon',
                        116=>'Lnon',
                        117=>'Lnon',
                        119=>'Lnon',
                        120=>'Lnon',
                        121=>'Lnon',
                        122=>'Lnon',
                        128=>'Lnon',
                        129=>'Lnon',
                        130=>'Lnon',
                        131=>'Lnon',
                        132=>'Lnon',
                        133=>'Lnon',
                        149=>'Lnon',
                        150=>'Lnon',
                        153=>'Lnon',
                        161=>'Lnon',
                        162=>'Lnon',
                        163=>'Lnon',
                        164=>'Lnon',
                        165=>'Lnon',
                        166=>'Lnon',
                        167=>'Lnon',
                        169=>'Lnon',
                        170=>'Lnon',
                        171=>'Lnon',
                        172=>'Lnon',
                        173=>'Lnon',
                        181=>'Lnon',
                        182=>'Lnon',
                        188=>'Lnon',
                        191=>'Lnon',
                       );
  require MyFSM;
  my $fsm = MyFSM->new(table => {%table},
                       initial => '1',
                       accepting => {%accepting_Rnon},
                       title => 'f Rnon',
                       flow => 'east',
                      );
  my $fsm_Lnon = MyFSM->new(table => {%table},
                            initial => '1',
                            accepting => {%accepting_Lnon},
                            title => 'f Lnon',
                            flow => 'east',
                           );
  print "Lnon\n";
  # $fsm_Lnon->eventually_accepting (verbose => 1, merge => 1);
  # $fsm_Lnon->simplify (verbose => 0);
  # $fsm_Lnon->renumber_states;
  # $fsm_Lnon->view;

  print "Rnon\n";
  # $fsm->eventually_accepting (verbose => 1, merge => 1);
  # $fsm->simplify (verbose => 0);
  # # $fsm = $fsm->reverse;
  # # $fsm->simplify (verbose => 1);
  # # $fsm->view;
  # # $fsm->print_tikz;

  print "union\n";
  my $fsm_bridges = $fsm->union($fsm_Lnon);
  # $fsm->renumber_states;
  $fsm->simplify (verbose => 1);
  $fsm->renumber_states;
  $fsm->view;

  exit 0;
}

{
  # f enclosed, by hulls all around
  # simplify to 91 states

  my %table = (
               1=>{0=>2,1=>3},
               2=>{0=>4,1=>5},
               3=>{0=>6,1=>7},
               4=>{0=>4,1=>5},
               5=>{0=>8,1=>9},
               6=>{0=>10,1=>11},
               7=>{0=>12,1=>13},
               8=>{0=>10,1=>14},
               9=>{0=>15,1=>16},
               10=>{0=>17,1=>18},
               11=>{0=>19,1=>20},
               12=>{0=>21,1=>22},
               13=>{0=>23,1=>13},
               14=>{0=>24,1=>25},
               15=>{0=>26,1=>27},
               16=>{0=>28,1=>29},
               17=>{0=>17,1=>30},
               18=>{0=>31,1=>32},
               19=>{0=>33,1=>34},
               20=>{0=>35,1=>36},
               21=>{0=>37,1=>38},
               22=>{0=>39,1=>40},
               23=>{0=>41,1=>22},
               24=>{0=>33,1=>34},
               25=>{0=>35,1=>36},
               26=>{0=>37,1=>38},
               27=>{0=>42,1=>43},
               28=>{0=>44,1=>45},
               29=>{0=>46,1=>29},
               30=>{0=>31,1=>47},
               31=>{0=>48,1=>49},
               32=>{0=>50,1=>51},
               33=>{0=>52,1=>53},
               34=>{0=>54,1=>55},
               35=>{0=>56,1=>57},
               36=>{0=>58,1=>59},
               37=>{0=>60,1=>61},
               38=>{0=>62,1=>63},
               39=>{0=>64,1=>65},
               40=>{0=>58,1=>59},
               41=>{0=>66,1=>67},
               42=>{0=>68,1=>69},
               43=>{0=>70,1=>71},
               44=>{0=>72,1=>73},
               45=>{0=>74,1=>75},
               46=>{0=>76,1=>77},
               47=>{0=>78,1=>79},
               48=>{0=>17,1=>18},
               49=>{0=>80,1=>81},
               50=>{0=>76,1=>82},
               51=>{0=>83,1=>79},
               52=>{0=>17,1=>18},
               53=>{0=>84,1=>85},
               54=>{0=>86,1=>87},
               55=>{0=>88,1=>89},
               56=>{0=>90,1=>91},
               57=>{0=>92,1=>93},
               58=>{0=>94,1=>95},
               59=>{0=>96,1=>59},
               60=>{0=>97,1=>98},
               61=>{0=>99,1=>100},
               62=>{0=>101,1=>102},
               63=>{0=>103,1=>104},
               64=>{0=>52,1=>53},
               65=>{0=>105,1=>106},
               66=>{0=>97,1=>107},
               67=>{0=>62,1=>63},
               68=>{0=>52,1=>108},
               69=>{0=>109,1=>110},
               70=>{0=>111,1=>112},
               71=>{0=>113,1=>114},
               72=>{0=>115,1=>116},
               73=>{0=>117,1=>118},
               74=>{0=>119,1=>120},
               75=>{0=>121,1=>89},
               76=>{0=>122,1=>123},
               77=>{0=>74,1=>75},
               78=>{0=>76,1=>82},
               79=>{0=>124,1=>29},
               80=>{0=>125,1=>126},
               81=>{0=>88,1=>89},
               82=>{0=>74,1=>127},
               83=>{0=>128,1=>129},
               84=>{0=>125,1=>126},
               85=>{0=>130,1=>114},
               86=>{0=>131,1=>132},
               87=>{0=>90,1=>133},
               88=>{0=>134,1=>135},
               89=>{0=>136,1=>137},
               90=>{0=>90,1=>114},
               91=>{0=>138,1=>139},
               92=>{0=>140,1=>141},
               93=>{0=>142,1=>36},
               94=>{0=>143,1=>144},
               95=>{0=>145,1=>146},
               96=>{0=>147,1=>95},
               97=>{0=>97,1=>148},
               98=>{0=>149,1=>150},
               99=>{0=>151,1=>152},
               100=>{0=>153,1=>154},
               101=>{0=>122,1=>123},
               102=>{0=>155,1=>156},
               103=>{0=>26,1=>27},
               104=>{0=>157,1=>79},
               105=>{0=>158,1=>159},
               106=>{0=>35,1=>36},
               107=>{0=>149,1=>100},
               108=>{0=>80,1=>160},
               109=>{0=>161,1=>159},
               110=>{0=>35,1=>36},
               111=>{0=>143,1=>112},
               112=>{0=>99,1=>150},
               113=>{0=>162,1=>114},
               114=>{0=>90,1=>114},
               115=>{0=>90,1=>91},
               116=>{0=>113,1=>133},
               117=>{0=>128,1=>129},
               118=>{0=>117,1=>51},
               119=>{0=>52,1=>108},
               120=>{0=>163,1=>137},
               121=>{0=>90,1=>114},
               122=>{0=>90,1=>164},
               123=>{0=>117,1=>118},
               124=>{0=>128,1=>45},
               125=>{0=>52,1=>108},
               126=>{0=>163,1=>165},
               127=>{0=>90,1=>114},
               128=>{0=>122,1=>123},
               129=>{0=>74,1=>127},
               130=>{0=>134,1=>135},
               131=>{0=>115,1=>166},
               132=>{0=>90,1=>114},
               133=>{0=>121,1=>89},
               134=>{0=>90,1=>114},
               135=>{0=>90,1=>114},
               136=>{0=>90,1=>164},
               137=>{0=>130,1=>114},
               138=>{0=>90,1=>114},
               139=>{0=>90,1=>114},
               140=>{0=>131,1=>167},
               141=>{0=>168,1=>139},
               142=>{0=>169,1=>170},
               143=>{0=>60,1=>171},
               144=>{0=>172,1=>173},
               145=>{0=>174,1=>170},
               146=>{0=>58,1=>59},
               147=>{0=>60,1=>175},
               148=>{0=>176,1=>173},
               149=>{0=>177,1=>152},
               150=>{0=>70,1=>71},
               151=>{0=>90,1=>114},
               152=>{0=>178,1=>179},
               153=>{0=>111,1=>112},
               154=>{0=>180,1=>137},
               155=>{0=>181,1=>182},
               156=>{0=>142,1=>36},
               157=>{0=>44,1=>129},
               158=>{0=>131,1=>167},
               159=>{0=>168,1=>183},
               160=>{0=>130,1=>114},
               161=>{0=>90,1=>91},
               162=>{0=>131,1=>132},
               163=>{0=>134,1=>87},
               164=>{0=>90,1=>133},
               165=>{0=>88,1=>89},
               166=>{0=>113,1=>114},
               167=>{0=>138,1=>139},
               168=>{0=>162,1=>114},
               169=>{0=>90,1=>91},
               170=>{0=>92,1=>93},
               171=>{0=>99,1=>150},
               172=>{0=>151,1=>152},
               173=>{0=>70,1=>71},
               174=>{0=>131,1=>167},
               175=>{0=>172,1=>173},
               176=>{0=>177,1=>152},
               177=>{0=>131,1=>132},
               178=>{0=>115,1=>141},
               179=>{0=>142,1=>36},
               180=>{0=>162,1=>164},
               181=>{0=>52,1=>53},
               182=>{0=>54,1=>184},
               183=>{0=>121,1=>89},
               184=>{0=>130,1=>114},
              );

  my %accepting = (
                   90 => 'non',
                   114 => 'non',
                  );
  require MyFSM;
  my $fsm = MyFSM->new(table => \%table,
                       initial => '1',
                       accepting => \%accepting,
                       title => 'fractional segment enclosed',
                       flow => 'east',
                      );
  $fsm->eventually_accepting (verbose => 1, merge => 1);

  $fsm->simplify (verbose => 1);
  # $fsm = $fsm->reverse;
  # $fsm->simplify (verbose => 1);

  $fsm->renumber_states;
  $fsm->view;
  $fsm->print_tikz;

  exit 0;
}

{
  # Rpred / Lpred of sqrts
  #
  # 1, 1,0, 1,0,0,0, 1,0,0,0, 0,0,0,0, 1,0,0,0, 0,0,0,0, 0,0,0,0, 0,0,0,0, 1
  # A036987 Kempner-Mahler sum 1/2^(2^k)), transcendental per Kempner

  require Math::NumSeq::SqrtDigits;

  $|=1;
  my $max = 0;
  foreach my $root (2 .. 50000) {
    next if sqrt($root)==int(sqrt($root));
    my $seq = Math::NumSeq::SqrtDigits->new (sqrt=>$root, radix=>2);
    my $Rpred_i = RLpred_non_at($seq, \@Rpred_state_table);
    # print "$Rpred_i,";
    if ($Rpred_i > $max) {
      print "$root  $Rpred_i\n";
      $max = $Rpred_i;
    } elsif ($Rpred_i == -1) {
      print "$root  $Rpred_i\n";
    }
  }
  print "\n";


  my $count_Rpred = 0;
  my $count_Lpred = 0;
  foreach my $root (2 .. 100) {
    next if sqrt($root)==int(sqrt($root));
    my $seq = Math::NumSeq::SqrtDigits->new (sqrt=>$root, radix=>2);

    my $Rpred_i = RLpred_non_at($seq, \@Rpred_state_table);
    $seq->rewind;
    my $Lpred_i = RLpred_non_at($seq, $Lpred_state_aref);
    $seq->rewind;
    my @values = next_list($seq, 50);
    print "sqrt $root\n";
    print " .",@values,"\n";
      print " Rpred $Rpred_i  Lpred $Lpred_i\n";
      print "\n";
    $count_Rpred += $Rpred_i == -1;
    $count_Lpred += $Lpred_i == -1;
  }
  print "count Rpred $count_Rpred Lpred $count_Lpred\n";
  exit 0;

  sub next_list {
    my ($seq,$count) = @_;
    $seq->rewind;
    my @ret;
    for (1 .. $count) {
      my ($i,$value) = $seq->next;
      push @ret, $value;
    }
    return @ret;
  }
  sub ith_list {
    my ($seq, $i_from, $i_to) = @_;
    return map {$seq->ith($_)} $i_from .. $i_to;
  }
  sub RLpred_of_seq {
    my ($seq, $table) = @_;
    return RLpred_non_at($seq, $table) == -1;
  }
  sub RLpred_non_at {
    my ($seq, $table) = @_;
    my $state = 0;
    foreach (1 .. 500) {
      my ($i,$bit) = $seq->next;
      $state = $table->[$state]->[$bit];
      if ($state == 10) { return $i; }
    }
    return -1;
  }
}

{
  # twindragon area tree parent -- direction sequence

  require Graph::Maker::TwindragonAreaTree;
  require MyGraphs;
  my $k = 10;
  my $graph = Graph::Maker->new('twindragon_area_tree', level=>$k,
                                undirected=>1);
  # print "$graph\n";
  my @parent;
  {
    $parent[0] = -1;
    my @pending = (0);
    my $count = 0;
    while (@pending) {
      $count++;
      my $parent = pop @pending;
      my @neighbours = $graph->neighbours($parent);
      @neighbours = grep {! defined $parent[$_]} @neighbours;
      foreach my $n (@neighbours) {
        $parent[$n] = $parent;
      }
      push @pending, @neighbours;
    }
    print "parents traced through $count\n";
  }
  print join(',',@parent[1..30]),"\n";

  my @values;
  foreach my $v (2 .. 150) {
    my $to_upper = TAVertexToUpper($v);
    my $to_lower = TAVertexToLower($v);
    my $parent = $parent[$v];
    my $dir;
    if (defined $to_upper && $parent == $to_upper) {
      $dir = 1;
    } elsif (defined $to_lower && $parent == $to_lower) {
      $dir = 3;
    } else {
      if ($v & 1) { $dir = 2; } else { $dir = 0; }
    }

    my $value;
    $value = $dir & 1;       # when vertical
    $value = ($dir & 1) ^ 1; # when horizontal
    $value = ($dir == 3 ? 1 : 0);
    push @values, $value;
  }
  print @values,"\n";
  require Math::OEIS::Grep;
  Math::OEIS::Grep->search(array => \@values, verbose=>1);
  exit 0;
}
{
  # twindragon area tree parent

  # 10... aim for 1011111
  #     reduce to 0100000
  # 11... aim for 1101111
  #     reduce to 1010000

  # 111001110000
  #  ^-------------want 0111... to flip down top
  #   ^------------want 0111... to flip down 1 above
  #    ^-----------want 0, good
  #     ^----------want 1, aim below 10000
  #      ^

  # want0   0 = good, stay want0
  #         1 = ->want01 below to flip
  # want01  0 = good, ->want1
  #         1 = ->want01 below to flip
  # want1   0 = want10 below to flip
  #         1 = good, stay want1
  # want10  0 = ->want10 below to flip
  #         1 = good, ->want0
  #
  #         0s      1     1s     0    1s     0     0s     1
  #        want0  ----> want01 ----> want1 ----> want10  --+
  #          ^---------------------------------------------+
  #        to upper      flip       to lower      flip
  #
  # 11110 [111] 00001 [000]
  #
  #
  sub TA_parent_want {
    my ($n) = @_;
    if ($n == 0) { return -1; }
    my @bits = reverse bit_split_lowtohigh($n);  # high to low
    my $state = 'want0';
    foreach my $i (0 .. $#bits) {
      my $bit = $bits[$i];
      if ($state eq 'want0') {
        if ($bit==0) {  }
        else         { $state = 'want01'; }
      } elsif ($state eq 'want01') {
        if ($bit==0) { $state = 'want1'; }
        else         {  }
      } elsif ($state eq 'want1') {
        if ($bit==0) { $state = 'want10'; }
        else         {  }
      } elsif ($state eq 'want10') {
        if ($bit==0) {  }
        else         { $state = 'want0'; }
      }
      print "$bit  $state\n";
    }
    my $bits = $n & 3;
    if ($state eq 'want01') {
      print "flip low\n";
      $n ^= 1;

    } elsif ($state eq 'want10') {
      print "flip low\n";
      $n ^= 1;

    } elsif ($state eq 'want0') {
      print "to upper\n";
      $n = TAVertexToUpper($n);
    } elsif ($state eq 'want1') {
      print "to lower\n";
      $n = TAVertexToLower($n);
    }
    printf " %b final (%s %02b)\n", $n, $state, $bits;
    return $n;
  }

  require Graph::Maker::TwindragonAreaTree;
  require MyGraphs;
  my $k = 12;
  my $graph = Graph::Maker->new('twindragon_area_tree', level=>$k,
                                undirected=>1);
  # print "$graph\n";
  my @parent;
  {
    $parent[0] = -1;
    my @pending = (0);
    my $count = 0;
    while (@pending) {
      $count++;
      my $parent = pop @pending;
      my @neighbours = $graph->neighbours($parent);
      @neighbours = grep {! defined $parent[$_]} @neighbours;
      foreach my $n (@neighbours) {
        $parent[$n] = $parent;
      }
      push @pending, @neighbours;
    }
    print "parents traced through $count\n";
  }
  print join(',',@parent[2..30]),"\n";

  my $show_parent_sequence = sub {
    my ($v) = @_;
    printf "v=%d %b\n", $v,$v;
    my $prev;
    while ($v >= 0) {
      my $parent = $parent[$v] // die "oops, no parent for $v";
      my $diff = $parent - $v;
      my $sign = ($diff > 0 ? '+' : '-');
      $diff = abs($diff);
      my $flip = '';
      if ($parent == ($v ^ 1)) {
        $flip = "flip";
      }
      my $to_upper = TAVertexToUpper($v);
      my $to_lower = TAVertexToLower($v);
      my $ul =
        (defined $to_upper ? sprintf("upper=%b%s", $to_upper,
                                     $to_upper==$parent ? "pp" : "")
         : '')
        . (defined $to_lower && defined $to_upper ?
           ($to_upper < $to_lower ? " < " : " > ")
           : "  ")
        . (defined $to_lower ? sprintf("lower=%b%s", $to_lower,
                                       $to_lower==$parent ? "pp" : "")
           : '')
        . (defined $to_lower && defined $to_upper ? " ***both" : "");
      my $transitions = count_bit_transitions($v) & 1;
      my $parity_1s = count_1_bits($v) & 1;
      my $parity_0s = count_0_bits($v) & 1;
      printf " %*b  %s%b         %d %s %s\n",
        $k, $v, $sign,$diff, $transitions, $flip, $ul;
      # my $got_parent = TA_parent_want($v);
      # if ($got_parent != $parent) {
      #   printf "wrong\n %*b\n", $k,$parent;
      #   exit 0;
      # }
      $v = $parent;
    }
  };
  # $show_parent_sequence->(2**$k-1);
  my $v = 0b111001101111;
  $show_parent_sequence->($v);
  exit 0;

  sub count_1_bits {
    my ($n) = @_;
    my $count = 0;
    while ($n) {
      $count += $n & 1;
      $n >>= 1;
    }
    return $count;
  }
  sub count_0_bits {
    my ($n) = @_;
    my $count = 0;
    while ($n) {
      $count += ! ($n & 1);
      $n >>= 1;
    }
    return $count;
  }
  sub count_bit_transitions {
    my ($n) = @_;
    my $count = 0;
    my $prev = $n & 1;
    for (;;) {
      $n >>= 1;
      last if $n == 0;
      my $bit = $n & 1;
      $count += ($prev != $bit);
      $prev = $bit;
    }
    return $count;
  }
}

{
  # twindragon area tree parent -- want states FSM

  require MyFSM;
  my %table = ( want0  => {0=>'want0',1=>'want01'},
                want01 => {0=>'want1',1=>'want01'},
                want1  => {0=>'want10',1=>'want1'},
                want10 => {0=>'want10',1=>'want0'},
              );
  my %accepting = ( want0  => 'to upper',
                    want1  => 'to lower',
                    want01  => 'flip',
                    want10  => 'flip',
                  );
  my $fsm = MyFSM->new(table => \%table,
                       initial => 'want0',
                       accepting => \%accepting,
                       title => 'TA want',
                       flow => 'east',
                      );

  $fsm->simplify (verbose => 1);

  # $fsm = $fsm->reverse;
  # $fsm->simplify (verbose => 1);
  # $fsm->renumber_states;

  $fsm->view;
  $fsm->show;
  exit 0;
}

{
  # triple 15ths differences by states
  # not working

  my $xyxy_to_segment = sub {
    my ($x1,$y1, $x2,$y2) = @_;
    abs($x1-$x2) + abs($y1-$y2) == 1
      or croak "bad segment $x1,$y1 to $x2,$y2";
    (($x1+$y1) & 1) == 0
      or croak "start not even $x1,$y1 to $x2,$y2";
    return "$x1,$y1,$x2,$y2";
  };
  my $segment_to_xyxy = sub {
    my ($segment) = @_;
    return split /,/, $segment;
  };

  my @segments_1_3_11 = (map {$xyxy_to_segment->(@$_)}
                         [0,0, 1,0],
                         [0,0, 0,1],
                         [0,0, 0,-1]);
  my @segments_2_6_8 = (map {$xyxy_to_segment->(@$_)}
                        [0,0, 1,0],
                        [0,0, 0,1],
                        [1,-1, 0,-1]);
  my @segments_4_12_14 = (map {$xyxy_to_segment->(@$_)}
                          [0,0, 1,0],
                          [0,0, 0,1],
                          [1,-1, 1,0]);
  my @segments_7_9_13 = (map {$xyxy_to_segment->(@$_)}
                         [0,0, 1,0],
                         [1,-1, 1,0]);

  my $segment_shift = sub {
    my ($s, $dx,$dy) = @_;
    my ($x1,$y1, $x2,$y2) = $segment_to_xyxy->($s);
    $x1 += $dx; $y1 += $dy;
    $x2 += $dx; $y2 += $dy;
    return $xyxy_to_segment->($x1,$y1, $x2,$y2);
  };
  my $segment_rotate_plus90 = sub {
    my ($s) = @_;
    my ($x1,$y1, $x2,$y2) = $segment_to_xyxy->($s);
    ($x1,$y1) = xy_rotate_plus90($x1,$y1);
    ($x2,$y2) = xy_rotate_plus90($x2,$y2);
    return $xyxy_to_segment->($x1,$y1, $x2,$y2);
  };
  my $segment_rotate_minus90 = sub {
    my ($s) = @_;
    my ($x1,$y1, $x2,$y2) = $segment_to_xyxy->($s);
    ($x1,$y1) = xy_rotate_minus90($x1,$y1);
    ($x2,$y2) = xy_rotate_minus90($x2,$y2);
    return $xyxy_to_segment->($x1,$y1, $x2,$y2);
  };
  my $segment_rotate_180 = sub {
    my ($s) = @_;
    my ($x1,$y1, $x2,$y2) = $segment_to_xyxy->($s);
    return $xyxy_to_segment->(-$x1,-$y1, -$x2,-$y2);
  };
  my $segment_expand = sub {
    my ($s) = @_;
    my ($x1,$y1, $x2,$y2) = $segment_to_xyxy->($s);
    my $dx = $x2-$x1;
    my $dy = $y2-$y1;
    ($x1,$y1) = mul($x1,$y1);
    ($x2,$y2) = mul($x2,$y2);
    my $mx = $x1 + $dx;
    my $my = $y1 + $dy;
    return ($xyxy_to_segment->($x1,$y1, $mx,$my),
            $xyxy_to_segment->($x2,$y2, $mx,$my));
  };

  my %want;
  foreach my $s (@segments_1_3_11) {
    $want{$s}                            = 1;
    $want{$segment_rotate_minus90->($s)} = 1;
    $want{$segment_rotate_plus90->($s)}  = 1;
  }
  foreach my $s (@segments_2_6_8) {
    $want{$s}                            = 1;
    $want{$segment_rotate_minus90->($s)} = 1;
    my $t = $segment_shift->($s, -1, 1);
    $t = $segment_rotate_180->($t);
    $want{$t}                            = 1;
  }
  foreach my $s (@segments_4_12_14) {
    $want{$s}                            = 1;
    $want{$segment_rotate_minus90->($s)} = 1;
    my $t = $segment_shift->($s, -1, 1);
    $t = $segment_rotate_minus90->($t);
    $want{$t}                            = 1;
  }
  foreach my $s (@segments_7_9_13) {
    $want{$s}                            = 1;
    my $t = $segment_shift->($s, -1, 1);
    $t = $segment_rotate_minus90->($t);
    $want{$t}                            = 1;
  }

  # consecutives added
  my @want_set = sort keys %want;
  foreach my $s (@want_set) {
    $want{$segment_rotate_minus90->($s)} = 1;
    $want{$segment_rotate_plus90->($s)}  = 1;
    my $t = $segment_rotate_minus90->($s);
    $t = $segment_shift->($t, 1, 1);
    $want{$t}                            = 1;
    $t = $segment_rotate_plus90->($s);
    $t = $segment_shift->($t, 1, -1);
    $want{$t}                            = 1;
  }

  # extras needed
  $want{$xyxy_to_segment->(0,-2, 0,-1)} = 1;
  $want{$xyxy_to_segment->(0,-2, -1,-2)} = 1;

  # expand
  @want_set = sort keys %want;
  %want = ();
  foreach my $s (@want_set) {
    my @expand = $segment_expand->($s);
    $want{$expand[0]} = 1;
    $want{$expand[1]} = 1;
  }

  foreach my $s (sort keys %want) {
    my ($x1,$y1, $x2,$y2) = $segment_to_xyxy->($s);
    print "  \\draw[->] ($x1,$y1)--($x2,$y2);\n";
  }
  exit 0;
}

{
  # triple 15ths union of configurations

  require MyPrintwrap;
  require MyLineSegments;
  my $segments = MyLineSegments->new;
  my $fill_1_3_11 = sub {
    my ($x,$y, $dx,$dy) = @_;
    $segments->add($x,$y, $x+$dx, $y+$dy);

    my ($rx,$ry) = xy_rotate_minus90($dx,$dy);
    $segments->add($x,$y, $x+$rx, $y+$ry);

    my ($lx,$ly) = xy_rotate_plus90($dx,$dy);
    $segments->add($x,$y, $x+$lx, $y+$ly);
  };
  my $fill_2_6_8 = sub {
    my ($x,$y, $dx,$dy) = @_;
    $segments->add($x,$y, $x+$dx, $y+$dy);         # 6

    my ($lx,$ly) = xy_rotate_plus90($dx,$dy);     # 2
    $segments->add($x,$y, $x+$lx, $y+$ly);

    my ($rx,$ry) = xy_rotate_minus90($dx,$dy);      # 8
    $x += $rx;
    $y += $ry;
    $segments->add($x,$y, $x+$dx, $y+$dy);
  };
  my $fill_4_12_14 = sub {
    my ($x,$y, $dx,$dy) = @_;
    $segments->add($x,$y, $x+$dx, $y+$dy);         # 12

    my ($lx,$ly) = xy_rotate_plus90($dx,$dy);      # 4
    $segments->add($x,$y, $x+$lx, $y+$ly);

    my ($rx,$ry) = xy_rotate_minus90($dx,$dy);     # 14
    $x += $dx;
    $y += $dy;
    $segments->add($x,$y, $x+$rx, $y+$ry);
  };
  my $fill_7_9_13 = sub {
    my ($x,$y, $dx,$dy) = @_;
    $segments->add($x,$y, $x+$dx, $y+$dy);         # 7,9

    my ($lx,$ly) = xy_rotate_plus90($dx,$dy);      # 13
    my ($rx,$ry) = xy_rotate_minus90($dx,$dy);
    $x += $dx;
    $y += $dy;
    $segments->add($x,$y, $x+$rx, $y+$ry);
  };

  my $fill_segment = sub {
    my ($x,$y, $dx,$dy) = @_;

    my ($rx,$ry) = xy_rotate_minus90($dx,$dy);
    my ($lx,$ly) = xy_rotate_plus90($dx,$dy);

    # 1,3,11
    $fill_1_3_11->($x,$y, $dx,$dy);                    # 3
    $fill_1_3_11->($x,$y, xy_rotate_plus90($rx,$ry));  # 11
    $fill_1_3_11->($x,$y, xy_rotate_minus90($dx,$dy)); # 1

    # 2,6,8
    $fill_2_6_8->($x,$y, $dx,$dy);                     # 6
    $fill_2_6_8->($x,$y, xy_rotate_minus90($dx,$dy));  # 2
    $fill_2_6_8->($x+$dx+$rx,$y+$dy+$ry, -$dx,-$dy);   # 8

    # 4,12,14
    $fill_4_12_14->($x,$y, $dx,$dy);                     # 12
    $fill_4_12_14->($x,$y, xy_rotate_minus90($dx,$dy));  # 4
    $fill_4_12_14->($x+$dx+$lx,$y+$dy+$ly, $rx,$ry);   # 14

    # 7,9,13
    $fill_7_9_13->($x,$y, $dx,$dy);                    # 7,9
    $fill_7_9_13->($x+$dx+$rx,$y+$dy+$ry, $lx,$ly);     # 13
  };

  $fill_segment->(0,0, 1,0);
  $fill_segment->(0,0, 0,1);
  $fill_segment->(0,0, 0,-1);

  $fill_segment->(1,1, 0,-1);
  $fill_segment->(0,-1, 0,1);

  $MyPrintwrap::Printwrap = 2;
  MyPrintwrap::printwrap("\\draw ");
  $MyPrintwrap::Printwrap += 2;
  $segments->printwrap;
  MyPrintwrap::printwrap(";\n");

  exit 0;
}

{
  # join squares left and right of crossing line
  #
  # left side
  # recurrence_guess([1,3,6,8,14,26,43,71,123,209,352])
  # 2*1+6
  # 2*3+8
  # 2*6+14
  # 2*8+26 == 42   \\ actual is bigger
  # 2*14+43
  # 2*26+71
  # 2*43+123
  # 2*71+209 == 351   \\ actual is bigger

  $|=1;
  my $path = Math::PlanePath::DragonCurve->new;
  my @values_left;
  my @values_right;
  foreach my $k (17..20,4 .. 16) {
    my ($n_lo, $n_hi) = $path->level_to_n_range($k);
    my $n_mid = $n_hi / 2;

    my ($sx,$sy) = BlobStart_xy($k);
    my ($ex,$ey) = BlobEnd_xy($k);
    my ($sx_horizontal,$sy_horizontal) = transform_to_horizontal($sx,$sy, $k+1);
    my ($ex_horizontal,$ey_horizontal) = transform_to_horizontal($ex,$ey, $k+1);
    $sy_horizontal == $ey_horizontal or die;

    my $count = 0;
    my $count_left = 0;
    my $count_right = 0;
    my %seen;
    foreach my $n ($n_lo .. $n_mid-1) {
      my ($x,$y) = $path->n_to_xy($n);
      my ($dx,$dy) = $path->n_to_dxdy($n);
      my ($lx,$ly) = xy_rotate_plus90($dx,$dy);
      $x += $dx/2 + $lx/2;
      $y += $dy/2 + $ly/2;
      next if $seen{"$x,$y"}++;

      next unless
        path_n_is_left_boundary($path, $n, $n_mid)
        && ! path_n_is_left_boundary($path, $n, $n_hi);

      ($x,$y) = transform_to_horizontal($x,$y, $k+1);
      if ($y > $sy_horizontal) {
        $count_left++;
      } else {
        $count_right++;
      }
      $count++;
    }
    # $count == JA($k-1)
    #   or die "oops count=$count vs JA=",JA($k);

    print "k=$k  $count  $count_left  $count_right\n";
    push @values_left, $count_left;
    push @values_right, $count_right;
  }
  require Math::OEIS::Grep;
  Math::OEIS::Grep->search(array => \@values_left, verbose=>1);
  Math::OEIS::Grep->search(array => \@values_right, verbose=>1);
  exit 0;
}

{
  # Dpred next, for t=0
  #  !t 10000 t 10000 t 1 0000
  #     01111   01111

  # DpredNext0(n)=n++;while(!DpredInf(n)||turn(n)!=1,n++);n;
  # n=from_binary(101111001111101);print(to_binary(n));
  # for(i=1,90, printf("%20s\n",to_binary(n)); n=DpredNext0(n))
  # 101111001111101
  # 101111010000000
  # 101111010000001
  # 101111010000010
  # 101111010000100
  # 101111010000101   low
  # 101111010001000   push up by drop run
  # 101111010001001
  # 101111010001010
  my $debug = 1;
  my $len = 5;

  my $n;
  $n = 0b1_01111_0_01111_0_1_00;
  $n = 0b1_01111_0_1_000;
  $n = 0b1_01111_0_011111_0_1;
  my $step = sub {
    my ($n) = @_;

    {
      my $mask = MaskLowestOneAndBelow($n);
      if ($debug) { printf "mask   = %25b\n", $mask & ((1<<20)-1); }

      my $lab = $mask + 1;
      if ($debug) { printf "lab    = %25b\n", $lab; }

      my $lu = $lab << 1;
      if ($debug) { printf "lu     = %25b\n", $lu; }

      if (($n & $lu) && ($mask >= 0b1111)) {
        if ($debug) { print "case low then 01\n"; }
        $n |= 3;
        return $n;
      } elsif (!($n & $lu) && ($mask >= 0b111)) {
        if ($debug) { print "case low then 10\n"; }
        $n |= 1;
        return $n;
      }

      my $xor = 0b10010010010010010;

      my $mul = $lab * $xor;
      if ($debug) { printf "mul    = %25b\n", $mul; }

      my $x = ($n & ~$mask) ^ $mul;
      if ($debug) { printf "x      = %25b\n", $x; }

      $mask = MaskLowestOneAndBelow($x);
      if ($debug) { printf "mask   = %25b\n", $mask & ((1<<20)-1); }

      $lab = $mask + 1;
      if ($debug) { printf "lab    = %25b\n", $lab; }

      $lu = $lab << 1;
      if ($debug) { printf "lu     = %25b\n", $lu; }

      my $l = $lab >> 1;
      if ($debug) { printf "l      = %25b\n", $l; }

      if ($n & $l) {
        if ($debug) { print "case 01 inc\n"; }
        $n += $l;
        $n &= ~($mask >> 1);
        $n |= 1;
      } else {
        if ($debug) { print "case 10 raise\n"; }
        $n += $lab >> 3;
      }
      return $n;
    }
  };

  foreach (1 .. $len) {
    printf "n      = %25b\n", $n;
    $n = $step->($n);
  }
  printf "final\nn      = %25b\n", $n;

  if (! $debug) {
    $|=1;
    my $target = 1;
    my $count = 0;
    $n = 0b100;
    print "$n\n";
    while ($target <= (1<<16)) {
      while ($n < $target) {
        $count++;
        $n = $step->($n);
      }
      print "$count, ";
      $count = 0;
      $target *= 2;
    }
    print "\n";
  }
  exit 0;
}

{
  # two-runs binary
  # style 100
  #       110
  #      1000
  #      1100
  #     10000
  #     11000
  #    100000
  #    100100
  #    100110
  #    110000
  #    110100
  #    110110
  # 2/(1 - x - 2*x^3)

  #       *
  #    011011011  does not occur
  #      0111011
  #      0111011

  #        *
  #    011011011  does not occur
  #     11001011
  #      1101011

  #         *
  #    011011011
  #     11000011  up 11
  #      1100011  up 11
  #       110011  up 11
  #      1000011  up 11
  #       100011  up 11
  #       010011  10 inc

  #       ^^ to 11
  #
  #    0110
  my $debug = 1;
  my $len = 5;

  my $n;
  $n = 0b0100000001011;
  $n = 0b0100001101100;
  $n = 0b0100001001100;
  $n = 0b0100001100110;
  $n = 0b100110;
  $n = 0b1100110;
  $n = 0b100;
  $n = 0;
  $n = 0b11000110;
  my $step = sub {
    my ($n) = @_;

    # this bit not right
    # {
    #   my $xor = 0b110110110110110110;
    #   my $x = $n ^ $xor;
    #   if ($debug) { printf "x      = %20b\n", $x; }
    #
    #   my $mask = MaskLowestOneAndBelow($x);
    #   if ($debug) { printf "mask   = %20b\n", $mask & ((1<<20)-1); }
    #
    #   my $lab = $mask + 1;
    #   if ($debug) { printf "lab    = %20b\n", $lab; }
    #
    #   my $lu = $lab << 1;
    #   if ($debug) { printf "lu     = %20b\n", $lu; }
    #
    #   my $l = $lab >> 1;
    #   if ($debug) { printf "l      = %20b\n", $l; }
    #
    #   if (!($n & $lu) && ($n & $lab)) {
    #     if ($debug) { print "case 10\n"; }
    #     $n &= ~$mask;
    #     $n |= $lab >> 1;
    #   } else {
    #     if ($debug) { print "case 11 up\n"; }
    #     $n &= ~$mask;
    #     $n |= $lab >> 2;
    #   }
    #   return $n;
    # }

    {
      my $xor = 0b011011011011011011;
      my $mask = MaskLowestOneAndBelow($n);
      if ($debug) { printf "mask   = %20b\n", $mask & ((1<<20)-1); }

      my $lab = $mask + 1;
      if ($debug) { printf "lab    = %20b\n", $lab; }

      my $ten = ! ($n & $lab);
      if ($ten) {
        $mask >>= 1;
        $lab >>= 1;
        if ($debug) { printf "rmask  = %20b\n", $mask; }
        if ($debug) { printf "rlab   = %20b\n", $lab; }
      }

      if ($mask >= 0b11111) {
        if ($debug) { print "case low\n"; }
        return $n | 4;
      }

      # if ($ten) {
      #   if ($debug) { print "case 10 low\n"; }
      #   return $n | ($lab >> 1);
      # }

      my $lone = $lab >> 1;
      if ($debug) { printf "lone   = %20b\n", $lone; }

      my $mul = $lone * $xor;
      if ($debug) { printf "mul    = %20b\n", $mul; }

      my $x = $n ^ ($lone * $xor);
      if ($debug) { printf "x      = %20b\n", $x; }

      $mask = MaskLowestOneAndBelow($x);
      if ($debug) { printf "mask   = %20b\n", $mask & ((1<<20)-1); }

      $lab = $mask + 1;
      if ($debug) { printf "lab    = %20b\n", $lab; }

      my $lu = $lab << 1;
      if ($debug) { printf "lu     = %20b\n", $lu; }

      my $l = $lab >> 1;
      if ($debug) { printf "l      = %20b\n", $l; }

      if (!($n & $lu) && ($n & $lab)) {
        if ($debug) { print "case 10\n"; }
        $n &= ~$mask;
        $n |= $lab >> 1;
      } else {
        if ($debug) { print "case 11 up\n"; }
        $n &= ~$mask;
        $n |= $lab >> 2;
      }
      return $n;
    }
  };

  foreach (1 .. $len) {
    printf "n      = %20b\n", $n;
    $n = $step->($n);
  }
  printf "final\nn      = %20b\n", $n;

  if (! $debug) {
    $|=1;
    my $target = 1;
    my $count = 0;
    $n = 0b100;
    print "$n\n";
    while ($target <= (1<<16)) {
      while ($n < $target) {
        $count++;
        $n = $step->($n);
      }
      print "$count, ";
      $count = 0;
      $target *= 2;
    }
    print "\n";
  }
  exit 0;
}

{
  # ternary with 2 zeros below each non-zero digit
  my @values;
  foreach my $n (1 .. 5000) {
    if (n_is_ternary_2_zeros_below($n)) {
      my $n3 = cnv($n,10,3);
      printf "%10s\n", $n3;
      push @values, $n3/100;
    }
  }
  require Math::OEIS::Grep;
  Math::OEIS::Grep->search(array => \@values, verbose=>1);
  exit 0;

  # return true if $n has >= 2 zero ternary digits below each non-zero
  sub n_is_ternary_2_zeros_below {
    my ($n) = @_;
    my $n3 = cnv($n,10,3);
    return scalar($n3 =~ /^([12]0{2,})*$/);
  }
}

{
  # two-runs binary
  # style  01
  #        11
  #       010  <- not consecutive
  #       110
  # (x + x^2)/(1 - x - 2*x^3)

  my $debug = 0;

  my $xor = 0b011011011011011011;
  my $n = 0b01000011011;
  $n = 0b0100000101100;
  $n = 0b0100000001011;
  $n = 0;
  my $step = sub {
    my ($n) = @_;

    my $mask = MaskLowestOneAndBelow($n);
    if ($debug) { printf "mask   = %20b\n", $mask & ((1<<20)-1); }

    if ($mask > 0b111) {
      return $n+1;
    }

    my $lab = $mask + 1;
    if ($debug) { printf "lab    = %20b\n", $lab; }

    my $lone = $lab >> 1;
    if ($debug) { printf "lone   = %20b\n", $lone; }

    my $mul = $lone * $xor;
    if ($debug) { printf "mul    = %20b\n", $mul; }

    my $x = $n ^ ($lone * $xor);
    if ($debug) { printf "x      = %20b\n", $x; }

    $mask = MaskLowestOneAndBelow($x);
    if ($debug) { printf "mask   = %20b\n", $mask & ((1<<20)-1); }

    $lab = $mask + 1;
    if ($debug) { printf "lab    = %20b\n", $lab; }

    my $l = $lab >> 2;
    if ($debug) { printf "l      = %20b\n", $l; }

    if ($n & $l) {
      if ($debug) { print "case 01\n"; }
      $n &= ~($mask >> 2);
      $n |= $lab >> 1;
    } else {
      if ($debug) { print "case 11\n"; }
      $n &= ~($mask >> 3);
    }
    return $n;
  };

  foreach (1 .. 40) {
    printf "n      = %20b\n", $n;
    $n = $step->($n);
  }
  printf "final\nn      = %20b\n", $n;


  my $target = 1;
  my $count = 0;
  $n = 1;
  print "$n\n";
  while ($target <= (1<<16)) {
    while ($n < $target) {
      $count++;
      $n = $step->($n);
    }
    print "$count, ";
    $count = 0;
    $target *= 2;
  }
  print "\n";
  exit 0;

  sub MaskLowestOne {
    my ($n) = @_;
    return MaskAboveLowestOne($n)>>1;
  }
  sub MaskAboveLowestOne {
    my ($n) = @_;
    return (($n-1) ^ $n) + 1;
  }
  sub MaskLowestOneAndBelow {
    my ($n) = @_;
    return ($n-1) ^ $n;
  }
}



{
  # twindragon graph 18 nodes 27 edges, all degree 3
  # TA(4)==18
  #
  # diameter      2,4,6,10,14,22,30,42,58       TEndLength + 2
  # TEndLength    2,2,4, 8,12,20,28,40,56,80,
  # start to cL   1,2,4, 6,10,14,22,30,46
  # start to cR   1,2,2, 4, 8,12,20,28,40
  # opp cL to cR  2,2,4, 4, 8,12,20,28,44       2*start_to_cL(k-2)
  # corner ecc    2,3,5, 7,11,15,23,31,47

  # my $graph = twindragon_Graph(3);
  # # graph_show($graph);
  # print "num edges    ",scalar($graph->edges),"\n";
  # print "num vertices ",scalar($graph->vertices),"\n";
  # print "diameter     ",scalar($graph->diameter),"\n";

  my $path = Math::PlanePath::DragonCurve->new;

  my @values;
  foreach my $k (9, 0 .. 8) {
    (my $n_lo, my $n_hi) = $path->level_to_n_range($k);  # one dragon side
    my $graph = twindragon_Graph($k);
    my $start_v = Graph_xy_to_vertex(0,0);

    my ($cRx,$cRy) = $path->n_to_xy($n_hi);
    my $cR_v = Graph_xy_to_vertex($cRx,$cRy);
    my $cR_ecc = $graph->vertex_eccentricity($cR_v);

    my ($cLx,$cLy) = xy_rotate_plus90($cRx,$cRy);
    my $cL_v = Graph_xy_to_vertex($cLx,$cLy);
    my $cL_ecc     = 1||$graph->vertex_eccentricity($cL_v);

    my $start_to_cR = $graph->path_length($start_v, $cR_v);
    my $start_to_cL = $graph->path_length($start_v, $cL_v);
    my $cR_to_cL   = $graph->path_length($cL_v,$cR_v);

    my $diameter      = 1||$graph->diameter;
    my $radius        = 1||$graph->diameter;
    my $shortest_path = 1||$graph->shortest_path;
    my $longest_path  = 1||$graph->longest_path;
    print "k=$k  diameter=$diameter radius=$radius shortest=$shortest_path longest=$longest_path\n";
    print "  start to cL $start_to_cL cR $start_to_cR  cR_ecc $cR_ecc   cL to cR $cR_to_cL\n";
    push @values, $cR_ecc;
  }
  require Math::OEIS::Grep;
  Math::OEIS::Grep->search(array => \@values, verbose=>1);
  exit 0;
}

{
  # right boundary squares inertia

  my $path = Math::PlanePath::DragonCurve->new;
  my @values;
  foreach my $level (8,16) {
    ### $level
    my $n_start = $path->n_start;
    (my $n_lo, my $n_hi) = $path->level_to_n_range($level);
    my ($x_hi,$y_hi) = $path->n_to_xy($n_hi);
    my $x_mid = $x_hi/2;
    my $y_mid = $y_hi/2;

    my $count = 0;
    my %seen;
    my $Ix = 0;
    my $Iy = 0;
    my $Ixy = 0;
    my $x_total = 0;
    my $y_total = 0;
  N: foreach my $n ($n_lo .. $n_hi-1) {
      my ($x,$y) = $path->n_to_xy($n);
      my ($dx,$dy) = $path->n_to_dxdy($n);
      my $mx = $x;
      my $my = $y;

      my $untraversed = 0;
      foreach my $i (1 .. 4) {
        ### at: "$i     $x,$y and $dx,$dy"
        if ($x <= $mx && $y <= $my) {
          $mx = $x;
          $my = $y;
        }
        if (! path_xyxy_is_traversed_either($path, $x,$y, $x+$dx,$y+$dy, $n_hi)) {
          $untraversed = 1;
        }
        $x += $dx;
        $y += $dy;
        ($dx,$dy) = xy_rotate_minus90($dx,$dy);  # 90deg to the right
      }
      next unless $untraversed;

      ### key: "$mx,$my"
      next if $seen{"$mx,$my"}++;
      $count++;

      $x = $mx + .5;  # middle of RQ square
      $y = $my + .5;
      $Ix += $y*$y;
      $Iy += $x*$x;
      $Ixy += $x*$y;
      $x_total += $x;
      $y_total += $y;
    }
    $count == RQ($level) or die "count=$count want RQ($level)=",RQ($level);

    my $x_total_str = (2*$x_total) . "/2";
    my $y_total_str = (2*$y_total) . "/2";

    my $Ix_str = (4*$Ix) . "/4";
    my $Iy_str = (4*$Iy) . "/4";
    my $Ixy_str = (4*$Ixy) . "/4";

    my $gx = $x_total / $count;
    my $gy = $y_total / $count;
    print "k=$level  $count  centroid  $x_total_str,$y_total_str   $gx,$gy\n";
    print "  Ix=$Ix_str, Iy=$Iy_str, Ixy=$Ixy_str\n";
    $Ix /= $count * 2**$level;
    $Iy /= $count * 2**$level;
    $Ixy /= $count * 2**$level;
    print "  $Ix $Iy $Ixy\n";
  }
  exit 0;
}
{
  # blob inertia xy cross-product in each quadrant
  #
  # k=20
  #   centroid 585.143032892415, -292.572532115166
  #   1158155827.20154  18.3948191606045  count 23.3427429737643
  #   2299050969.56578  36.515489395207  count 29.4799107355732
  #   933909707.719445  14.8331509304255  count 19.6534327703444
  #   1904981419.00855  30.256540513763  count 27.5239135203181

  my $path = Math::PlanePath::DragonCurve->new;
  foreach my $k (5 .. 20) {
    print "k=$k\n";
    my ($n_lo, $n_hi) = $path->level_to_n_range($k);
    my $n_start = BlobN($k);
    my $n_end = BlobN($k+1)-3;
    my $blob_segments = $n_end - $n_start;
    $blob_segments == BlobSegments($k) or die;

    my $cx = 0;
    my $cy = 0;
    foreach my $n ($n_start .. $n_end-1) {
      my ($x,$y) = $path->n_to_xy($n + .5);
      ($x,$y) = transform_to_horizontal($x,$y, $k);
      if ($k & 1) { $x /= sqrt(2); $y /= sqrt(2); }
      $cx += $x;
      $cy += $y;
    }
    $cx /= $blob_segments;
    $cy /= $blob_segments;
    print "  centroid $cx, $cy\n";

    my @sum = (0,0,0,0,0);
    my @count = (0,0,0,0,0);
    foreach my $n ($n_start .. $n_end-1) {
      my ($x,$y) = $path->n_to_xy($n + .5);
      ($x,$y) = transform_to_horizontal($x,$y, $k);
      if ($k & 1) { $x /= sqrt(2); $y /= sqrt(2); }
      $x -= $cx;
      $y -= $cy;
      my $quadrant = ($y > 0 ? ($x > 0 ? 1 : 2)
                      : ($x > 0 ? 4 : 3));
      $sum[$quadrant] += abs($x*$y);
      $count[$quadrant]++;
    }
    my $total = sum(@sum);
    foreach my $q (1 .. 4) {
      my $percent = $sum[$q] / $total * 100;
      my $count_percent = $count[$q] / $blob_segments * 100;
      print "  $sum[$q]  $percent  count $count_percent\n";
    }
  }
  exit 0;
}
{
  # left boundary squares inertia

  my $path = Math::PlanePath::DragonCurve->new;
  my @values;
  foreach my $level (8,16) {
    ### $level
    my $n_start = $path->n_start;
    (my $n_lo, my $n_hi) = $path->level_to_n_range($level);
    my ($x_hi,$y_hi) = $path->n_to_xy($n_hi);
    my $x_mid = $x_hi/2;
    my $y_mid = $y_hi/2;

    my $count = 0;
    my %seen;
    my $Ix = 0;
    my $Iy = 0;
    my $Ixy = 0;
    my $x_total = 0;
    my $y_total = 0;
  N: foreach my $n ($n_lo .. $n_hi-1) {
      my ($x,$y) = $path->n_to_xy($n);
      my ($dx,$dy) = $path->n_to_dxdy($n);
      my $mx = $x;
      my $my = $y;

      my $untraversed = 0;
      foreach my $i (1 .. 4) {
        ### at: "$i     $x,$y and $dx,$dy"
        if ($x <= $mx && $y <= $my) {
          $mx = $x;
          $my = $y;
        }
        if (! path_xyxy_is_traversed_either($path, $x,$y, $x+$dx,$y+$dy, $n_hi)) {
          $untraversed = 1;
        }
        $x += $dx;
        $y += $dy;
        ($dx,$dy) = xy_rotate_plus90($dx,$dy);
      }
      next unless $untraversed;

      ### key: "$mx,$my"
      next if $seen{"$mx,$my"}++;
      $count++;

      $x = $mx + .5;
      $y = $my + .5;
      $Ix += $y*$y;
      $Iy += $x*$x;
      $Ixy += $x*$y;
      $x_total += $x;
      $y_total += $y;
    }
    $count == LQ($level) or die "count=$count want LQ($level)=",LQ($level);

    my $gx = $x_total / $count;
    my $gy = $y_total / $count;
    print "k=$level  $count   $x_total,$y_total   $gx,$gy\n";
    print "  $Ix $Iy $Ixy\n";
    $Ix /= $count * 2**$level;
    $Iy /= $count * 2**$level;
    $Ixy /= $count * 2**$level;
    print "  $Ix $Iy $Ixy\n";
  }
  exit 0;
}





{
  # EpredL left-side enclosure FSM

  require MyFSM;
  my %accepting = (ls3 => 'non', ls3rev => 'non');
  my %table = ( ls0    => {0=>'ls0',1=>'ls1rev'},
                ls1rev => {0=>'ls12',1=>'ls0rev'},
                ls0rev => {0=>'ls0',1=>'ls0rev'},
                ls12   => {0=>'ls12',1=>'ls2rev'},
                ls2rev => {0=>'ls3',1=>'ls1rev'},
                ls3    => {0=>'ls12',1=>'ls3rev'},
                ls3rev => {0=>'ls3',1=>'ls3rev'},
              );
  my $fsm = MyFSM->new(table => \%table,
                       initial => 'ls0',
                       accepting => \%accepting,
                       title => 'EpredL',
                       flow => 'east',
                      );

  # my @accepting = keys %accepting;
  # foreach my $i (1 .. $#accepting) {
  #   $fsm->merge_state($accepting[0],$accepting[1]);
  # }

  $fsm->simplify (verbose => 1);

  $fsm = $fsm->reverse;
  $fsm->simplify (verbose => 1);
  $fsm->renumber_states;

  $fsm->view;
  $fsm->show;
  exit 0;
}

{
  # EpredL left-side enclosure sequence

  my $path = Math::PlanePath::DragonCurve->new;
  my $n_is_enc = sub {
    my ($n, $side) = @_;
    my ($x,$y) = $path->n_to_xy($n);
    my ($dx,$dy) = $path->n_to_dxdy($n);
    my ($x2,$y2) = ($x+$dx, $y+$dy);
    my ($dxS,$dyS) = ($side eq 'left'
                      ? xy_rotate_plus90($dx,$dy)
                      : xy_rotate_minus90($dx,$dy));
    return
      (path_xyxy_is_traversed_either($path, $x,$y, $x+$dxS,$y+$dyS, $n)
       && path_xyxy_is_traversed_either($path, $x2,$y2, $x2+$dxS,$y2+$dyS, $n)
       && path_xyxy_is_traversed_either($path, $x+$dxS,$y+$dyS, $x2+$dxS,$y2+$dyS, $n));
  };

  foreach my $side ('left') {
    foreach my $n (0 .. 20000) {
      my $is_enc = $n_is_enc->($n,$side) ? 1 : 0;
      my $got_enc = n_is_enc_by_states_low_to_high($n,$side) ? 1 : 0;
      my $diff = $is_enc == $got_enc ? '' : ' ***';
      if ($diff || $n < 32) {
        print "$n $is_enc $got_enc$diff\n";
      }
    }
  }
  exit 0;

  # binary(10) == [1,0,1,0]
  sub n_is_enc_by_states_low_to_high {
    my ($n, $side) = @_;
    my %table = ( initial => ['1','initial'],
                  1 => [2,3],
                  2 => [2,2],
                  3 => [4,'initial'],
                  4 => [4,5],
                  5 => [6,3],
                  6 => [6,6],
                );
    my @bits = bit_split_lowtohigh($n);  # high to low
    my $state = 'initial';
    foreach my $bit (@bits) {
      $state = $table{$state}->[$bit] // die $state;
    }
    return $state eq 5 || $state eq 6;
  }

  # binary(20) == [1, 0, 1, 0, 0]
  sub n_is_enc_by_states {
    my ($n, $side) = @_;
    my %table = ( ls0    => ['ls0','ls1rev'],
                  ls1rev => ['ls12','ls0rev'],
                  ls0rev => ['ls0','ls0rev'],
                  ls12   => ['ls12','ls2rev'],
                  ls2rev => ['ls3','ls1rev'],
                  ls3    => ['ls12','ls3rev'],
                  ls3rev => ['ls3','ls3rev'],
                );
    my @bits = reverse bit_split_lowtohigh($n);  # high to low
    my $state = 'ls0';
    foreach my $bit (@bits) {
      $state = $table{$state}->[$bit] // die $state;
    }
    return $state eq 'ls3' || $state eq 'ls3rev';
  }

  sub n_is_enc_by_bits {
    my ($n, $side) = @_;
    my @bits = reverse bit_split_lowtohigh($n);  # high to low
    shift @bits || return 0;
    for (;;) {
      # state ls1rev
      my $bit = shift @bits // return 0;
      while (@bits && $bits[0] == $bit) { shift @bits; }  # run of $bit
      shift @bits;   # final opposite

      my $nextbit = shift @bits // return 0;
      if ($bit==0 && $nextbit==0) {
        # ls2rev and 0 to enc
        while (@bits) {  # must be all 1-bits
          if (! shift @bits) { return 0; }
        }
        return 1;
      }
      while ($bit==1 && $nextbit==0) {
        # skip 0-bits in ls0, and 1-bit going back to ls1rev
        $nextbit = shift @bits // return 0;
      }
      return 0;
    }
  }
}
{
  # triple 15ths count of each configuration occurrences
  #
  # At each even point in the curve consider segments in following directions,
  # and rotation by 0,1,2,3.
  # When rotation 0,2 direction along the curve is to leave vertically from O.
  # When rotation 1,3 the opposite.
  # Allow for curve direction when applying the x/15 offset
  #
  #     ^
  #     |
  #   L |
  #     |   M
  #     o ------>
  #     |        ^
  #   R |        | F
  #     |        |
  #     v   B    |
  #      <-------
  #
  # Odds(k) = if(k<1,0, 2^(k-1));
  # Odds(1) == 1
  # Odds(2) == 2
  # Dodd(k) = if(k<1,0, dD(k-1));
  # Dodd(4) == 1
  # gDodd(x) = x*gdD(x);
  # Deven(k) = D(k) - Dodd(k);
  # Deven(4) == 0
  # Deven(5) == 1
  # gDeven(x) = gD(x) - gDodd(x);
  # Ocount(k) = Dodd(k)*2 + Odds(k);
  # vector(10,k,k--; Ocount(k))
  #    0 1 2
  # v=[0,1,1,2,6,12,24,52,108,220,435]; \\ new O
  # vO=[0,1,2,4,10,22,46,98,206,426,878,1802]; \\ total O
  # vector(#vO,i,my(k=i-1); vO[i]-Odds(k))/2
  # vector(#vO,i,my(k=i-1); Dodd(k))
  # vE=[0,0,0,0,0,4,16,44,112,268,608,1340] \\ total E
  # vector(#vE,i,my(k=i-1); Deven(k))*4
  # vB=[0,0,0,1,2,5,16,43,106,253,580,1287]  \\ total B
  # vT=[0,0,1,2,5,12,27,62,141,312,679,1458] \\ total T
  # vector(min(#vT,#vB),i,my(k=i-1); if(i<2,0, vB[i-1]+vO[i-1]))
  # vtotal=[0,1,3,7,17,43,105,247,565,1259,2745,5887]
  # O(k) = 2^(k-1) + 2*Dodd(k)
  # E(k) = Deven(k)
  # B(k) = E(k-1) + T(k-1)
  # T(k) = O(k-1) + B(k-1)
  #      = O(k-1) + E(k-2) + T(k-2)
  # gOcount(x) = x*gTwoPow(x) + 2*gDodd(x);
  # gEcount(x) = 4*gDeven(x);
  # gT(x) = x*gO(x) + x^2*gE(x) + x^2*gT(x)
  # gTcount(x) = 1/(1-x^2) * (x*gOcount(x) + x^2*gEcount(x));
  # gf_terms(gTcount(x),#vT)
  # gBcount(x) = 1/(1-x^2) * (x^2*gOcount(x) + x*gEcount(x));
  # gf_terms(gBcount(x),#vB)
  # gtotal(x) = gEcount(x) + gBcount(x) + gTcount(x) + gOcount(x);
  # gf_terms(gtotal(x),#vtotal)
  # 4*2^n + 2*n + 3 + gf (-7 - 5*x - 8*x^2)/(1 - x - 2*x^3)

  my $n_hi = 2**16;
  # $n_hi = int(32768 / 15);
  my $path = Math::PlanePath::DragonCurve->new;

  my $xy_dxdy_to_ns = sub {
    my ($x,$y, $dx,$dy) = @_;
    foreach my $sign (1, -1) {
      ### consider: "xy=$x,$y dxdy=$dx,$dy  sign=$sign"
      my $n = $path->xyxy_to_n($x,$y, $x+$dx,$y+$dy);
      if (defined $n) {
        ### yes: ($n + ($sign<0)) . "  sign=$sign"
        return ($n + ($sign<0), $sign);
      }
      ($x,$y) = ($x+$dx, $y+$dy);
      ($dx,$dy) = (-$dx, -$dy);
    }
    return;
  };

  my @count;
  my %seen_union;
  my %seen_triple;
  my %seenxy;
  for (my $n = 0; $n <= $n_hi; $n+=2) {
    my ($x,$y) = $path->n_to_xy($n);
    ### at: "n=$n  $x,$y"
    next if $seenxy{"$x,$y"}++;

    foreach my $dir4 (0 .. 3) {
      my $dx = $dir4_to_dx[$dir4];
      my $dy = $dir4_to_dy[$dir4];

      my ($lx,$ly) = xy_rotate_plus90($dx,$dy);
      my ($rx,$ry) = xy_rotate_minus90($dx,$dy);
      ### dir: "$dir4 dxdy=$dx,$dy  left $lx,$ly right $rx,$ry"

      my $ox = $x + $dx + $rx;
      my $oy = $y + $dy + $ry;

      my @L = $xy_dxdy_to_ns->($x,$y, $lx,$ly);
      my @M = $xy_dxdy_to_ns->($x,$y, $dx,$dy);
      my @R = $xy_dxdy_to_ns->($x,$y, $rx,$ry);
      my @F = $xy_dxdy_to_ns->($ox,$oy, $lx,$ly);
      my @B = $xy_dxdy_to_ns->($ox,$oy, -$dx,-$dy);
      ### @L
      ### @M
      ### @R
      ### @F

      my $triple = sub {
        my ($type, @elems) = @_;
        my @values;
        my %n_list;
        foreach my $elem (@elems) {
          ### $elem
          @$elem == 3 or return;
          my ($offset, $n, $sign) = @$elem;
          push @values, 15*$n + $offset*$sign;
          if ($sign < 0) { $n--; }
          $n_list{$n} = 1;
        }
        @values = sort {$a<=>$b} @values;
        my $key = join(', ',@values);
        return if $seen_triple{$key};

        $seen_triple{$key} = \@values;
        foreach (@values) { $seen_union{$_} = 1; }

        my $max_value = max(@values);
        my $level = value15_to_level($max_value);
        $count[$level]->{$type}++;
        $count[$level]->{'total'}++;
        # my $type = join('-',map{$_->[0]} @_);

        if (scalar(keys %seen_triple) < 40) {
          my @n_list = keys %n_list;
          @n_list = sort {$a<=>$b} @n_list;
          my $n_list = join(',',@n_list);
          print "$key   n=${n}[$n_list] $x,$y dir=$dir4  type $type level=$level\n";
        }
      };

      $triple->('E', [1,@L], [3,@M], [11,@R]);
      $triple->('B', [2,@L], [6,@M], [8,@B]);
      $triple->('T', [4,@L], [12,@M], [14,@F]);
      $triple->('O', [7,@M], [9,@M], [13,@F]);

      ($dx,$dy) = xy_rotate_plus90($dx,$dy);
    }
  }

  require Tie::IxHash;
  tie my %values, 'Tie::IxHash';
  print "count by level\n";
  foreach my $k (0 .. $#count) {
    printf "k=%2d ", $k;
    foreach my $type ('E','B','T','O','total') {
      if ($k > 0) { $count[$k]->{$type} += $count[$k-1]->{$type} || 0; }
      my $count = $count[$k]->{$type} || 0;
      push @{$values{$type}}, $count;
      printf " %5d", $count;
    }
    print "\n";
  }

  require Math::OEIS::Grep;
  foreach my $key (keys %values) {
    my $aref = $values{$key};
    pop @$aref;
    Math::OEIS::Grep->search(array => $aref, verbose=>1, name=>$key);
  }

  my @triples = values %seen_triple;
  @triples = sort {$a->[0] <=> $b->[0]} @triples;

  require Math::OEIS::Stripped;
  my @A260748 = Math::OEIS::Stripped->anum_to_values('A260748');
  my @A260749 = Math::OEIS::Stripped->anum_to_values('A260749');
  my @A260750 = Math::OEIS::Stripped->anum_to_values('A260750');

  my @want_triples
    = map {[ $A260748[$_], $A260749[$_], $A260750[$_] ]}
    0 .. min($#A260748, $#A260749, $#A260750);
  print "stripped ",scalar(@want_triples),"  got ",scalar(@triples),"\n";

  my $bad = 0;
  foreach my $i (0 .. $#want_triples) {
    my $got = $triples[$i];
    my $want = $triples[$i];
    $got = join(', ',@$got);
    $want = join(', ',@$want);
    if ($got ne $want) {
      print "diff i=$i  got=$got  want=$want\n";
      $bad++;
    }
  }
  print "bad $bad\n";

  # 21 twice to 30720 which is 2048 ?
  # diff=21 at 52 to 73 = 3+7/15 to 4+13/15
  #                       4-8/15 to 5-2/15
  my @union = keys %seen_union;
  @union = grep {$_ < 15*$n_hi/2} @union;
  @union = sort {$a<=>$b} @union;
  my @diffs = map {$union[$_]-$union[$_-1]} 1 .. $#union;
  require Text::Wrap;
  print Text::Wrap::wrap('','', join(', ',@diffs)), "\n";
  exit 0;

  # $value is some 15*n+offset.  Return smallest k where n < 2^k.
  sub value15_to_level {
    my ($value) = @_;
    my $n = int($value / 15);
    my ($pow,$exp) = round_down_pow($n,2);
    return $exp+1;
  }
}


{
  # other n state transitions

  require MyFSM;
  my %table;
  my %accepting = (same => 1);
  my @pending = ([1,-1, 0,0, 0], [1,-1, 0,0, 1]);
  my $state_to_name = sub {
    my ($state) = @_;
    my ($old_s,$new_s, $delta_x,$delta_y, $bit) = @$state;
    # if ($old_s == $new_s && $delta_x == 0 && $delta_y == 0) {
    #   return "same";
    # }
    my $out = ($bit ^ ($delta_x+$delta_y)) & 1;
    my $flip = ($bit == $out ? "same" : "flip");
    return "$old_s,$new_s $delta_x,$delta_y $flip$bit$out";
    return "$old_s,$new_s $delta_x,$delta_y ${bit}out$out";
  };
  my $initial = $state_to_name->($pending[0]);
  my %seen;
  my %allow_delta = ('0,0' => 1,
                     '1,0' => 1,
                     '-1,0' => 1,
                     '0,1' => 1,
                     '0,-1' => 1,
                     '-1,1' => 1,
                     '1,-1' => 1,
                    );
  my $b = Math::Complex->new(1,1);
  while (@pending) {
    my $state = shift @pending;
    my ($old_s,$new_s, $delta_x,$delta_y, $bit) = @$state;
    my $from = $state_to_name->($state);
    next if $seen{$from}++;

    print "$from\n";
    if ($old_s == $new_s && $delta_x == 0 && $delta_y == 0) {
      print "  is accepting\n";
      $accepting{$from} = "same";
    }

    foreach my $bit_above (0,1) {
      ($old_s,$new_s, $delta_x,$delta_y) = @$state;
      my $delta = Math::Complex->new($delta_x,$delta_y);
      if ($bit && ! $bit_above) { $old_s = - $old_s; }

      my $z = $bit_above*$b + $bit + $delta;
      if (($z->Re % 2) == 1 && ($z->Im % 2) == 0) {
        $new_s = - $new_s;
      }
      my $new_bit = ($z->Re + $z->Im) % 2;
      $delta += $bit*$old_s - $new_bit*$new_s;
      $delta /= $b;
      $delta_x = $delta->Re;
      $delta_y = $delta->Im;

      my $new_state = [$old_s,$new_s, $delta_x,$delta_y, $bit_above];
      my $to = $state_to_name->($new_state);
      $table{$from}->{$bit_above} = $to;
      print "  $bit_above -> $to\n";
      if(! $allow_delta{"$delta_x,$delta_y"}) {
        print "bits $bit_above $bit  unexpected delta $delta_x,$delta_y\n";
        print "from  $from\n";
        print "to    $to\n";
        die;
      }
      push @pending, $new_state;
    }
  }

  my $fsm = MyFSM->new(table => \%table,
                       initial => $initial,
                       accepting => \%accepting,
                       title => 'N Other Transitions',
                       flow => 'west',
                      );

  # my @accepting = keys %accepting;
  # foreach my $i (1 .. $#accepting) {
  #   $fsm->merge_state($accepting[0],$accepting[1]);
  # }

  $fsm->simplify (verbose => 1);

  # Reversal fairly wild.
  # $fsm = $fsm->reverse;
  # $fsm->renumber_states;
  # $fsm->simplify (verbose => 1);
  # $fsm->renumber_states;

  $fsm->view;
  exit 0;
}

{
  # twindragon area tree, depths off diameter spine
  # distinct values 1,2,3,6,11,18,31,54,91 = JA
  # 2,4,8,16,28,48,84
  # distributed 2 2 2 4 8 12 20 36 60 100        A203175
  #
  # k=12 count of 1s = 100 = L(8)
  # 292+172 == 464
  # my(k=5000); print("diameter ",TAdiameter(k)); \
  #   sum(i=1,k,my(depth=JA(i),count=if(k-1-i<0,2,2*LQ(k-1-i))); \
  #       /* print("depth ",depth," count ",count); */ \
  #       depth*count) /TAdiameter(k)*1.0 / k
  # mean as sum r^(k-i)*r^i / r^k -> something*k
  # 0.191...

  require Graph::Maker::TwindragonAreaTree;
  require MyGraphs;
  foreach my $k (1 .. 16) {
    print "k=$k  ";
    my $graph = Graph::Maker->new('twindragon_area_tree', level=>$k,
                                  undirected=>1);
    my %seen;
    my $parent = 0;
    my $v = 1;
    my @depth_list = (0);
    my $diameter = 0;
    for (;;) {
      $diameter++;
      ### at: "parent=$parent  v=$v"
      my @children = MyGraphs::Graph_subtree_children($graph,$parent,$v)
        or last;
      ### @children
      if (@children >= 2) {
        my @depths = map {MyGraphs::Graph_subtree_depth($graph,$v,$_)}@children;
        if ($depths[0] < $depths[1]) {
          ($depths[0],$depths[1]) = ($depths[1],$depths[0]);
          ($children[0],$children[1]) = ($children[1],$children[0]);
        }
        print "$depths[1],";
        $seen{$depths[1]}++;
        push @depth_list, $depths[1];
      } if (@children == 1) {
        print "0,";
        push @depth_list, 0;
        $seen{0}++;
      }

      ### along diameter ...
      $parent = $v;
      $v = $children[0];
    }
    print "\n";

    my $num_depth_list = scalar(@depth_list);
    if ($diameter) {
      print "  diameter $diameter count $num_depth_list mean ",sum(@depth_list)/$diameter,"\n";
    }
    print "  unique ",join(',',sort {$a<=>$b} keys %seen),"\n";
    print "  distributed\n";
    foreach my $depth (sort {$a<=>$b} keys %seen) {
      print "    $depth  count $seen{$depth}\n";
    }
  }
  exit 0;

  sub Graph_tree_eccentricities_hash {
    my ($graph, $v) = @_;
    my %depths = ($v => 0);
    my @pending = ($v);
    my $depth = 1;
    do {
      @pending = map {$graph->neighbours($_)} @pending;
      @pending = grep {! exists $depths{$_}} @pending;
      foreach my $c (@pending) { $depths{$c} = $depth; }
      $depth++;
    } while (@pending);
    return $depth;
  }
}



{
  # blob convex hull
  # hull vertices 11,10,11,12  as k mod 4
  # 2*area = 2,9,23,57,125,267,559,1161,2357,4779,9663,19497,39125,78507,157439,315561,631637
  # 2*minrect area = 2,12,30,77,154,345,690,1457,2914,5985,11970,24257,48514,97665,195330
  # minrect even/odd = straight/diagonal
  # minrect even short side 2^k-1, long side 11,15,47,63,191,255
  #   255 = 11111111
  #   191 = 10111111
  #    63 =   111111
  #    47 =   101111
  #    15 =     1111
  #    11 =     1011
  # minrect odd short side 1/2*(2^k-1), 2*long side 47,95,191,383 = 3*2^n - 1
  #   255 = 11111111
  #   191 = 10111111
  #    63 =   111111
  #    47 =   101111
  #    15 =     1111
  #    11 =     1011

  require Math::Geometry::Planar;
  my $path = Math::PlanePath::DragonCurve->new;
  my @values;
  foreach my $k (4 .. 18) {
    print "$k\n";
    my $n_lo = BlobN($k) + 3;
    my $n_hi = BlobN($k+1);
    my $points = [];
    foreach my $n ($n_lo .. $n_hi) {
      my ($x,$y) = $path->n_to_xy($n);
      push @$points, [$x,$y];
    }
    my $planar = Math::Geometry::Planar->new;
    $planar->points($points);
    if (@$points > 4) {
      $planar = $planar->convexhull2;
      $points = $planar->points;
    }

    my ($x_hi,$y_hi) = $path->n_to_xy(2**$k);
    $points = hull_points_sort_by_angle ($points, $x_hi,$y_hi);

    my $length = scalar(@$points);
    my $area = $planar->area;

    my $diffs = [ map{ [$points->[$_]->[0] - $points->[$_-1]->[0],
                        $points->[$_]->[1] - $points->[$_-1]->[1] ] }
                  0 .. $#$points
                ];

    $planar = $planar->minrectangle;
    my $minrect = $planar->area;
    my $rect_points = $planar->points;
    $rect_points = hull_points_sort_by_angle ($rect_points, $x_hi,$y_hi);
    my $p1 = $rect_points->[0];
    my $p2 = $rect_points->[1];
    my $p3 = $rect_points->[2];
    my $mdx = $p2->[0] - $p1->[0];
    my $mdy = $p2->[1] - $p1->[1];
    if ($mdx < 0) {
      $mdx = - $mdx;
      $mdy = - $mdy;
    }
    my $modx = $p3->[0] - $p2->[0];
    my $mody = $p3->[1] - $p2->[1];
    if ($modx < 0) {
      $modx = - $modx;
      $mody = - $mody;
    }

    print "len=$length  ",points_str($points),"\n";
    print "   ",points_str($diffs),"\n";
    print "   area=$area  minrect area=$minrect direction $mdx,$mdy  other $modx,$mody\n";

    # push @values,2*$area;
    # push @values,2*$minrect;
    if ($k % 2 == 1) {
      push @values, 2*max(abs($mdy),abs($mody));
    }
  }

  print join(',',@values),"\n";
  shift @values;
  shift @values;
  shift @values;
  require Math::OEIS::Grep;
  Math::OEIS::Grep->search(array => \@values, verbose=>1);
  exit 0;
}

{
  # twindragon GraphViz2
  my $graphviz2 = twindragon_GraphViz2(4);
  $graphviz2->run(format => 'xlib',
                  driver => 'neato',
                 );
  exit 0;
}

{
  # double-visited other bits
  require Math::PlanePath::DragonCurve;
  my $path = Math::PlanePath::DragonCurve->new;
  for (my $n = 0; $n < 2000; $n++) {
    my ($x,$y) = $path->n_to_xy($n);
    my @n_list = $path->xy_to_n_list($x,$y);
    next unless $n_list[0] == $n;
    next unless @n_list == 2;
    next unless $n % 2;

    printf "* %4d %12s\n", $n_list[0], n_to_signs($n_list[0]);
    printf "  %4d %12s\n", $n_list[1], n_to_signs($n_list[1]);
    print "\n";

    # printf "* %4d %12b\n", $n_list[0], $n_list[0];
    # printf "  %4d %12b    sum %12b\n", $n_list[1], $n_list[1],  $n_list[0]+$n_list[1];
    # print "\n";
  }
  exit 0;

  sub n_to_signs {
    my ($n) = @_;
    my @bits = reverse bit_split_lowtohigh($n);
    my $prev = '0';
    my $s = '+';
    foreach my $i (0 .. $#bits) {
      my $bit = $bits[$i];
      if ($bit) { $bits[$i] = $s; }
      if ($prev == 0 && $bit == 1) {
        $s = ($s eq '+' ? '-' : '+');
      }
      $prev = $bit;
    }
    return join('',@bits);
  }
}

{
  # boundary squares 1,2,3 sides sequence
  #               1          1
  #              9----8    5---4
  #           1  |    |  3 |   | 2
  #             10--11,7---6   3---2
  #      1         3  |            | 1
  #   17---16   13---12        0---1
  # 1  |    | 3  |               1
  #   18-19,15-14,22-23
  #      2  |    |    |
  #        20--21,25-24
  #           2  |
  #             26---27
  #                   |
  #      --32   29---28
  #         |    |
  #        31---30

  # turn after
  # 1e L    1o L
  # 2eA R  2eB L
  # 3eA R  3eB R  3eC L
  #

  $| = 1;
  my $path = Math::PlanePath::DragonCurve->new;
  my $sides = 0;
  require Math::NumSeq::PlanePathTurn;
  my $seq = Math::NumSeq::PlanePathTurn->new(planepath_object => $path,
                                             turn_type => 'Left');
  my @values;
  for (my $n = 0; $n < 100; $n++) {
    next unless path_n_is_right_boundary($path, $n);
    if ($seq->ith($n+1) && path_n_is_right_boundary($path, $n+1)) {
      print "$sides,";
      if (@values < 200) { push @values, $sides; }
      # my $calc = ith_RQ_sides($#values);
      # $calc == $sides or die "calc $calc want $sides";
      $sides = 1;
    } else {
      $sides++;
    }
  }
  print "\n";
  require Math::OEIS::Grep;
  Math::OEIS::Grep->search(array => \@values, verbose => 0);

  my %expand = ('1e' => ['1e','1o'],
                '1o' => ['2e'],
                '2e' => ['1e','3o'],
                '3o' => ['3e','3o'],
                '3e' => ['2e']);
  my @array = ('1e');
  for (1 .. 7) {
    @array = map { @{$expand{$_} // die} } @array;
  }

  foreach (@array) { tr/eo/,,/ }
  print @array,"\n";

  exit 0;

  use Smart::Comments;
  sub ith_RQ_sides {      # wrong
    my ($i) = @_;
    ### ith_RQ_sides(): $i

    my $k = 0;
    until (RQ($k) > $i) { $k++; }

    my $state = 'RQ';
    my $opposite = 0;
    while ($k > 2) {
      ### at: "k=$k i=$i state=$state opposite=$opposite"
      if ($state eq 'RQ') {
        $k--;
        if ($i < RQ($k)) {
        } else {
          $i -= RQ($k);
          $state = 'LQrev';
        }

      } elsif ($state eq 'LQrev') {
        $k -= 2;
        my $cmp = RQ($k+1) + LQ($k+1);
        ### $cmp
        if ($i < $cmp) {
          $state = 'RQ';
        } else {
          $i -= $cmp;
          $state = 'RQrev';
          $opposite ^= 1;
        }

      } elsif ($state eq 'RQrev') {
        $k--;
        if ($i < LQ($k)) {
          $state = 'LQ';
        } else {
          $i -= LQ($k);
          $opposite ^= 1;
        }

      } elsif ($state eq 'LQ') {
        $k -= 2;
        if ($i < RQ($k)) {
          $state = 'RQrev';
        } else {
          $i -= RQ($k);
          $state = 'RQ';
        }

      } else {
        die;
      }
    }
    ### end: "k=$k i=$i state=$state opposite=$opposite"

    my %table = (RQ    => [ [1], [1,1], [1,1,2] ],
                 RQrev => [ [1], [1,1], [2,1,1] ],
                 LQ    => [ [1], [2], [3,1] ],
                 LQrev => [ [1], [2], [1,3] ]);
    my $aref = $table{$state}->[$k];
    if ($i > $#$aref) {
      die "oops $i > $#$aref";
    }
    my $sides = $table{$state}->[$k]->[$i];
    if ($opposite) { $sides = 4 - $sides; }
    return $sides;
  }
  no Smart::Comments;
}

{
  # 15ths triple numerators

  my @A260482 = (7,                                             # 1
                 13, 14,                                        # 2
                 26, 27, 28, 33, 37, 47, 52, 53, 54, 56, 57,    # 8

                 66, 67, 69, 71, 73, 74, 77, 87, 93, 94, 97, 103, 104, 106,
                 107, 108, 109, 111, 112, 113, 114, 123, 127,   # 23

                 132, 133, 134, 138, 139, 141, 142, 146, 147, 148, 149, 151,
                 153, 154, 157, 167, 173, 174, 177, 186, 187, 188, 189, 191,
                 193, 194, 197, 206, 207, 208, 209, 211, 212, 213, 214, 216,
                 217, 218, 219, 221, 222, 223, 224, 226, 227, 228);
  my $target = 2;
  my $count = 0;
  foreach my $a (@A260482) {
    while ($a >= $target) {
      print "$count\n";
      $count = 0;
      $target *= 2;
    }
    $count++;
  }

  exit 0;
}

{
  # convex hull vertex numbers FSM
  require MyFSM;
  my %table;
  $table{'all'}->{0} = '10to4';
  $table{'all'}->{1} = '2to8';

  $table{'10to4'}->{0} = '10to3';
  $table{'10to4'}->{1} = '6to8';
  $table{'2to8'}->{0} = '1to4';
  $table{'2to8'}->{1} = '2to4';

  $table{'10to3'}->{0} = '10to2';
  $table{'10to3'}->{1} = '6to8';
  $table{'6to8'}->{0} = 'non';
  $table{'6to8'}->{1} = '2to4';
  #
  $table{'2to4'}->{0} = '1to3';
  $table{'2to4'}->{1} = 'non';
  $table{'1to4'}->{0} = '10to3';
  $table{'1to4'}->{1} = '7to8';

  $table{'10to2'}->{0} = '10to1';
  $table{'10to2'}->{1} = '6to8';
  $table{'1to3'}->{0} = '10to2';
  $table{'1to3'}->{1} = '7to8';

  $table{'10to1'}->{0} = 'non';
  $table{'10to1'}->{1} = '6to7';
  $table{'1to2'}->{0} = '10to1';
  $table{'1to2'}->{1} = '7to8';
  $table{'7to8'}->{0} = 'non';
  $table{'7to8'}->{1} = '3to4';

  $table{'6to7'}->{0} = 'non';
  $table{'6to7'}->{1} = '2to3';
  $table{'2to3'}->{0} = '1to2';
  $table{'2to3'}->{1} = 'non';
  $table{'3to4'}->{0} = '2to3';
  $table{'3to4'}->{1} = 'non';

  my $fsm = MyFSM->new(table => \%table,
                       initial => 'all',
                       accepting => { non => 1 },
                       title => 'Hull Vertices');
  $fsm->simplify (verbose => 1);
  # $fsm->view;

  my @low_vertices = (undef,
                      '10001',  # P1 = 17
                      '01001',  # P2 = 9  extras
                      '00100',  # P3 = 4
                      '00010',  # P4 = 2
                      '00001',  # P5 = 1
                      '11100',  # P6 = 28
                      '11110',  # P7 = 30
                      '11111',  # P8 = 31
                      '11111',  # P9 = 31
                      '10010',  # P10 = 18
                     );
  my @low_extra_vertices = (undef,
                            [],  # 1
                            ['01000','00101'],  # P2to3 8,5
                            [],  # 3
                            [],  # 4
                            [],  # 5
                            [],  # 6
                            [],  # 7
                            [],  # 8
                            [],  # 9
                            [],  # 10
                           );
  #        9--8  5--4
  #        |  |  |  |
  #       10-11--6  3--2
  #           |        |
  # 17-16 13-12     0--1
  #  |  |  |
  # 18-19-22-23
  #     |  |  |
  #    20-25-24
  #        |
  #       26-27
  #           |
  #    32 29-28
  #     |  |
  #    31-30


  my $is_hull_vertex_by_fsm = sub {
    my ($k, $n) = @_;
    my @bits = bit_split_lowtohigh($n);
    while (@bits < $k) { push @bits, 0; }
    if (@bits > $k) { die "n too big"; }
    my $state = 'all';
    foreach my $i (reverse 5 .. $k-1) {
      if ($n == 0 || $n == (1 << $i)) { return 0; }
      my $bit = ($n & (1 << $i)) ? 1 : 0;  # $bits[$i];
      $n &= (1<<$i) - 1;
      $state = $table{$state}->{$bit} // die;
      if ($state eq 'non') { return 0; }
      if ($bit == 1) {
        $n = (1<<$i) - $n;
      }
    }
    # my $low = join('',reverse @bits[0..4]);
    my $low = sprintf '%05b', $n;
    if ($state eq 'all') { $state = '6to5'; }
    my ($from,$to) = ($state =~ /^(\d+)to(\d+)$/) or die "$state";
    ### $low
    ### $state
    ### $from
    ### $to
    ### low_vertices: $low_vertices[$from]
    while ($from != $to) {
      if ($low eq $low_vertices[$from]) { return 1; }
      foreach my $v (@{$low_extra_vertices[$from]}) {
        if ($low eq $v) { return 1; }
      }
      $from++; if ($from > 10) { $from = 1; }
    }
    if ($low eq $low_vertices[$to]) { return 1; }
    return 0;
  };

  my $HBV_by_fsm = sub {
    my ($k) = @_;
    my $HBV = 0;
    foreach my $n (0 .. 2**$k-1) {
      if ($is_hull_vertex_by_fsm->($k,$n)) { $HBV++; }
    }
    return $HBV;
  };

  {
    # counts
    foreach my $k (5 .. 24) {
      my $got = $HBV_by_fsm->($k);
      my $want = HBV($k);
      my $diff = ($got==$want ? '' : ' ***');
      print "$k  $got $want$diff\n";
    }
  }

  {
    # smallest N on hull boundary
    foreach my $k (5 .. 24) {
      foreach my $n (0 .. 2**$k-1) {
        if ($is_hull_vertex_by_fsm->($k,$n)) {
          my $P5N = P5N($k);
          printf "k=%d N=%d %0*b  cf P5=%d %0*b\n",
            $k, $n, $k,$n, $P5N, $k,$P5N;
          last;
        }
      }
    }
    exit 0;
  }
  {
    # P5N on hull boundary
    my $path = Math::PlanePath::DragonCurve->new;
    foreach my $k (# 10,
                   5 .. 60,
                  ) {
      my $P5N = P5N($k);
      my ($P5_x,$P5_y) = P5_xy($k);
      my ($x,$y) = $path->n_to_xy($P5N);
      $x==$P5_x && $y==$P5_y or die;
      my $got = $is_hull_vertex_by_fsm->($k,$P5N) ? 1 : 0;
      printf "k=%d  $P5_x,$P5_y  P5=%d %0*b  got %d\n",
        $k, $P5N, $k,$P5N, $got;
    }
    exit 0;
  }

  {
    # 2,4,9,17, [18,33], 34,36, [41], 60,62,63
    #  to_binary(P1N(6))  == 100010   \\ N=34       also
    #  to_binary(P2N(6))  == 010001   \\ N=17
    #  to_binary(P3N(6))  == 001001   \\ N=9
    #  to_binary(P4N(6))  == 000100   \\ N=4
    #  to_binary(P5N(6))  == 000010   \\ N=2
    #  to_binary(P6N(6))  == 110111   \\ N=55
    #  to_binary(P7N(6))  == 111100   \\ N=60
    #  to_binary(P8N(6))  == 111110   \\ N=62
    #  to_binary(P9N(6))  == 111111   \\ N=63
    #  to_binary(P10N(6)) == 100100   \\ N=36


    #        9--8  5--4
    #        |  |  |  |
    #       10-11--6  3--2
    #           |        |
    # 17-16 13-12     0--1
    #  |  |  |
    # 18-19-22-23
    #     |  |  |
    #    20-25-24
    #        |
    #       26-27 46-47
    #           |  |  |
    # 33-32 29-44-49-48
    #  |  |  |  |  |
    # 34-35-38-43-50-51 54-55
    #     |  |  |  |  |  |  |
    #    36-37 40-41 52-57-56
    #                    |
    #                   58-59
    #                       |
    #                64 61-60
    #                 |  |
    #                63-62

    print "k=6\n";
    my $HBV = 0;
    foreach my $n (0 .. 2**6-1
                  ) {
      if ($is_hull_vertex_by_fsm->(6,$n)) {
        print " N=$n\n";
        $HBV++;
      }
    }
    print " HBV=$HBV\n";
  }
  exit 0;
}

{
  # convex hull vertex numbers by FLAT::NFA
  require FLAT::NFA;
  my $flat = FLAT::NFA->new;

  my $add_chain = sub {
    my @tails = @_;
    my $p = $flat->add_states(1);
    foreach my $tail (@tails) {
      my $from = $p;
      foreach my $bit (@$tail) {
        my $to = $flat->add_states(1);
        $flat->set_transition($from, $to, $bit);
        $from = $to;
      }
      $flat->set_accepting($from);
    }
    return $p;
  };

  my %tails = (p1 => [ 1,0,0,0,1 ],
               p2 => [ 0,1,0,0,1 ],
               p3 => [ 0,0,1,0,0 ],
               p4 => [ 0,0,0,1,0 ],
               p5 => [ 0,0,0,0,1 ],
               p6 => [ 1,1,1,0,0 ],
               p7 => [ 1,1,1,1,0 ],
               p8 => [ 1,1,1,1,1 ],
               p9 => [ 1,1,1,1,1 ],
               p10 => [ 1,0,0,1,0],
              );
  my $p1 = $add_chain->($tails{'p1'}, $tails{'p2'});
  my $p2 = $add_chain->($tails{'p2'}, $tails{'p3'});
  my $p3 = $add_chain->($tails{'p3'}, $tails{'p4'});
  my $p4 = $add_chain->($tails{'p4'}, $tails{'p5'});
  # my $p5 = $add_chain->(0,0,0,0,1);
  my $p6 = $add_chain->($tails{'p6'}, $tails{'p7'});
  my $p7 = $add_chain->($tails{'p7'}, $tails{'p8'});
  my $p8 = $add_chain->($tails{'p8'}, $tails{'p9'});
  # my $p9 = $add_chain->(1,1,1,1,1);
  my $p10 = $add_chain->($tails{'p10'}, $tails{'p1'});

  $flat->set_starting($p1);

  $flat->set_transition($p8, $p4, 1);
  $flat->set_transition($p4, $p3, 0);

  $flat->set_transition($p1, $p7, 0);
  $flat->set_transition($p7, $p3, 1);
  $flat->set_transition($p3, $p2, 0);

  $flat->set_transition($p1, $p10, 1);
  $flat->set_transition($p10, $p6, 1);
  $flat->set_transition($p6, $p2, 1);

  $flat->set_transition($p2, $p1, 0);

  # $flat = $flat->reverse;
  # $flat = $flat->as_min_dfa;
  { my @pruned = $flat->prune;
    print scalar(@pruned)," pruned\n"; }

  # print $flat->as_summary;

  my $graphviz = $flat->as_graphviz;
  require IPC::Run;
  IPC::Run::run(['dot','-Txlib'], '<',\$graphviz);

  exit 0;
}

{
  # convex hull vertex numbers

  my $limit = 30;

  my $path = Math::PlanePath::DragonCurve->new;
  require Math::Geometry::Planar;
  my $points = [ [0,0], [1,0] ];
  my $ex = 1;
  my $ey = 0;
  foreach my $k (0 .. $limit) {
    # print "k=$k  ";
    foreach my $p ($points->[8]//[0,0]
                   # @$points
                  ) {
      my ($x,$y) = @$p;
      my $n = $path->xy_to_n($x,$y);
      print "$n, ";
    }
    print "\n";

    ($points,$ex,$ey) = hull_expand($points,$ex,$ey);
  }
  exit 0;
}

{
  # convex hull -- number of boundary points visited

  # HBP_formula(k) = 1/6 * vector_modulo([4, 5],k) * 2^floor(k/2) \
  #                + vector_modulo([5, 6, 7, 9],k) * 2^floor(k/4) \
  #                - 1/3 * vector_modulo([17, 19, 16, 17],k)
  # HBP(k) = if(k<4,HBP_samples[k+1], HBP_formula(k));
  # vector(20,k,k--;HBP(k))
  # HBP_samples = [2,3,5,6,7,9,14,19,25,31,44,57,77,95,136,173,245,303,448,565,837,1039,1584,1989,3045,3791,5904,7397,11557,14415,22736]
  # vector(#HBP_samples,k,k--; HBP(k)) == HBP_samples
  # vector(4,k,k--; HBP_formula(k))
  #
  # ratio 4/14 = 0.2857142857142857142857142857   even
  # ratio 5/19 = 0.2631578947368421052631578947   odd

  # my(k=1000); HBP(k)/HP(k)*1.0
  # my(k=1001); HBP(k)/HP(k)*1.0

  my $limit = 17;

  require Math::Geometry::Planar;
  my $points = [ [0,0], [1,0] ];
  my $ex = 1;
  my $ey = 0;
  my $path = Math::PlanePath::DragonCurve->new;
  my @values;
  foreach my $k (# 0 .. 2,
                 0 .. $limit,
                ) {
    my $count = 0;
    my @boundary_points = polygon_to_boundary_points($points);
    my $num_boundary_points = scalar(@boundary_points);
    ### @boundary_points
    foreach my $p (@boundary_points) {
      my ($x,$y) = @$p;
      my @n_list = $path->xy_to_n_list($x,$y);
      if (@n_list) {
        $count++;
      }
    }

    my $frac = $count / $num_boundary_points;
    print "$k  $count of $num_boundary_points   $frac\n";

    ($points,$ex,$ey) = hull_expand($points,$ex,$ey, unsorted=>1);
    push @values, $count;
  }
  print join(',',@values),"\n";

  require Math::OEIS::Grep;
  Math::OEIS::Grep->search(array => \@values, verbose=>1);
  exit 0;

  # $points is an arrayref [ [$x1,$y1], [x2,$y2], ... ] which is a polygon.
  # Return the number of integer point x,y points on its boundary.
  # The polygon sides should not cross, but they can be empty.
  sub polygon_to_boundary_points {
    my ($points) = @_;
    my @ret;
    foreach my $i (0 .. $#$points) {
      my $p1 = $points->[$i-1];
      my $p2 = $points->[$i];
      my ($x1,$y1) = @$p1;
      my ($x2,$y2) = @$p2;
      my $dx = $x2 - $x1;
      my $dy = $y2 - $y1;

      next unless $dx == 0 || $dy == 0 || abs($dx)==abs($dy);
      $dx = ($dx <=> 0) || 0;
      $dy = ($dy <=> 0) || 0;
      ### side: "$x1,$y1 to $x2,$y2 by $dx,$dy"

      # $x1 += $dx;
      # $y1 += $dy;

      while ($x1 != $x2 || $y1 != $y2) {
        push @ret, [$x1,$y1];
        $x1 += $dx;
        $y1 += $dy;
      }
    }
    return @ret;
  }
}



{
  # twindragon area tree count nodes at depth
  # start:  1,1,1,2,2,2,1,2,3,2,1,1,2,3,2,3,4,2,2,2,2,2,3,4,3,4,3,3,5,3,2,1,2,3,2,2,2,3,5,4,4,5,4,4,3,3,2,2,3,1
  # corner: 1,2,2,2,2,1,2,3,2,1,1,2,3,2,3,4,2,2,2,2,2,3,4,3,4,3,3,5
  #
  # height is corner to corner is 2*RQ(k-1)-1
  # 1,2,3,5,9,17,29,49,85,145,245,417,709,1201,2037
  # vector(14,k, 2*RQ(k-1)-1)

  require Graph::Maker::TwindragonAreaTree;
  require Graph::Maker::TwindragonAreaTreeByPath;
  my @heights;
  foreach my $k (0 .. 16) {
    my $graph = Graph::Maker->new('twindragon_area_tree', level=>$k,
                                  undirected=>1);
    my @counts = Graph_num_vertices_at_depths($graph,
                                         0             # start
                                         # 2**$k-2     # corner
                                        );
    push @heights, scalar(@counts);

    # {
    #   print "ones at ";
    #   foreach my $i (0 .. $#counts) {
    #     if ($counts[$i] == 1) { print "$i,"; }
    #   }
    #   print "\n";
    # }

     print join(',',@counts[0..min(90,$#counts)]),"\n";
  }

  require Math::OEIS::Grep;
  Math::OEIS::Grep->search(array => \@heights, verbose=>1);

  exit 0;

  # $graph is a Graph.pm, $root is the name of a vertex in it.
  # Return a list which is counts of how many vertices at depth from $root,
  # ($count0, $count1, $count2, ...)
  # $count0==1 always, being the single vertex $root.
  # If $graph is cyclic then the shortest path to each vertex is reckoned its
  # depth.
  sub Graph_num_vertices_at_depths {
    my ($graph, $root) = @_;
    my @counts;
    my %seen;
    my @pending = ($root);
    while (@pending) {
      push @counts, scalar(@pending);
      my @new_pending;
      foreach my $v (@pending) {
        $seen{$v} = 1;
        foreach my $child ($graph->neighbours($v)) {
          next if $seen{$child};
          push @new_pending, $child;
        }
      }
      @pending = @new_pending;
    }
    return @counts;
  }
}

{
  # TA num nodes at depth -- wrong

  my $level = 4;
  foreach my $depth (2) {
    my $count = count_depth_in_level($level, $depth);
    print "$depth  $count\n";
  }
  exit 0;

  sub count_depth_in_level {
    my ($k, $depth) = @_;
    ### count_depth_in_level(): "k=$k, depth=$depth"

    if ($depth < 0) {
      return 0;
    }
    if ($depth == 0) {
      ### node ...
      return 1;
    }
    if ($k <= 1) {
      return 0;
    }

    my $count = 0;
    my $lq = LQ($k-2);
    ### $lq
    ### forward ...
    $count += count_depth_in_level($k-1, $depth-$lq);
    ### backward ...
    $count += count_depth_in_level($k-1, $depth - $lq + 1);
    return $count;
  }
}

{
  # num representations using L(k) terms
  # 292,172,100,60,36,20,12,8,4,2,1
  # 1,2,4,6,10,

  my @LQ = map {LQ($_)} reverse 1 .. 10;
  print join(',',@LQ),"\n";

  foreach my $n (0 .. 15) {
    my $count = count_sums_using($n, \@LQ, 0);
    print "$n  $count\n";
  }
  exit 0;

  sub count_sums_using {
    my ($n, $aref, $astart) = @_;
    my $count = 0;
    foreach my $i ($astart .. $#$aref) {
      my $a = $aref->[$i];
      my $nrem = $n - $a;
      if ($nrem == 0) { $count++; last; }
      if ($nrem > 0) {
        $count += count_sums_using($nrem, $aref, 0);
      }
    }
    return $count;
  }
}

{
  # complex base convex hull
  # area 1,3,7,19,43,91,187,395,811,1643,3307,6699
  # Plus 1,2,5,11,25,49,101,205,421,837,1685,3381
  # diff [-1, -1, -3, -3, -7, -7, -15, -15, -31, -31, -63, -63]
  # Four flat sides of twindragon add 1/2
  #
  require Math::Geometry::Planar;
  require Math::PlanePath::ComplexPlus;
  my $path = Math::PlanePath::ComplexPlus->new;
  my @values;
  foreach my $k (0 .. 10) {
    print "$k\n";
    my ($n_lo, $n_hi) = $path->level_to_n_range($k);
    my ($x_hi,$y_hi) = $path->n_to_xy($n_hi);
    my $points = [];
    foreach my $n ($n_lo .. $n_hi) {
      push @$points, [$path->n_to_xy($n)];
    }
    my $area = 0;
    my $diffs = [];
    if (@$points >= 3) {
      my $planar = Math::Geometry::Planar->new;
      $planar->points($points);
      if (@$points >= 5) {
        $planar = $planar->convexhull2;
        $points = $planar->points;
        ### $points
      }
      $points = hull_points_sort_by_angle ($points, $x_hi,$y_hi);

      $area = $planar->area;
      $diffs = [ map{ [$points->[$_]->[0] - $points->[$_-1]->[0],
                       $points->[$_]->[1] - $points->[$_-1]->[1] ] }
                 0 .. $#$points
               ];
    }
    my $num_points = scalar(@$points);
    print "len=$num_points  ",points_str($points),"\n";
    print "   ",points_str($diffs),"\n";
    print "   area=$area\n";
    push @values,$area;
  }

  print "k=10 P ",points_str([[P1_xy(10)],[P2_xy(10)],[P3_xy(10)],[P4_xy(10)]]),"\n";
  print "k=11 P ",points_str([[P1_xy(11)],[P2_xy(11)],[P3_xy(11)],[P4_xy(11)]]),"\n";

  print join(',',@values),"\n";
  shift @values;
  shift @values;
  shift @values;
  require Math::OEIS::Grep;
  Math::OEIS::Grep->search(array => \@values, verbose=>1);
  exit 0;
}
{
  # twindragon graph diameter path tikz

  require Graph;
  my $path = Math::PlanePath::DragonCurve->new;
  foreach my $k (9,
                 0 .. 8) {
    my $rotate = -45*($k+1);
    print "  \\begin{scope}[rotate=$rotate,shift=(pos-$k)]\n";

    my ($n_lo, $n_hi) = $path->level_to_n_range($k+1);
    my ($x_hi,$y_hi) = $path->n_to_xy($n_hi);
    print "    \\draw[my gray] ";
    my $join = '';
    foreach my $n ($n_lo .. $n_hi) {
      my ($x,$y) = $path->n_to_xy($n);
      print "$join($x,$y)";
      $join = '--';
    }
    foreach my $n ($n_lo+1 .. $n_hi-1) {
      my ($x,$y) = $path->n_to_xy($n);
      $x = $x_hi - $x;
      $y = $y_hi - $y;
      print "$join($x,$y)";
      $join = '--';
    }
    print "${join}cycle;\n";

    foreach my $n ($n_lo, $n_hi) {
      my ($x,$y) = $path->n_to_xy($n);
      print "    \\node at ($x,$y) [my dot];\n";
    }

    my $graph = twindragon_Graph($k);
    my $apsp = $graph->APSP_Floyd_Warshall;
    my @longest_path = $apsp->longest_path;
    print "    \\draw[ultra thick] ";
    $join = '';
    foreach my $v (@longest_path) {
      my ($x,$y) = Graph_vertex_to_xy($v);
      print "$join($x,$y)";
      $join = '--';
    }
    print ";\n";

    print "  \\end{scope}\n";
  }
  exit 0;
}

{
  # blob graph diameter path for tikz

  require Graph;
  my $path = Math::PlanePath::DragonCurve->new;
  foreach my $k (11,
                 4 .. 10) {
    print "  \\begin{scope}[shift=(pos-$k)]\n";

    my $n_lo = BlobN($k);
    my $n_hi = BlobN($k+1) - 3;
    print "    \\draw[my gray] ";
    my $join = '';
    foreach my $n ($n_lo .. $n_hi) {
      my ($x,$y) = $path->n_to_xy($n);
      print "$join($x,$y)";
      $join = '--';
    }
    print ";\n";

    foreach my $n ($n_lo, $n_hi) {
      my ($x,$y) = $path->n_to_xy($n);
      print "    \\node at ($x,$y) [my dot];\n";
    }

    my $graph = blob_Graph($k);
    my $apsp = $graph->APSP_Floyd_Warshall;
    my @longest_path = $apsp->longest_path;
    print "    \\draw[ultra thick] ";
    $join = '';
    foreach my $v (@longest_path) {
      my ($x,$y) = Graph_vertex_to_xy($v);
      print "$join($x,$y)";
      $join = '--';
    }
    print ";\n";

    print "  \\end{scope}\n";
  }
  exit 0;
}

{
  # blob graph
  # diameter   2,5,8,12,18,26,36,52,70
  # start ecc  2,3,5, 8,12,17,23,34,46
  # end ecc    2,4,5, 9,12,22,27,43,54

  my $graph = blob_Graph(6);
  Graph_view($graph);
  print "num edges    ",scalar($graph->edges),"\n";
  print "num vertices ",scalar($graph->vertices),"\n";
  print "diameter     ",scalar($graph->diameter),"\n";
  my $path = Math::PlanePath::DragonCurve->new;

  my @values;
  my @start_eccentricity;
  my @end_eccentricity;
  foreach my $k (4 .. 12) {
    my $graph = blob_Graph($k);
    my $apsp = $graph->APSP_Floyd_Warshall;
    my $diameter      = $apsp->diameter;
    my $radius        = $apsp->diameter;
    my $shortest_path = $apsp->shortest_path;
    my $average_path  = $apsp->average_path_length;
    my $longest_path  = $apsp->longest_path;
    my @longest_path  = $apsp->longest_path;
    my @centre        = $apsp->center_vertices;

    my $n_lo = BlobN($k);
    my $n_hi = BlobN($k+1) - 3;
    my ($x_lo,$y_lo) = $path->n_to_xy($n_lo);
    my ($x_hi,$y_hi) = $path->n_to_xy($n_hi);
    my $start_eccentricity  = $apsp->vertex_eccentricity(Graph_xy_to_vertex($x_lo,$y_lo));
    my $end_eccentricity  = $apsp->vertex_eccentricity(Graph_xy_to_vertex($x_hi,$y_hi));

    print "k=$k  diameter=$diameter radius=$radius shortest=$shortest_path longest=$longest_path avg=$average_path ecc $start_eccentricity $end_eccentricity\n";
    print "  centre  ",join(' ',@centre),"\n";
    print "  longest ",join(' ',@longest_path),"\n";

    push @values, $diameter;
    push @start_eccentricity, $start_eccentricity;
    push @end_eccentricity, $end_eccentricity;
  }
  require Math::OEIS::Grep;
  Math::OEIS::Grep->search(array => \@values, verbose=>1, name=>'diameter');
  Math::OEIS::Grep->search(array => \@start_eccentricity, verbose=>1, name=>'start_eccentricity');
  Math::OEIS::Grep->search(array => \@end_eccentricity, verbose=>1, name=>'end_eccentricity');
  exit 0;

}

{
  # twindragon diameter pictures, diameter by Graph.pm
  # 2,4,6,10,14,22,30,42,58
  #
  # 0,1,2,3,6,
  # detour(h) = if(h==0,0, h==1,1, 2*detour(h-1) - if(h%2,1,0));
  # vector(8,h,detour(h))
  # diam_even(h) = 2^h + sum(j=0,h,2*detour(j)) + if(h>=3,2,0);
  # vector(10,h,diam_even(h))
  # diam(k) = if(k%2==0, diam_even(k/2), 0);
  # vector(8,k,diam(k))
  #
  # 0,2,4,8,10,12
  #   0 2 2  2  2
  #
  # diam_even_powers(h) = 7/3*2^h + h + vector_modulo([2/3,1/3],h);
  # vector(10,h,diam_even_powers(h))

  # periodic terms
  # + (14/3)* vector_modulo([1, 1, 1, 1],k) * 2^k
  # + (1/2)* vector_modulo([1, 1, 1, 1],k)
  # + (1/6)* vector_modulo([-1, 1, -1, 1],k)


  foreach my $k (9) {
    my $graph = twindragon_Graph($k);
    my $apsp = $graph->APSP_Floyd_Warshall;
    my $diameter = $apsp->diameter;

    print "k=$k  diameter=$diameter\n";

    require Image::Base::GD;
    my $size = 600;
    my $scale = 10;
    my $image = Image::Base::GD->new (-height => $size,
                                      -width  => $size);
    $image->rectangle(0,0, $size-1,$size-1, 'black');
    my $transform = sub {
      my ($x,$y) = @_;
      $x *= $scale;
      $y *= $scale;
      $x += 200;
      $y = 300-$y;
      return ($x,$y);
    };
    foreach my $edge ($graph->edges) {
      my ($v1,$v2) = @$edge;
      my ($x1,$y1) = Graph_vertex_to_xy($v1);
      my ($x2,$y2) = Graph_vertex_to_xy($v2);
      $image->line($transform->($x1,$y1),
                   $transform->($x2,$y2),
                   'white');
    }
    my $path = Math::PlanePath::DragonCurve->new;
    my ($n_lo, $n_hi) = $path->level_to_n_range($k+1);
    my ($x_hi,$y_hi) = $path->n_to_xy($n_hi);
    foreach my $xy ([0,0], [$x_hi,$y_hi]) {
      my ($x,$y) = $transform->(@$xy);
      $image->rectangle($x-2,$y-2, $x+2,$y+2, 'green', 1);
    }
    my @diameter = $apsp->diameter;
    foreach my $i (0 .. $#diameter-1) {
      my $v1 = $diameter[$i];
      my $v2 = $diameter[$i+1];
      my ($x1,$y1) = Graph_vertex_to_xy($v1);
      my ($x2,$y2) = Graph_vertex_to_xy($v2);
      $image->line($transform->($x1,$y1),
                   $transform->($x2,$y2),
                   'red');
    }
    my $filename = "/tmp/tdiam-$k.png";
    $image->save($filename);
  }
  require IPC::Run;
  push @ipc, IPC::Run::start(['xzgv',glob('/tmp/tdiam*.png')],'&');
  exit 0;
}

{
  # blob Hamiltonian path
  my $path = Math::PlanePath::DragonCurve->new;
  foreach my $k (5) {
    my $blob_lo = BlobN($k);
    my $blob_hi = BlobN($k+1) - 3;

    my $xy_to_v_upto = 0;
    my %xy_to_v;
    my @to;
    my $prev;
    foreach my $n ($blob_lo .. $blob_hi) {
      my ($x,$y) = $path->n_to_xy($n);
      my $v = ($xy_to_v{"$x,$y"} //= $xy_to_v_upto++);
      ### at: "$x,$y is $v"
      if (defined $prev) {
        push @{$to[$prev]}, $v;
        push @{$to[$v]}, $prev;
      }
      $prev = $v;
    }
    ### @to
    my $num_vertices = scalar(@to);
    my $hamiltonian = v_is_hamiltonian(\@to);
    print "$k N=$blob_lo to $blob_hi is $num_vertices  hamiltonian=$hamiltonian\n";
  }
  exit 0;

  sub v_is_hamiltonian {
    my ($aref) = @_;
    # my @vertices = keys %$href;
    # foreach my $start (@vertices) {
    #   my %seen = ($start => 1);
    #   my @seen = ($start);
    #   my @upto = (undef, -1);
    #   my $pos = 0;
    #   my $v = $start;
    #   for (;;) {
    #     # go forward
    #     my $next = $href->{$v}->[++$upto[$pos]];
    #     if (! defined $next) {
    #       # no more forwards
    #       $seen{$v} = 0;
    #       $pos--;
    #       if ($pos < 0) { last; }
    #       next;
    #     }
    #     if ($seen{$next}) {
    #       next;
    #     }
    #
    #     $v = $next;
    #     $seen{$v} = 1;
    #     $pos++;
    #     if ($pos > $#vertices) {
    #       return 1;
    #     }
    #   }
    # }
    return 0;
  }
}

{
  # enclosed squares consecutive sides -- types

  my $path = Math::PlanePath::DragonCurve->new;
  my @types;
  foreach my $i (0,1) {
    foreach my $j (0,1) {
      foreach my $k (0,1) {
        my $type = "$i,$j,$k";
        push @types, $type;
        printf "%6s", $type;
      }
    }
  }
  print "\n";
  my %values;

  foreach my $k (0 .. 14) {
    my ($n_lo, $n_hi) = $path->level_to_n_range($k);
    my %count = map {$_=>0} @types;
    my %seen;
    foreach my $n ($n_lo .. $n_hi-1) {
      my ($x,$y) = $path->n_to_xy($n);
      next if $seen{"$x,$y"}++;

      # horizontals
      my $n1 = $path->xyxy_to_n_either($x,$y,   $x+1,$y) // next;
      my $n3 = $path->xyxy_to_n_either($x,$y+1, $x+1,$y+1) // next;

      # verticals
      my $n2 = $path->xyxy_to_n_either($x,  $y,   $x,$y+1) // next;
      my $n4 = $path->xyxy_to_n_either($x+1,$y, $x+1,$y+1) // next;

      my @n_list = ($n1, $n2, $n3, $n4);
      next if grep {$_>=$n_hi} @n_list;

      @n_list = sort {$a<=>$b} @n_list;

      my @diff = map {$n_list[$_]-$n_list[$_-1]} 1 .. $#n_list;
      @diff = map {$_==1 ? 1 : 0} @diff;
      my $type = join(',', @diff);

      $count{$type}++;
    }
    sum(values %count) == A($k) or die;
    foreach my $type (@types) {
      push @{$values{$type}}, $count{$type};
    }
    foreach my $type (@types) {
      printf "%6d", $count{$type} || 0;
    }
    print "\n";
  }
  require Math::OEIS::Grep;
  foreach my $type (@types) {
    Math::OEIS::Grep->search(array => $values{$type}, verbose => 0,
                             name => "consecutive=$type");
  }
  exit 0;
}
{
  # enclosed squares consecutive sides
  # consecutive=1  A(k-2)
  # consecutive=2
  # consecutive=3  2^(k-3)-1
  #
  my $path = Math::PlanePath::DragonCurve->new;
  my @values;
  foreach my $k (0 .. 12) {
    my ($n_lo, $n_hi) = $path->level_to_n_range($k);
    my @count = (0,0,0,0);
    my %seen;
    foreach my $n ($n_lo .. $n_hi-1) {
      my ($x,$y) = $path->n_to_xy($n);
      next if $seen{"$x,$y"}++;

      # horizontals
      my $n1 = $path->xyxy_to_n_either($x,$y,   $x+1,$y) // next;
      my $n3 = $path->xyxy_to_n_either($x,$y+1, $x+1,$y+1) // next;

      # verticals
      my $n2 = $path->xyxy_to_n_either($x,  $y,   $x,$y+1) // next;
      my $n4 = $path->xyxy_to_n_either($x+1,$y, $x+1,$y+1) // next;

      my @n_list = ($n1, $n2, $n3, $n4);
      next if grep {$_>=$n_hi} @n_list;

      @n_list = sort {$a<=>$b} @n_list;

      my @diff = map {$n_list[$_]-$n_list[$_-1]} 1 .. $#n_list;
      my $consecutive = grep {$_==1} @diff;

      $count[$consecutive]++;
    }
    $count[4] = sum(@count);
    foreach my $i (0 .. $#count) {
      push @{$values[$i]}, $count[$i];
    }
    foreach my $i (0 .. $#count) {
      printf "%6d", $count[$i] || 0;
    }
    print "\n";
  }
  require Math::OEIS::Grep;
  foreach my $i (0 .. $#values) {
    Math::OEIS::Grep->search(array => $values[$i], verbose => 0, name => "consecutive=$i");
  }
  exit 0;
}

{
  # enclosed squares consecutive sides
  # 1, 4, 11, 28, 67, 152, 335, 724 area
  # v = [1, 3, 7, 17, 43, 105, 247, 565]; \\ ?
  # w = vector(7,i, v[i+1]-2*v[i]);
  # p = vector(6,i, w[i+1]-2*w[i]);
  # gp(x) = Polrev([-1, 1, 3, 1, -1, -3])*1/(1-x^6);
  # Polrev(w)
  # gf_terms(gp(x),10)
  # gw(x) - 2*x*gw(x) = 1 + gp(x)*x
  # gw(x) = 1/(1-2*x) * (gp(x)*x + 1)
  # gf_terms(gw(x),10)
  # gv(x) = 1/(1-2*x) * (gw(x)*x + 1)
  # gf_terms(gv(x),10)
  # V(k) = if(k<5,v[k+1], 4*V(k-1) - 4*V(k-2) - V(k-3) + 4*V(k-4) - 4*V(k-5));
  # vector(10,k,k--; V(k))
  # 4*247 - 4*105 - 43 + 4*17 - 4*7

  my $path = Math::PlanePath::DragonCurve->new;
  require Math::NumSeq::PlanePathTurn;
  my $seq = Math::NumSeq::PlanePathTurn->new(planepath_object=>$path,
                                             turn_type => 'Left');
  my @values;
  foreach my $k (0 .. 13) {
    my ($n_lo, $n_hi) = $path->level_to_n_range($k);
    my $run = 0;
    my $prev = -1;
    my $count = 0;
    foreach my $n ($n_lo+1 .. $n_hi-1) {
      my $value = $seq->ith($n);
      if ($value == $prev) {
        $run++;
      } else {
        $run = 1;
      }
      if ($run == 3) {
        $count++;
      }
      $prev = $value;
    }
    push @values, $count;
  }
  require Math::OEIS::Grep;
  Math::OEIS::Grep->search(array => \@values, verbose => 1);
  exit 0;
}


{
  # points on right boundary
  # P_samples  = [2,3,5,9, 16,29, 54,101,190,361,690];
  # PR_samples = [2,3,5,9, 16,27, 45, 76,129,218]
  #              [0,0,0,0,  0, 2,  9];
  # PR(k) = R(k)+1 - HQAR(k);
  # vector(11,k,k--; PR(k))
  # vector(11,k,k--; P(k)-PR(k)) == [0,0,0,0, 1,1, 7,23,59,141]
  # pol_ascending_print(pol_partial_fractions(gR(x) + 1/(1-x) - gHQAR(x)))

  # PL(k) = L(k)+1 - HQAL(k);
  # vector(11,k,k--; PL(k))
  # P_samples  = [2,3,5,9, 16,29, 54,101,190,361,690];
  # PL_samples = [2,3,5,9, 13,21, 35, 57, 93,155,261];
  #              [0,0,0,0,  3, 8, 19];
  # pol_ascending_print(pol_partial_fractions(gL(x) + 1/(1-x) - gHQAL(x)))

  # PB_formula(k) = PL(k) + PR(k) - 4*2*k + 22;
  # PB(k) = if(k<6,P(k), PB_formula(k));
  # vector(14,k,k--; PB(k))

  my @values;
  foreach my $k (0 .. 10) {
    my $path = Math::PlanePath::DragonCurve->new;
    my ($n_lo, $n_hi) = $path->level_to_n_range($k);
    my $count = 0;
    my %seen;
    foreach my $n ($n_lo .. $n_hi) {
      my ($x,$y) = $path->n_to_xy($n);
      next if $seen{"$x,$y"}++;

      my ($dx,$dy) = (1,0);
      my $boundary = 0;
      foreach my $i (1 .. 4) {
        my $n = $path->xyxy_to_n_either($x,$y, $x+$dx,$y+$dy);
        if (defined $n
            && $n < $n_hi
            && path_n_is_right_boundary($path, $n, $n_hi)) {
          $boundary = $i;
          last;
        }
        ($dx,$dy) = xy_rotate_plus90($dx,$dy);  # left
      }
      ### $n
      ### $boundary
      if ($boundary) {
        $count++;
      } else {
      }
    }
    print "k=$k  $count\n";
    push @values, $count;
  }
  require Math::OEIS::Grep;
  Math::OEIS::Grep->search(array => \@values, verbose => 1);
  exit 0;
}
{
  # double visited points on the boundary
  # DB_samples = [ 0,0,0,0,1,4,11,26,57,114,217,398,709,1242,2153 ]
  #
  # B(k) boundary points, hanging counted twice
  # - S(k) singles
  # = 1/2*B(k)-1 doubles
  # blob start,end counted twice
  # vector(15,k,k--; B(k)-S(k) - (2*k-7) - HQ(k) - HQ2(k) - HQ4(k))

  # BQ1(k) = sum(j=0,k,LQ1(j));
  # BQ2(k) = sum(j=0,k,LQ2(j));
  # BQ3(k) = sum(j=0,k,LQ3(j));
  # vector(#DB_samples,k,k--; LQ2(k))
  # vector(#DB_samples,k,k--; BQ2(k))
  # vector(#DB_samples,k,k--; BQ3(k))
  # vector(#DB_samples,k,k--; BQ2(k) + 2*BQ3(k) - HQA(k) - 8*k+24   - DB_samples[k+1])
  # PB_formula(k) = B(k) - HQA(k) - 8*k+24;
  # PB(k) = if(k<4,2^k+1, k==4,16, k==5,29, PB_formula(k));
  # DB(k) = PB(k) - S(k);
  # vector(#DB_samples,k,k--; DB(k) - DB_samples[k+1])

  my @values;
  foreach my $k (0..14) {
    my $path = Math::PlanePath::DragonCurve->new;
    my ($n_lo, $n_hi) = $path->level_to_n_range($k);
    my $count = 0;
    my %seen;
    foreach my $n ($n_lo .. $n_hi-1) {
      my ($x,$y) = $path->n_to_xy($n);
      next if $seen{"$x,$y"}++;
      my @n_list = $path->xy_to_n_list($x,$y);
      @n_list = grep {$_<=$n_hi} @n_list;
      next unless @n_list == 2;
      ### $n

      my ($dx,$dy) = (1,0);
      my $boundary = 0;
      foreach (1 .. 4) {
        if (path_xyxy_is_left_boundary($path, $x,$y, $x+$dx,$y+$dy, $n_hi)
            || path_xyxy_is_right_boundary($path, $x,$y, $x+$dx,$y+$dy, $n_hi)) {
          $boundary = 1;
          last;
        }
        ($dx,$dy) = xy_rotate_plus90($dx,$dy);  # left
      }
      if ($boundary) {
        $count++;
      }
    }
    print "k=$k  $count\n";
    push @values, $count;
  }
  require Math::OEIS::Grep;
  Math::OEIS::Grep->search(array => \@values, verbose => 1);
  exit 0;
}

{
  # twindragon segment midpoints -- inertia by points
  # 1, 1.50004768535297, 0.500009537070593

  # minimum
  # 8993.71675286547, 23646.2832471345, -126.260772831788    (0, 0)  at 31

  my $k = 7;

  my $path = Math::PlanePath::DragonCurve->new;
  my ($n_lo, $n_hi) = $path->level_to_n_range($k+1);
  my ($x_hi,$y_hi) = $path->n_to_xy($n_hi);
  my $gx = $x_hi/2;
  my $gy = $y_hi/2;
  print "centroid  $gx, $gy\n";

  my $scale = 2**$k;
  $scale *= 13107;
  $scale = 1;
  foreach my $angle (0,

                     map {deg2rad($_)} -10 .. 60
                     # deg2rad(20),
                     # deg2rad(22.5),
                     # deg2rad(25),
                     # -1.69328565,
                     # -0.1224893,
                     # 0.1224893,
                     # 0.239999,      # 13 degrees
                    ) {
    my $Ix = 0;
    my $Iy = 0;
    my $Ixy = 0;
    my $sum_x = 0;
    my $sum_y = 0;
    foreach my $n ($n_lo .. $n_hi-1) {
      my ($x1,$y1) = $path->n_to_xy($n);
      my ($x2,$y2) = $path->n_to_xy($n+1);
      foreach (1, 2) {
        my $x = ($x1+$x2)/2 - $gx;
        my $y = ($y1+$y2)/2 - $gy;
        ($x,$y) = xy_rotate_degrees($x,$y, -$angle);

        $Ix += $y*$y;
        $Iy += $x*$x;
        $Ixy += -2*$x*$y;
        $sum_x += $x;
        $sum_y += $y;

        $x1 = $x_hi - $x1;
        $y1 = $y_hi - $y1;
        $x2 = $x_hi - $x2;
        $y2 = $y_hi - $y2;
      }
    }
    $Ix /= $scale;
    $Iy /= $scale;
    $Ixy /= $scale;
    my $angle_degrees = rad2deg($angle);
    print "$Ix, $Iy, $Ixy    ($sum_x, $sum_y)  at $angle_degrees\n";
  }
  exit 0;

  sub xy_rotate_degrees {
    my ($x,$y, $angle) = @_;
    # {
    #   my $z = Math::Complex->make($x,$y);
    #   $z *= Math::Complex->emake(1, $angle);
    #   return ($z->Re, $z->Im);
    # }
    {
      return ($x*cos($angle) - $y*sin($angle),
              $x*sin($angle) + $y*cos($angle));
    }
  }
  CHECK {
    my ($x,$y) = xy_rotate_degrees(2,1, deg2rad(90));
    nearly_equal($x, -1) or die;
    nearly_equal($y, 2) or die;
    ($x,$y) = xy_rotate_degrees(1,1, -deg2rad(45));
    nearly_equal($x, sqrt(2)) or die;
    nearly_equal($y, 0) or die;
  }
  sub nearly_equal {
    my ($x, $y) = @_;
    return abs($x - $y) < 1e-6;
  }
}

{
  # twindragon convex hull inertia by points

  # k=7
  # -5,0 -5,-5 0,-10 11,-10 21,0 21,5 16,10 5,10
  # centroid 8  0
  # Ix = 18608
  # Iy = 12440
  # Ixy = 4360

  # k=15
  # -85,0 -85,-85 0,-170 171,-170 341,0 341,85 256,170 85,170
  # Ix = 1 198 748 328
  # Iy =   838 032 680
  # Ixy =  303 649 920

  my $k = 7;
  my $scale = 19456844;
  my $resolution = 200;

  my $path = Math::PlanePath::DragonCurve->new;
  my ($n_lo, $n_hi) = $path->level_to_n_range($k+1);
  my ($x_hi,$y_hi) = $path->n_to_xy($n_hi);

  my $P1 = complex_to_point(P1_formula($k+1));
  my $P2 = complex_to_point(P2_formula($k+1));
  my $P3 = complex_to_point(P3_formula($k+1));
  my $P4 = complex_to_point(P4_formula($k+1));
  my $P1dash = [ $x_hi - $P1->[0], $y_hi - $P1->[1] ];
  my $P2dash = [ $x_hi - $P2->[0], $y_hi - $P2->[1] ];
  my $P3dash = [ $x_hi - $P3->[0], $y_hi - $P3->[1] ];
  my $P4dash = [ $x_hi - $P4->[0], $y_hi - $P4->[1] ];

  my $points = [ $P4, $P3, $P2, $P1,
                 $P4dash, $P3dash, $P2dash, $P1dash ];
  $points = [ [-5/6, 0],
              [-5/6, -1/3],
              [-1/2, -2/3],
              [1/6, -2/3],
              [5/6, 0],
              [5/6, 1/3],
              [1/2, 2/3],
              [-1/6, 2/3],
            ];
  foreach my $p (@$points) {
    $p->[0] *= $resolution;
    $p->[1] *= $resolution;
  }
  print points_str($points),"\n";

  require Math::Geometry::Planar;
  my $planar = Math::Geometry::Planar->new;
  $planar->points($points);
  my $G = $planar->centroid;
  foreach my $p (@$points) {
    $p->[0] -= $G->[0];
    $p->[1] -= $G->[1];
  }
  print "centroid $G->[0]  $G->[1]\n";

  $planar = Math::Geometry::Planar->new;
  $planar->points($points);

  require MyPlanar;
  my ($xmin,$ymin, $xmax,$ymax) = $planar->MyPlanar::bbox_xyxy;
  my $Ix = 0;
  my $Iy = 0;
  my $Ixy = 0;
  foreach my $x ($xmin .. $xmax) {
    foreach my $y ($ymin .. $ymax) {
      if ($planar->isinside([$x,$y])) {
        $Ix += $y*$y;
        $Iy += $x*$x;
        $Ixy += $x*$y;
      }
    }
  }
  $Ix /= $scale;
  $Iy /= $scale;
  $Ixy /= $scale;
  print "$Ix, $Iy, $Ixy\n";
  exit 0;

  sub complex_to_point {
    my ($z) = @_;
    return [ round_maybe($z->Re), round_maybe($z->Im) ];
  }
  sub round_maybe {
    my ($x) = @_;
    my $int = int($x+($x>=0 ? .5 : -.5));
    if (abs($x - $int) < 1e-10) { return $int; }
    return $x;
  }
}
{
  # twindragon area tree geometric-arithmetic mean TAGA

  require Math::BigRat;
  require Graph::Maker::TwindragonAreaTree;
  my @values;
  foreach my $k (1..9) {
    my $graph = Graph::Maker->new('twindragon_area_tree', level=>$k,
                                  undirected=>1);
    my %degrees = map {$_ => $graph->vertex_degree($_)} $graph->vertices;
    my $total;
    my $num_edges = 0;
    my %count_types;
    my @total_by_sqrt = (0) x 10;
    foreach my $edge ($graph->edges) {
      my $d1 = $degrees{$edge->[0]};
      my $d2 = $degrees{$edge->[1]};
      my $num = $d1 * $d2;
      my $den = $d1 + $d2;
      $total += 2* sqrt($num) / $den;
      $num_edges++;
      if ($d1 > $d2) { ($d1,$d2) = ($d2,$d1) }
      # $count_types{"$d1,$d2"}++;
      $count_types{"$num/$den"}++;
      $total_by_sqrt[$num] += Math::BigRat->new("2/$den");
    }
    my $f = $total / $num_edges;
    print "$k $total  $f\n";
    foreach my $num (1 .. $#total_by_sqrt) {
      if ($total_by_sqrt[$num]) {
        print "  $num $total_by_sqrt[$num]\n";
      }
    }
    # foreach my $type (sort keys %count_types) {
    #   print "  $type $count_types{$type}\n";
    # }
  }
  exit 0;
}
{
  # complex i+1 blocks on boundary -- starting from single
  #
  #
  #  k=2      <------<------
  #          |c  d          ^    n=0b 10  ad
  #          |b 10 e   11   |
  #          v   a f        |
  #    <------      +------>
  #   |             ^
  #   |  00    01   |
  #   v             |
  #    ------>----->
  #
  #  c  |  d  | h              d   | d   |  e  |  e  |
  #     |     |                          |           |
  # ----+-----+----      ----------+-----+-----+-----+
  #     |     |                    |           |
  #  b  |  0  | e        c     c   |  0  |  1  |  f  |  f
  #     |     |                    |           |
  # ----+-----+----       ---------+-----+-----+-----
  #     |     |                          |
  #  g  |  a  | f              b   |  b  |  a  |  a
  #

  my $a = 1 << 0;
  my $b = 1 << 1;
  my $c = 1 << 2;
  my $d = 1 << 3;
  my $e = 1 << 4;
  my $f = 1 << 5;
  my $g = 1 << 6;
  my $h = 1 << 7;

  my $all = $a|$b|$c|$d|$e|$f|$g|$h;
  my $initial = 0;

  my $expand = sub {
    my ($s) = @_;
    return ((  ($s & $b) && $a)
            | (($s & $c) && $b)
            | (($s & $d) && $c)
            | (($s & $d) && $d)
            | ((1      ) && $e)
            | (($s & $a) && $f)
            | (($s & $b) && $g)
            | (($s & $e) && $h),

            (  ($s & $a) && $a)
            | ((1      ) && $b)
            | (($s & $d) && $c)
            | (($s & $e) && $d)
            | (($s & $f) && $e)
            | (($s & $a) && $f)
            | (($s & $b) && $g)
            | (($s & $e) && $h));
  };

  my $n_to_state_by_high_to_low = sub {
    my ($n, $k) = @_;
    my $s = $initial;
    my @bits = bit_split_lowtohigh($n);
    for (my $i = $k-1; $i >= 0; $i--) {
      my $bit = $bits[$i];   # high to low
      my @e = $expand->($s);
      $s = $e[$bit||0];
    }
    return $s;
  };
  require Math::PlanePath::ComplexPlus;
  my $path = Math::PlanePath::ComplexPlus->new;
  my @path_offsets = ([ 0,-1, $a],
                      [-1, 0, $b],
                      [-1, 1, $c],
                      [ 0, 1, $d],
                      [ 1, 0, $e],
                      [ 1,-1, $f],
                      [-1,-1, $g],
                      [ 1, 1, $h]);
  my $n_to_state_by_path = sub {
    my ($n, $k) = @_;
    my ($n_lo, $n_hi) = $path->level_to_n_range($k);
    my ($x,$y) = $path->n_to_xy($n);
    my $s = 0;
    foreach my $elem (@path_offsets) {
      my ($dx,$dy, $s_bit) = @$elem;
      my $n_neighbour = $path->xy_to_n($x+$dx,$y+$dy);
      if (defined $n_neighbour && $n_neighbour <= $n_hi) {
        $s |= $s_bit;
      }
    }
    return $s;
  };
  foreach my $k (0 .. 10) {
    my ($n_lo, $n_hi) = $path->level_to_n_range($k);
    foreach my $n ($n_lo .. $n_hi) {
      my $s_expand = $n_to_state_by_high_to_low->($n, $k);
      my $s_path   = $n_to_state_by_path->($n, $k);
      # printf "k=%d n=%*b  exp=%6b  path=%6b\n", $k, $k, $n, $s_expand, $s_path;
      if ($s_path != $s_expand) {
        die;
      }
    }
  }

  {
    # a
    #                 0  -> enclosed
    # x 1 1,0s x 0 0,1s  -> enclosed
    #
    my $a_enclosed_by_low_to_high = sub {
      my ($n, $k) = @_;
      # return $n_to_state_by_path->($n,$k) & $a ? 1 : 0;

      my @bits = bit_split_lowtohigh($n);
      $#bits = $k-1;
      for (;;) {
        while (@bits && $bits[0]) { shift @bits};  # skip 1s
        @bits || return 0;  # [end]
        shift @bits;                               # 0
        @bits || return 0;  # [end]
        if (shift @bits) { return 1; }             # 0
        shift @bits;                               # any

        while (@bits && ! $bits[0]) { shift @bits};  # skip 0s
        @bits || return 0;  # [end]
        shift @bits;                                 # 1
        @bits || return 0;  # [end]
        if (! shift @bits) { return 1; }             # 1
        shift @bits;                                 # any

      }
    };
    my $a_enclosed_by_path = sub {
      my ($n, $k) = @_;
      my ($n_lo, $n_hi) = $path->level_to_n_range($k);
      my ($x,$y) = $path->n_to_xy($n);
      my $n_neighbour = $path->xy_to_n($x,$y-1);
      return (defined $n_neighbour && $n_neighbour <= $n_hi
              ? 1 : 0);
    };
    foreach my $k (0 .. 10) {
      my ($n_lo, $n_hi) = $path->level_to_n_range($k);
      foreach my $n ($n_lo .. $n_hi) {
        my $a_expand = $a_enclosed_by_low_to_high->($n, $k);
        my $a_path   = $a_enclosed_by_path->($n, $k);
        # printf "k=%d n=%*b  exp=%6b  path=%6b\n", $k, $k, $n, $a_expand, $a_path;
        if ($a_path != $a_expand) {
          die;
        }
      }
    }
  }
  {
    # b
    #                 0  -> enclosed
    # 0,1s x 1 1,0s x 0  -> enclosed
    #
    my $b_enclosed_by_low_to_high = sub {
      my ($n, $k) = @_;
      # return $n_to_state_by_path->($n,$k) & $b ? 1 : 0;

      my @bits = bit_split_lowtohigh($n);
      $#bits = $k-1;
      for (;;) {
        @bits || return 0;  # [end]
        if (shift @bits) { return 1; }               # 0
        shift @bits;                                 # any
        while (@bits && ! $bits[0]) { shift @bits};  # skip 0s
        @bits || return 0;  # [end]
        if (! shift @bits) { return 1; }             # 1

        @bits || return 0;  # [end]
        if (! shift @bits) { return 1; }           # 1
        shift @bits;                               # any
        while (@bits && $bits[0]) { shift @bits};  # skip 1s
        @bits || return 0;  # [end]
        if (shift @bits) { return 1; }             # 0

      }
    };
    my $b_enclosed_by_path = sub {
      my ($n, $k) = @_;
      my ($n_lo, $n_hi) = $path->level_to_n_range($k);
      my ($x,$y) = $path->n_to_xy($n);
      my $n_neighbour = $path->xy_to_n($x-1,$y);
      return (defined $n_neighbour && $n_neighbour <= $n_hi
              ? 1 : 0);
    };
    foreach my $k (0 .. 10) {
      my ($n_lo, $n_hi) = $path->level_to_n_range($k);
      foreach my $n ($n_lo .. $n_hi) {
        my $b_expand = $b_enclosed_by_low_to_high->($n, $k);
        my $b_path   = $b_enclosed_by_path->($n, $k);
        # printf "k=%d n=%*b  exp=%6b  path=%6b\n", $k, $k, $n, $b_expand, $b_path;
        if ($b_path != $b_expand) {
          die;
        }
      }
    }
  }
  {
    # c
    #                 0  -> enclosed
    # 1,0s x 0 0,1s x 1  -> enclosed
    #
    my $c_enclosed_by_low_to_high = sub {
      my ($n, $k) = @_;
      # return $n_to_state_by_path->($n,$k) & $c ? 1 : 0;

      my @bits = bit_split_lowtohigh($n);
      $#bits = $k-1;
      for (;;) {
        shift @bits;                                 # any
        while (@bits && ! $bits[0]) { shift @bits};  # skip 0s
        @bits || return 0;  # [end]
        shift @bits;                                 # 1

        @bits || return 0;  # [end]
        if (! shift @bits) { return 1; }           # 1
        shift @bits;                               # any
        while (@bits && $bits[0]) { shift @bits};  # skip 1s
        @bits || return 0;  # [end]
        shift @bits;                               # 0

        @bits || return 0;  # [end]
        if (shift @bits) { return 1; }               # 0
      }
    };
    my $c_enclosed_by_path = sub {
      my ($n, $k) = @_;
      my ($n_lo, $n_hi) = $path->level_to_n_range($k);
      ### $k
      ### $n_lo
      ### $n_hi
      my ($x,$y) = $path->n_to_xy($n);
      ### $x
      ### $y
      my $n_neighbour = $path->xy_to_n($x-1,$y+1);
      ### $n_neighbour
      return (defined $n_neighbour && $n_neighbour <= $n_hi
              ? 1 : 0);
    };
    foreach my $k (0 .. 10) {
      my ($n_lo, $n_hi) = $path->level_to_n_range($k);
      foreach my $n ($n_lo .. $n_hi) {
        my $c_expand = $c_enclosed_by_low_to_high->($n, $k);
        my $c_path   = $c_enclosed_by_path->($n, $k);
        # printf "k=%d n=%*b  exp=%6b  path=%6b\n", $k, $k, $n, $c_expand, $c_path;
        if ($c_path != $c_expand) {
          die;
        }
      }
    }
  }
  {
    # d
    #                 0  -> enclosed
    # 1,0s x 0 0,1s x 1  -> enclosed
    #
    my $d_enclosed_by_low_to_high = sub {
      my ($n, $k) = @_;
      # return $n_to_state_by_path->($n,$k) & $d ? 1 : 0;

      my @bits = bit_split_lowtohigh($n);
      $#bits = $k-1;
      for (;;) {
        while (@bits && ! $bits[0]) { shift @bits};  # skip 0s
        @bits || return 0;  # [end]
        shift @bits;                                 # 1

        @bits || return 0;  # [end]
        if (! shift @bits) { return 1; }           # 1
        shift @bits;                               # any
        while (@bits && $bits[0]) { shift @bits};  # skip 1s
        @bits || return 0;  # [end]
        shift @bits;                               # 0

        @bits || return 0;  # [end]
        if (shift @bits) { return 1; }               # 0
        shift @bits;                                 # any
      }
    };
    my $d_enclosed_by_path = sub {
      my ($n, $k) = @_;
      my ($n_lo, $n_hi) = $path->level_to_n_range($k);
      ### $k
      ### $n_lo
      ### $n_hi
      my ($x,$y) = $path->n_to_xy($n);
      ### $x
      ### $y
      my $n_neighbour = $path->xy_to_n($x,$y+1);
      ### $n_neighbour
      return (defined $n_neighbour && $n_neighbour <= $n_hi
              ? 1 : 0);
    };
    foreach my $k (0 .. 10) {
      my ($n_lo, $n_hi) = $path->level_to_n_range($k);
      foreach my $n ($n_lo .. $n_hi) {
        my $d_expand = $d_enclosed_by_low_to_high->($n, $k);
        my $d_path   = $d_enclosed_by_path->($n, $k);
        # printf "k=%d n=%*b  exp=%6b  path=%6b\n", $k, $k, $n, $d_expand, $d_path;
        if ($d_path != $d_expand) {
          die;
        }
      }
    }
  }
  {
    # e
    #                 0  -> enclosed
    # 1,0s x 0 0,1s x 1  -> enclosed
    #
    my $e_enclosed_by_low_to_high = sub {
      my ($n, $k) = @_;
      # return $n_to_state_by_path->($n,$k) & $e ? 1 : 0;

      my @bits = bit_split_lowtohigh($n);
      $#bits = $k-1;
      for (;;) {
        @bits || return 0;  # [end]
        if (! shift @bits) { return 1; }           # 1
        shift @bits;                               # any
        while (@bits && $bits[0]) { shift @bits};  # skip 1s
        @bits || return 0;  # [end]
        if (shift @bits) { return 1; }             # 0

        @bits || return 0;  # [end]
        if (shift @bits) { return 1; }               # 0
        shift @bits;                                 # any
        while (@bits && ! $bits[0]) { shift @bits};  # skip 0s
        @bits || return 0;  # [end]
        if (! shift @bits) { return 1; }             # 1
      }
    };
    my $e_enclosed_by_path = sub {
      my ($n, $k) = @_;
      my ($n_lo, $n_hi) = $path->level_to_n_range($k);
      my ($x,$y) = $path->n_to_xy($n);
      my $n_neighbour = $path->xy_to_n($x+1,$y);
      return (defined $n_neighbour && $n_neighbour <= $n_hi
              ? 1 : 0);
    };
    foreach my $k (0 .. 10) {
      my ($n_lo, $n_hi) = $path->level_to_n_range($k);
      foreach my $n ($n_lo .. $n_hi) {
        my $e_expand = $e_enclosed_by_low_to_high->($n, $k);
        my $e_path   = $e_enclosed_by_path->($n, $k);
        # printf "k=%d n=%*b  exp=%6b  path=%6b\n", $k, $k, $n, $e_expand, $e_path;
        if ($e_path != $e_expand) {
          die;
        }
      }
    }
  }

  my $s_to_name = sub {
    my ($s) = @_;
    if ($s == 0) { return 'zero'; }
    return join('',
                ($s & $a ? 'a' : ()),
                ($s & $b ? 'b' : ()),
                ($s & $c ? 'c' : ()),
                ($s & $d ? 'd' : ()),
                ($s & $e ? 'e' : ()),
                ($s & $f ? 'f' : ()),
                ($s & $g ? 'g' : ()),
                ($s & $h ? 'h' : ()));
  };

  {
    my $s = $all;
    my ($s0,$s1) = $expand->($s);
    printf "%b -> %b, %b\n", $s, $s0, $s1;
  }

  require MyFSM;
  my %table;
  foreach my $s (0 .. $all) {
    my ($s0,$s1) = $expand->($s);
    $table{$s_to_name->($s)}{0} = $s_to_name->($s0);
    $table{$s_to_name->($s)}{1} = $s_to_name->($s1);
  }

  my %accepting;
  my $title;
  if (1) {
    foreach my $s (0 .. $all) {
      if ($s == $all) {
        $accepting{$s_to_name->($s)} = 'non';
      }
    }
    $title = 'i+1 eight';
  }
  if (0) {
    # accepting a,b,d,e
    foreach my $s (0 .. $all) {
      if (($s & $a)
          && ($s & $b)
          && ($s & $d)
          && ($s & $e)) {
        $accepting{$s_to_name->($s)} = 'non';
      }
    }
    $title = 'i+1 NSEW';
  }
  if (0) {
    # accepting any "b" side non-boundary
    foreach my $s (0 .. $all) {
      if (($s & $b) == 0) {
        $accepting{$s_to_name->($s)} = 'non';
      }
    }
  }
  if (0) {
    # accepting single boundary
    my $target = $e;
    foreach my $s (0 .. $all) {
      if (($s & $target) != 0) {
        $accepting{$s_to_name->($s)} = 'non';
      }
    }
    $title = 'accept '.$s_to_name->($target);
  }

  my $fsm = MyFSM->new(table => \%table,
                       initial => $s_to_name->($initial),
                       accepting => \%accepting,
                       title => $title);

  # {
  #   my $count = 0;
  #   my @done;
  #   foreach my $s (0 .. $all) {
  #     my $s2 = $rotate->($s);
  #     next if $done[$s]++;
  #     next if $done[$s2]++;
  #     if ($s != $s2) {
  #       print "merge: $s $s2\n";
  #       $fsm->merge_state($s_to_name->($s), $s_to_name->($s2));
  #       $fsm->rename_state($s_to_name->($s),
  #                          $s_to_name->($s).'+'.$s_to_name->($s2));
  #       # last if $count++ > 6;
  #     }
  #   }
  # }

  # $fsm = $fsm->digit_delta;
  # $fsm->delete_unreached_states;
  $fsm = $fsm->reverse;
  $fsm->simplify;
  $fsm->renumber_states;

    $fsm->rename_state(1,'t1');
    $fsm->rename_state(2,'t2');
    $fsm->rename_state(3,'f1');
    $fsm->rename_state(6,'f2');

    $fsm->rename_state(4,'t3');
    $fsm->rename_state(5,'t4');
    $fsm->rename_state(9,'t5');
    $fsm->rename_state(10,'t6');
    $fsm->rename_state(16,'t7');
    $fsm->rename_state(15,'t8');

    $fsm->rename_state(12,'f7');
    $fsm->rename_state(11,'f8');
    $fsm->rename_state(17,'f3');
    $fsm->rename_state(7,'f4');
    $fsm->rename_state(8,'f5');
    $fsm->rename_state(14,'f6');


    $fsm->rename_state(13,'a');
    $fsm->rename_state(19,'b');
    $fsm->rename_state(21,'c');
    $fsm->rename_state(18,'d');
    $fsm->rename_state(20,'e');
    $fsm->rename_state(22,'f');

  # $fsm->print_tikz;
  # $fsm->show;
  $fsm->view;
  exit;
  $fsm = $fsm->reverse;
  $fsm->simplify;
  $fsm->view;
  $fsm->print_tikz;

  exit 0;
}
{
  # twindragon area tree mean distance between nodes

  require Graph::Maker::TwindragonAreaTree;
  my @values;
  my @graphs;
  foreach my $k (1..9) {
    $graphs[$k] ||= Graph::Maker->new('twindragon_area_tree', level=>$k,
                                      undirected=>1);
    my $graph = $graphs[$k];
    my $apsp = $graph->APSP_Floyd_Warshall;
    my $diameter = $apsp->diameter;
    my @vertices = $graph->vertices;

    # average_path_length() is between distinct vertices, so the count it
    # divides by is v^2-v.
    my $average_path_length = $apsp->average_path_length;
    my $ff = $average_path_length / $diameter;

    my $total = 0;
    my $count = 0;
    my $for = $apsp->for_shortest_paths(sub {
                                          my ($t, $u,$v, $n) = @_;
                                          my $length = $t->path_length($u,$v);
                                          # if ($k <= 2) {
                                          #   print "  for $length\n";
                                          # }
                                          $total += $length;
                                          $count++;
                                        });
    my $mean = $total/$count;

    my $f = $mean / $diameter;

    print "$k $f  (distinct $ff)\n";
    # print "  num_vertices ",scalar(@vertices),"\n";
    # print "  mean  $mean $average_path_length\n";
    # print "  total $total $average_path_length_total  count $count $average_path_length_count  for=$for\n";
  }
  exit 0;
}
{
  # twindragon area tree Wiener index
  # k=0  *         W=0 E=0
  #
  # k=1  *---*           W=1 WC=1 E=1
  #
  # k=2  *---*---*---*   W=10  WC=4 E=2
  #
  # k=3  *---*---*---*   W=  WC= E=
  #              |
  #      *---*---*---*

  require Graph::Maker::TwindragonAreaTree;
  my @values;
  my @graphs;
  foreach my $k (0 .. 6) {
    $graphs[$k] ||= Graph::Maker->new('twindragon_area_tree', level=>$k,
                                  undirected=>1);
    my $graph = $graphs[$k];
    my $apsp = $graph->APSP_Floyd_Warshall;
    my $diameter = $apsp->diameter;
    my @vertices = $graph->vertices;

    $graphs[$k+1] ||= Graph::Maker->new('twindragon_area_tree', level=>$k+1,
                                  undirected=>1);
    my $next_graph = $graphs[$k+1];
    my $vc;
    foreach my $v (@vertices) {
      if ($next_graph->vertex_degree($v) > $graph->vertex_degree($v)) {
        $vc = $v;
        last;
      }
    }
    $vc || die;

    my $w = 0;
    foreach my $u (@vertices) {
      foreach my $v (@vertices) {
        $w += $apsp->path_length($u,$v);
      }
    }
    $w /= 2;

    my $ws = 0;
    foreach my $v (@vertices) {
      $ws += $apsp->path_length('0,0',$v);
    }

    my $wc = 0;
    foreach my $v (@vertices) {
      $wc += $apsp->path_length($vc,$v);
    }
    my $wsc = $ws + $wc;

    my $f = Wiener($k);
    my $wc_func = WC($k);
    my $ws_func = WS($k);
    my $ecc = TA_ecc($k);
    print "$k  ${w}[$f] WSC=$wsc WC=${wc}[$wc_func] WS=${ws}[$ws_func] E=$ecc vc=$vc\n";
    push @values, $w;
  }
  require Math::OEIS::Grep;
  Math::OEIS::Grep->search(array => \@values, verbose => 1);
  exit 0;

  # WS(k) = if(k==0,0, WS(k-1)+WC(k-1) + 2^(k-1)*RQ(k-1))
  # WC(k) = if(k==0,0, WS(k-1)+WC(k-1) + 2^(k-1)*LQ(k-1))
  # WS(k)+WC(k) = 2*(WS(k-1)+WC(k-1)) + 2^(k-1)*(LQ(k-1)+RQ(k-1))
  #
  # WSC(k) = WS(k) + WC(k)
  # WSC(k) = 2^(k-1)*BQsum(k-1)
  # vector(15,k,k--; WSC(k))
  # gWSC(x) = x*gBQsum(2*x)
  # gWSC(x) == (-5/4)/(1 - 2*x) + (5/4 + 2*x + 6*x^2)/(1 - 2*x - 16*x^3)
  # gf_terms(gWSC(x),15)
  #
  # WC(k) = if(k==0,0, WSC(k-1) + 2^(k-1)*LQ(k-1))
  # WC(k) = if(k==0,0, k==1,1, 2^(k-2)*BQsum(k-2) + 2^(k-1)*LQ(k-1))
  # vector(15,k,k--; WC(k))
  # gWC(x) = x*gWSC(x) + x*gLQ(2*x)
  # gf_terms(gWC(x),15)
  #
  # recurrence k-5 [1024, -768, 160, -32, 10] k-1
  # Wiener(k) = if(k==0,0, 2*Wiener(k-1) + 2^k*WC(k-1) + 4^(k-1))
  # Wiener(k) = if(k==0,0, 2^k*sum(j=0,k-1, WC(j)) + 1/2*2^k*(2^k-1) )
  # Wiener(k) = if(k==0,0, 1/136* (-12*2^k - 17*4^k + ( 57*dJA(k+1) + 48*dJA(k) + 58*dJA(k-1) )*4^k ))
  # Wiener(k) = 1/136*( -12*2^k - 17*4^k + (   LQ(k) + 19*LQ(k+1) + 9/2*LQ(k+2) )*4^k )
  # Wiener(k) = 1/136*( -12*2^k - 17*4^k + ( 8*RQ(k) - 18*RQ(k+1) + 19*RQ(k+2) )*4^k )
  # vector(12,k,k--; Wiener(k))
  # matsolve(mattranspose([2, 4, 6;4, 6, 10;6, 10, 18]),[105; 163; 277]) == [1;19;9/2]  \\ LQ
  # matsolve(mattranspose([3,5,9;5,9,15;9,15,25]),[105; 163; 277]) == [8,-18,19] \\ RQ
  # matsolve(mattranspose([0,1,2;1,2,3;2,3,6]),[105; 163; 277]) == [-19,49,28] \\ JA
  #
  # gWiener(x) = 2*x*gWiener(x) + 2*x*gWC(2*x) + 1/4*1/(1-4*x)
  # gWiener(x) = x/(1-2*x)*( 2*gWC(2*x) + 1/(1-4*x) )
  # gf_terms(gWiener(x),15)
  # gWiener(x) == (-3/17)/(1-2*x) + (-1/2)/(1-4*x) + (57/34 + 96/17*x + 464/17*x^2)/(1 - 4*x - 128*x^3)
  # gWiener(x) == (-3/17)/(1-2*x) + (-1/2)/(1-4*x) + (57/34 + 24/17*(4*x) + 29/17*(4*x)^2)/(1 - (4*x) - 2*(4*x)^3)
  # gWiener(x) == (-3/17)/(1-2*x) + (-1/2)/(1-4*x) + 9*gLQ(4*x) + x*gLQ(4*x) + x^2*47*gLQ(4*x)
  # gWiener(x) == (-3/17)/(1-2*x) + (-1/2)/(1-4*x) + 67*gLQ(4*x) + 47*x*gLQ(4*x) + x^2*49*gLQ(4*x)
  # matsolve(mattranspose([1, 0, 1;1, 1, 2;1, 1, 1]),[57;48;58]) == [9;1;47] \\ LQ down
  # matsolve(mattranspose([1, 0, 1;1, 1, 0;0, 1, 1]),[57;48;58]) == 1/2*[67;47;49]   \\ LQ up
  # matsolve([1, 1, 1;1, 1, 2;1, 0, 1],[57;48;58]) == [67;-1;-9]   \\ RQ up

  #
  # LQpow2(k) = 2^k*LQ(k)
  # vector(150,k,k--;LQpow2(k))
  # gLQpow2(x) = (1 + 4*x^2)/(1 - 2*x - 16*x^3)
  # gLQpow2(x) == gLQ(2*x)
  #
  # BQsum(k) = sum(j=0,k,BQ(j))
  # BQsum(k) = 1/2*BQ(k+3) - 5/2
  # vector(15,k,k--; BQsum(k))
  # gBQsum(x) = 1/(1-x)*gBQ(x)
  # gBQsum(x) == (-5/2)/(1 - x) + 1/2*1/x^3*(gBQ(x)-2 -3*x - 5*x^2)

  # gf_terms(gBQsum(x),15)
  sub Wiener {
    my ($k) = @_;
    {
      if ($k == 0) { return 0; }
      return (2*Wiener($k-1) + 2**$k*WC($k-1) + 2**(2*$k-2)
             );
    }
    {
      if ($k == 0) { return 0; }
      return (2*Wiener($k-1)
              + (WC($k-1) + 2**($k-1))*2**($k-1) + 2**($k-1)*WC($k-1)
             );
    }
  }
  sub WC {
    my ($k) = @_;
    {
      if ($k==0) { return 0; }
      return (WS($k-1) + WC($k-1)
              + 2**($k-1)*LQ($k-1)
             );
    }
    {
      if ($k==0) { return 0; }
      return (WC($k-1)
              + 2**($k-1)*LQ($k-1) + WS($k-1)
             );
    }
    {
      my $prod = 1;
      foreach my $j (0 .. $k) {
        $prod *= TA_ecc($j) + 1;
      }
      return $prod;
    }
  }
  sub WS {
    my ($k) = @_;
    {
      if ($k==0) { return 0; }
      return (WS($k-1) + WC($k-1)
              + 2**($k-1)*RQ($k-1)
             );
    }
    {
      if ($k==0) { return 0; }
      return (WS($k-1)
              + 2**($k-1)*RQ($k-1) + WC($k-1)
             );
    }
  }
  sub TA_ecc {
    my ($k) = @_;
    return RQ($k) - 1;
  }
}

{
  # vertex distance plot
  # Data
  my $k = 13;
  my $path = Math::PlanePath::DragonCurve->new;
  my ($n_lo, $n_hi) = $path->level_to_n_range($k);
  my ($x_hi,$y_hi) = $path->n_to_xy($n_hi);
  my $x_mid = $x_hi/2;
  my $y_mid = $y_hi/2;
  my $a_mid = atan2($y_mid,$x_mid);

  my @x;
  my @y;
  my $i = 0;
  foreach my $n ($n_lo .. $n_hi-1) {
    my ($x,$y) = $path->n_to_xy($n);
    $x -= $x_mid;
    $y -= $y_mid;
    my $a = atan2($y,$x);
    $a -= $a_mid;
    if ($a > pi()) { $a -= 2*pi(); }
    if ($a < -pi()) { $a += 2*pi(); }
    next unless path_n_is_right_boundary($path, $n, $n_hi);
    push @x, $a;
    push @y, sqrt($x*$x + $y*$y);
    $i++;
  }

  require Chart::Gnuplot;
  my $chart = Chart::Gnuplot->new(
                                  # output => "/tmp/simple.png",
                                  terminal => 'x11',
                                  title  => "Simple testing",
                                  xlabel => "N",
                                  ylabel => "Distance",
                                 );
  my $dataSet = Chart::Gnuplot::DataSet->new(xdata => \@x,
                                             ydata => \@y,
                                             title => "k=$k",
                                             #style => "lines",
                                             style => "points",
                                            );
  $chart->plot2d($dataSet);
  exit 0;
}

{
  # twindragon unit squares with 4 double-point corners

  my $path = Math::PlanePath::DragonCurve->new;
  my @values;
  foreach my $k (0 .. 14) {
    my ($n_lo, $n_hi) = $path->level_to_n_range($k+1);
    my ($x_hi,$y_hi) = $path->n_to_xy($n_hi);
    ### $n_hi
    ### end: "$x_hi, $y_hi"

    my $xy_to_twindragon_n_list = sub {
      my ($x,$y) = @_;
      my @n_list = ($path->xy_to_n_list($x, $y),
                    $path->xy_to_n_list($x_hi-$x, $y_hi-$y));
      @n_list = grep {$_ <= $n_hi} @n_list;
      return @n_list;
    };
    my $xy_to_twindragon_n_count = sub {
      my ($x,$y) = @_;
      my @n_list = $xy_to_twindragon_n_list->($x,$y);
      return scalar(@n_list);
    };

    my $count = 0;
    my %seen;
    foreach my $n ($n_lo .. $n_hi-1) {
      my ($x,$y) = $path->n_to_xy($n);
      foreach (1,2) {
        next if (($x+$y) % 2) == 1;
        next if $seen{"$x,$y"}++;

        if ($xy_to_twindragon_n_count->($x,$y) == 2
            && $xy_to_twindragon_n_count->($x+1,$y) == 2
            && $xy_to_twindragon_n_count->($x,$y+1) == 2
            && $xy_to_twindragon_n_count->($x+1,$y+1) == 2) {
          if ($k==6) {
            my @n_list = $xy_to_twindragon_n_count->($x,$y);
            print "  n=$n  $x,$y  is ",join(',',@n_list),"\n";
          }
          $count++;
        }
        ($x,$y) = ($x_hi-$x, $y_hi-$y);
      }
    }
    push @values, $count;
    print "k=$k  count=$count\n";
  }
  require Math::OEIS::Grep;
  Math::OEIS::Grep->search(array => \@values, verbose => 1);
  exit 0;
}

{
  # twindragon area tree 10
  require Graph::Maker::TwindragonAreaTree;
  my $graph = Graph::Maker->new('twindragon_area_tree', level=>10,
                             undirected=>1);

  require Graph::Writer::Graph6;
  my $writer = Graph::Writer::Graph6->new;
  $writer->write_graph($graph, '/tmp/twindragon-area-tree-10.g6');

  # my $diameter = $graph->diameter;
  # print "diameter $diameter\n";
  exit 0;
}

{
  # N in +/- RQ base as JA increments

  my @width;
  foreach my $k (0 .. 10) {
    $width[$k] = length(RQ($k+1)) + 1;
  }

  foreach my $n (RQ(8) .. RQ(9)+10) {
    # foreach my $n (JA(10) .. JA(11)+1) {
    # foreach my $n (55) {
    my @r = n_to_RQ_base($n);
    printf "n=%2d  ", $n;
    foreach my $i ($#r+1 .. 8) {
      printf " %*s", $width[8-$i], '';
    }
    my $nonzero = 0;
    foreach my $i (0 .. $#r) {
      my $r = $r[$i];
      if ($r == 0 && ! $nonzero) { $r = ''; }
      printf " %*s", $width[$#r-$i], $r;
      $nonzero ||= $r;
    }
    print "\n";
  }
  exit 0;

  sub n_to_RQ_base {
    my ($n) = @_;
    my @ret;
    my $k = 3;
    while ($n > JA($k)) { $k++; }
    $k--;
    push @ret, (0) x (12-$k);
    while ($k >= 3) {
      $k--;
      ### at: "n=$n  k=$k  JA(k)=".JA($k)." RQ(k-2)=".RQ($k-2)
      my $r = 0;
      if (abs($n) > JA($k)) {
        $r = RQ($k-2);
        ### subtract: $r
        if ($n < 0) { $r = -$r; }
        $n -= $r;
      }
      push @ret, $r;
    }
    if ($n != 0) { die "oops, left n=$n"; }
    return @ret;
  }
}
{
  # boundary hanging squares
  #
  # right hanging squares
  # H_initial = [0,0,0,0,1];    \\ same recurrence as JA increment
  # H(k) = if(k<#H_initial,return(H_initial[k+1])); H(k-1) + 2*H(k-3) + 1;
  # vector(16,k,H(k))
  # [0,0,0,0,1,2,3,6,11,18,31,54,91,154,263]   446,755
  # = A003479 = JA
  #
  # left hanging squares
  # H_initial = [0,0,0,0,2];
  # H(k) = if(k<#H_initial,return(H_initial[k+1])); H(k-1) + 2*H(k-3) + 2;
  # vector(16,k,H(k))
  # [0,0,0,0,0,0,2,4,6,12,22,36,62,108,182]
  # = 2* A003479 = 2*JA
  # = A005303
  #
  # left or right hanging 2 or 4 squares same delayed
  #
  #                            +1 0 +1 +2  0 +1 +4  +3  +3 +10 +15
  # Blob right [0,0,0,0,1,1,1,3,4,6,12,20,31,55,95,157,266,456,770,1302

  #
  # right squares with all 4 sides on right boundary, or some such
  # H_initial = [0,0,0,0,1,2,4];
  # H(k) = if(k<#H_initial,return(H_initial[k+1])); H(k-1) + 2*H(k-3) + 3;
  # vector(16,k,H(k))
  # [0, 0, 0, 0, 1, 2, 4, 9, 16, 27, 48, 83, 140, 239, 408, 691, 1172]
  #
  # left squares with all 4 sides on left boundary, or some such
  # H_initial = [0,0,0, 0,0,0, 2,4,8,18];
  # H(k) = if(k<#H_initial,return(H_initial[k+1])); H(k-1) + 2*H(k-3) + 6;
  # vector(16,k,H(k))
  # [0, 0, 0, 0, 0, 2, 4, 8, 18, 32, 54, 96, 166, 280, 478, 816]
  # half      1, 2, 4, 9, 16, 27, 48, 83, 140, 239, 408]
  #
  # turn left BitAboveLowestOne(n)==0
  # for n=  111    1 right            cf 0011      110     1100     11000
  #     n= 1000    0 left                0100 =0   111 =1  1101 =0  11001=0
  #     n= 1001    0 left                0101 =0           1110 =1  11010=0
  #     n= 1010    0 left                0110 =1                    11011=1
  # Rpred 111,000 are triplets
  #
  my @turn;
  @turn = (-1,1,1,1, -1,1,1);
  @turn = (1,-1,-1,1,1,1,-1,1,1,-1,-1,1,1,1,-1);
  @turn = (1,1,1);
  my $side;
  $side = 'left';
  $side = 'right';
  my $path = Math::PlanePath::DragonCurve->new;
  my @values;
  foreach my $level (0 .. 15) {
    my $n_start = $path->n_start;
    my ($n_lo, $n_hi) = $path->level_to_n_range($level);

    my $blob_lo = BlobN($level);
    my $blob_hi = BlobN($level+1) - 3;

    # return true if $x,$y is a single-visited point within level $k
    my $xy_is_single_visited = sub {
      my ($x,$y) = @_;
      my @n_list = $path->xy_to_n_list($x,$y);
      @n_list = grep {$_ <= $n_hi} @n_list;
      return @n_list == 1;
    };

    my $n_is_boundary_square = sub {
      my ($n) = @_;
      ### n_is_boundary_square(): $n

      my ($x,$y) = $path->n_to_xy($n);
      my ($dx,$dy) = $path->n_to_dxdy($n);
      my $mx = $x;  # \ will be lower left corner
      my $my = $y;  # /

      foreach my $side (0 .. 3) {
        ### at: "  side=$side  $x,$y and $dx,$dy"
        $mx = min($mx, $x);
        $my = min($my, $y);
        if ($side > 0 && ! $xy_is_single_visited->($x,$y)) {
          ### not single-visited ...
          return 0;
        }
        my $is_boundary
          = ($side eq 'left'
             ? path_xyxy_is_left_boundary($path, $x,$y, $x+$dx,$y+$dy, $n_hi)
             : path_xyxy_is_right_boundary($path, $x,$y, $x+$dx,$y+$dy, $n_hi));
        if (! $is_boundary) {
          ### not boundary ...
          return 0;
        }
        $x += $dx;
        $y += $dy;
        if ($side eq 'left') {
          ($dx,$dy) = xy_rotate_minus90($dx,$dy);
        } else {
          ($dx,$dy) = xy_rotate_plus90($dx,$dy);
        }
      }
      ### yes ...
      return "$mx,$my";
    };

    my $n_is_turns = sub {
      my ($n) = @_;
      ### n_is_turns(): $n

      my ($x,$y) = $path->n_to_xy($n);
      my ($dx,$dy) = $path->n_to_dxdy($n);
      my $mx = $x;
      my $my = $y;

      foreach my $t (0 .. $#turn) {
        my $is_boundary
          = ($side eq 'left'
             ? path_xyxy_is_left_boundary($path, $x,$y, $x+$dx,$y+$dy, $n_hi)
             : path_xyxy_is_right_boundary($path, $x,$y, $x+$dx,$y+$dy, $n_hi));
        if (! $is_boundary) {
          ### not boundary ...
          return 0;
        }
        my $turn = ($side eq 'left' ? -$turn[$#turn-$t] : $turn[$t]);
        $x += $dx;
        $y += $dy;
        my ($want_dx,$want_dy) = ($turn == 1
                                  ? xy_rotate_plus90($dx,$dy)
                                  : xy_rotate_minus90($dx,$dy));
        $n++;
        ($dx,$dy) = $path->n_to_dxdy($n);
        if ($dx != $want_dx || $dy != $want_dy) {
          ### not turn: "want dxdy=$want_dx,$want_dy got $dx,$dy"
          return 0;
        }
      }
      my $is_boundary
        = ($side eq 'left'
           ? path_xyxy_is_left_boundary($path, $x,$y, $x+$dx,$y+$dy, $n_hi)
           : path_xyxy_is_right_boundary($path, $x,$y, $x+$dx,$y+$dy, $n_hi));
      if (! $is_boundary) {
        ### not boundary ...
        return 0;
      }
      ### yes ...
      return "$mx,$my";
    };

    my $count = 0;
    my %seen;
    # foreach my $n ($n_lo .. $n_hi-1) {
   foreach my $n ($blob_lo .. $blob_hi-1) {
      # my $mxy = $n_is_boundary_square->($n) or next;
      # next if $seen{"$mxy"}++;

      my $mxy = $n_is_turns->($n) or next;

      if ($level <= 8) {
        print "  hanging n=$n at $mxy\n"
      }
      $count++;
    }
    print "k=$level  $count\n";
    push @values, $count;
  }

  require Math::OEIS::Grep;
  Math::OEIS::Grep->search(array => \@values, verbose => 1);
  exit 0;
}
{
  # triangle regions surrounding 8 segment combinations

  my $path = Math::PlanePath::DragonCurve->new;
  my @values;
  my %seen;
  my $side = 'right';
  foreach my $k (0 .. 10) {
    print " k=$k\n";
    my ($n_lo, $n_hi) = $path->level_to_n_range($k);

    my $xyxy_is_traversed = sub {
      my ($x1,$y1, $x2,$y2) = @_;
      my $n = $path->xyxy_to_n_either($x1,$y1, $x2,$y2);
      return (defined $n && $n < $n_hi ? 1 : 0);
    };

    my $found = 0;
    foreach my $n ($n_lo .. $n_hi-1) {
      my ($x,$y) = $path->n_to_xy($n);
      my ($dx,$dy) = $path->n_to_dxdy($n);
      my ($rx,$ry) = xy_rotate_minus90($dx,$dy);
      my ($lx,$ly) = xy_rotate_plus90($dx,$dy);

      my $str
        = $xyxy_is_traversed->($x,$y, $x+$rx,$y+$ry)
        . $xyxy_is_traversed->($x,$y, $x+$lx,$y+$ly)
        . $xyxy_is_traversed->($x,$y, $x-$dx,$y-$dy)

        . $xyxy_is_traversed->($x+$dx,$y+$dy, $x+$dx +$rx,$y+$dy +$ry)
        . $xyxy_is_traversed->($x+$dx,$y+$dy, $x+$dx +$lx,$y+$dy +$ly)
        . $xyxy_is_traversed->($x+$dx,$y+$dy, $x+$dx +$dx,$y+$dy +$dy);
      if ($side eq 'right') {
        $str .= $xyxy_is_traversed->($x+$rx,$y+$ry,
                                     $x+$rx + $dx,$y+$ry + $dy);
      }
      $seen{$str}++;
    }
    my $c = scalar(keys %seen);
    print "  combinations $c\n";
  }
  exit 0;
}

{
  # triangle regions first full weight

  my $path = Math::PlanePath::DragonCurve->new;
  my @values;
  foreach my $side ('right','left') {
    print "$side\n";
    foreach my $k (0 .. 10) {
      print " k=$k\n";
      my ($n_lo, $n_hi) = $path->level_to_n_range($k);

      my $xyxy_is_traversed = sub {
        my ($x1,$y1, $x2,$y2) = @_;
        my $n = $path->xyxy_to_n_either($x1,$y1, $x2,$y2);
        return (defined $n && $n < $n_hi ? 1 : 0);
      };

      my $found = 0;
      foreach my $n ($n_lo .. $n_hi-1) {
        my ($x,$y) = $path->n_to_xy($n);
        my ($dx,$dy) = $path->n_to_dxdy($n);
        my ($rx,$ry) = xy_rotate_minus90($dx,$dy);
        my ($lx,$ly) = xy_rotate_plus90($dx,$dy);

        if ($xyxy_is_traversed->($x,$y, $x+$rx,$y+$ry)
            && $xyxy_is_traversed->($x,$y, $x+$lx,$y+$ly)
            && $xyxy_is_traversed->($x,$y, $x-$dx,$y-$dy)

            && $xyxy_is_traversed->($x+$dx,$y+$dy, $x+$dx +$rx,$y+$dy +$ry)
            && $xyxy_is_traversed->($x+$dx,$y+$dy, $x+$dx +$lx,$y+$dy +$ly)
            && $xyxy_is_traversed->($x+$dx,$y+$dy, $x+$dx +$dx,$y+$dy +$dy)

            && ($side eq 'left'
                || $xyxy_is_traversed->($x+$rx,$y+$ry,
                                        $x+$rx + $dx,$y+$ry + $dy))
           ) {
          print "  n=$n\n";
          $found = 1;
        }
      }
      last if $found;
    }
  }
  exit 0;
}

{
  # convex hull of four arms inward, mill wheel

  require Math::Geometry::Planar;
  my $path = Math::PlanePath::DragonCurve->new;

  my $full_points = [ [0,0], [1,0] ];
  my $ex = 1;
  my $ey = 0;

  my $limit = 30;
  my @re;
  my @im;
  my @re_values;
  my @im_values;
  my @values;

  # gf_terms((-10/3 - 16/3*x)/(1 - 2*x^2)+ (      -  2/3*x)/(1 + 2*x^2),20)*3
  # vector(20,k,k--; vector_modulo([-10,-18,-10,-14],k)*2^floor(k/2))
  my $area_func = sub {
    my ($k) = @_;
    # + (14/3)/(1 - 2*x)
    # + (-10/3 - 16/3*x)/(1 - 2*x^2)
    # + (      -  2/3*x)/(1 + 2*x^2)
    # + (1/3)/(1 - x)
    # + (1/3 + 1/3*x)/(1 + x^2)
    return (14 * 2**$k
            - [10,18,10,14]->[$k%4] * 2**int($k/2)
            + [2,2,0,0]->[$k%4])
      / 3;
  };

  foreach my $k (0 .. $limit) {
    ### at: "k=$k  end=$ex,$ey  hull=".points_str($full_points)
    my $points = [];
    foreach my $p (@$full_points) {
      my ($x,$y) = @$p;
      ($x,$y) = ($ex-$x, $ey-$y);
      ### reverse to: "$x, $y"
      foreach my $rot (1 .. 4) {
        push @$points, [$x,$y];
        ($x,$y) = xy_rotate_plus90($x,$y);
      }
    }
    ### four: points_str($points)
    $points = points_to_hull($points);
    my $num_points = scalar(@$points);
    my $points_str = points_str($points);

    my $full_num_points = scalar(@$full_points);
    my $full_points_str = points_str($full_points);

    print "$k  $num_points = $points_str\n";
    print "    cf  full $full_num_points = $full_points_str\n";

    my $area = MyPlanar::points_to_area($points);
    my $area_by_formula = $area_func->($k);
    my $area_diff = ($area == $area_by_formula ? '' : ' ***');
    # print "k=$k area $area cf formula $area_by_formula$area_diff\n";

    if ($k == 10) {
      foreach my $p (@$points) {
        print "($p->[0],$p->[1])--";
      }
      print "cycle\n";
    }

    push @values, $area;

    ($full_points,$ex,$ey) = hull_expand($full_points,$ex,$ey);
  }
  print join(',',@values),"\n";
  require Math::OEIS::Grep;
  Math::OEIS::Grep->search(array => \@values, verbose=>1);
  exit 0;
}

{
  # 4-cycle area

  #   <----
  #  |     ^
  #  v     |
  #   ---->

  require Math::PlanePath::DragonCurve;
  my $path = Math::PlanePath::DragonCurve->new;
  my @values;
  foreach my $k (0 .. 10) {
    my ($n_lo, $n_hi) = $path->level_to_n_range($k);
    my ($x_hi,$y_hi) = $path->n_to_xy($n_hi);
    my @points;

    foreach my $n ($n_lo .. $n_hi-1) {
      next unless path_n_is_right_boundary($path, $n, $n_hi);
      my ($x2,$y2) = $path->n_to_xy($n+1);
      push @points, [$x2,$y2];
    }

    my @new_points = @points;
    foreach my $p (@points) {
      my ($x,$y) = @$p;
      ($x,$y) = xy_rotate_plus90($x,$y);
      $x += $x_hi;
      $y += $y_hi;
      push @new_points, [$x,$y];
    }
    @points = @new_points;

    ($x_hi,$y_hi) = $path->n_to_xy(2*$n_hi);
    @new_points = @points;
    foreach my $p (@points) {
      my ($x,$y) = @$p;
      ($x,$y) = xy_rotate_180($x,$y);
      $x += $x_hi;
      $y += $y_hi;
      push @new_points, [$x,$y];
    }
    @points = @new_points;

    my $area_formula = ($k==0 ? 1 : 4*A($k-1) + 4*2**($k-1));
     $area_formula = ($k==0 ? 1 : 4*A($k-1) + 2.2**$k);

    my $area = MyPlanar::points_to_area(\@points);
    print "k=$k  area $area  formula $area_formula\n";
    push @values, $area;
  }

  require Math::OEIS::Grep;
  Math::OEIS::Grep->search(array => \@values,
                           verbose => 1);
  exit 0;
}

{
  # sides in square regions count

  #   4      2      5             *      *------*
  #       ------->                |      |
  #   3      1      6             v      |
  #                               *----->*
  # real((x+I*y)*conj(xh+yh*I))
  # imag((x+I*y)*conj(xh+yh*I))

  my $path = Math::PlanePath::DragonCurve->new;
  my @values;
  foreach my $k (0 .. 16) {
    my ($n_lo, $n_hi) = $path->level_to_n_range($k);
    my ($x_hi,$y_hi) = $path->n_to_xy($n_hi);
    ### $n_hi
    my $segment_to_QR = sub {
      my ($x,$y) = @_;
      ### segment_to_QR(): "$x, $y"
      ($x, $y) = ($x*$x_hi + $y*$y_hi, # complex multiply x,y * conj(x_hi,y_hi)
                  - $x*$y_hi + $y*$x_hi);
      ### multiply: "$x, $y"
      if ($y < 0) {
        if ($x < 0) { return 3; }
        return ($x < $n_hi ? 1 : 6);
      } else {
        if ($x < 0) { return 4; }
        return ($x < $n_hi ? 2 : 5);
      }
    };
    my @count = ((0) x 7);
    my $x = 0;
    my $y = 0;
    foreach my $n ($n_lo .. $n_hi-1) {
      my ($dx,$dy) = $path->n_to_dxdy($n);
      {
        my ($rx,$ry) = xy_rotate_minus90($dx,$dy);
        my $sr = $segment_to_QR->($x+$dx/2+$rx/4, $y+$dy/2+$ry/4);
        $count[$sr]++;
        ### $sr
      }
      {
        my ($lx,$ly) = xy_rotate_plus90($dx,$dy);
        my $sl = $segment_to_QR->($x+$dx/2+$lx/4, $y+$dy/2+$ly/4);
        $count[$sl]++;
        ### $sl
      }
      $x += $dx;
      $y += $dy;
    }
    foreach my $s (1 .. $#count) {
      printf "%6d", $count[$s];
      push @{$values[$s]}, $count[$s];
    }
    my $sum = sum(@count);
    print "    $sum cf n_hi=$n_hi\n";
  }
  print "\n";

  require Math::OEIS::Grep;
  foreach my $s (1 .. $#values) {
    my $aref = $values[$s];
    shift @$aref;
    shift @$aref;
    shift @$aref;
    Math::OEIS::Grep->search(array => $aref, verbose=>1,
                             name => $s);
  }
  exit 0;
}

# {
#   # triangles in regions -- count
#
#   my $path = Math::PlanePath::CCurve->new;
#   {
#     my ($x,$y) = $path->n_to_xy($n_end);
#     $x == -$len or die "$x";
#     $y == 0 or die;
#   }
#   foreach my $k (0 .. 10) {
#     my ($n_lo, $n_hi) = $path->level_to_n_range($k+1);
#     my ($x_hi,$y_hi) = $path->n_to_xy($n_hi);
#     my @count = ((0) x 15);
#     my $mx = 0;
#     my $my = 0;
#     foreach my $n ($n_lo .. $n_hi-1) {
#       ($dx,$dy) = $path->n_to_dxdy($n);
#       $x = $mx;
#       $y = $my;
#       my $part = seg_to_part($x,$y,$dx,$dy);
#       # print "x=$mx y=$my s=",$mx+$my," dx=$dx dy=$dy  part $part\n";
#       $count[$part]++;
#       $mx += $dx;
#       $my += $dy;
#     }
#   }
#   print "\n";
#   print join(' ',@count),"\n";
#   print "total $n_end\n";
#   printcounts(\@count);
#   exit 0;
#
#   sub seg_to_part {
#     my ($x,$y,$dx,$dy) = @_;
#     if (defined (my $part = seg_to_quad($x,$y,$dx,$dy, 20,4,7,20))) {
#       return $part;
#     }
#     $x += $seg_len;
#     if (defined (my $part = seg_to_quad($x,$y,$dx,$dy, 0,2,6,3))) {
#       return $part;
#     }
#     $x += $seg_len;
#     if (defined (my $part = seg_to_quad($x,$y,$dx,$dy, 20,20,5,1))) {
#       return $part;
#     }
#     $x -= 2*$seg_len;
#     $y += $seg_len;
#     if (defined (my $part = seg_to_quad($x,$y,$dx,$dy, 10,14,20,20))) {
#       return $part;
#     }
#     $x += $seg_len;
#     if (defined (my $part = seg_to_quad($x,$y,$dx,$dy, 9,12,20,13))) {
#       return $part;
#     }
#     $x += $seg_len;
#     if (defined (my $part = seg_to_quad($x,$y,$dx,$dy, 8,20,20,11))) {
#       return $part;
#     }
#     return 20;
#   }
# }

{
  # Graph.pm layout as diameter line (bit slow on its edge stuff)
  require MyGraphs;
  require Graph::Maker::TwindragonAreaTree;

  my $k = 4;
  foreach my $k (11 .. 11) {
    my $graph = Graph::Maker->new('twindragon_area_tree', level=>$k,
                                  undirected=>1,
                                  # unionfind=>1,
                                  graph_maker => sub {
                                    return Graph::Hashed->new(@_);
                                  },
                                 );
    ### class: ref $graph
    if ($graph->can('cache_neighbours')) { $graph->cache_neighbours; }
    $graph->set_graph_attribute(name => "Twindragon$k");
    print "vertices ",scalar($graph->vertices),"\n";

    require Graph::Writer::Graph6;
    my $writer = Graph::Writer::Graph6->new;
    open my $fh, '>', \my $str or die;
    $writer->write_graph($graph, $fh);
    print "graph6 length: ", length($str), "\n";
    # print $str;

    my ($eccentricty, @centre) = MyGraphs::Graph_tree_centre_vertices($graph);
    @centre = sort @centre;
    print "centre ecc=$eccentricty  ", join('  ',@centre), "\n";

    print "Graph: ecc=", join(',',map{$graph->vertex_eccentricity($_)} @centre), "\n";
    print "Graph: centre ", join('  ',sort $graph->centre_vertices), "\n";

    my $l = Graph_layout_diameterline($graph);
    ### $l
    foreach my $v ($graph->vertices) {
      $l->{$v} or die "no layout for $v";
    }

    # foreach my $v (sort keys %$l) {
    #   print "$v  ",join(', ',@{$l->{$v}}),"\n";
    # }

    Graph_draw_layout($graph, $l);
    # Graph_view($graph);
  }
  exit 0;

  # $graph is a Graph.pm
  # $layout is a hashref { $v => [$x,$y] } where $x,$y is where to place
  # vertex $v.
  sub Graph_draw_layout {
    my ($graph, $layout) = @_;
    require Image::Base::GD;
    my $height = 700;
    my $width = 800;
    my $scale = 40;

    my $x_min = min(map {$_->[0]} values %$layout);
    my $x_max = max(map {$_->[0]} values %$layout);
    my $x_width = $x_max - $x_min + 2;

    my $y_min = min(map {$_->[1]} values %$layout);
    my $y_max = max(map {$_->[1]} values %$layout);
    my $y_height = $y_max - $y_min + 2;

    # use Smart::Comments;
    ### $x_width
    ### $y_height
    ### w: $width/$x_width
    ### h: $height/$y_height
    $scale = max(1, int(min($scale, ($width/$x_width), ($height/$y_height))));

    $scale = 3;

    my $ellipse_size = max(1, int($scale/3));
    my $ellipse_sub = int($ellipse_size/2);
    my $ellipse_add = $ellipse_size - $ellipse_sub - 1;
    print "scale $scale\n";
    print "ellipse $ellipse_size  $ellipse_sub .. $ellipse_add\n";

    $width = $scale*$x_width;

    my $image = Image::Base::GD->new (-height => $height,
                                      -width  => $width);
    $image->rectangle(0,0, $width-1,$height-1, 'black');
    my $transform = sub {
      my ($x,$y) = @_;
      $x -= $x_min;
      $y -= $y_min;
      $x++; $y++;
      $x *= $scale;
      $y *= $scale;
      $y = $height-1 - $y;  # up the screen
      return ($x,$y);
    };
    my $line_colour = 'blue';
    $line_colour = 'white';
    my $dot_colour = 'white';
    foreach my $edge ($graph->edges) {
      my ($v1,$v2) = @$edge;
      my ($x1,$y1) = @{$layout->{$v1}} or die;
      my ($x2,$y2) = @{$layout->{$v2}} or die;
      $image->line($transform->($x1,$y1),
                   $transform->($x2,$y2),
                   $line_colour);
    }
    foreach my $v ($graph->vertices) {
      my ($x,$y) = @{$layout->{$v}} or die;
      ($x,$y) = $transform->($x,$y);
      $image->ellipse($x-$ellipse_sub, $y-$ellipse_sub,
                      $x+$ellipse_add, $y+$ellipse_add,  $dot_colour, 1);
      if (! $graph->degree($v)) {
        die $v;
      }
    }
    require File::Temp;
    my $tempfh = File::Temp->new (UNLINK => 0, TMPDIR => 1,
                                  TEMPLATE => 'graph-XXXXXX',
                                  SUFFIX => '.png');
    my $filename = $tempfh->filename;
    $image->save($filename);
    require IPC::Run;
    push @ipc, IPC::Run::start(['xzgv',$filename],'&');
  }

}



{
  # Dekking Recurrent Sets 12 states + 4 empty for complex shape boundary

  #   dbar*bbar
  #       |   c3
  # *-----*-----*
  #             | b3
  #       *-----*
  #          a2
  my %expand = (a1   => [ 'a2', 'b2' ],
                a2   => [ 'a2', 'b3' ],
                a3   => [ 'a1', 'bbar' ],
                abar => [ 'abar', 'bbar' ],

                b1   => [ 'c1', 'b1' ],
                b2   => [ 'c3', 'b1' ],
                b3   => [ 'c3', 'bbar' ],
                bbar => [ 'cbar', 'bbar' ],

                c1   => [ 'd2', 'c2' ],
                c2   => [ 'd3', 'c2' ],
                c3   => [ 'dbar', 'c1' ],
                cbar => [ 'dbar', 'cbar' ],

                d1   => [ 'd1', 'a1' ],
                d2   => [ 'd1', 'a3' ],
                d3   => [ 'dbar', 'a3' ],
                dbar => [ 'dbar', 'abar' ],
               );

  my @s = ('a2', 'b3', 'c3', 'd1');
  my $expand = sub {
    @s = map {@{$expand{$_}}} @s;
  };

  my $k = 5;
  my $step = 40;
  foreach (1 .. $k) {
    $expand->();
  }
  if (@s < 50) {
    print join(' ',@s),"\n";
  }

  require Language::Logo;
  my $lo = Logo->new(update => 2, port => 8200 + (time % 100),
                     width => 1200, height => 900);
  my $draw;
  # $lo->command("seth 135; backward 200; seth 90");
  $lo->command("pendown");
  $lo->command("forward 2;
                left 90; forward 2;
                left 90; forward 4;
                left 90; forward 4;
                left 90; forward 4;
                left 90; forward 2;
                right 90; backward 2");
  $lo->command("hideturtle");
  my $angle = 0;
  foreach my $s (@s) {
    $s =~ /[a-d](bar)?/ or die;
    next if $1;
    my $dir = ord(substr($s,0,1)) - ord('a');
    $angle = -$dir * 90 + 90;
    $lo->command("seth $angle; forward $step");
  }
  $lo->disconnect("Finished...");
  exit 0;
}

{
  # twindragon area tree

  my $path = Math::PlanePath::DragonCurve->new;

  # +----+        k=1
  # |    |
  # |    |0,1
  # +---  ----+
  #      |    |
  #      |    |
  #      +----+

  #        -2,1  -1,1  0,1       k=2
  #     +----+    +----+
  #     |    |    |    |
  #     |    |    |    |
  #     +---  ----  ---  ----+
  #  -3,1    |    |     |    |
  #          |    |     |    |
  #          +----+     +----+
  #        -2,0        0,0
  foreach my $k (10,
                 # 0 .. 6,
                ) {
    print "k=$k\n";
    my ($n_lo, $n_hi) = $path->level_to_n_range($k+1);
    my ($x_hi,$y_hi) = $path->n_to_xy($n_hi);
    ### $n_hi
    ### end: "$x_hi, $y_hi"

    my @matrix;
    my %xy_to_node;
    my @node_to_xy;
    my $nodes = 0;
    my $xy_to_node = sub {
      my ($x,$y) = @_;
      my $key = "$x,$y";
      return ($xy_to_node{$key} //= do {
        my $node = $nodes++;
        $node_to_xy[$node] = $key;
        $node
      });
    };

    my $twindragon_xyxy_to_n = sub {
      my ($x1,$y1, $x2,$y2) = @_;
      $path->xyxy_to_n_either($x1,$y1, $x2,$y2)
        // $path->xyxy_to_n_either($x_hi-$x1, $y_hi-$y1,
                                   $x_hi-$x2, $y_hi-$y2);
    };

    my $nn_is_consecutive = sub {
      my ($n1, $n2) = @_;
      return (defined $n1 && defined $n2 && ($n2-$n1) == 1);
    };

    #     |                 |
    #    /                   \
    # ---   ----        ---   ----
    #      /               \
    #     |                 |
    #

    my $twindragon_xy_is_join = sub {
      my ($x,$y) = @_;

      my $n_up = $twindragon_xyxy_to_n->($x,$y, $x,$y+1);
      my $n_down = $twindragon_xyxy_to_n->($x,$y, $x,$y-1);
      my $n_left = $twindragon_xyxy_to_n->($x,$y, $x-1,$y);
      my $n_right = $twindragon_xyxy_to_n->($x,$y, $x+1,$y);

      ### twindragon_xy_is_join(): "$x,$y"
      ### $n_up
      ### $n_left
      ### $n_down
      ### $n_right

      if ($nn_is_consecutive->($n_up,$n_left)
          && $nn_is_consecutive->($n_down,$n_right)) {
        ### is join up,left ...
        return ($x-1,$y-1, $x,$y);
      }

      if ($nn_is_consecutive->($n_right,$n_up)
          && $nn_is_consecutive->($n_left,$n_down)) {
        ### is join up,right ...
        return ($x-1,$y, $x,$y-1);
      }

      ### no join ...
      return;
    };

    foreach my $n ($n_lo .. $n_hi-1) {
      my $degree = 0;
      my ($x,$y) = $path->n_to_xy($n);
      for (0,1) {
        my ($x1,$y1, $x2,$y2) = $twindragon_xy_is_join->($x,$y)
          or next;

        my $node1 = $xy_to_node->($x1,$y1);
        my $node2 = $xy_to_node->($x2,$y2);
        $matrix[$node1][$node2] = 1;
        $matrix[$node2][$node1] = 1;
        ### join at: "$x,$y  $x1,$y1 to $x2,$y2 nodes $node1 $node2"

        $x = $x_hi - $x;
        $y = $y_hi - $y;
      }
    }

    my @count_degree = (0,0,0,0);
    foreach my $i (0 .. $nodes-1) {
      my $degree = 0;
      foreach my $j (0 .. $nodes-1) {
        if ($matrix[$i][$j]) { $degree++; }
      }
      $count_degree[$degree]++;
    }
    print "count_degree ",join(', ',@count_degree),"\n";

    # /tmp/tree.g6
    if (1) {
      my $str = "n=$nodes\nm\n";
      foreach my $i (0 .. $nodes-1) {
        foreach my $j (0 .. $nodes-1) {
          $str .= $matrix[$i][$j] || 0;
        }
        $str .= "\n";
      }
      if (length($str) < 1000) {
        print $str;
      }
      require File::Slurp;
      File::Slurp::write_file('/tmp/tree.matrix',$str);
      require IPC::Run;
      # '-s' for sparse6
      IPC::Run::run(['nauty-amtog','-s','/tmp/tree.matrix','/tmp/tree.g6']);
      if (-s '/tmp/tree.g6' < 2000) {
        system('cat /tmp/tree.g6');
      }
      IPC::Run::run(['nauty-labelg','/tmp/tree.g6']);
      IPC::Run::run(['nauty-labelg','-i2','/tmp/tree.g6']);
      IPC::Run::run(['nauty-showg','/tmp/tree.g6']);
      IPC::Run::run(['nauty-showg','-t','/tmp/tree.g6']);
    }

    # display with graphviz and gv
    if (0) {
      require Graph::Easy;
      my $graph = Graph::Easy->new({undirected => 1});
      $graph->{_warn_on_unknown_attributes} = 1;
      foreach my $i (0 .. $nodes-1) {
        foreach my $j ($i+1 .. $nodes-1) {
          if ($matrix[$i][$j]) {
            $graph->add_edge($i,$j);
            print "edge $i $j  is $node_to_xy[$i] to $node_to_xy[$j]\n";
          }
        }
      }
      my $graphviz_str = $graph->as_graphviz;
      print $graphviz_str;
      require File::Temp;
      my $temp = File::Temp->new (UNLINK => 0, SUFFIX => '.ps');
      my $temp_filename = $temp->filename;
      require IPC::Run;
      IPC::Run::run(['neato','-Tps'], '<',\$graphviz_str, '>',$temp_filename);
      push @ipc, IPC::Run::start(['gv','--scale=.1',$temp_filename],'&');
    }

    # display with drawing and xzgv
    if (0) {
      require Image::Base::GD;
      my $size = 900;
      my $scale = 4;
      my $image = Image::Base::GD->new (-height => $size,
                                        -width  => $size);
      $image->rectangle(0,0, $size-1,$size-1, 'black');
      my $transform = sub {
        my ($x,$y) = @_;
        ($x,$y) = mul($x,$y);
        $x *= $scale;
        $y *= $scale;
        $x += $size/2;
        $y = $size/2 - $y;
        return ($x,$y);
      };
      my $colour = 'white';
      foreach my $i (0 .. $nodes-1) {
        foreach my $j ($i+1 .. $nodes-1) {
          if ($matrix[$i][$j]) {
            my ($x1,$y1) = split /,/, $node_to_xy[$i];
            my ($x2,$y2) = split /,/, $node_to_xy[$j];
            $image->line($transform->($x1,$y1),
                         $transform->($x2,$y2),
                         $colour);
          }
        }
      }
      $image->save('/tmp/tree.png');
      require IPC::Run;
      push @ipc, IPC::Run::start(['xzgv','/tmp/tree.png'],'&');
    }
  }
  exit 0;
}

{
  # UMT N= 1101010...10 or 101
  # UAD N= 111...111 21,21,20,20  ternary
  # FB  N= 1010...10 022,0220

  # Bird N=11000 10101010101010101010101 or ...
  # require Math::PlanePath::RationalsTree;
  # my $path = Math::PlanePath::RationalsTree->new
  #   (tree_type => 'Drib');

  require Math::PlanePath::PythagoreanTree;
  my $path = Math::PlanePath::PythagoreanTree->new
    (tree_type => 'UMT',
     coordinates => 'PQ',
     # digit_order => "LtoH",
    );

  foreach my $k (2 .. 20) {
    my $x = to_bigint(sG_real($k));
    my $y = to_bigint(sG_imag($k));
    my $n = $path->xy_to_n($x,$y);
    my $n3 = cnv($n,10,3);
    # print "k=$k $x,$y n=$n [$n3]\n";
    print "$n3\n";
  }
  exit 0;
}

{
  # some graph eigenvalues

  # Terdragon k=2 Laplacian largest eigenvalue
  # polroots(x^3 - 8*x^2 + 15*x - 4 )

  require Graph::Maker::TwinDragon;
  require Graph::Writer::Matrix;
  require Graph::Maker::PlanePath;
  my $writer = Graph::Writer::Matrix->new (format => 'pari',
                                           matrix_type => 'Laplacian',
                                          );
  foreach my $k (0 .. 3) {
    my $graph;
    $graph = Graph::Maker->new('twindragon', level=>$k, arms=>1,
                               undirected=>1);

    $graph = Graph::Maker->new('planepath',
                               undirected=>1,
                               level=>$k,
                               planepath=>'TerdragonCurve');

    print "mateigen(";
    $writer->write_graph($graph, \*STDOUT);
    print ",1)[1]\n";
  }
  exit 0;
}

{
  # convex hull of left and right boundary segments

  require Math::Geometry::Planar;
  my $path = Math::PlanePath::DragonCurve->new;

  my $full_points = [ [0,0], [1,0] ];
  my $ex = 1;
  my $ey = 0;

  my $limit = 50;
  my @re;
  my @im;
  my @re_values;
  my @im_values;
  foreach my $k (0 .. $limit) {
    my $points = [];
    my ($n_lo, $n_hi) = $path->level_to_n_range($k);
    foreach my $n ($n_lo .. $n_hi-1) {
      my ($x,$y) = $path->n_to_xy($n);
       next unless path_n_is_right_boundary($path, $n, $n_hi);
      # next unless path_n_is_left_boundary($path, $n, $n_hi);
      push @$points, [$x,$y];
    }
    push @$points, [$path->n_to_xy($n_hi)];

    if (@$points >= 5) {
      ### convexhull2: points_str($points)
      my $planar = Math::Geometry::Planar->new;
      $planar->points($points);
      $planar = $planar->convexhull2;
      $points = $planar->points;
    }
    $points = hull_points_sort_by_angle($points, $ex,$ey);
    $full_points = hull_points_sort_by_angle($full_points, $ex,$ey);

    my $points_str = points_str($points);
    my $full_points_str = points_str($full_points);
    my $diff = ($full_points_str eq $points_str ? '' : '   ***');

    my $num_points = scalar(@$points);
    print "$k  $num_points   $points_str\n";
    print "    cf  $full_points_str$diff\n";

    if ($k == 10) {
      foreach my $p (@$points) {
        print "($p->[0],$p->[1])--";
      }
      print "cycle\n";
    }

    ($full_points,$ex,$ey) = hull_expand($points,$ex,$ey, unsorted=>1);
  }
  exit 0;
}

{
  # convex hull centroid -- fractal

  require Math::Geometry::Planar;
  my $points = [
                [2/3,  -2/3],
                [0,    -2/3],
                [-1/3, -1/3],
                [-1/3,    0],
                [-1/6,  1/6],
                [2/3,   1/3],
                [3/3,   1/3],
                [7/6,   1/6],
                [7/6,     0],
                [5/6,  -3/6],
               ];
  $points = points_to_bigrat($points);
  my $centroid = points_to_centroid($points);
  print $centroid->[0],"\n";
  print $centroid->[1],"\n";

  print 305/756,"\n";
  print -59/378,"\n";
  exit 0;
}

{
  # dragon convex hull centroid

  require Math::Geometry::Planar;
  my $points = [
                [to_bigrat(0),to_bigrat(0)],
                [to_bigrat(1),to_bigrat(0)],
               ];
  $points = points_to_bigrat($points);
  my $ex = to_bigrat(1);
  my $ey = to_bigrat(0);

  my $limit = 50;
  my @re;
  my @im;
  my @re_values;
  my @im_values;
  foreach my $k (0 .. $limit) {
    print "$k  ",points_str($points),"\n";

    my $centroid = points_to_centroid($points);
    my $area = MyPlanar::points_to_area($points);
    ### $centroid
    push @re, $centroid->[0];
    push @im, $centroid->[1];
    if ($k >= 6) {
      push @re_values, $centroid->[0] * $area * 6;
      push @im_values, $centroid->[1] * $area * 6;
    }

    ($points,$ex,$ey) = hull_expand($points,$ex,$ey, unsorted=>1);
  }
  require Math::OEIS::Grep;
  Math::OEIS::Grep->search(array => \@re_values, verbose=>1);
  Math::OEIS::Grep->search(array => \@im_values, verbose=>1);

  print "% GP-DEFINE  HG_samples = [ ";
  my $join = '';
  foreach my $i (0 .. $#re) {
    print "$join$re[$i]+$im[$i]*I";
    $join = ', ';
  }
  print " ]\n";
  exit 0;
}

{
  # left boundary segment states -- low to high mappings graph

  #               x 0 1 0* 0 0 0
  #             x 0 1 0* 0 1 0 0
  #                          1 0
  #                          0 1
  #               x 0 0 1* 1 1 1
  # zeros =         0 1 0*  non
  #         zeros 0 1 1 0*
  #         ones  1 1 1 0*
  # ones  = zeros     0 1*
  #
  # zeros     0 0 0
  # zeros   0 1 0 0
  # ones    1 1 0 0
  # zeros     1 0 0
  # ones  1 1 1 0 0
  # zeros     0 1 0
  #  mix      1 1 0
  # zeros 0 0 1 1 0
  # ones    1 1 1 0
  #
  # zeros     0 0 1
  # zeros   0 1 0 1
  #  mix    1 1 0 1
  # zeros   0 0 1 1
  #  mix    1 0 1 1
  # ones      1 1 1
  #

  my %table;
  foreach my $digit (0,1) {
    foreach my $state (0 .. 10) {
      $table{$state}->{$digit} = $Lpred_state_aref->[$state]->[$digit];
    }
  }
  require MyFSM;
  my $fsm = MyFSM->new(table => \%table,
                       initial => 0,
                       accepting => { 10 => 'non' });
  # $fsm->simplify;
  print $fsm->as_graph6;
  print $fsm->as_sparse6;
  $fsm->view;

  $fsm = $fsm->reverse;
  $fsm->simplify;

  # initial=0
  if (1) {
    $fsm->rename_state('4,0,6,2,4,0,10,2,10,4,10','m1');
    $fsm->rename_state('3,1,8,1,7,9,10,9,3,10,10','m2');
    $fsm->rename_state('4,0,10,2,4,0,10,2,10,4,10','m3');
    $fsm->rename_state('0,0,2,0,4,10,10,10,0,10,10','m4');
    $fsm->rename_state('2,0,10,0,2,4,10,4,2,10,10','m5');
    $fsm->rename_state('0,0,0,0,10,10,10,10,0,10,10','m6');
    $fsm->rename_state('0,0,10,10,0,0,10,10,10,0,10','m7');
    $fsm->rename_state('10,0,10,0,10,0,10,0,10,10,10','m8');
    $fsm->rename_state('0,0,10,0,0,10,10,10,0,10,10','m9');
  }

  # initial=4
  if (0) {
    $fsm->rename_state('4,4,0,6,2,0,10,2,10,4,10','m1');
    $fsm->rename_state('7,3,1,8,1,9,10,9,3,10,10','m2');
    $fsm->rename_state('4,4,4,10,6,4,10,6,10,4,10','m3');
    $fsm->rename_state('2,2,0,10,0,4,10,4,2,10,10','m4');
    $fsm->rename_state('4,4,0,10,2,0,10,2,10,4,10','m5');
    $fsm->rename_state('4,0,0,2,0,10,10,10,0,10,10','m6');
    $fsm->rename_state('0,0,0,10,10,0,10,10,10,0,10','m7');
    $fsm->rename_state('6,6,4,10,4,4,10,4,6,10,10','m8');
    $fsm->rename_state('10,10,0,10,0,0,10,0,10,10,10','m9');
    $fsm->rename_state('4,4,4,6,4,10,10,10,4,10,10','m10');
    $fsm->rename_state('10,4,4,4,4,10,10,10,4,10,10','m11');
  }

  $fsm->show;
  $fsm->view;
  # MyFSM::fsm_replace($fsm, '10,10,10,10,10,10,10,10','right');
  print $fsm->as_graph6;
  print $fsm->as_sparse6;
  exit 0;
}

{
  # right boundary segment states -- high to low delta

  # 1111 00 1111 00 ...
  # 1111 000  non
  # 1111 01     to s5
  # 1 0000 101  to s5
  #                1. non
  #                01 non
  #                001 001 s5
  #                000 back to start
  # 1 0 1 0 1 non
  #    ^
  #    010
  #
  # 1 0 1 0 1 non
  #        ^
  #        010

  require MyFSM;
  my $table = {};
  foreach my $digit (0,1) {
    foreach my $state (0 .. 10) {
      $table->{$state}->{$digit} = $Rpred_state_table[$state]->[$digit];
    }
  }
  my $fsm = MyFSM->new
    (table => $table,
     accepting => { 10 => 'non' },
     initial   => 0,
    );

  $fsm->simplify;
  print $fsm->as_graph6;
  print $fsm->as_sparse6;
  $fsm->show;
  $fsm->view;

  $fsm = $fsm->digit_delta;
  $fsm->merge_state("10-prev0","10-prev1");
  $fsm->simplify;

  $fsm->show;
  $fsm->view;
  print $fsm->as_graph6;
  print $fsm->as_sparse6;
  exit 0;
}
{
  # blob bridges -- right and left boundary by states high to low

  require MyFSM;
  my %right_table = ('1e'  => { 0 => '1e',  1 => '1o' },
                     '1o'  => { 0 => '2eA', 1 => '2eB_3oC' },
                     '2eA' => { 0 => '1e', 1 => '3oA' },
                     '3oA' => { 0 => '3eA', 1 => '3eB' },
                     '3oB' => { 0 => '3eC', 1 => '3oA' },
                     '2eB_3oC' => { 0 => '3oB', 1 => '2eB_3oC' },
                     '3eA' => { 0 => '2eA', 1 => 'non' },
                     '3eB' => { 0 => 'non', 1 => 'non' },
                     '3eC' => { 0 => 'non', 1 => '2eB_3oC' },
                     'non' => { 0 => 'non', 1 => 'non' },
                    );
  my $right_fsm = MyFSM->new(table => \%right_table,
                             initial => '1e',
                             accepting => { non => 'non' });

  my %left_table;
  foreach my $digit (0,1) {
    foreach my $state (keys %right_table) {
      $left_table{$state}->{$digit} = $right_table{$state}->{1-$digit};
    }
  }
  my $left_fsm = MyFSM->new(table => \%left_table,
                            initial => '2eB_3oC',
                            accepting => { non => 'non' });

  my $fsm = $right_fsm->union($left_fsm);
  foreach my $state (keys %right_table) {
    next if $state eq 'non';
    $fsm->merge_state("non+non","$state+non");
    $fsm->merge_state("non+non","non+$state");
  }
  $fsm->delete_unreached_states;
  $fsm->simplify;

  $fsm->rename_state('1e+2eB_3oC',  'b1');
  $fsm->rename_state('1o+3oB',      'b2');
  $fsm->rename_state('2eB_3oC+3eC', 'b3');
  $fsm->rename_state('3oB+2eB_3oC', 'b4');
  $fsm->rename_state('3eC+2eB_3oC', 'b5');
  $fsm->rename_state('non+non', 'non');
  $fsm->rename_state('2eA+3oA', 'b6');
  $fsm->rename_state('3oA+3oB', 'b7');

  $fsm->rename_state('3oA+3eA', 'b8');
  $fsm->rename_state('3eA+3oA', 'b9');
  $fsm->rename_state('1e+3eB',  'b10');
  $fsm->view;
  print $fsm->as_graph6;
  print $fsm->as_sparse6;
  exit 0;
}

{
  # pictures of right boundary minimum distance to centre
  #

  require Image::Base::GD;
  my $size = 900;
  my $scale = 1/7 * 3;

  my $path = Math::PlanePath::DragonCurve->new;
  my $bb;
  BEGIN { $bb = Math::Complex->make(1,1); }

  foreach my $k (0 .. 12) {
    my $filename = sprintf "/tmp/min-point-%02d.png", $k;
    my $image = Image::Base::GD->new (-height => $size,
                                      -width  => $size);
    $image->rectangle(0,0, $size-1,$size-1, 'black');

    my ($n_lo, $n_hi) = $path->level_to_n_range($k);
    my ($x_hi,$y_hi) = $path->n_to_xy($n_hi);
    my ($x_mid,$y_mid) = ($x_hi/2, $y_hi/2);

    my %seen;
    my $min_norm;
    my $min_norm_count = 0;
    my @min_norm_xy;
    my @min_norm_n;
    foreach my $n ($n_lo .. $n_hi-1) {
      my ($x,$y) = $path->n_to_xy($n);
      my $norm = ($x-$x_mid)**2 + ($y-$y_mid)**2;
      next unless path_n_is_right_boundary($path, $n, $n_hi);
      if (! defined $min_norm || $norm < $min_norm) {
        $min_norm = $norm;
        @min_norm_xy = ();
        @min_norm_n = ();
        %seen = ();
      }
      if ($norm == $min_norm && ! $seen{"$x,$y"}++) {
        push @min_norm_xy, $x,$y;
        push @min_norm_n, $n;
      }
    }

    my $transform = sub {
      my ($x,$y) = @_;
      {
        # unrotated and centre on x_mid,y_mid
        my $z = Math::Complex->make($x - $x_mid, $y - $y_mid) / sqrt(2)**$k
          * $scale * $size;
        return ($size/2 + $z->Re, $size/2 - $z->Im);
      }
      {
        # unrotated and centre on min_norm
        my $z = Math::Complex->make($x - $min_norm_xy[0], $y - $min_norm_xy[1])
          / sqrt(2)**$k
          * $scale * $size;
        return ($size/2 + $z->Re, $size/2 - $z->Im);
      }
      {
        # unrotated centre on b^k/2
        my $z = (Math::Complex->make($x,$y) - $bb**$k/2) / sqrt(2)**$k
          * $scale * $size;
        return ($size/2 + $z->Re, $size/2 - $z->Im);
      }

    };

    foreach my $n ($n_lo .. $n_hi-1) {
      my ($x1,$y1) = $path->n_to_xy($n);
      my ($x2,$y2) = $path->n_to_xy($n+1);
      $image->line($transform->($x1,$y1),
                   $transform->($x2,$y2),
                   'blue');
    }

    print "k=$k n=",join(',',@min_norm_n),"  dxdy=";
    pairmap {
      my $x = $a;
      my $y = $b;
      my $dx = $x - $x_mid;
      my $dy = $y - $y_mid;
      print " $dx,$dy";
      ($x,$y) = $transform->($x,$y);
      $image->ellipse($x-2, $y-2, $x+2,$y+2, 'red', 1);
    } @min_norm_xy;
    print "\n";

    {
      my ($x,$y) = $transform->($x_mid,$y_mid);
      $image->line($x-5, $y-5, $x+5,$y+5, 'green');
      $image->line($x+5, $y-5, $x-5,$y+5, 'green');
    }

    $image->save($filename);
  }
  require IPC::Run;
  push @ipc, IPC::Run::start(['xzgv',glob('/tmp/min-point*.png')],'&');
  exit 0;
}

{
  # min,max to centre

  my $path;
  BEGIN { $path = Math::PlanePath::DragonCurve->new; }
  my @values_max;
  my @values_min;
  foreach my $k (13 .. 20) {
    my ($n_lo, $n_hi) = $path->level_to_n_range($k);
    my ($x_hi,$y_hi) = $path->n_to_xy($n_hi);
    my ($x_mid,$y_mid) = ($x_hi/2, $y_hi/2);

    my ($min_norm_n, $min_norm_x, $min_norm_y,
        $max_norm_n, $max_norm_x, $max_norm_y) = centre_min_max($k);

    my $P3_z = P3_formula($k);
    my ($P3_x,$P3_y) = ($P3_z->Re, $P3_z->Im);
    $P3_x = round_nearest($P3_x);
    $P3_y = round_nearest($P3_y);

    my $max_norm = ($max_norm_x-$x_mid)**2 + ($max_norm_y-$y_mid)**2;
    my $min_norm = ($min_norm_x-$x_mid)**2 + ($min_norm_y-$y_mid)**2;
    my $min_dx = $min_norm_x - $x_mid;
    my $min_dy = $min_norm_y - $y_mid;

    print "k=$k  max $max_norm n=$max_norm_n $max_norm_x,$max_norm_y P3=$P3_x,$P3_y   min $min_norm n=$min_norm_n $min_norm_x,$min_norm_y  mid $x_mid,$y_mid dxdy=$min_dx,$min_dy\n";
    push @values_min, $min_norm*4;
    push @values_max, $max_norm*4;
  }
  require Math::OEIS::Grep;
  Math::OEIS::Grep->search(array => \@values_min, verbose=>1, name => 'min');
  Math::OEIS::Grep->search(array => \@values_max, verbose=>1, name => 'max');
  exit 0;

  sub centre_min_max {
    my ($k) = @_;

    my ($n_lo, $n_hi) = $path->level_to_n_range($k);
    my ($x_hi,$y_hi) = $path->n_to_xy($n_hi);
    my ($x_mid,$y_mid) = ($x_hi/2, $y_hi/2);

    my $max_norm;
    my $max_norm_n;
    my $max_norm_x;
    my $max_norm_y;
    my $min_norm;
    my $min_norm_count = 0;
    my $min_norm_n;
    my $min_norm_x;
    my $min_norm_y;
    foreach my $n ($n_lo .. $n_hi-1) {
      my ($x,$y) = $path->n_to_xy($n);
      my $norm = ($x-$x_mid)**2 + ($y-$y_mid)**2;
      next unless path_n_is_right_boundary($path, $n, $n_hi);
      if (! defined $max_norm || $max_norm < $norm) {
        $max_norm = $norm;
        $max_norm_n = $n;
        $max_norm_x = $x;
        $max_norm_y = $y;
      }
      if (! defined $min_norm || $norm < $min_norm) {
        $min_norm = $norm;
        $min_norm_n = $n;
        $min_norm_x = $x;
        $min_norm_y = $y;
        $min_norm_count = 0;
      }
      if ($norm == $min_norm) {
        $min_norm_count++;
      }
    }
    return ($min_norm_n, $min_norm_x, $min_norm_y,
            $max_norm_n, $max_norm_x, $max_norm_y,
            $min_norm_count);
  }
}






{
  # Cheeger constant, vertices each side of bridge segments

  foreach my $k (4 .. 16) {
    print "k=$k\n";
    foreach my $b ($k-1, $k, $k+1) {
      next if $b < 4;
      next if $b > $k+1;
      my $before = BlobPrecedingP($b);
      my $after = P($k) - $before;
      my $P = P($k);

      # my $before = BlobN($b);
      # my $after = 2**$k - $before;
      print "  blob $b  before $before after $after (total $P)\n";
    }
  }
  exit 0;
}

{
  # twindragon skin area
  require Math::Geometry::Planar;
  my $planar = Math::Geometry::Planar->new;
  $planar->points([
                   [1+1/5,  1-3/5],
                   [1-1/5,  1+3/5],
                   [  1/5,  2-3/5],
                   [ -1/5,    3/5],
                   [  1/5,   -3/5],
                   [1-1/5, -1+3/5],
                  ]);
  my $area = $planar->area;
  print "   area=$area\n";
  exit 0;
}

{
  # twindragon area tree count of horizontal or vertical degree-2 squares

  my $path = Math::PlanePath::DragonCurve->new;
  my @count_degree;
  foreach my $k (0 .. 10) {
    print "k=$k\n";
    my ($n_lo, $n_hi) = $path->level_to_n_range($k+1);
    my ($x_hi,$y_hi) = $path->n_to_xy($n_hi);
    ### $n_hi
    ### end: "$x_hi, $y_hi"

    my @matrix;
    my %xy_to_node;
    my @node_to_xy;  # elements strings "$x,$y"
    my $nodes = 0;
    my $xy_to_node = sub {
      my ($x,$y) = @_;
      my $key = "$x,$y";
      return ($xy_to_node{$key} //= do {
        my $node = $nodes++;
        $node_to_xy[$node] = $key;
        $node
      });
    };
    my $node_to_xy = sub {
      my ($node) = @_;
      return split /,/, $node_to_xy[$node];
    };

    my $twindragon_xyxy_to_n = sub {
      my ($x1,$y1, $x2,$y2) = @_;
      $path->xyxy_to_n_either($x1,$y1, $x2,$y2)
        // $path->xyxy_to_n_either($x_hi-$x1, $y_hi-$y1,
                                   $x_hi-$x2, $y_hi-$y2);
    };

    my $nn_is_consecutive = sub {
      my ($n1, $n2) = @_;
      return (defined $n1 && defined $n2 && ($n2-$n1) == 1);
    };

    #     |                 |
    #    /                   \
    # ---   ----        ---   ----
    #      /               \
    #     |                 |
    #

    my $twindragon_xy_is_join = sub {
      my ($x,$y) = @_;

      my $n_up = $twindragon_xyxy_to_n->($x,$y, $x,$y+1);
      my $n_down = $twindragon_xyxy_to_n->($x,$y, $x,$y-1);
      my $n_left = $twindragon_xyxy_to_n->($x,$y, $x-1,$y);
      my $n_right = $twindragon_xyxy_to_n->($x,$y, $x+1,$y);

      ### twindragon_xy_is_join(): "$x,$y"
      ### $n_up
      ### $n_left
      ### $n_down
      ### $n_right

      if ($nn_is_consecutive->($n_up,$n_left)
          && $nn_is_consecutive->($n_down,$n_right)) {
        ### is join up,left ...
        return ($x-1,$y-1, $x,$y);
      }

      if ($nn_is_consecutive->($n_right,$n_up)
          && $nn_is_consecutive->($n_left,$n_down)) {
        ### is join up,right ...
        return ($x-1,$y, $x,$y-1);
      }

      ### no join ...
      return;
    };

    $xy_to_node->(0,0);
    foreach my $n ($n_lo .. $n_hi-1) {
      my $degree = 0;
      my ($x,$y) = $path->n_to_xy($n);
      for (0,1) {
        my ($x1,$y1, $x2,$y2) = $twindragon_xy_is_join->($x,$y)
          or next;

        my $node1 = $xy_to_node->($x1,$y1);
        my $node2 = $xy_to_node->($x2,$y2);
        $matrix[$node1][$node2] = 1;
        $matrix[$node2][$node1] = 1;
        ### join at: "$x,$y  $x1,$y1 to $x2,$y2 nodes $node1 $node2"

        $x = $x_hi - $x;
        $y = $y_hi - $y;
      }
    }
    my $A = 1;
    my $B = 2;
    my $C = 4;
    my $D = 8;
    my $edge_to_label = sub {
      my ($node1, $node2) = @_;
      my ($x1,$y1) = $node_to_xy->($node1);
      my ($x2,$y2) = $node_to_xy->($node2);
      my $dx = $x2 - $x1;
      my $dy = $y2 - $y1;
      # print "edge $dx, $dy\n";
      if ($dx == -1 && $dy == -1) { return $A; }
      if ($dx == 1 && $dy == -1) { return $B; }
      if ($dx == 1 && $dy == 1) { return $C; }
      if ($dx == -1 && $dy == 1) { return $D; }

      # ($x1,$y1) = div($x1,$y1);
      # ($x2,$y2) = div($x2,$y2);
      # if ($x1 == $x2 && abs($y1-$y2) == 2) { return 1; }
      # if ($y1 == $y2 && abs($x1-$x2) == 2) { return 0; }
      die "got $dx, $dy";
    };
    my @count_types = (0) x 16;
    foreach my $i (0 .. $nodes-1) {
      my $type = 0;
      foreach my $j (0 .. $nodes-1) {
        if ($matrix[$i][$j]) {
          $type += $edge_to_label->($i,$j)
        }
      }
      $count_types[$type]++;
    }
    print "count_types ",join(', ',@count_types),"\n";

    $count_degree[$k][0] = $count_types[0];
    $count_degree[$k][1] = $count_types[$B] + $count_types[$D];
    $count_degree[$k][2] = $count_types[$C+$D] + $count_types[$B+$C]
      + $count_types[$A+$B] + $count_types[$A+$D];
    $count_degree[$k][3] = $count_types[$A+$B+$C] + $count_types[$A+$C+$D];
    $count_degree[$k][4] = $count_types[$C+$D] + $count_types[$A+$B]; # horiz
    $count_degree[$k][5] = $count_types[$B+$C] + $count_types[$A+$D]; # vert
    print "count_degree ",join(', ',@{$count_degree[$k]}),"\n";
  }
  my $k_len = scalar(@count_degree);
  foreach my $d (0 .. 5) {
    my $dname = ($d == 4 ? '2_H' : $d == 5 ? '2_V' : $d);
    print "% GP-Test  vector($k_len,k,my(k=k-1); TDegCount$dname(k)) == [";
    my $join = '';
    foreach my $k (0 .. $#count_degree) {
      print $join,$count_degree[$k][$d];
      $join = ',';
    }
    print "]\n";
  }
  exit 0;
}

{
  # dragon covering by 2-clique pairs

  my $path = Math::PlanePath::DragonCurve->new;
  my @values;
  K: foreach my $k (0 .. 10) {
    my $graph = dragon_Graph($k);

    # my ($n_lo, $n_hi) = $path->level_to_n_range($k);
    # my ($x_hi,$y_hi) = $path->n_to_xy($n_hi);
    # $graph->delete_vertices(Graph_xy_to_vertex(0,0),
    #                         Graph_xy_to_vertex($x_hi,$y_hi));

    my $P = $graph->vertices;
    print "k=$k  points=$P\n";
    # print "$graph\n";

    my $count = 0;
  NODE: for (;;) {
      my @vertices = sort($graph->vertices);
      last unless @vertices;

      foreach my $v (@vertices) {
        my @neighbours = $graph->neighbors($v);
        ### @neighbours
        if (@neighbours == 1) {
          my $v2 = $neighbours[0];
          $graph->delete_vertices($v, $v2);
          next NODE;
        }
      }
      # print "  no degree 1 node\n";
      $count++;

      foreach my $v (@vertices) {
        my @neighbours = $graph->neighbors($v);
        if (@neighbours == 2) {
          @neighbours = sort @neighbours;
          my $v2 = $neighbours[0];
          $graph->delete_vertices($v, $v2);
          next NODE;
        }
      }
      print "  no degree 2 node\n";
      my $vertices = $graph->vertices;
      print "  left over $vertices vertices $graph\n";
      next K;
    }
    # push @values, $count;
    print "  is covered\n";
  }
  require Math::OEIS::Grep;
  #  Math::OEIS::Grep->search(array => \@values, verbose=>1);
  exit 0;
}

{
  # twindragon lines lengths histogram

  require MyLineSegments;
  my $path = Math::PlanePath::DragonCurve->new;
  my @values;
  foreach my $k (0 .. 16) {
    my $segments = MyLineSegments->new;
    my ($n_lo, $n_hi) = $path->level_to_n_range($k);
    my ($x_hi,$y_hi) = $path->n_to_xy($n_hi);
    foreach my $n ($n_lo .. $n_hi-1) {
      my ($x1,$y1) = $path->n_to_xy($n);
      my ($x2,$y2) = $path->n_to_xy($n+1);
      $segments->add(      $x1,      $y1,       $x2,      $y2);
      $segments->add($x_hi-$x1,$y_hi-$y1, $x_hi-$x2,$y_hi-$y2);
    }
    my @counts;
    my @lines = $segments->lines;
    foreach my $line (@lines) {
      my $length = abs($line->[0] - $line->[2]) + abs($line->[1] - $line->[3]);
      $counts[$length]++;
    }
    print "k=$k  ",;
    foreach my $length (0 .. $#counts) {
      my $count = $counts[$length] || 0;
      print "$count,";
    }
    my $max_length = $#counts;
    print "  max length $max_length\n";

    push @values, $counts[1] - BridgeCount($k);
  }
  require Math::OEIS::Grep;
  Math::OEIS::Grep->search(array => \@values, verbose=>1);
  @values = first_differences(@values);
  Math::OEIS::Grep->search(array => \@values, verbose=>1);
  @values = first_differences(@values);
  Math::OEIS::Grep->search(array => \@values, verbose=>1);
  @values = first_differences(@values);
  Math::OEIS::Grep->search(array => \@values, verbose=>1);
  exit 0;

  sub first_differences {
    my $prev = shift;
    return map {my $ret = $_ - $prev; $prev = $_; $ret} @_;
  }
}

{
  # pictures of join tree
  # num nodes A033138 = 100100100... periodic 3
  #         = A155803 = A023001 interleaved with 2*A023001 and 4*A023001.
  #

  require Image::Base::GD;
  require Math::Geometry::Planar;
  my $size = 600;
  my $scale = 1/3;

  my $path = Math::PlanePath::DragonCurve->new;
  my @values;
  my $b;
  BEGIN { $b = Math::Complex->make(1,1); }

  foreach my $k (0 .. 12) {
    my $filename = sprintf "/tmp/join-tree-%02d.png", $k;
    my $image = Image::Base::GD->new (-height => $size,
                                      -width  => $size);
    $image->rectangle(0,0, $size-1,$size-1, 'black');

    my ($n_lo, $n_hi) = $path->level_to_n_range($k+1);

    my $is_enclosed_square = sub {
      my ($x,$y) = @_;    # lower left corner
      my $dx = 1;
      my $dy = 0;
      for (1 .. 4) {
        my $x2 = $x + $dx;
        my $y2 = $y + $dy;
        if (! path_xyxy_is_traversed_either($path, $x,$y, $x2,$y2, $n_hi)) {
          return 0;
        }
        ($dx,$dy) = xy_rotate_plus90($dx,$dy);
        $x = $x2;
        $y = $y2;
      }
      return 1;
    };

    my $consecutive = sub {
      my ($n1, $n2) = @_;
      return (defined $n1
              && defined $n2
              && $n1 < $n_hi
              && $n2 < $n_hi
              && abs($n1-$n2) == 1);
    };

    my $square_neighbours = sub {
      my ($x,$y) = @_;
      ### square_neighbours: "$x, $y"
      my @ret;
      {
        #  .  .
        #
        #  *--*
        #     |
        my $n = $path->xyxy_to_n_either ($x,$y, $x+1,$y);
        my $n2 = $path->xyxy_to_n_either ($x+1,$y, $x+1,$y-1);
        if ($consecutive->($n,$n2)) {
          push @ret, $x+1,$y-1;
        }
      }
      {
        #  .  *--
        #     |
        #  .  *
        my $n = $path->xyxy_to_n_either ($x+1,$y, $x+1,$y+1);
        my $n2 = $path->xyxy_to_n_either ($x+1,$y+1, $x+2,$y+1);
        if ($consecutive->($n,$n2)) {
          push @ret, $x+1,$y+1;
        }
      }
      {
        #  |
        #  *--*
        #
        #  .  *
        my $n = $path->xyxy_to_n_either ($x+1,$y+1, $x,$y+1);
        my $n2 = $path->xyxy_to_n_either ($x,$y+1, $x,$y+2);
        if ($consecutive->($n,$n2)) {
          push @ret, $x-1,$y+1;
        }
      }
      {
        #    *  .
        #    |
        #  --*  .
        my $n = $path->xyxy_to_n_either ($x,$y+1, $x,$y);
        my $n2 = $path->xyxy_to_n_either ($x,$y, $x-1,$y);
        if ($consecutive->($n,$n2)) {
          push @ret, $x-1,$y-1;
        }
      }
      return @ret;
    };

    my $jeq_z = JEQ_complex($k-1);
    my $J_direction;

    my $transform = sub {
      my ($x,$y) = @_;
      {
        # rotate to pattern of 6, centre on b^(k-1)
        my $z = (Math::Complex->make($x,$y) - $b**($k-1)) / 16
          * $scale * $size
          * Math::Complex->make(0,1) ** ($k - int($k/3));
        if ($J_direction) {
          $z /= $J_direction * abs($J_direction);
        }
        return ($size/2 + $z->Re, $size/2 - $z->Im);
      }
      {
        # unrotated centre on b^k
        my $z = (Math::Complex->make($x,$y) - $b**$k) / sqrt(2)**$k
          * $scale * $size;
        return ($size/2 + $z->Re, $size/2 - $z->Im);
      }
      {
        # rotate and centre on 1/2
        my $z = (Math::Complex->make($x,$y) / $b**$k
                 - Math::Complex->make(.5,.5))
          * $scale * $size;
        return ($size/2 + $z->Re, $size/2 - $z->Im);
      }
    };

    my %seen;
    my ($jec_x,$jec_y) = JEC($k-1);
    ### JEQ: "$jeq_z"
    ### JEC: "$jec_x, $jec_y"
    my @pending = ($jec_x,$jec_y);
    {
      my $x = shift @pending;
      my $y = shift @pending;
      my @new = $square_neighbours->($x,$y);
      while (@new) {
        my $x2 = shift @new;
        my $y2 = shift @new;
        next unless $is_enclosed_square->($x2,$y2);
        push @pending, $x2,$y2;
      }
    }

    my $nodes = 0;
    my $colour = 'green';
    while (@pending) {
      ### @pending
      my $x = shift @pending;
      my $y = shift @pending;
      next if $seen{"$x,$y"}++;
      $nodes++;
      ### node at: "$x, $y"
      my @new = $square_neighbours->($x,$y);
      my $new_count = scalar(@new) / 2;
      while (@new) {
        my $x2 = shift @new;
        my $y2 = shift @new;
        next unless $is_enclosed_square->($x2,$y2);

        if ($colour eq 'green') {
          $J_direction = Math::Complex->make($x2-$x, $y2-$y);
        }

        $image->line($transform->($x,$y),
                     $transform->($x2,$y2),
                     $colour);
        $colour = 'red';
        push @pending, $x2,$y2;
      }
    }

    my $nodes2 = cnv($nodes,10,2);
    print "k=$k  $nodes [$nodes2]\n";
    $image->save($filename);
    push @values, $nodes;
  }
  require Math::OEIS::Grep;
  Math::OEIS::Grep->search(array => \@values, verbose=>1);
  exit 0;

  {
    my $b;
    BEGIN { $b = Math::Complex->make(1,1) }
    sub JEQ_complex {
      my ($k) = @_;
      return (Math::Complex->make(3/5, 1/5) * $b**$k
              - Math::Complex->make(1/10, -3/10) * (-1)**$k);
    }
  }
  sub JEC {
    my ($k) = @_;
    my $z = JEQ_complex($k) - Math::Complex->make(1/2, 1/2);
    return (floor(.5+$z->Re), floor(.5+$z->Im));
  }
  CHECK {
    # JEC(2): JEC(2)
    # JEQ(2): "".JEQ_complex(2)
    my ($x,$y) = JEC(0);   $x == 0 && $y == 0 or die;
    ($x,$y) = JEC(1);   $x == 0 && $y == 0 or die;
    ($x,$y) = JEC(2);   $x == -1 && $y == 1 or die;
    # -2, \;
    # -3 {-} i, \;
    # -2 {-} 4 i, \;
    #  1 {-} 5 i, \;
    #  6 {-} 4 i, \;
  }
}

{
  # blob lines by cycles
  # 1,2,1,3,3,2,5,5,9,8,21,21,37

    require MyLineSegments;
  my $path = Math::PlanePath::DragonCurve->new;
  my @values;
  foreach my $k (4 .. 16) {
    my $n_lo = BlobN($k)        + 0;
    my $n_hi = BlobN($k+1) - 3  - 0;

    my $segments = MyLineSegments->new;
    foreach my $n ($n_lo .. $n_hi-1) {
      $segments->add($path->n_to_xy($n),
                     $path->n_to_xy($n+1));
    }

    my @lines_joined = $segments->lines_joined;
    my $lines_joined = scalar(@lines_joined);
    print "k=$k  lines_joined $lines_joined\n";
    push @values, $lines_joined;
  }
  require Math::OEIS::Grep;
  Math::OEIS::Grep->search(array => \@values, verbose=>1);
  exit 0;
}

{
  require Graph::Maker;
  require Graph::Maker::Dragon;
  my $graph = Graph::Maker->new('dragon', level=>4, undirected=>1);
  print $graph->is_directed ? "directed\n" : "undirected\n";
  Graph_view($graph);
  exit 0;
}

{
  # Mandelbrot twindragon skin

  #        *
  #       / \
  #  *   /   *
  #   \ /
  #    *

  # LQ(k) = JA(k+1) + JA(k)
  # RQ(k) = JA(k+2) + JA(k)
  #
  # LQ(k) = JA(k) + JA(k-2) + JA(k-2) + JA(k)
  # JA(k+1) = JA(k) + 1 + 2*JA(k-2)
  # r1 = 1/sqrt(2)    r2=r1^3 = 1/2*r1 = 1/sqrt(2)^3
  exit 0;
}

{
  # blob maximum distance to previous level nodes
  my $path = Math::PlanePath::DragonCurve->new;

  my $step_k = 2;
  foreach my $k (6 .. 16) {
    ### $k
    my %original_vertices;
    my $original_n_lo = BlobN($k-$step_k);
    my $original_n_hi = BlobN($k-$step_k+1)-3;
    {
      my $n_factor = 2**$step_k;
      ### $original_n_lo
      ### $original_n_hi
      ### 4*n_lo: $original_n_lo * $n_factor
      ### 4*n_hi: $original_n_hi * $n_factor
      foreach my $n ($original_n_lo .. $original_n_hi-1) {
        my ($x,$y) = $path->n_to_xy($n * $n_factor);
        my $v = Graph_xy_to_vertex($x,$y);
        $original_vertices{$v} = 1;
      }
    }
    ### original: join('  ',keys %original_vertices)

    my $n_lo = 4*$original_n_lo;      # 4*original points
    my $n_hi = 4*$original_n_hi;
    # my $n_lo = BlobN($k);
    # my $n_hi = BlobN($k+1)-3;
    ### $n_lo
    ### $n_hi
    my %neighbours;
    foreach my $n ($n_lo .. $n_hi-1) {
      my ($x,$y) = $path->n_to_xy($n);
      my ($x2,$y2) = $path->n_to_xy($n+1);
      my $v = Graph_xy_to_vertex($x,$y);
      my $v2 = Graph_xy_to_vertex($x2,$y2);
      push @{$neighbours{$v}}, $v2;
      push @{$neighbours{$v2}}, $v;
    }

    my $max_distance = 0;
    my $max_v = 0;
    my $second_distance = 0;
    my $second_v = 0;
    foreach my $new_v (sort keys %neighbours) {
      ### consider v: $new_v
      my @pending = ($new_v);
      my $distance = 0;
    MAX: for (;;) {
        ### pending: join('  ',@pending)
        my @new_pending;
        foreach my $v (@pending) {
          if ($original_vertices{$v}) {
            last MAX;
          }
          push @new_pending, @{$neighbours{$v}};
        }
        @pending = @new_pending;
        $distance++;
        if ($distance > 5) {
          die "oops";
        }
      }
      ### $max_distance
      if ($distance > $max_distance) {
        $second_distance = $max_distance;
        $second_v = $max_v;
        $max_distance = $distance;
        $max_v = $new_v;
      } elsif ($distance > $second_distance) {
        $second_distance = $distance;
        $second_v = $new_v;
      }
    }

    my ($max_x,$max_y) = Graph_vertex_to_xy($max_v);
    my $max_n = $path->xy_to_n($max_x,$max_y);
    my ($second_x,$second_y) = Graph_vertex_to_xy($second_v);
    my $second_n = $path->xy_to_n($second_x,$second_y);
    my $total = $max_distance + $second_distance;

    print "k=$k n=$n_lo to $n_hi   max $max_distance at $max_v n=$max_n   second $second_distance at $second_v n=$second_n  total $total\n";
  }
  exit 0;
}

{
  # dragon 8 graph
  my $graph = dragon_Graph(5);
  Graph_view($graph);
  exit 0;
}




{
  # dragon graph
  # diameter is PathLen, start to end 1,2,4,8,12,18,26,36,52,70
  #
  # centre 0+0 1+0   # 0
  #        1+0       # 1
  #        1+1       # 2
  #        0+2       # 3
  #        -1+1      # 4
  #        -3+0      # 5
  #        -2-3      # 6
  #        0-6       # 7
  #        7-5 6-4   # 8
  #        11+0      # 9


  my $graph = dragon_Graph(3);
  # graph_show($graph);
  print "num edges    ",scalar($graph->edges),"\n";
  print "num vertices ",scalar($graph->vertices),"\n";
  print "diameter     ",scalar($graph->diameter),"\n";

  my @values;
  foreach my $k (0 .. 10) {
    my $graph = dragon_Graph($k);
    my $apsp = $graph->APSP_Floyd_Warshall;
    my $diameter      = $apsp->diameter;
    my $radius        = $apsp->diameter;
    my $shortest_path = $apsp->shortest_path;
    my $average_path  = $apsp->average_path_length;
    my $longest_path  = $apsp->longest_path;
    my @longest_path  = $apsp->longest_path;
    my @centre        = $apsp->center_vertices;
    print "k=$k  diameter=$diameter radius=$radius shortest=$shortest_path longest=$longest_path avg=$average_path\n";
    print "  centre  ",join(' ',@centre),"\n";
    print "  longest ",join(' ',@longest_path),"\n";
    push @values, $diameter;
  }
  require Math::OEIS::Grep;
  Math::OEIS::Grep->search(array => \@values, verbose=>1);
  exit 0;

}

{
  # shortest path
  my $path = Math::PlanePath::DragonCurve->new;

  my $up = [0,1];
  my $down = [0,-1];
  my $left = [-1,0];
  my $right = [1,0];

  my @start = ($right, $up, $left, $up, $left, $down, $left);
  my @end = ($left, $down, $left, $up);

  my @bridge = ([ $up, $right, $up],  # 8
                [ $up, $left, $up],   # 9
                [ $left, $up, $left], # 10
                [ $left, $down, $left], # 3
                [ $down, $left, $down], # 4
                [ $down, $right, $down], # 5
                [ $right, $down, $right], # 6
                [ $right, $up, $right], # 7
               );


  my @blob = ([ $right, $up ],    # 8    NE
              [ $up ],            # 9    N
              [ $up, $left ],     # 10   NW
              [ $left ],          # 11   W
              #
              [ $down, $left ],   # 12   SW
              [ $down ],          # 5    S
              [ $down, $right ],  # 6    SE
              [ $right ],         # 7    E
             );

  my @values;
  foreach my $k (5 .. 16) {
    my ($n_lo, $n_hi) = $path->level_to_n_range($k);
    my ($x_hi,$y_hi) = $path->n_to_xy($n_hi);

    my $xyxy_is_traversed = sub {
      my ($x1,$y1, $x2,$y2) = @_;
      my $n = $path->xyxy_to_n_either ($x1,$y1, $x2,$y2) // return 0;
      if ($n >= $n_hi) { return 0; }
      return 1;
    };

    my $x = 0;
    my $y = 0;
    my $length = 0;

    my $move = sub {
      my ($d) = @_;
      my ($dx,$dy) = @$d;
      unless ($xyxy_is_traversed->($x,$y, $x+$dx,$y+$dy)) {
        die "oops $x, $y no segment to $dx, $dy";
      }
      $x += $dx;
      $y += $dy;
      $length += abs($dx) + abs($dy);
    };

    foreach my $d (@start) {
      $move->($d);
    }
    foreach my $j (4 .. $k) {
      {
        ### start blob: $j
        my $aref = $blob[$j % 8];
        foreach my $i (1 .. BlobPathLen($j)) {
          my $d = $aref->[$i % scalar(@$aref)];
          $move->($d);
        }
      }
      {
        ### start bridge: $j
        my $aref = $bridge[$j % 8];
        foreach my $d (@$aref) {
          $move->($d);
        }
      }
    }

    foreach my $j (reverse 4 .. $k-2) {
      {
        ### end blob: $j
        my $aref = $blob[($j+2) % 8];
        ### pathlen: BlobPathLen($j)
        foreach my $i (1 .. BlobPathLen($j)) {
          my $d = $aref->[$i % scalar(@$aref)];
          $move->($d);
        }
      }
      {
        ### end bridge: $j
        my $aref = $bridge[($j+1) % 8];
        foreach my $d (@$aref) {
          $move->($d);
        }
      }
    }

    foreach my $d (@end) {
      ### end: $d
      $move->($d);
    }

    $x == $x_hi && $y == $y_hi
      or die;

    my $pathlen = PathLen($k);
    my $diff = $length == $pathlen ? "" : "  ***";
    print "k=$k  len=$length$diff\n";

    push @values, $length;
  }
  require Math::OEIS::Grep;
  Math::OEIS::Grep->search(array => \@values, verbose=>1);
  exit 0;

}

{
  # BlobN
  require Math::OEIS::Grep;
  my @BlobN = map {BlobSegments($_)} 6 .. 25;
  for my $offset (-16 .. 16) {
    my @values = map {$_+$offset} @BlobN;
    Math::OEIS::Grep->search(array => \@values, name=> "$offset");
  }
  exit 0;
}
{
  # TD twindragon graph
  my @matrix;
  my $set = sub {
    my ($i,$j) = @_;
    $matrix[$i][$j] = 1;
    $matrix[$j][$i] = 1;
  };
  $set->(0,1);
  $set->(0,2);
  $set->(1,2);
  $set->(3,1);
  $set->(3,2);

  $set->(4,5);
  $set->(4,6);
  $set->(5,6);
  $set->(7,5);
  $set->(7,6);

  $set->(0,4);

  $set->(8,9);
  $set->(8,10);
  $set->(9,10);
  $set->(11,9);
  $set->(11,10);

  $set->(12,13);
  $set->(12,14);
  $set->(13,14);
  $set->(15,13);
  $set->(15,14);

  $set->(8,12);

  $set->(16,17);
  $set->(3,16);
  $set->(3+4,16);

  $set->(8+3,17);
  $set->(8+3+4,17);
  my $nodes = 18;

  require Graph::Easy;
  my $graph = Graph::Easy->new({undirected => 1});
  $graph->{_warn_on_unknown_attributes} = 1;
  foreach my $i (0 .. $nodes-1) {
    foreach my $j ($i+1 .. $nodes-1) {
      if ($matrix[$i][$j]) {
        $graph->add_edge($i,$j);
        print "edge $i $j\n";
      }
    }
  }
  my $graphviz_str = $graph->as_graphviz;
  print $graphviz_str;
  require File::Temp;
  my $temp = File::Temp->new (UNLINK => 0, SUFFIX => '.ps');
  my $temp_filename = $temp->filename;
  require IPC::Run;
  IPC::Run::run(['neato','-Tps'], '<',\$graphviz_str, '>',$temp_filename);
  push @ipc, IPC::Run::start(['gv','--scale=.1',$temp_filename],'&');

  my $str = "n=$nodes\nm\n";
  foreach my $i (0 .. $nodes-1) {
    foreach my $j (0 .. $nodes-1) {
      $str .= $matrix[$i][$j] || 0;
    }
    $str .= "\n";
  }
  print $str;
  require File::Slurp;
  File::Slurp::write_file('/tmp/twin.matrix',$str);
  require IPC::Run;
  # '-s',
  IPC::Run::run(['nauty-amtog','/tmp/twin.matrix','/tmp/twin.g6']);
  system('cat /tmp/twin.g6');
  IPC::Run::run(['nauty-showg','/tmp/twin.g6']);
  IPC::Run::run(['nauty-showg','-t','/tmp/twin.g6']);

  exit 0;
}

{
  # left boundary squares in complex base i+1

  require Math::PlanePath::ComplexPlus;
  my $complex = Math::PlanePath::ComplexPlus->new;

  my $path = Math::PlanePath::DragonCurve->new;
  my @values;
  foreach my $level (10) {
    ### $level
    my $n_start = $path->n_start;
    (my $n_lo, my $n_hi) = $path->level_to_n_range($level);

    my $count = 0;
    my %seen;
    my $x_total = 0;
    my $y_total = 0;
    my %found;
  N: foreach my $n ($n_lo .. $n_hi-1) {
      my ($x,$y) = $path->n_to_xy($n);
      my ($dx,$dy) = $path->n_to_dxdy($n);
      my $mx = $x;
      my $my = $y;

      my $untraversed = 0;
      foreach my $i (1 .. 4) {
        ### at: "$i     $x,$y and $dx,$dy"
        if ($x <= $mx && $y <= $my) {
          $mx = $x;
          $my = $y;
        }
        if (! path_xyxy_is_traversed_either($path, $x,$y, $x+$dx,$y+$dy, $n_hi)) {
          $untraversed = 1;
        }
        $x += $dx;
        $y += $dy;
        ($dx,$dy) = xy_rotate_plus90($dx,$dy);
      }
      next unless $untraversed;

      ### key: "$mx,$my"
      next if $seen{"$mx,$my"}++;
      $count++;

      $mx += .5;    # midpoint of square
      $my += .5;

      ($mx,$my) = mul($mx,$my);
      ($mx,$my) = (-$mx,-$my);  # 180
      $my += 1;
      # ($mx,$my) = xy_rotate_minus90($mx,$my);
      # ($mx,$my) = xy_rotate_minus90($mx,$my);
      # $mx -= 1;
      $mx /= 2;
      $my /= 2;

      my $n = $complex->xy_to_n($mx,$my);
      my $n2 = $n && cnv($n,10,2);
      $n2 //= '[undef]';
      my $str = sprintf "%3d %3d  %*s", $mx, $my, $level, $n2;
      $found{$n} = $str;

      push @values, $n%2;
      # push @values, $diff;
    }
    $count == LQ($level) or die "count=$count want LQ($level)=",LQ($level);
    print "LQ($level) = $count\n";

    my $prev_n = 0;
    foreach my $n (sort {$a<=>$b} keys %found) {
      my $diff = ($n && ($n - $prev_n))||0;
      printf "%s  %+d\n", $found{$n}, $diff;
      $prev_n = $n;
    }

    $prev_n = 0;
    my $other = 0;
    foreach my $n (0 .. 2**($level-1)) {
      next if $found{$n};
      my $diff = ($n && ($n - $prev_n))||0;
      printf "%*b  %+d\n", $level, $n, $diff;
      $other++;
      $prev_n = $n;
    }
    print "$other\n";
  }

  splice @values, 0,30;
  require Math::OEIS::Grep;
  Math::OEIS::Grep->search(array => \@values, verbose=>1);
  exit 0;
}




{
  # blob links, segments on both left and right boundary
  #
  # 25,50,100,203,409,818,1636,3275,6553,13106
  my $path = Math::PlanePath::DragonCurve->new;
  my $prev_run = 0;
  my $prev = -1;
  my @values;
  foreach my $n (0 .. 2**14) {
    if (path_n_is_left_boundary($path, $n, 4*$n+16)
        && path_n_is_right_boundary($path, $n, 4*$n+16)) {

      if ($n > $prev+1) {
        my $offset = $n - 2*$prev_run;
        print "  *2 + $offset\n";
        $prev_run = $n;
        if ($n > 16) {
          push @values, $n;
        }
      }
      my $n2 = sprintf '%b', $n;
      print "$n [$n2]\n";

      $prev = $n;
    }
  }
  require Math::OEIS::Grep;
  Math::OEIS::Grep->search(array => \@values, verbose=>1);
  exit 0;
}

{
  # right boundary by bit pattern

  my $bad = 0;
  my $path = Math::PlanePath::DragonCurve->new;
  foreach my $n (0 .. 2**14) {
    my $is_bits = is_right_boundary_by_low_bits($n);
    my $is_path = path_n_is_right_boundary($path,$n,2**30);
    if ($is_bits != $is_path) {
      my $n2 = sprintf '%b', $n;
      print "oops n=$n [$n2]  is_path=$is_path is_bits=$is_bits\n";
      die if $bad++ > 10;
    }
  }
  exit 0;

  # works
  sub is_right_boundary_by_low_bits {
    my ($n) = @_;
    ### is_right_boundary_by_low_bits(): $n
    my $n2 = cnv($n,10,2);
    ### $n2
    $n2 =~ /.*(101|110|111|000)/ or return 1;   # lowest triplet
    my $pos = $-[1];
    ### $pos
    $n2 = substr($n2,0,$pos+1);
    ### $n2
    # print "$n2\n";
    return ($n2 =~ /1(01+|10+)(0(01+|10+))*$/ ? 0 : 1); # 1 between runs is non

    return ($n2 =~ /101|110/ ? 0 : 1);  # WRONG
  }
}

{
  my $bad = 0;
  my $path = Math::PlanePath::DragonCurve->new;
  foreach my $n (0 .. 2**14) {
    my $state_htol = left_boundary_state_HtoL($n);
    my $state_path = left_boundary_state_by_path($path,$n);
    if ($state_htol != $state_path) {
      my $bits = left_boundary_bits_by_path($path,$n);
      my $bits2 = sprintf '%04b', $bits;
      print "n=$n  HtoL $state_htol  path $state_path bits=$bits2\n";
      die if $bad++ > 10;
    }
  }
  exit 0;

  sub left_boundary_state_HtoL {
    my ($n) = @_;
    my $state = 4;
    foreach my $bit (reverse bit_split_lowtohigh($n)) {
      $state = $Lpred_state_aref->[$state]->[$bit];
    }
    return $state;
  }

  my @left_boundary_bits_to_state;
  BEGIN {
    $left_boundary_bits_to_state[0b0000] = 0;
    $left_boundary_bits_to_state[0b0001] = 1;

    $left_boundary_bits_to_state[0b1000] = 2;
    $left_boundary_bits_to_state[0b1100] = 2;   # b no effect
    $left_boundary_bits_to_state[0b1001] = 3;

    $left_boundary_bits_to_state[0b0010] = 4;
    $left_boundary_bits_to_state[0b0110] = 4;   # b no effect
    $left_boundary_bits_to_state[0b0011] = 5;

    $left_boundary_bits_to_state[0b1010] = 6;
    $left_boundary_bits_to_state[0b1110] = 10;  # non

    $left_boundary_bits_to_state[0b1011] = 7;
    $left_boundary_bits_to_state[0b1101] = 8;
    $left_boundary_bits_to_state[0b0111] = 9;

    $left_boundary_bits_to_state[0b1111] = 10;  # non
  }

  sub left_boundary_state_by_path {
    my ($path, $n) = @_;
    my $bits = left_boundary_bits_by_path($path,$n);
    return $left_boundary_bits_to_state[$bits]
      // die "oops n=$n bits=" . sprintf('%04b',$bits);
  }
  sub left_boundary_bits_by_path {
    my ($path, $n) = @_;
    my ($x,$y) = $path->n_to_xy($n);
    my ($dx,$dy) = $path->n_to_dxdy($n);

    #        b
    #     *-----*
    #   a |     | c
    #     *-----*
    #
    my ($dxL,$dyL) = xy_rotate_plus90($dx,$dy);

    my $bits = 0;
    if (($x + $y) % 2) {
      $bits |= 0b0001;
    }

    if (path_xyxy_is_traversed_either($path, $x,$y, $x+$dxL,$y+$dyL)) {
      $bits |= 0b1000;    # a
    }
    my $x2 = $x + $dx;
    my $y2 = $y + $dy;
    if (path_xyxy_is_traversed_either($path, $x2,$y2, $x2+$dxL,$y2+$dyL)) {
      $bits |= 0b0010;    # c
    }
    if (path_xyxy_is_traversed_either($path, $x+$dxL,$y+$dyL, $x2+$dxL,$y2+$dyL)) {
      $bits |= 0b0100;    # b
    }
    return $bits;
  }

}

{
  # left boundary segment states -- high to low mappings graph

  my %table;
  foreach my $digit (0,1) {
    foreach my $state (0 .. 10) {
      $table{$state}->{$digit} = $Lpred_state_aref->[$state]->[$digit];
    }
  }
  require MyFSM;
  my $fsm = MyFSM->new(table => \%table,
                       initial => 0);
  $fsm->view;
  exit 0;
}



{
  # right boundary segment states -- low to high mappings graph
  #
  #                 101, 110, 111
  # m7 1 m4 1.         m7=11.           zeros none
  #      0
  # m7 1 m5 01         m7=101           zeros 1        zeros 2 continue
  #      0
  # m6 0 m3 00         m6=000 100*      zeros 3 or more
  #      1
  #
  #    10 1* m7    101=non
  #    11 0* m6    110=non
  # m9 00 1* m7      m7 = previous was 1
  # m9 01 0* m6      m6 = previous was 0

  require MyFSM;

  my $table = {};
  foreach my $digit (0,1) {
    foreach my $state (0 .. 10) {
      $table->{$state}->{$digit} = $Rpred_state_table[$state]->[$digit];
    }
  }
  my $fsm = MyFSM->new
    (table => $table,
     accepting => { 10 => 'non' },
     initial   => 0,
    );


  $fsm->show;
  $fsm->view;

  $fsm = $fsm->reverse;
  $fsm->simplify;

  $fsm->rename_state('0,4,2,6,0,9,8,10,10,4,10','m1');
  $fsm->rename_state('1,3,1,3,5,7,5,10,3,10,10','m2');

  $fsm->rename_state('0,0,0,8,0,4,10,10,10,0,10','m3');
  $fsm->rename_state('0,8,0,8,4,10,4,10,8,10,10','m4');
  $fsm->rename_state('0,4,0,4,0,10,8,10,10,4,10','m5');

  $fsm->rename_state('0,0,0,10,0,0,10,10,10,0,10','m6');
  $fsm->rename_state('0,0,0,0,10,10,10,10,0,10,10','m7');

  $fsm->rename_state('0,10,0,10,0,10,0,10,10,10,10','m8');
  $fsm->rename_state('0,0,0,0,0,10,10,10,10,0,10','m9');

  $fsm->show;
  $fsm->view;
  exit 0;
}



{
  # right boundary states -- high to low

  my $bad = 0;
  my $path = Math::PlanePath::DragonCurve->new;
  foreach my $n (0 .. 2**14) {
    my $state_htol = right_boundary_state_HtoL($n);
    my $state_path = right_boundary_state_by_path($path,$n);
    if ($state_htol != $state_path) {
      my $bits = right_boundary_bits_by_path($path,$n);
      my $bits2 = sprintf '%04b', $bits;
      my $n2 = sprintf '%b', $n;
      print "n=$n [$n2]  HtoL $state_htol  path $state_path bits=$bits2\n";
      die if $bad++ > 10;
    }
  }
  exit 0;

  sub right_boundary_state_HtoL {
    my ($n) = @_;
    my $state = 0;
    foreach my $bit (reverse bit_split_lowtohigh($n)) {
      $state = $Rpred_state_table[$state]->[$bit];
    }
    return $state;
  }

  my @right_boundary_bits_to_state;
  BEGIN {
    $right_boundary_bits_to_state[0b0000] = 0;
    $right_boundary_bits_to_state[0b0001] = 1;

    $right_boundary_bits_to_state[0b1000] = 2;
    $right_boundary_bits_to_state[0b1001] = 3;

    $right_boundary_bits_to_state[0b0010] = 4;
    $right_boundary_bits_to_state[0b0011] = 5;

    $right_boundary_bits_to_state[0b1010] = 6;
    $right_boundary_bits_to_state[0b1011] = 7;

    $right_boundary_bits_to_state[0b1100] = 8;
    $right_boundary_bits_to_state[0b1101] = 3;   # b no effect, no left touch

    $right_boundary_bits_to_state[0b0110] = 9;
    $right_boundary_bits_to_state[0b0111] = 5;   # b no effect, no left touch

    $right_boundary_bits_to_state[0b1110] = 10;  # non
    $right_boundary_bits_to_state[0b1111] = 10;  # non
  }

  my %warned;
  sub right_boundary_state_by_path {
    my ($path, $n) = @_;
    my $bits = right_boundary_bits_by_path($path,$n);
    my $state = $right_boundary_bits_to_state[$bits];
    if (! defined $state) {
      if (! $warned{$bits}++) {
        warn "oops n=$n bits=" . sprintf('%04b',$bits);
      }
      $state = 99;
    }
    return $state;
  }
  sub right_boundary_bits_by_path {
    my ($path, $n) = @_;
    my ($x,$y) = $path->n_to_xy($n);
    my ($dx,$dy) = $path->n_to_dxdy($n);

    #        b
    #     *-----*
    #   a |     | c
    #     *-----*
    #
    my ($dxL,$dyL) = xy_rotate_minus90($dx,$dy);

    my $bits = 0;
    if (($x + $y) % 2) {
      $bits |= 0b0001;
    }

    if (path_xyxy_is_traversed_either($path, $x,$y, $x+$dxL,$y+$dyL)) {
      $bits |= 0b1000;    # a
    }
    my $x2 = $x + $dx;
    my $y2 = $y + $dy;
    if (path_xyxy_is_traversed_either($path, $x2,$y2, $x2+$dxL,$y2+$dyL)) {
      $bits |= 0b0010;    # c
    }
    if (path_xyxy_is_traversed_either($path, $x+$dxL,$y+$dyL, $x2+$dxL,$y2+$dyL)) {
      $bits |= 0b0100;    # b
    }
    return $bits;
  }

}

{
  # turn sequence down RationalsTree values traversed
  require Math::PlanePath::RationalsTree;
  my $path = Math::PlanePath::RationalsTree->new (tree_type => 'SB');
  my $n = $path->n_start;
  $n *= 2;   # 0=left
  my ($x,$y) = $path->n_to_xy($n);
  print "start $x / $y\n";

  for my $i (1 .. 25) {
    my $bit = 1 - BitAboveLowestOne($i);
    for (1,2) {
      $n *=2; $n += $bit;
      my ($x,$y) = $path->n_to_xy($n);
      my $frac = $x/$y;
      print "$bit  to $x / $y    $frac\n";
    }
  }
  exit 0;
}

{
  # A088431 and A007400 continued fraction
  require Math::ContinuedFraction;
  require Math::NumSeq::PlanePathTurn;
  require Math::NumSeq::GolayRudinShapiro;
  require Math::NumSeq::OEIS;

  my @runlengths = (0,1);
  # my $seq = Math::NumSeq::PlanePathTurn->new(planepath=>'DragonMidpoint');
  my $seq = Math::NumSeq::PlanePathTurn->new(planepath=>'DragonCurve');
  # my $seq = Math::NumSeq::GolayRudinShapiro->new;
  # my $seq = Math::NumSeq::OEIS->new (anum => 'A203531');

  my (undef, $prev) = $seq->next;
  my $count = 1;
  while (@runlengths < 100) {
    my (undef, $turn) = $seq->next;
    if ($turn != $prev) {
      push @runlengths, $count * 2;
      $count = 0;
      $prev = $turn;
    }
    $count++;
  }

  # @runlengths = (0, 1, 4, 2, 4, 4, 6, 4, 2, 4, 6, 2, 4, 6, 4, 4, 2, 4, 6, 2, 4, 4, 6, 4,
  #                2, 6, 4, 2, 4, 6, 4, 4, 2, 4, 6, 2, 4, 4, 6, 4, 2, 4, 6, 2, 4, 6, 4, 4,
  #                2, 6, 4, 2, 4, 4, 6, 4, 2, 6, 4, 2, 4, 6, 4, 4, 2, 4, 6, 2, 4, 4, 6, 4,
  #                2, 4, 6, 2, 4, 6, 4, 4, 2, 4, 6, 2, 4, 4, 6, 4, 2, 6, 4, 2, 4, 6, 4, 4,
  #               );

  my $cf = Math::ContinuedFraction->new(\@runlengths);
  my $cfstr = $cf->to_ascii;
  print "cf $cfstr\n";

  foreach my $i (1 .. $#runlengths) {
    my ($num, $den) = $cf->convergent($i);
    my $numstr = $num->as_bin;
    $numstr =~ s/^0b//;
    my $denstr = $den->as_bin;
    $denstr =~ s/^0b//;
    # printf "%3d %-40.70s\n", $i, $numstr;
    # printf "    %-40.70s\n", $denstr;

    my $approx = ($num << 256) / $den;
    my $bits = $approx->as_bin;
    $bits =~ s/^0b//;
    print "approx:   $bits\n";
  }

  my ($num, $den) = $cf->convergent($#runlengths);
  my $approx = $num->numify / $den->numify;
  print "$approx\n";

  $num *= Math::BigInt->new(2) ** 70;
  $num /= $den;
  my $bits = $num->as_bin;
  $bits =~ s/^0b//;
  print "d:   $bits\n";

  exit 0;
}
{
  # turn sequence as RationalsTree
  #
  # n=0b10111100111100001111110000110000111111001111000000111100001100001111110011110000111111000011000000111100111100000011110000110000 # sb
  # n=0b10000110000111100000011110011110000001100001111110000111100111111000011000011110000001111001111110000110000111111000011110011110
  # s  0b10000110000111100000011110011110000001100001111110000111100111100000011000011110000001111001111110000110000111111000011110011110
  # x=0b1101000100000001000000000000000100000000000000000000000000000001
  # y=0b10000000000000000000000000000000000000000000000000000000000000000

  # l = [1,0;1,1]
  # r = [1,1;0,1]
  # u = [l,r,r,r,r,l,r,r]
  # vecprod(v) = my(ret=1);for(i=1,#v,ret*=v[i]);ret
  # vecinv(v) = apply(m->m^-1, v)
  # vecprod(concat(Vecrev(u), vecinv(Vecrev(u))))
  # vecprod(u)*[1;1]
  #
  # compare sum 1/2^2^n = 0.8164215090218931...

  require Math::BigInt;
  require Math::PlanePath::RationalsTree;
  my $path = Math::PlanePath::RationalsTree->new (tree_type => 'SB');
  {
    my ($x,$y) = $path->n_to_xy(0b101111011);
    print "$x  $y\n";
  }

  foreach my $k (4 .. 8) {
    print "\n";
    my $str = '0b10';
    my $n_lo = 1;
    my $n_hi = 2**$k-1;
    foreach my $n ($n_lo .. $n_hi) {
      my $bit = BitAboveLowestOne($n);
      $bit = 1-$bit;
      $str .= $bit;
      # $str .= $bit;   # twice
    }

    # print "s $str\n";
    # $str = substr($str,0,-1);
    # print "s $str\n";

    my $n = Math::BigInt->new($str);
    # print "n ",$n->as_bin,"\n";

    # $str = "178272474837746701834932157648948135838";
    #  $n = Math::BigInt->new($str);
    # print "n ",$n->as_bin,"\n";

    my ($x,$y) = $path->n_to_xy($n);
    print "x ",$x->as_bin,"\n";
    print "y ",$y->as_bin,"\n";

    require Math::BigFloat;
    my $f = Math::BigFloat->new(0);
    $f->precision(200);
    $f->accuracy(200);
    $f += "$x";
    $f /= "$y";
    print"f $f\n";
  }
  exit 0;
}

{
  # A088431 2, 1, 2, 2, 3, 2, 1, 2,
  # run lengths of paperfolding sequence
  # continued fraction of sum 1/2^2^n is 2* run lengths,
  # so following Stern-Brocot down twice each dragon turn gives that sum

  require Math::ContinuedFraction;
  require Math::BigInt;
  require Math::BigRat;
  my $half = Math::BigRat->new('1/2');
  my $rat = Math::BigRat->new('1');
  for (my $exp = 1; $exp <= 16; $exp *= 2) {
    $rat += $half ** $exp;
  }
  print "$rat\n";

  my $num = $rat->numerator;
  my $den = $rat->denominator;
  print "num ",$num->as_bin,"\n";
  print "den ",$den->as_bin,"\n";
  my $cf = Math::ContinuedFraction->from_ratio($num,$den);
  my $cfstr = $cf->to_ascii;
  my $cfaref = $cf->to_array;
  my $cflen = scalar(@$cfaref);
  print "$cflen  $cfstr\n";
  $,=',';

  foreach (@$cfaref) { $_ /= 2 }
  print @$cfaref,"\n";
  exit 0;
}


{
  # right boundary states -- high to low regexp
  # not working

  # s0  0*100 rep
  #     0*10 to s4
  #     0*11 to s3
  # s3  1*001 rep
  #     1*0110 non
  #     1*0111 non
  #     1*0101 non
  #     1*0100 to s4
  # s4  0 to s0
  #     100 rep
  #     110 non
  #     111 non
  #     101 non
  #
  # s0  100+ rep
  #     11 to 1*001 rep
  #           1*0110 non
  #           1*0111 non
  #           1*0101 non
  #           1*0100 to 100 rep
  #                     0+  to s0
  #                     110 non
  #                     111 non
  #                     101 non
  #     10 to 100 rep
  #           0+  to s0
  #           110 non
  #           111 non
  #           101 non

  my $s5 = qr{
              (001)*     # back to s5
              (1.        # s7 non
               |01       # s9 non
              )
             }x;
  my $s6 = qr{
              (01+0)*     # back to s6
               (00        # s8 non
                | 1 $s5   # across to s5
               )
              }x;
  my $re = qr{^
              (100+          # back to s0
               |10(100)*0+   # through s5 back to s0
              | )*
              1          # to s1
              (1+0 $s6   # to s6
               | 01 $s5  # to s5
              )
             }x;
  # 10100010101

  my $bad = 0;
  my $path = Math::PlanePath::DragonCurve->new;
  foreach my $n (0 .. 2**14) {
    my $state = right_boundary_state_by_path($path,$n);
    my $pr = ($state == 10 ? 0 : 1);
    my $n2 = cnv($n,10,2);
    my $hr = ($n2 =~ $re ? 0 : 1);
    if ($hr != $pr) {
      my $bits = right_boundary_bits_by_path($path,$n);
      print "n=$n [$n2]  path=$pr  re=$hr\n";
      die if $bad++ > 10;
    }
  }
  exit 0;
}


{
  # triple point location

  # x3,y3     x2,y2
  #   3<-------2
  #   ^        |
  #   |        |
  #   |     3  |           X=153/256 -- 154/256   1-102/256,103/256
  #   |        v           Y=51/256 -- 52/256
  #   0------->1
  # x,y     x1,y1
  #
  #
  #
  my $path = Math::PlanePath::DragonCurve->new;
  my $level = 2*8;
  my ($n_lo, $n_hi) = $path->level_to_n_range($level);
  my $n12_lo = $n_hi;
  my $n12_hi = 2*$n_hi;
  my ($x_hi,$y_hi) = $path->n_to_xy($n_hi);
  my ($x12_hi,$y12_hi) = $path->n_to_xy($n12_hi);
  print "$n_hi  $n12_hi  xy=$x_hi,$y_hi and $x12_hi,$y12_hi\n";

  foreach my $n ($n_lo .. $n_hi-1) {
    my ($x0,$y0) = $path->n_to_xy($n);
    my ($dx,$dy) = $path->n_to_dxdy($n);
    my $x1 = $x0 + $dx;
    my $y1 = $y0 + $dy;
    ($dx,$dy) = xy_rotate_plus90($dx,$dy);
    my $x2 = $x1 + $dx;
    my $y2 = $y1 + $dy;
    my $x3 = $x0 + $dx;
    my $y3 = $y0 + $dy;
    my $n12 = $path->xyxy_to_n_either ($x1,$y1, $x2,$y2) // next;
    ($n12 >= $n12_lo && $n12 < $n12_hi) or next;

    print "triple at $x3,$y3 ----- $x2,$y2\n";
    print "           |          |      $n12\n";
    print "          $x0,$y0 ---- $x1,$y1\n";
    print "                $n\n";
    last;
  }
  exit 0;
}

{
  # Midpoint turn sequence summation
  # sum Mturn(n)/2^n = 0.552208564605602

  # Finch:
  # sum s(n)/2^n = 0.850736188  A143347
  # s(n) = 1 left, 0 right = 1 - BitAboveLowestOne(n)
  #
  #   inf     1            1
  # = sum  ------- * -------------    from Finch book
  #   k=0   2^2^k            1
  #                   1 - -------
  #                        2^2^k
  #
  # A038189 bit above lowest 1, gf by Ralf Stephan
  # needing e terms for n to 2^e
  # gBitAboveLowestOne(x) = sum(k=0,9, (x^2^k)^3/(1 - (x^2^k)^4))
  # gBitAboveLowestOne(x) = sum(k=0,9, x^(3*2^k)/(1 - x^(4*2^k)))
  # gBitAboveLowestOne(x) = sum(k=0,9, 1/x^(2^k)/(1/x^(2^(k+2)) - 1))
  # gf_terms(gBitAboveLowestOne(x),500) == vector(500,n,if(n==1,0,BitAboveLowestOne(n-1)))
  # vector(20,n,BitAboveLowestOne(ceil(n/2)))
  # vector(20,n,BitAboveLowestOne(n + (n%2)))
  #
  # gf_term_0(g) = polcoeff(numerator(g),0) / polcoeff(denominator(g),0);
  #
  # k=6; g = x^(3*2^k)/(1 - x^(4*2^k)) + x^(3*2^(k+1))/(1 - x^(4*2^(k+1)))
  # g == x^(3*2^k)*(1 + x^(4*2^k))/(1 - x^(4*2^k)) / (1 + x^(4*2^k)) + x^(3*2^(k+1))/(1 - x^(4*2^(k+1)))
  # g == x^(3*2^k)*(1 + x^(4*2^k))/(1 - x^(2*4*2^k)) + x^(3*2^(k+1))/(1 - x^(4*2^(k+1)))
  # g == ( x^(3*2^k)*(1 + x^(4*2^k)) + x^(3*2^(k+1)) )/(1 - x^(4*2^(k+1)))
  # g == ( x^(3*2^k) + x^(3*2^k)*x^(4*2^k) + x^(3*2^(k+1)) )/(1 - x^(4*2^(k+1)))
  # g == ( x^(3/2*2^(k+1)) + x^(7/2*2^(k+1)) + x^(3*2^(k+1)) )/(1 - x^(4*2^(k+1)))
  # g == ( x^(3/2*2^(k+1)) + x^(7/2*2^(k+1)) + x^(3*2^(k+1)) )/(1 - x^(4*2^(k+1)))
  #
  # gPaperFolding(x) = x/(1-x) - gBitAboveLowestOne(x)      \\ 0<->1 for n>=1
  #   = x
  # gTurn(x) = x/(1-x) - 2*gBitAboveLowestOne(x)
  # gf_terms(gTurn(x),501)[^1] == vector(500,n,turn(n))
  # gf_terms(gTurn(x),10)
  # CgTurn = 1/(1-(1/2)) - 2*CgBit
  #
  # BitAboveLowestOne(n+1) = BitAboveLowestZero(n)
  # gBitAboveLowestZero(x) = 1/x*gBitAboveLowestOne(x)
  # gf_terms(gBitAboveLowestOne(x),20)
  # gf_terms(gBitAboveLowestZero(x),20)
  # gBitAboveLowestOne(.5)
  # gBitAboveLowestZero(.5)
  # 1 - gBitAboveLowestOne(.5)
  #
  # gMturn(x) = x/(1-x) - gBitAboveLowestOne(x) - gBitAboveLowestZero(x)
  # gMturn(x) = x/(1-x) - gBitAboveLowestOne(x) - 1/x*gBitAboveLowestOne(x)
  # gMturn(x) = x^2/(x*(1-x)) - (1-x)*(1 + x)/(x*(1-x))*gBitAboveLowestOne(x)
  # gMturn(x) = 1/(x*(1-x)) * ( x^2 - (1-x)*(1+x)*gBitAboveLowestOne(x) )
  # gMturn(x) = 1/(x*(1-x)) * ( x^2 - (1-x^2)*gBitAboveLowestOne(x) )
  #
  # gMturn(x) = 1/2*(gTurn(x) + gTurn(x)/x - 1)
  # gMturn(x) = -1/2 + 1/2*(1+x)/x*gTurn(x)
  # gf_terms(gMturn(x),501)[^1] == vector(500,n,Mturn(n))
  # gf_terms(gMturn(x),10)
  #
  # gMturn(x) == -1/2 + 1/2*(1+x)/x*( x/(1-x) - 2*gBitAboveLowestOne(x) )
  # gMturn(1/2)*1.0
  #     0.0 -1/2 + 3/2*( 1 - 2*CgBit )
  #     1.0 - 3*CgBit
  #     1.0 - 3*(1-Cgs)
  #     3*Cgs - 2.0   /* = Cgs */
  #
  # gTurn(1/2)*1.0
  # gBitAboveLowestOne(1/2)*2.0+1/2
  # CgBit = gBitAboveLowestOne(1/2)*1.0
  # gf_terms(1/(1-x) - gBitAboveLowestOne(x) - 1, 10)
  #
  # gs(x) = (1/(1-x) - gBitAboveLowestOne(x) - 1)
  # gf_terms(gs(x), 10)
  # Cgs = gs(1/2)*1.0
  #       1/(1-(1/2)) - CgBit - 1
  #       1 - CgBit
  #
  #
  # gMturn(x) = 1/2*(x*gTurn(x) + gTurn(x))
  # gf_terms(gMturn(x), 10)
  # 3*(1 - gBitAboveLowestOne(1/2)*1.0) - 2
  # 3 - 3*gBitAboveLowestOne(1/2) - 2.0
  # 1.0 - 3*gBitAboveLowestOne(1/2)
  # 3*0.8587361882 - 2

  # t^3/t^4 / (1/t^4 - t^4/t^4)
  # = 1/t * 1 / (1/t^4 - 1)
  # = - 1/t * 1 / (1 - 1/t^4)
  #
  # 1/2/(1-1/2)
  # - t^3/ (1 - t^4)
  # - t*1/(1-t^4) + t*1/(1-t^4) - t^3/ (1 - t^4)
  # - t*1/(1-t^4) + (t - t^3)/ (1 - t^4)
  # sum(k=0,4, my(t=1/2^2^k); (t + t^3)/ (1 - t^4)*1.0)  \\ -> -1
  #
  # (t - t^3)/ (1 - t^4)  +  (t^2 - (t^2)^3)/ (1 - (t^2)^4)
  # (1 + t^4)*(t - t^3)/ (1 - t^8)  +  (t^2 - (t^2)^3)/ (1 - t^8)
  # ( (1 + t^4)*(t - t^3)  +  (t^2 - (t^2)^3)) / (1 - t^8)
  # (-t^7 - t^6 + t^5 - t^3 + t^2 + t) / (1 - t^8)
  #
  # gf_terms(gBitAboveLowestOne_term(x,0), 32)   n%4==3 is 1
  # gf_terms(gBitAboveLowestOne_term(x,1), 32)   n%8==3 is 1
  # gf_terms(gBitAboveLowestOne_term(x,2), 32)   n%16==12 is 1
  # gBitAboveLowestOne_term(x,1)
  # gBitAboveLowestOne_term(x,2)
  # gBitAboveLowestOne_term(x,2)

  # gBitAboveLowestOne(x)
  # g(x,k) = n=0 to 2^k inclusive      BitAboveLowestOne(2^k)=0
  # reverse x^(2^k)*g(1/x,k)
  # shift up x^(2^k)*x^(2^k)*g(1/x,k)
  # add  g(x,k+1) = g(1/x,k) + x^(2^k)*x^(2^k)*g(1/x,k)

  my $sum = 0;
  foreach my $n (1 .. 256) {
    $sum += (1-BitAboveLowestOne($n)) / 2**$n;
  }
  print "$sum      ",1-$sum,"\n";
  $sum = 0;
  foreach my $n (1 .. 256) {
    $sum += BitAboveLowestOne($n) / 2**$n;
  }
  print "$sum\n";

  print "Mturn\n";

  $sum = 0;
  foreach my $n (1 .. 256) {
    $sum += Mturn($n) / 2**$n;
  }
  print "$sum\n";

  $sum = 0;
  foreach my $n (1 .. 256) {
    $sum += (1 - bit_above_lowest_zero($n) - BitAboveLowestOne($n)) / 2**$n;
  }
  print "$sum\n";

  $sum = 0;
  foreach my $n (1 .. 256) {
    $sum += (1 - (($n&2)>>1) - BitAboveLowestOne($n+ ($n&1))) / 2**$n;
  }
  print "$sum\n";

  $sum = 0;
  foreach my $m (0 .. 256) {
    if ($m > 0) {
      $sum += (1 - 0 - BitAboveLowestOne(4*$m + 0)) / 2**(4*$m);
    }
    $sum += (1 - 0 - BitAboveLowestOne(4*$m+1 + 1)) / 2**(4*$m+1);
    $sum += (1 - 1 - BitAboveLowestOne(4*$m+2 + 0)) / 2**(4*$m+2);
    $sum += (1 - 1 - BitAboveLowestOne(4*$m+3 + 1)) / 2**(4*$m+3);
  }
  print "$sum\n";

  $sum = 0;
  foreach my $m (0 .. 256) {
    if ($m > 0) {
      $sum += (1 - BitAboveLowestOne(4*$m + 0)) / 2**(4*$m);
    }
    $sum += (2 - 2*BitAboveLowestOne(4*$m+2)) / 2**(4*$m+2);
    $sum += (    - BitAboveLowestOne(4*$m+2)) / 2**(4*$m+2);
    $sum += (    - BitAboveLowestOne(4*$m+4)) / 2**(4*$m+3);
  }
  print "$sum\n";

  $sum = 0;
  foreach my $m (0 .. 256) {
    $sum += (1 - BitAboveLowestOne(4*$m + 4)) / 2**(4*$m+4);
    $sum += (2 - 3*BitAboveLowestOne(4*$m+2)) / 2**(4*$m+2);
    $sum += (    - BitAboveLowestOne(4*$m+4)) / 2**(4*$m+3);
  }
  print "$sum\n";

  $sum = 0;
  foreach my $m (0 .. 256) {
    $sum += (2 - 3*BitAboveLowestOne(4*$m+2)) / 2**(4*$m+2);
    $sum += (  - 2*BitAboveLowestOne(4*$m+4)) / 2**(4*$m+4);
    $sum += (1 -   BitAboveLowestOne(4*$m+4)) / 2**(4*$m+4);
  }
  print "$sum\n";

  $sum = 0;
  foreach my $m (0 .. 256) {
    $sum += (2 - 3*BitAboveLowestOne(4*$m+2)) / 2**(4*$m+2);
    $sum += (1 - 3*BitAboveLowestOne(4*$m+4)) / 2**(4*$m+4);
  }
  print "$sum\n";

  $sum = 0;
  foreach my $m (0 .. 256) {
    $sum += 2 / 2**(4*$m+2);
    $sum += 1 / 2**(4*$m+4);
    $sum += -3*BitAboveLowestOne(4*$m+2) / 2**(4*$m+2);
    $sum += -3*BitAboveLowestOne(4*$m+4) / 2**(4*$m+4);
  }
  print "$sum\n";

  $sum = 0;
  foreach my $m (0 .. 256) {
    $sum += 9/16 / 2**(4*$m);
    $sum += -3*BitAboveLowestOne(4*$m+2) / 2**(4*$m+2);
    $sum += -3*BitAboveLowestOne(4*$m+4) / 2**(4*$m+4);
  }
  print "$sum\n";

  $sum = 0;
  foreach my $m (0 .. 256) {
    $sum += 9/16 / 2**(4*$m);
    $sum += -3/4 *($m&1) / 2**(4*$m);
    $sum += -3*BitAboveLowestOne($m+1) / 2**(4*$m+4);
  }
  print "$sum\n";

  $sum = 9/15;
  foreach my $m (0 .. 256) {
    $sum += -3/4 *($m&1) / 2**(4*$m);
    $sum += -3*BitAboveLowestOne($m+1) / 2**(4*$m+4);
  }
  print "$sum\n";

  $sum = 47/85;
  foreach my $m (0 .. 256) {
    $sum += -3/16 * BitAboveLowestOne($m+1) / 2**(4*$m);
  }
  print "$sum\n";

  exit 0;
}

{
  # bridge points, points which are on both left and right boundary
  # 0,1,2,3,4,5,6, 12,13, 26,27, 51,52, 101,102, 204,205, 410,411,
  # 819,820, 1637,1638, 3276,3277, 6554,6555
  #
  # even values = 2* A007910 G.f.: 1/((1-2*x)*(1+x^2)).
  # binary 110011001100...
  #     1, 2, 3, 6, 13, 26, 51, 102, 205, 410, 819, 1638, 3277, 6554, 13107,
  #     26214, 52429, 104858, 209715, 419430, 838861, 1677722, 3355443,
  #
  # odd values = A133190 a(n)=2a(n-1)-a(n-2)+2a(n-3).
  # binary 110011001100...
  #     1, 3, 3, 5, 13, 27, 51, 101, 205, 411, 819, 1637, 3277, 6555, 13107,
  #     26213, 52429, 104859, 209715, 419429, 838861, 1677723, 3355443,

  require Math::NumSeq::PlanePathCoord;

  my $path = Math::PlanePath::DragonCurve->new;
  my $prev = -1;
  my @n_right_list = (0);
  my @n_left_list = (0);
  my $dir_right = 0;
  my $dir_left = 0;
  my $x_right = 0;
  my $y_right = 0;
  my $x_left = 0;
  my $y_left = 0;
  my $pos_right = 0;
  my $pos_left = 0;

  my $any_consecutive = sub {
    my ($aref, $bref) = @_;
    foreach my $a (@$aref) {
      foreach my $b (@$bref) {
        if (abs($a-$b) == 1) {
          return 1;
        }
      }
    }
    return 0;
  };

  my $advance_right = sub {
    ### advance_right() ...
    $dir_right = ($dir_right - 1) % 4;  # perhaps boundary turn right
    for (;;) {
      ### $dir_left
      my $dx = $dir4_to_dx[$dir_right];
      my $dy = $dir4_to_dy[$dir_right];
      my @next_n_list = $path->xy_to_n_list($x_right+$dx,$y_right+$dy);
      if ($any_consecutive->(\@n_right_list, \@next_n_list)) {
        @n_right_list = @next_n_list;
        $x_right += $dx;
        $y_right += $dy;
        $pos_right++;
        return;
      }
      $dir_right = ($dir_right + 1) % 4;
    }
  };
  my $advance_left = sub {
    ### advance_left() ...
    $dir_left = ($dir_left + 1) % 4;  # perhaps boundary turn left
    for (;;) {
      ### $dir_left
      my $dx = $dir4_to_dx[$dir_left];
      my $dy = $dir4_to_dy[$dir_left];
      my @next_n_list = $path->xy_to_n_list($x_left+$dx,$y_left+$dy);
      if ($any_consecutive->(\@n_left_list, \@next_n_list)) {
        @n_left_list = @next_n_list;
        $x_left += $dx;
        $y_left += $dy;
        $pos_left++;
        return;
      }
      $dir_left = ($dir_left - 1) % 4;
    }
  };

  my @values;
  for (;;) {
    ### at: "n_list_right=".join(',',@n_right_list)."  n_list_left=".join(',',@n_left_list)
    if (@n_right_list == 1 && @n_left_list == 1
        && $n_right_list[0] == $n_left_list[0]) {
      my $n = $n_right_list[0];
      last if $n > 10000;
      if ($n > $prev+1) {
        print "\n";
      } else {
      }

      push @values, $n;    # N
      #  push @values, $pos_right;    # positions

      printf "%4d %8b   %b %b\n", $n,$n, $pos_left,$pos_right;
      $prev = $n;
      &$advance_right();
      &$advance_left();

    } elsif ($n_right_list[0] < $n_left_list[0]) {
      &$advance_right();
    } else {
      &$advance_left();
    }
  }
  while ($values[0] < 20) { shift @values };
  print join(',',@values),"\n";
  require Math::OEIS::Grep;
  Math::OEIS::Grep->search(array => \@values, verbose => 1);

  # odd/even values
  foreach my $mod (0, 1) {
    my @filtered_values = grep {$_%2==$mod} @values;
    Math::OEIS::Grep->search(array => \@filtered_values, verbose => 1,
                             name => "==$mod mod 2");
  }

  # odd/even terms
  foreach my $mod (0, 1) {
    my @filtered_values = map {$_%2==$mod ? $values[$_] : ()} 0 .. $#values;
    Math::OEIS::Grep->search(array => \@filtered_values, verbose => 1,
                             name => "positions $mod mod 2");
  }
  exit 0;

  # use constant SURROUND_4 => [ 1,0, 0,1, -1,0, 0,-1 ];
  # sub path_n_is_bridge {
  #   my ($path, $n) = @_;
  #   my @n_list = $path->xy_to_n_list($path->n_to_xy($n));
  #   return @n_list == 1
  #     && Math::NumSeq::PlanePathCoord::_path_n_surround_count($path,$n,SURROUND_4) == 2;
  # }
}

{
  # sum 1/2^(2^i)) from RationalsTree

  # [2, 1, 2, 2, 3, 2, 1, 2, 3, 1, 2, 3, 2, 2, 1, 2, 3, 1, 2, 2, 3, 2, 1, 3 ]
  # [4, 2, 4, 4, 6, 4, 2, 4, 6, 2, 4, 6, 4, 4, 2, 4, 6, 2, 4, 4, 6, 4, 2, 6 ]

  require Math::BigInt;
  require Math::BigRat;
  require Math::PlanePath::RationalsTree;

  foreach my $tree_type ('HCS',
                         'AYT',
                         'CW',
                         'SB',
                         # 'Bird','Drib',
                        ) {
    foreach my $k (6 .. 6) {
      print "k=$k $tree_type\n";

      my $f = Math::BigRat->new(0);
      foreach my $i (0 .. $k) {
        my $y = Math::BigRat->new(2) ** (2**$i);
        $f += 1/$y;
      }
      print "f=$f\n";

      my $x = $f->numerator;
      my $y = $f->denominator;
      my $path = Math::PlanePath::RationalsTree->new (tree_type => $tree_type);
      my $n = $path->xy_to_n($x,$y);
      print "n=$n\n";
      print "n=",$n->as_bin,"\n";

      print "o    ";
      foreach my $n (1 .. 64) {
        my $bit = BitAboveLowestOne($n);
        print $bit;
        print $bit;
        # if ($bit) { print "0"; }
      }
      print "\n";

      print "x=$x\ny=$y\n";
      print "x=",$x->as_bin,"\n";
      print "y=",$y->as_bin,"\n";

      my @values;
      $n = substr($n,2);
      my $run = 0;
      foreach my $i (0 .. length($n)-1) {
        if (substr($n,$i,1)) {
          if ($run) {
            $run++;  # including "1"
            print "$run,";
            push @values, $run;
          }
          $run = 0;
        } else {
          $run++;
        }
      }
      print "\n";
      # require Math::OEIS::Grep;
      # Math::OEIS::Grep->search(array => \@values, verbose=>1,
      #                          name => "k=$k $tree_type");
    }
  }
  print "\n";
  exit 0;
}

{
  # enclosed squares consecutiveness

  my $path = Math::PlanePath::DragonCurve->new;
  my @values;
 N: for (my $n = 0; $n < 5000000; $n++) {
    my ($x,$y) = $path->n_to_xy($n);
    my ($dx,$dy) = $path->n_to_dxdy($n);
    my @n_list = $n;
    for (1 .. 3) {
      $x += $dx;
      $y += $dy;
      ($dx,$dy) = xy_rotate_minus90($dx,$dy);
      my $n2 = $path->xyxy_to_n_either ($x,$y, $x+$dx,$y+$dy)
        // next N;
      # $n2 <= $n_hi or next N;
      push @n_list, $n2;
    }
    @n_list == 4 or die;

    my $consecutive = 0;
    foreach my $i (0 .. $#n_list) {
      if (abs($n_list[$i] - $n_list[$i-1]) == 1) {
        $consecutive++;
      }
    }
    if ($consecutive == 4) {
      print "n=$n  $consecutive\n";
    }
  }
  # require Math::OEIS::Grep;
  # Math::OEIS::Grep->search(array => \@values);
  exit 0;
}

{
  # turn sequence how many runs in level $k
  # DragonCurve  2^k-1
  # DragonMidpoint  3*2^k  A111286
  #
  #   k=2 runs=2           k=3 runs=4
  #    .                  .  +__+
  #    |                  |  |  |
  #    -__+               -__-  -__+
  #       |                        |
  #    .__+                     .__+
  #
  # Runs(k) = 2*Runs(k-1) since middle new "+" joins final "--" invert
  #
  # TerdragonCurve  2*3^k
  # TerdragonMidpoint  2*3^n - 1
  #

  require Math::NumSeq::PlanePathTurn;
  foreach my $pathname ('DragonCurve',
                        'DragonMidpoint',
                        'TerdragonMidpoint',
                        'TerdragonCurve') {
    print "$pathname\n";
    my $seq = Math::NumSeq::PlanePathTurn->new(planepath=>$pathname,
                                               turn_type => 'LSR');
    my $path = $seq->{'planepath_object'};
    my @values;
    foreach my $k (2 .. 10) {
      my $n = 1;
      my $prev = $seq->ith($n++);
      my ($n_lo, $n_hi) = $path->level_to_n_range($k);
      my @runs;
      my $length = 1;
      my $count_runs = 1;
      for ( ; $n < $n_hi; ) {
        my $turn = $seq->ith($n++);
        if ($turn == $prev) {
          $length++;
        } else {
          if (@runs < 80) {
            push @runs, $length;
          }
          $length = 1;
          $count_runs++;
          $prev = $turn;
        }
      }
      push @runs, $length;
      push @values, $count_runs;
      print "k=$k  $count_runs\n";

      if (@runs < 70) {
        scalar(@runs) == $count_runs or die;
        @runs = map {$_==3 ? '_' : $_} @runs;
        print "   ",@runs,"\n";
      }
    }
    require Math::OEIS::Grep;
    Math::OEIS::Grep->search(array => \@values,verbose=>1,name=>$pathname);
  }
  exit 0;
}

{
  # DragonMidpoint turn sequence run lengths
  # A157196

  require Math::NumSeq::PlanePathTurn;
  my $seq = Math::NumSeq::PlanePathTurn->new(planepath=>'DragonMidpoint',
                                             turn_type => 'LSR');

  my @values;
  my (undef, $prev) = $seq->next;
  my $count = 1;
  while (@values < 200) {
    my (undef, $turn) = $seq->next;
    if ($turn != $prev) {
      push @values, $count;
      $count = 0;
      $prev = $turn;
    }
    $count++;
  }
  require Math::OEIS::Grep;
  Math::OEIS::Grep->search(array => \@values,verbose=>1);
  exit 0;
}

{
  # left boundary squares centroid

  my $path = Math::PlanePath::DragonCurve->new;
  my @values;
  foreach my $level (0 .. 9) {
    ### $level
    my $n_start = $path->n_start;
    (my $n_lo, my $n_hi) = $path->level_to_n_range($level);

    my $count = 0;
    my %seen;
    my $x_total = 0;
    my $y_total = 0;
  N: foreach my $n ($n_lo .. $n_hi-1) {
      my ($x,$y) = $path->n_to_xy($n);
      my ($dx,$dy) = $path->n_to_dxdy($n);
      my $mx = $x;
      my $my = $y;

      my $untraversed = 0;
      foreach my $i (1 .. 4) {
        ### at: "$i     $x,$y and $dx,$dy"
        if ($x <= $mx && $y <= $my) {
          $mx = $x;
          $my = $y;
        }
        if (! path_xyxy_is_traversed_either($path, $x,$y, $x+$dx,$y+$dy, $n_hi)) {
          $untraversed = 1;
        }
        $x += $dx;
        $y += $dy;
        ($dx,$dy) = xy_rotate_plus90($dx,$dy);
      }
      next unless $untraversed;

      ### key: "$mx,$my"
      next if $seen{"$mx,$my"}++;
      $count++;

      $x_total += $mx + .5;
      $y_total += $my + .5;
    }
    $count == LQ($level) or die "count=$count want LQ($level)=",LQ($level);

    my $gx = $x_total / $count;
    my $gy = $y_total / $count;
    print "k=$level  $count   $x_total,$y_total   $gx,$gy\n";
    push @values, $gx;
  }

  require Math::OEIS::Grep;
  Math::OEIS::Grep->search(array => \@values);
  exit 0;
}

{
  # join area ascii art

  my $path = Math::PlanePath::DragonCurve->new;
  require Image::Base::Text;
  foreach my $k (6 .. 8) {
    print "k=$k\n";
    my $width = 78;
    my $height = 40;
    my $image = Image::Base::Text->new (-width => $width, -height => $height);

    my $draw = sub {
      my ($x,$y, $char) = @_;
      $x *= 2;
      $y *= 2;
      $x += 30;
      $y += 25;
#      $y = $height - $y;
      $image->xy($x,$y, $char);
    };
    my ($n_lo, $n_hi) = $path->level_to_n_range($k);
    my ($x_hi,$y_hi) = $path->n_to_xy($n_hi);
    my ($next_n_lo, $next_n_hi) = $path->level_to_n_range($k+1);

    my $xy_is_join_square = sub {
      my ($x,$y, $dx,$dy) = @_;
      ### xy_is_join_square(): "$x,$y  $dx,$dy"
      my $other = 0;
      my $x_min = $x;
      my $y_min = $y;
      foreach (1 .. 3) {
        $x += $dx;
        $y += $dy;
        ($dx,$dy) = xy_rotate_plus90($dx,$dy);
        ### consider: "$x,$y  $dx,$dy"
        if (path_xyxy_is_traversed_either($path, $x,$y, $x+$dx,$y+$dy, $n_hi)) {
          ### current side ...
        } elsif (path_xyxy_is_traversed_either($path, $x,$y, $x+$dx,$y+$dy, $next_n_hi)) {
          ### next side ...
          $other = 1;
        } else {
          ### not enclosed ...
          return 0;
        }
        $x_min = min($x_min,$x);
        $y_min = min($y_min,$y);
      }
      ### return: $other
      return $other && "$x_min,$y_min";
    };

    my $count = 0;
    my %seen;
    foreach my $n ($n_lo .. $n_hi-1) {
      ### $n
      my ($x,$y) = $path->n_to_xy($n);
      my ($dx,$dy) = $path->n_to_dxdy($n);

      $draw->($x,$y, '.');
      if ($dy) {
        $draw->($x+$dx/2,$y+$dy/2, '|');
      } else {
        $draw->($x+$dx/2,$y+$dy/2, '-');
      }

      my $key = $xy_is_join_square->($x,$y, $dx,$dy) || next;
      next if $seen{$key}++;

      $x += $dx/2;
      $y += $dy/2;
      ($dx,$dy) = xy_rotate_plus90($dx,$dy);
      $x += $dx/2;
      $y += $dy/2;
      $draw->($x,$y, 'J');
      $count++;
    }
    $count == JA($k)
      or die "oops count=$count vs JA=",JA($k);

    $draw->(0,0, '0');
    $draw->($x_hi,$y_hi, 'E');
    $image->save('/dev/stdout');
  }
  exit 0;
}

{
  # join area centroid

  #            9----8    5---4               2
  #            |    |    |   |
  #           10--11,7---6   3---2           1
  #                 |            |
  #      16   13---12        0---1       <- Y=0
  #       |    |
  #      15---14
  #
  #      -4   -3   -2   -1   0   1

  my $path = Math::PlanePath::DragonCurve->new;
  my @vx;
  my @vy;
  foreach my $k (0 .. 12) {
    my ($n_lo, $n_hi) = $path->level_to_n_range($k);
    my ($x_hi,$y_hi) = $path->n_to_xy($n_hi);
    my ($next_n_lo, $next_n_hi) = $path->level_to_n_range($k+1);

    my $xy_is_join_square = sub {
      my ($x,$y, $dx,$dy) = @_;
      ### xy_is_join_square(): "$x,$y  $dx,$dy"
      my $other = 0;
      my $x_min = $x;
      my $y_min = $y;
      foreach (1 .. 3) {
        $x += $dx;
        $y += $dy;
        ($dx,$dy) = xy_rotate_plus90($dx,$dy);
        ### consider: "$x,$y  $dx,$dy"
        if (path_xyxy_is_traversed_either($path, $x,$y, $x+$dx,$y+$dy, $n_hi)) {
          ### current side ...
        } elsif (path_xyxy_is_traversed_either($path, $x,$y, $x+$dx,$y+$dy, $next_n_hi)) {
          ### next side ...
          $other = 1;
        } else {
          ### not enclosed ...
          return 0;
        }
        $x_min = min($x_min,$x);
        $y_min = min($y_min,$y);
      }
      ### return: $other
      return $other && "$x_min,$y_min";
    };

    my $count = 0;
    my $x_total = 0;
    my $y_total = 0;
    my %seen;
    foreach my $n ($n_lo .. $n_hi-1) {
      ### $n
      my ($x,$y) = $path->n_to_xy($n);
      my ($dx,$dy) = $path->n_to_dxdy($n);
      my $key = $xy_is_join_square->($x,$y, $dx,$dy) || next;
      next if $seen{$key}++;

      $x *= 2;
      $y *= 2;
      $x += $dx;
      $y += $dy;
      ($dx,$dy) = xy_rotate_plus90($dx,$dy);
      $x += $dx;
      $y += $dy;
      $count++;

      $x_total += $x;
      $y_total += $y;
    }
    $count == JA($k)
      or die "oops count=$count vs JA=",JA($k);
    push @vx, $x_total;
    push @vy, $y_total;
    printf "k=%2d  1/2*  %4d %4d\n", $k, $x_total, $y_total;
  }
  require Math::OEIS::Grep;
  Math::OEIS::Grep->search(array => \@vx, verbose=>1, name=>"x");
  Math::OEIS::Grep->search(array => \@vy, verbose=>1, name=>"y");

  print "GJA_samples = 1/2*[ ";
  foreach my $k (0 .. $#vx) {
    print "$vx[$k]+$vy[$k]*I, ";
  }
  print "];\n";
  exit 0;
}

{
  # twindragon minimum area rectangle
  #
  # *---*   1         *     2*2/2 = 2
  # |   |            / \
  # *---*           *   *
  #                  \ /
  #                   *

  # *---*      4       *    2*4/2 = 4
  # |   |             / \
  # *---*---*        *   *
  #     |   |         \ /
  #     *---*          *
  #                   / \
  #                  *   *
  #                   \ /
  #                    *
  #
  my $path = Math::PlanePath::DragonCurve->new;
  my @v0;
  my @v1;
  my @vmin;
  my @vmax;
  foreach my $k (0 .. 16) {
    my ($n_lo, $n_hi) = $path->level_to_n_range($k+1);
    my ($x_hi,$y_hi) = $path->n_to_xy($n_hi);

    printf "k=%2d", $k;
    my @area;
    foreach my $rot (0, 1) {
      my $transform = sub {
        my ($x,$y) = @_;
        if ($rot) {
          return mul($x,$y);
        } else {
          return ($x,$y);
        }
      };
      my $factor = ($rot ? 2 : 1);

      my $x_min = 0;
      my $y_min = 0;
      my $x_max = 0;
      my $y_max = 0;
      foreach my $n ($n_lo .. $n_hi) {
        my ($x,$y) = $path->n_to_xy($n);
        ($x,$y) = $transform->($x,$y);
        $x_min = min($x_min,$x);
        $y_min = min($y_min,$y);
        $x_max = max($x_max,$x);
        $y_max = max($y_max,$y);

        ($x,$y) = $path->n_to_xy($n);
        ($x,$y) = ($x_hi-$x, $y_hi-$y);
        ($x,$y) = $transform->($x,$y);
        $x_min = min($x_min,$x);
        $y_min = min($y_min,$y);
        $x_max = max($x_max,$x);
        $y_max = max($y_max,$y);
      }
      my $width = $x_max - $x_min;
      my $height = $y_max - $y_min;
      my $area = $width*$height / $factor;
      # if ($k <= 1) {
      #   print "\n  $width x $height / $factor = $area\n";
      # }
      $area[$rot] = $area;
      printf "  %6d", $area;
    }
    my $min = min($area[0],$area[1]);
    my $max = max($area[0],$area[1]);
    printf "  %6d %6d", $min, $max;
    push @v0, $area[0];
    push @v1, $area[1];
    push @vmin, $min;
    push @vmax, $max;
    if ($area[0] < $area[1]) {
      print "  ***";
    }
    print "\n";
  }
  require Math::OEIS::Grep;
  Math::OEIS::Grep->search(array => \@v0, verbose=>1, name=>"v0");
  Math::OEIS::Grep->search(array => \@v1, verbose=>1, name=>"v1");
  Math::OEIS::Grep->search(array => \@vmin, verbose=>1, name=>"vmin");
  Math::OEIS::Grep->search(array => \@vmax, verbose=>1, name=>"vmax");
  exit 0;
}

{
  # twindragon minimum area rectangle -- by planar
  #
  # 1,4,8,24,48,108,216,468,936,1924,3848,7844,15688,31588
  #
  require Math::Geometry::Planar;
  my $path = Math::PlanePath::DragonCurve->new;
  my @values;
  foreach my $k (0 .. 16) {
    my ($n_lo, $n_hi) = $path->level_to_n_range($k+1);
    my ($x_hi,$y_hi) = $path->n_to_xy($n_hi);
    my $points = [];
    foreach my $n ($n_lo .. $n_hi) {
      my ($x,$y) = $path->n_to_xy($n);
      push @$points, [$x,$y], [$x_hi-$x, $y_hi-$y];
    }
    my $planar = Math::Geometry::Planar->new;
    $planar->points($points);
    if (@$points > 4) {
      $planar = $planar->convexhull2;
      $points = $planar->points;
    }
    $planar = $planar->minrectangle;
    my $area = $planar->area;
    my $rect_points = $planar->points;
    push @values, $area;
    print "$k  $area\n";
  }
  require Math::OEIS::Grep;
  Math::OEIS::Grep->search(array => \@values, verbose=>1);
  exit 0;
}



{
  # twindragon convex hull pictures calc vs formulas
  #
  require Image::Base::GD;
  require Math::Complex;
  require Math::Geometry::Planar;
  my $size = 600;
  my $scale = 1/3;

  my $path = Math::PlanePath::DragonCurve->new;
  my @values;
  my $b;
  BEGIN { $b = Math::Complex->make(1,1); }

  foreach my $k (0 .. 15) {
    print "k=$k\n";
    my $filename = sprintf "/tmp/twin-hull-%02d.png", $k;
    my $image = Image::Base::GD->new (-height => $size,
                                      -width  => $size);
    $image->rectangle(0,0, $size-1,$size-1, 'black');

    my $bk = $b ** $k;
    $bk = sqrt(2) ** $k * 2;
    $bk = 2**4;
    my $transform = sub {
      my ($x,$y) = @_;
      my $z = (Math::Complex->make($x,$y) / $bk - Math::Complex->make(.5,.5))
        * $scale * $size;
      return ($size/2 + $z->Re, $size/2 - $z->Im);
    };

    my ($n_lo, $n_hi) = $path->level_to_n_range($k+1);
    my ($x_hi,$y_hi) = $path->n_to_xy($n_hi);

    foreach my $n ($n_lo .. $n_hi-1) {
      my ($x1,$y1) = $path->n_to_xy($n);
      my ($x2,$y2) = $path->n_to_xy($n+1);
      $image->line($transform->($x1,$y1),
                   $transform->($x2,$y2),
                   'red');
      $image->line($transform->($x_hi-$x1,$y_hi-$y1),
                   $transform->($x_hi-$x2,$y_hi-$y2), 'blue');
    }

    Math::Complex->display_format(format => '%.0f');

    {
      # by formulas
      my @zs = (P1_formula($k+1),
                P2_formula($k+1),
                P3_formula($k+1),
                P4_formula($k+1),
                $b**($k+1) - P1_formula($k+1),
                $b**($k+1) - P2_formula($k+1),
                $b**($k+1) - P3_formula($k+1),
                $b**($k+1) - P4_formula($k+1),
               );
      # my @zs = (P1_formula($k),
      #           P2_formula($k),
      #           P3_formula($k),
      #           P4_formula($k),
      #           P5_formula($k),
      #           P6_formula($k),
      #           P7_formula($k),
      #           P8_formula($k),
      #           P9_formula($k),
      #           P10_formula($k) );
      ### @zs
      foreach my $i (0 .. $#zs) {
        my $zprev = $zs[$i-2];
        my $z1 = $zs[$i-1];
        my $z2 = $zs[$i];
        ### $z1
        ### $z2
        if (abs($z1 - $z2) < .01) {
          print "  same points $z1  $z2\n";
        }
        if ($z2 != $z2 && $zprev != $z1) {
          my $d1 = $z1 - $zprev;
          my $d2 = $z2 - $z1;
          my $ratio = $d1 / $d2;
          if (abs($ratio->Im) < .001) {
            print "  colinear  $zprev  $z1  $z2\n";
          }
        }
        $image->line($transform->($z1->Re,$z1->Im),
                     $transform->($z2->Re,$z2->Im), 'green');
      }
      if ($k < 5) {
        foreach my $i (0 .. $#zs) {
          print "  $zs[$i]";
        }
        print "\n";
      }
    }

    {
      # actual hull
      my $points = [];
      foreach my $n ($n_lo .. $n_hi) {
        my ($x,$y) = $path->n_to_xy($n);
        push @$points, [$x,$y], [$x_hi-$x, $y_hi-$y];
      }
      my $planar = Math::Geometry::Planar->new;
      $planar->points($points);
      if (@$points > 4) {
        $planar = $planar->convexhull2;
        $points = $planar->points;
      }
      foreach my $i (0 .. $#$points) {
        my ($x1,$y1) = @{$points->[$i-1]};
        my ($x2,$y2) = @{$points->[$i]};
        $image->line($transform->($x1,$y1), $transform->($x2,$y2), 'white');
      }
      my $length = scalar(@$points);
      print "  length=$length\n";
    }

    $image->save($filename);
  }
  exit 0;

  sub P1_formula  { my ($k) = @_; return - ($b**($k+3) + p($k+3)) /3; }
  sub P2_formula  { my ($k) = @_; return - ($b**($k+2) + p($k+2)) /3; }
  sub P3_formula  { my ($k) = @_; return - ($b**($k+1) + p($k+1)) /3; }
  sub P4_formula  { my ($k) = @_; return - ($b**($k+0) + p($k+0)) /3; }
  sub P5_formula  { my ($k) = @_; return - ($b**($k-1) + p($k-1)) /3; }
  sub P6_formula  { my ($k) = @_; return (Math::Complex->make(2,1)*$b**$k     + Math::Complex->i*p($k+1)) /3; }
  sub P7_formula  { my ($k) = @_; return (Math::Complex->make(3,1)*$b**$k     + Math::Complex->i*p($k  )) /3; }
  sub P8_formula  { my ($k) = @_; return (Math::Complex->make(3,4)*$b**($k-1) + Math::Complex->i*p($k-1)) /3; }
  sub P9_formula  { my ($k) = @_; return (Math::Complex->make(0,7)*$b**($k-2) + Math::Complex->i*p($k-2)) /3; }
  sub P10_formula { my ($k) = @_; return (Math::Complex->make(4,1)*$b**($k-1) +   p($k)  ) /3; }
  BEGIN {
    require Math::Complex;
    my $b = Math::Complex->make(1,1);
    my @p_vector = (Math::Complex->make(-1,0),
                    -$b,
                    -$b**2,
                    -$b**3-3,
                    Math::Complex->make(1,0),
                    $b,
                    $b**2,
                    $b**3+3);
    sub p { my ($k) = @_; return $p_vector[$k % 8]; }
  }
}

{
  # twindragon convex hull
  # area 1,3,7,19,43,91,187,395,811,1643,3307,6699
  # Plus 1,2,5,11,25,49,101,205,421,837,1685,3381
  # diff [-1, -1, -3, -3, -7, -7, -15, -15, -31, -31, -63, -63]
  # Four flat sides of twindragon add 1/2
  #
  require Math::Geometry::Planar;
  my $path = Math::PlanePath::DragonCurve->new;
  my @values;
  foreach my $k (0 .. 11) {
    print "$k\n";
    my ($n_lo, $n_hi) = $path->level_to_n_range($k+1);
    my ($x_hi,$y_hi) = $path->n_to_xy($n_hi);
    my $points = [];
    foreach my $n ($n_lo .. $n_hi) {
      my ($x,$y) = $path->n_to_xy($n);
      push @$points, [$x,$y], [$x_hi-$x, $y_hi-$y];
    }
    my $planar = Math::Geometry::Planar->new;
    $planar->points($points);
    if (@$points > 4) {
      $planar = $planar->convexhull2;
      $points = $planar->points;
    }
    $points = hull_points_sort_by_angle ($points, $x_hi,$y_hi);

    my $length = scalar(@$points);
    my $area = $planar->area;

    my $diffs = [ map{ [$points->[$_]->[0] - $points->[$_-1]->[0],
                        $points->[$_]->[1] - $points->[$_-1]->[1] ] }
                  0 .. $#$points
                ];

    print "len=$length  ",points_str($points),"\n";
    print "   ",points_str($diffs),"\n";
    print "   area=$area\n";
    push @values,$area;
  }

  print join(',',@values),"\n";
  shift @values;
  shift @values;
  shift @values;
  require Math::OEIS::Grep;
  Math::OEIS::Grep->search(array => \@values, verbose=>1);
  exit 0;
}



{
  # twindragon counts of adjacent diamonds

  # 2 neighbours: 0,0,2,2,2,6,10,14,26,46,74,126,218
  # A052537 = LQ2(k)
  # 2*A003229
  #
  # 4 neighbours: match 0,0,0,0,0,4,16,44,112,268,608,1340,2896
  # = 4*A003230 = 4*A(k)
  #
  # 1 neighbours: match 0,2,2,4,6,8,14,24,38,64,110,184,310
  #                half 0,1,1,2,3,4,7,12,19,32,55,92,155
  # H(k) = H(k-1) + 2*H(k-3) - 1
  #
  # 3 neighbours: match 0,0,0,2,8,14,24,46,80,134,232,398,672
  #                half 0,0,0,1,4,7,12,23,40,67,116,199,336
  # H(k) = H(k-1) + 2*H(k-3) + 3

  require Math::PlanePath::ComplexPlus;
  my $path = Math::PlanePath::ComplexPlus->new;
  my @values;
  foreach my $k (0 .. 10) {
    my ($n_lo, $n_hi) = $path->level_to_n_range($k);
    my $is_visited = sub {
      my ($x,$y) = @_;
      my $n = $path->xy_to_n($x,$y);
      return (defined $n && $n <= $n_hi ? 1 : 0);
    };
    my @count = (0,0,0,0,0);
    foreach my $n ($n_lo .. $n_hi) {
      my ($x,$y) = $path->n_to_xy($n);
      my $neighbours = ($is_visited->($x+1,$y)
                        + $is_visited->($x-1,$y)
                        + $is_visited->($x,$y+1)
                        + $is_visited->($x,$y-1));
      $count[$neighbours]++;
    }
    print "k=$k  ",join('  ',@count),"\n";
    foreach my $neighbours (0 .. 4) {
      push @{$values[$neighbours]}, $count[$neighbours]/2;
    }
  }
  foreach my $neighbours (1 .. 4) {
    require Math::OEIS::Grep;
    Math::OEIS::Grep->search(array => $values[$neighbours],
                             name => "$neighbours neighbours",
                             verbose => 1);
  }
  exit 0;
}



{
  # join end square position

  my $path = Math::PlanePath::DragonCurve->new;
  foreach my $k (0 .. 16) {
    my $n = JN($k);
    $n -= 1;
    my ($x,$y) = $path->n_to_xy($n);
    my ($dx,$dy) = $path->n_to_dxdy($n);

    my $mx = $x;
    my $my = $y;
    foreach (1 .. 4) {
      if ($x <= $mx && $y <= $my) {
        $mx = $x;
        $my = $y;
      }
      $x += $dx;
      $y += $dy;
      ($dx,$dy) = xy_rotate_plus90($dx,$dy);
    }

    my ($cx,$cy) = J_end_formula($k);

    my ($gx,$gy) = GS_xy($k+1);
    $gx = $gx->numify;
    $gy = $gy->numify;
    $gx = sprintf '%.2f', $gx;
    $gy = sprintf '%.2f', $gy;

    my $diff = ($cx==$mx && $cy==$my ? '' : '  ***');
    my $within = ($cx<=$gx && $cx+1>=$gx && $cy<=$gy && $cy+1>=$gy ? '' : '  ***');
    print "$k JN $mx, $my  formula $cx,$cy$diff  centroid $gx, $gy $within\n";
  }

  exit 0;

  # c[k+2] = b^2*c[k] + (1-i)
  #        = (1-i)*( 1 + b^2 + ... )
  # c[k] = (1-i) * (b^k - b^(k%2))/(b^2 - 1)
  #      = 1/5*(3+I) * (b^k - b^(k%2))
  #
  # G[k+1] = (2/5 - 1/5*I)*b^(k+1) + (1/10 + 1/5*I)*conj(b/2)^(k+1);
  # G[k+1] = (2/5 - 1/5*I)*b*b^k + (1/10 + 1/5*I)*conj(b/2)^(k+1);
  # G[k+1] = 1/5*(3+I)*b^k + (1/10 + 1/5*I)*conj(b/2)^(k+1);
  #
  # G[k+1] - c[k] = (1/10 + 1/5*I)*conj(b/2)^(k+1) - 1/5*(3+I) * - b^(k%2)
  #               = (3/20 + 1/20*I)*conj(b/2)^k + 1/20*(3+I) * 4*b^(k%2)
  #               = (3/20 + 1/20*I)*(conj(b/2)^k + 4*b^(k%2))
  #
  sub J_end_formula {
    my ($k) = @_;
    my $x = 0;
    my $y = 0;
    foreach my $i (1 .. $k-1) {
      ($x,$y) = ($x - $y, $y + $x);
      if ($i >= 1 && $i % 2 == 1) {
        $x -= 1;
        $y += 1;
      }
    }
    return ($x,$y);
  }
}

{
  # GL centroid of left segments

  my $path = Math::PlanePath::DragonCurve->new;
  require Math::BigRat;
  my $half = Math::BigRat->new(1) / 2;
  my (@gx,@gy,@lq, @rbx,@rby, @lbx,@lby);
  my @values;
  foreach my $k (0 .. 12) {
    my ($n_lo, $n_hi) = $path->level_to_n_range($k);
    my $x_total = 0;
    my $y_total = 0;
    my $count = 0;
    foreach my $n ($n_lo .. $n_hi-1) {
      next unless path_n_is_left_boundary($path, $n, $n_hi);
      my ($x,$y) = $path->n_to_xy($n);
      my ($dx,$dy) = $path->n_to_dxdy($n);
      $x_total += $x + $dx*$half;
      $y_total += $y + $dy*$half;
      ### add: ($x + $dx*$half)."  ".($y + $dy*$half)
      $count++;
    }
    my $gx = $x_total / $count;
    my $gy = $y_total / $count;
    print "$k $count  at $gx, $gy    tot  $x_total, $y_total\n";
    push @values, $x_total;
  }
  shift @values;
  shift @values;
  shift @values;
  require Math::OEIS::Grep;
  Math::OEIS::Grep->search (array => \@values,verbose=>1);
  exit 0;
}

{
  # CLQ count boundary squares by direction
  #
  #  0    1   0   0   0      0   0   0   0      0   0   0   0    total 1
  #  1    0   0   0   0      1   0   0   0      0   0   0   0    total 1
  #  2    0   1   0   0      0   0   0   0      1   0   0   0    total 2
  #  3    0   1   1   0      0   0   0   0      1   1   0   0    total 4
  #  4    0   1   1   0      0   0   2   0      1   1   0   0    total 6
  #  5    0   1   1   2      0   0   2   0      1   1   2   0    total 10
  #  6    2   1   1   4      0   0   2   0      1   1   4   2    total 18
  #  7    4   1   1   6      4   0   2   0      1   1   6   4    total 30
  #  8    6   5   1   8      8   0   2   0      5   1   8   6    total 50
  #  9    8  13   5  10     12   0   2   0     13   5  10   8    total 86
  # 10   10  25  13  12     16   0  10   0     25  13  12  10    total 146
  # 11   12  41  25  22     20   0  26   0     41  25  22  12    total 246
  # 12   22  61  41  48     24   0  50   0     61  41  48  22    total 418
  # 13   48  85  61  98     44   0  82   0     85  61  98  48    total 710
  # 14   98 129  85 180     96   0 122   0    129  85 180  98    total 1202
  # 15  180 225 129 302    196   0 170   0    225 129 302 180    total 2038
  # 16  302 421 225 472    360   0 258   0    421 225 472 302    total 3458
  #

  my $path = Math::PlanePath::DragonCurve->new;
  require Math::BigRat;
  my (@clq);
  foreach my $k (0 .. 16) {
    my ($n_lo, $n_hi) = $path->level_to_n_range($k);
    my $x_total = 0;
    my $y_total = 0;
    my $lq = 0;
    my %seen;
    foreach my $n ($n_lo .. $n_hi-1) {
      next unless path_n_is_left_boundary($path, $n, $n_hi);
      my ($x,$y) = $path->n_to_xy($n);
      my ($dx,$dy) = $path->n_to_dxdy($n);
      my ($rdx,$rdy) = xy_rotate_plus90($dx,$dy);
      my $mx = $x + $dx*.5 + $rdx*.5;
      my $my = $y + $dy*.5 + $rdy*.5;
      if ($seen{"$mx,$my"}++) {
        ### seen: "n=$n  $mx,$my"
        next;
      }

      my $d = xy_to_dir4($dx,$dy);
      my $s = 0;
      foreach (1 .. 4) {
        if (path_xyxy_is_traversed_either($path, $x,$y, $x+$dx,$y+$dy, $n_hi)) {
          $s++;
        }
        $x += $dx;
        $y += $dy;
        ($dx,$dy) = xy_rotate_plus90($dx,$dy);
      }

      ### square: "n=$n  mid $mx,$my  dxdy=$dx,$dy  d=$d  s=$s"

      $clq[$k][$d][$s]++;
    }

    printf "%2d  ", $k;
    my $total = 0;
    foreach my $s (1,2,3) {
      foreach my $d (0..3) {
        my $c = $clq[$k][$d][$s] // 0;
        $total += $c;
        printf "%3d ", $c;
      }
      print "   ";
    }
    print "total $total\n";
  }

  foreach my $s (1,2,3) {
    foreach my $d (0..3) {
      my @values;
      foreach my $k (6 .. 16) {
        my $c = $clq[$k][$d][$s] // 0;
        push @values, $c;
      }
      require Math::OEIS::Grep;
      Math::OEIS::Grep->search (array => \@values,name=>"d=$d,s=$s");
    }
  }
  exit 0;
}

{
  # GLQ centroid of left squares
  my $path = Math::PlanePath::DragonCurve->new;
  require Math::BigRat;
  my $half = Math::BigRat->new(1) / 2;
  my (@gx,@gy,@lq, @rbx,@rby, @lbx,@lby);
  my @rq = (1);
  my $rbx = -2;
  my $rby = -2;
  my $lbx = -3;
  my $lby = 1;
  foreach my $k (0 .. 100) {
    my ($n_lo, $n_hi) = $path->level_to_n_range($k);
    my $x_total = 0;
    my $y_total = 0;
    my $lq = 0;
    my %seen;
    foreach my $n ($n_lo .. $n_hi-1) {
      next unless path_n_is_left_boundary($path, $n, $n_hi);
      my ($x,$y) = $path->n_to_xy($n);
      my ($dx,$dy) = $path->n_to_dxdy($n);
      $x += $dx*$half - $dy*$half;
      $y += $dy*$half + $dx*$half;
      next if $seen{"$x,$y"}++;
      $x_total += $x;
      $y_total += $y;
      $lq++;
    }
    my $gx = $x_total / $lq;
    my $gy = $y_total / $lq;
    $gx[$k] = $gx;
    $gy[$k] = $gy;
    $lq[$k] = $lq;
    $rq[$k+1] = $rq[$k] + $lq;
    $rbx[$k] = $rbx;
    $rby[$k] = $rby;
    $lbx[$k] = $lbx;
    $lby[$k] = $lby;
    print "$k $lq  at $gx, $gy\n";

    if ($k >= 4) {
      my $fx = ($lq[$k-1]*$gx[$k-1]
                - 2*$lq[$k-3]*$gx[$k-3]

                + $rbx[$k-4]*$rq[$k-3]
                + $lbx[$k-4]*$lq[$k-3]
               ) / $lq[$k];
      my $fy = ($lq[$k-1]*$gy[$k-1]
                - 2*$lq[$k-3]*$gy[$k-3]

                + $rby[$k-4]*$rq[$k-3]
                + $lby[$k-4]*$lq[$k-3]
               ) / $lq[$k];
      print "  formula $fx, $fy      lb = $lbx, $lby\n";
    }

    # mul (1+I)
    ($rbx,$rby) = ($rbx - $rby, $rby + $rbx);
    ($lbx,$lby) = ($lbx - $lby, $lby + $lbx);
  }
  exit 0;
}



{
  # convex hull N point numbers

  my $path = Math::PlanePath::DragonCurve->new;
  my $points = [
                [0,0],
                [1,0],
               ];
  my $ex = 1;
  my $ey = 0;

  my $limit = 40;
  my @P1;
  my @P;
  my @values;
  foreach my $k (0 .. $limit) {
    print "$k\n";
    my @hull_n;
    my $n_limit = 2**$k;
    foreach my $p (@$points) {
      my ($x,$y) = @$p;
      my @n_list = $path->xy_to_n_list($x,$y);
      @n_list = grep {$_ <= $n_limit} @n_list;
      if (@n_list != 1) { die; }
      push @hull_n, $n_list[0];
    }
    @hull_n = sort {$a<=>$b} @hull_n;
    foreach my $n (@hull_n) {
      printf "%*b\n", $k+2, $n;
    }

    ($points,$ex,$ey) = hull_expand($points,$ex,$ey);
  }
  exit 0;
}

{
  # moment of inertia of segment midpoint
  #
  # relative to centre of gravity
  # 1,3,15,123,683,2899,11011,40851,154195,603315
  #
  # b=1+w |b|=sqrt(3)
  # R = b^k/2 - b^(k-1)/2
  #   = (b-1)*b^(k-1)/2
  #   = w*b^(k-1)/2
  # |R| = sqrt(3)^(k-1)/2
  # |R|^2 = 3^(k-1)/4
  #       = 1/4, 3/4, 9/4
  #
  # norm(2,0) = (2^2+3*0^2)/4 = 1
  # norm(3,1) = (3^2+3*1^2)/4 = 3
  # norm(3,3) = (3^2+3*3^2)/4 = 9
  #
  # IC = m*r^2
  # IR = m/2((R + r cos)^2 + (r sin)^2)
  #    + m/2((R - r cos)^2 + (r sin)^2)
  #    = m*((r cos)^2 + (r sin^2)) + m*R^2
  #    = IC + m*R^2
  #
  # I[k] = I[k-1] + 2*(I[k-1] + 3^(k-1)*3^(k-1))
  #      = 3*I[k-1] + 2*3^(2k-2)      k >= 1
  #      = 3*I[k-1] + 2*9^(k-1)      k >= 1
  # I[0] = 0
  # I[1] = 2*1 = 2
  # I[2] = 3*2 + 2*3^2 = 24
  # I[3] = 3*24 + 2*3^4 = 234
  # I[4] = 3*234 + 2*3^6 = 2160
  #
  # IC(k) = 2*9^(k-1) + 3*2*9^(k-2) + ... + 3^(k-1)*2*9^0 + 3^k*I[0]
  #       = 2* (9^k - 3^k) / (9-3)
  # IC(k) = 3^(k-1) * (3^k - 1)
  #
  # relative to 0,0
  # IE(k) = IC(k) + 3^k*3^k
  # IE(k) = 3^(k-1) * (3^k - 1) + 3*3^(k-1)*3^k
  # IE(k) = 3^(k-1) * (3^k - 1  + 3*3^(k-1))
  # IE(k) = 3^(k-1) * (4*3^k - 1)
  #
  # radius of gyration
  # 3^k*r^2 = IC(k)
  # r^2 = 3^(k-1) * (3^k - 1) / 3^k
  # r^2 = (3^k - 1) / 3
  #
  require Math::BigRat;
  my $half = Math::BigRat->new(1) / 2;
  my $path = Math::PlanePath::DragonCurve->new;
  my @values;
  foreach my $level (0 .. 9) {
    my ($n_lo, $n_hi) = $path->level_to_n_range($level);
    # my ($mx,$my) = $path->n_to_xy($n_hi/2);  # relative to centre
    # my ($mx,$my) = (0,0);  # relative to origin
    my ($mx,$my) = GS_xy($level);
    my $sum = 0;
    foreach my $n ($n_lo .. $n_hi) {
      my ($x,$y) = $path->n_to_xy($n);
      my ($dx,$dy) = $path->n_to_dxdy($n);
      $x += $half * $dx;
      $y += $half * $dy;
      $x -= $mx;
      $y -= $my;
      $sum += $x**2 + $y**2;
    }
    push @values, $sum * 4**$level * 2;
    print "level=$level mid=$mx,$my   $sum\n";

    ($mx,$my) = mul($mx,$my);
  }
  # shift @values;
  require Math::OEIS::Grep;
  Math::OEIS::Grep->search (array => \@values,verbose=>1);
  exit 0;
}

{
  # N point number which is first to touch the next level
  # A057744 = 7,14,28,55,110,220
  # p = (1-2*x^3)/(1-2*x-x^3+2*x^4)
  # p == (1-2*x^3)/((1-x^3)*(1-2*x))
  # p == (1-2*x^3)/((1-2*x)*(1-x)*(1+x+x^2))
  # p == 1/21 * ( (-4 -5*x)/(1+x+x^2) + 7/(1-x) + 18/(1-2*x) )
  # (x - (-1/2 - sqrt(3)/2*I)) * (x - (-1/2 + sqrt(3)/2*I))
  # ((-1/2 - sqrt(3)/2*I) - x) * ((-1/2 + sqrt(3)/2*I) - x)
  #
  # opposite 1,2,4,11,22,44,91,182,364,731,1462
  # not in OEIS, it seems
  # binary 1011011011011011011...011   *2 and *4
  #
  # JN
  # JO
  # JND
  # JOD
  #
  # dist from midpoint 2^k - N
  # A155803 0,0,0,1,2,4,9,18,36,73,146
  #    = (8^n - 1)/7 interleave  2* and 4*
  #    = 100100100100100100...1001
  #    = octal 1111...111
  # 1/7*8^floor(k/3)*2^(k%3) - 1/7*2^(k%3)
  # = 1/7*2^(3*floor(k/3))*2^(k%3) - 1/7*2^(k%3)
  # = 1/7*2^k - 1/7*2^(k%3)
  # decimal_to_binary(n) = subst(Pol(digits(n,2)),'x,10)

  # vector(12,k, decimal_to_binary(( 2^k - 2^(k%3) )/7))
  #
  # opposite 2^k - N2
  #        0,0,0,3,6,12,27,54,108,219,438
  #  = 3*  0,0,0,1,2,4,9,18,36,73,146
  #  is same as first
  #
  #        |
  #        |
  #        v
  #   ---->
  #
  #        /
  #       /
  #      v
  #      ^
  #       \
  #        \*  unchanged, 2*prev
  #   \    /
  #    \  /
  #     vv
  #
  #      --->
  #          ^
  #          |
  #      <---*  unchanged, 2*prev
  #
  #   \    /\
  #    \  /  \
  #     vv    v     one earlier, 2*prev - 1
  #     ^^    ^
  #    /  \  /
  #   /    \/*
  #
  #
  require Math::PlanePath::DragonCurve;
  my $path = Math::PlanePath::DragonCurve->new;
  my @values;
 LEVEL: foreach my $level (0 .. 10) {
    my ($n_lo, $n_hi) = $path->level_to_n_range($level);
    my ($next_n_lo, $next_n_hi) = $path->level_to_n_range($level+1);
    my $n = $n_lo;
    my $n2 = $n_hi;
    for (;; $n++) {
      if ($n == $n_hi) {
        last;
      }
      my ($x,$y) = $path->n_to_xy ($n);
      my @n_list = $path->xy_to_n_list($x,$y);
      @n_list == 2 or next;
      $n2 = $n_list[1];
      if ($n_list[0] == $n
          && $path->n_to_level($n2) == $level+1) {
        last;
      }
    }
    my $f = level_to_first_touch($level);
    my $f2 = level_to_first_touch_other($level);
    print "level=$level  $n / $n2    f=$f $f2\n";
     push @values, $n2;
    # push @values, $n;
    # push @values, 2**$level - $n;
    # push @values, ($n2 - 2**$level)/3;
    next LEVEL;
  }
  require Math::OEIS::Grep;
  Math::OEIS::Grep->search(array => \@values, verbose => 1);
  exit 0;

  sub level_to_first_touch {
    my ($k) = @_;
    my $ret = 1 + (0*$k);
    foreach my $i (1 .. $k) {
      $ret *= 2;
      if ($i % 3 == 0) {
        $ret -= 1;
      }
      ### $ret
    }
    return $ret;
  }

  sub level_to_first_touch_other {
    my ($k) = @_;
    my $ret = (0*$k);
    foreach my $i (1 .. $k) {
      $ret *= 2;
      if ($i % 3 == 0) {
        $ret += 3;
      }
      ### $ret
    }
    return $ret + 2**$k;
  }
}

{
  # count repeated points
  # diff 4-term feedback 1/(1-2x)(1-x-2x^3)
  # = 4*x^4 - 2*x^3 + 2*x^2 - 3*x + 1
  # total 1/((1-x)*(1-2x)*(1-x-2x^3)).

  # A003476 diff of unrepeated a(n) = a(n-1) + 2a(n-3).
  # A003479 overlap points by next 2^n section,  x^4

  # position of first/last across 2^k
  # A155803 A023001 interleaved with 2*A023001 and 4*A023001.
  # A023001 (8^n - 1)/7.
  # ternary 100100100

  $|=1;
  require Math::PlanePath::DragonCurve;
  my $path = Math::PlanePath::DragonCurve->new;
  my $count = 0;
  my $prev = 0;
  my $prev_diff = 0;
  my $last_pos = 0;
  foreach my $n (0 .. 2**20) {
    if (is_pow2($n)) {
      my $diff = $count - $prev;
      my $diff_diff = $diff - $prev_diff;

      # printf "%d count=%d[%b] diff=%d[%b]   dd=%d\n",
      #   $n, $count,$count, $diff,$diff, $diff_diff;
      # print "$n  $count $diff\n";
      # print "$count,";
      # print "$diff,";
      # print "$last_pos,";
      $prev = $count;
      $prev_diff = $diff;
      $last_pos = 0;
    }
    my ($x, $y) = $path->n_to_xy ($n);
    my @n_list = $path->xy_to_n_list($x,$y);
    $count += (@n_list == 2 && $n_list[1] >= next_pow2($n));
    if (@n_list == 2 && $n_list[1] >= next_pow2($n)) {
      $last_pos = next_pow2($n) - $n;
      print "$n_list[0] $n_list[1]  at $n  last=$last_pos\n";
    }
  }
  exit 0;

  sub next_pow2 {
    my ($n) = @_;
    return 2*high_bit($n);
  }
}

{
  # convex hull minimum rectangle

  my $points = [ [0,0], [1,0] ];
  my $ex = 1;
  my $ey = 0;
  # $points = points_to_bigrat($points);

  my $limit = 40;
  my @values;
  foreach my $k (1 .. $limit) {
    ($points,$ex,$ey) = hull_expand($points,$ex,$ey);

    # my $area = MyPlanar::points_to_area($points);
    require Math::Geometry::Planar;
    my $planar = Math::Geometry::Planar->new;
    $planar->points($points);
    $planar = $planar->minrectangle;
    my $area = $planar->area;
    my $rect_points = $planar->points;

    push @values, $area;

    my $num_points = scalar(@$points);
    my $area_frac = $area / 2**($k-1);

    my $dx = ($rect_points->[1]->[0] - $rect_points->[0]->[0]);
    my $dy = ($rect_points->[1]->[1] - $rect_points->[0]->[1]);
    if (abs($dx) == 0) {
      ($dx,$dy) = xy_rotate_plus90($dx,$dy);
    }
    my $slope = $dy / $dx;

    print "k=$k num=$num_points   area=$area  $area_frac  slope=$slope\n";
  }
  require Math::OEIS::Grep;
  Math::OEIS::Grep->search(array => \@values, verbose=>1);

  print "0, ";
  foreach my $i (0 .. 12) {
    my $v = $values[$i];
    ### $v
    ### 5: $v*5
    ### i: int($v*5) == $v*5
    ### i: abs(round($v*5) - $v*5)
    if ($v != int($v) && abs($v*5 - round($v*5)) < 1e-6) {
      $v = round(5*$v);
      $v = "\\tfrac{$v}{5}";
    } elsif ($v != int($v) && $v*2 == int($v*2)) {
      $v *= 2;
      $v = "\\tfrac{$v}{2}";
    }
    print "$v, ";
  }
  print "\n";

  @values = @values[5..16];
  foreach my $i (0 .. $#values) {
    my $v = $values[$i];
    if ($v != int($v)) { $v = (2*$v).'/2'; }
    my $e = $i+6;
    $v .= "/2^$e";
    print "$v, ";
  }
  print "\n";

  exit 0;

  sub round {
    my ($x) = @_;
    return int($x + .5);
  }
}

{
  # convex hull by replications

  my $points = [
                [0,0],
                [1,0],
               ];
  my $ex = 1;
  my $ey = 0;

  my $limit = 40;
  my @P1;
  my @P;
  my @values;
  foreach my $k (0 .. $limit) {
    print "$k  ",points_str($points),"\n";
    push @values, 2*MyPlanar::points_to_area($points);

    $P[$k] = $points;
    push @P1, $points->[0];

    ($points,$ex,$ey) = hull_expand($points,$ex,$ey);

    if (@$points == 10) {
      # push @values, $points->[0]->[0];
    }
  }
  print "values ",join(', ',@values),"\n";
  require Math::OEIS::Grep;
  Math::OEIS::Grep->search(array => \@values, verbose=>1);

  my $x = -5;
  my $y = 0;
  foreach my $k (6 .. $limit-1) {
    my $diff = ($x == $P1[$k]->[0] && $y == $P1[$k]->[1] ? '' : '***');
    print "$k  $x,$y  $diff\n";
    ($x,$y) = mul($x,$y);
    if ($k % 8 == 0) {
      $x++;
    }
    if ($k % 8 == 1) {
      $y--;
    }
    if ($k % 8 == 4) {
      $x--;
    }
    if ($k % 8 == 5) {
      $y++;
    }
  }

  exit 0;
}


{
  # convex hull by replications

  my $points = [
                [0,0],
                [1,0],
               ];
  my $ex = 1;
  my $ey = 0;

  my $limit = 40;
  my @P1;
  my @P;
  my @values;
  foreach my $k (0 .. $limit) {
    print "$k  ",points_str($points),"\n";
    push @values, 2*MyPlanar::points_to_area($points);

    $P[$k] = $points;
    push @P1, $points->[0];

    ($points,$ex,$ey) = hull_expand($points,$ex,$ey);

    if (@$points == 10) {
      # push @values, $points->[0]->[0];
    }
  }
  print "values ",join(', ',@values),"\n";
  require Math::OEIS::Grep;
  Math::OEIS::Grep->search(array => \@values, verbose=>1);

  my $x = -5;
  my $y = 0;
  foreach my $k (6 .. $limit-1) {
    my $diff = ($x == $P1[$k]->[0] && $y == $P1[$k]->[1] ? '' : '***');
    print "$k  $x,$y  $diff\n";
    ($x,$y) = mul($x,$y);
    if ($k % 8 == 0) {
      $x++;
    }
    if ($k % 8 == 1) {
      $y--;
    }
    if ($k % 8 == 4) {
      $x--;
    }
    if ($k % 8 == 5) {
      $y++;
    }
  }

  exit 0;
}
{
  my $points = [ [ 0.25,0 ],
                 [ 0.5,0 ],
                 [ 0.75,0 ],
                 [ 1,0 ],
                 [ 0,0 ],
                 [ 1,1 ],
                 [ 0,1.75 ],
                 [ 0,1.5 ],
                 [ 0,1.25 ],
                 [ 0,1 ],
                 [ 0,2 ],
                 [ 1,1 ],
                 # [1,0], [0,0], [1,1], [1,1], [0,1], [0,2],
               ];
  print points_str($points),"\n";
  require Math::Geometry::Planar;
  my $planar = Math::Geometry::Planar->new;
  $planar->points($points);
  $planar = $planar->convexhull2;
  $points = $planar->points;
  # $points = points_hull($points);
  print points_str($points),"\n";
  exit 0;
}
{
  # triangle area in complex numbers
  #
  # A = i/4 det /    z1      z2     \
  #             \ conj(z1) conj(z2) /
  #   = i/4 [ z1*conj(z2) - z2*conj(z2) ]
  #   = i/4 [ z1*conj(z2) - conj( z1*conj(z2) ) ]
  #   = i/4 * 2*i*Im( z1*conj(z2) )
  #   = -1/2 * Im( z1*conj(z2) )
  #   = 1/2 * Im( conj(z1) * z2 )
  #
  require Math::Complex;
  require Math::Geometry::Planar;
  foreach my $a (-5 .. 5) {
    foreach my $b (-5 .. 5) {
      foreach my $c (-5 .. 5) {
        foreach my $d (-5 .. 5) {
          my $z1 = Math::Complex->make($a,$b);
          my $z2 = Math::Complex->make($c,$d);
          my $A = Math::Complex->make(0,1/4) * ($z1 * ~$z2 - $z2 * ~$z1);
          $A = (~ $z1 * $z2)->Im / 2;

          my $p1 = [$a,$b];
          my $p2 = [$c,$d];
          my $P = Math::Geometry::Planar::TriangleArea([ [0,0],$p1,$p2 ]);

          $A == $P or die "planar $P complex $A";
          # print "$a,$b $c,$d  $P   $A\n";
        }
      }
    }
  }
  print "ok\n";
  exit 0;
}

{
  # segments centroid, centre of gravity
  #
  # b = 1+i
  # B = 1-i
  # b-B/2 = 1+i - 1/2-(1/2)i
  #       = 1/2 + (3/2)i
  # 1/(b-B/2) = (1/2 - (3/2)i) / (1/4 + 9/4)
  #           = (1/2 - (3/2)i) *4/10
  #           = 4/20 - (12/20)i
  #           = 1/5 - (3/5)i
  #
  # G[1] = (1/2)*b + (B/2)*G[0]
  #
  # G[k] = (G[k-1] + b^k + -I*G[k-1])/2
  #      = (1/2)*b^k + (1/2)*(1-i)*G[k-1]
  #      = (1/2)*b^k + (B/2)*G[k-1]
  #      = (1/2)*b^k + (B/2)*( (1/2)*b^(k-1) + (B/2)*G[k-2] )
  #      = (1/2)*b^k + (1/2)*b^(k-1)*(B/2) + (B/2)^2*G[k-2]
  #
  #      = (1/2)*b^k + (1/2)*b^(k-1)*(B/2) + ... + (1/2)*b*(B/2)^(k-1) + (B/2)^k*G[0]
  #      = (1/2)*b^k + (1/2)*b^(k-1)*(B/2) + ... + (1/2)*b*(B/2)^(k-1) + (1/2)*(B/2)^k
  #      = (1/2)* (b^(k+1) - conj(b/2)^(k+1)) / (b - conj(b/2))
  #      = (1/10 - 3/10*I)* (b^(k+1) - conj(b/2)^(k+1))
  #      = (2/5 - 1/5*I)* b^k + (1/10 + 1/5*I)*conj(b/2)^k


  #      = (1/2)*( b^k + b^(k-1)*(B/2) + ... + (B/2)^k - (1/2)*(B/2)^k )
  #      = (1/2)*( b^k + b^(k-1)*(B/2) + ... + (B/2)^k ) - (1/4)*(B/2)^k
  #      = (1/2)*(b^(k+1) - (B/2)^(k+1))/(b-B/2) - (1/4)*(B/2)^k
  #      = (1/2)*(1/5 - (3/5)i)*(b^(k+1) - (B/2)^(k+1)) - (1/4)*(B/2)^k
  #      = (1/10 - (3/10)i)*(b^(k+1) - (B/2)^(k+1)) - (1/4)*(B/2)^k
  #      = (1/10 - 3/10*I)*b^(k+1) + (-3/20 + 1/5*I)*(B/2)^k
  # G(k) = (2/5 - 1/5*I)*b^k + (-3/20 + 1/5*I)*(B/2)^k
  #
  #      =

  require Math::BigRat;
  my $half = Math::BigRat->new('1/2');
  my $rat = Math::BigRat->new('1');

  exit 0;
}

sub path_n_is_right_boundary {
  my ($path, $n, $n_limit) = @_;
  ### path_n_is_right_boundary(): $n
  ### $n_limit
  if (defined $n_limit && $n >= $n_limit) {
    croak "path_n_is_right_boundary() $n >= limit $n_limit";
  }
  my ($x1,$y1) = $path->n_to_xy($n);
  my ($x2,$y2) = $path->n_to_xy($n+1);
  my ($dx,$dy) = $path->n_to_dxdy($n);
  ($dx,$dy) = xy_rotate_minus90($dx,$dy);  # to right side
  my $x1s = $x1 + $dx;
  my $y1s = $y1 + $dy;
  my $x2s = $x2 + $dx;
  my $y2s = $y2 + $dy;
  return (! path_xyxy_is_traversed_either($path, $x1,$y1, $x1s,$y1s, $n_limit)
          || ! path_xyxy_is_traversed_either($path, $x2,$y2, $x2s,$y2s, $n_limit)
          || ! path_xyxy_is_traversed_either($path, $x1s,$y1s, $x2s,$y2s, $n_limit));
}

sub path_n_is_left_boundary {
  my ($path, $n, $n_limit) = @_;
  ### path_n_is_left_boundary(): $n
  if ($n >= $n_limit) {
    croak "path_n_is_left_boundary() $n >= limit $n_limit";
  }
  my ($x1,$y1) = $path->n_to_xy($n);
  my ($x2,$y2) = $path->n_to_xy($n+1);
  my ($dx,$dy) = $path->n_to_dxdy($n);
  ($dx,$dy) = xy_rotate_plus90($dx,$dy);  # to left side
  my $x1s = $x1 + $dx;
  my $y1s = $y1 + $dy;
  my $x2s = $x2 + $dx;
  my $y2s = $y2 + $dy;
  if (! path_xyxy_is_traversed_either($path, $x1,$y1, $x1s,$y1s, $n_limit)) {
    ### x1 y1 to s ...
    return 1;
  }
  if (! path_xyxy_is_traversed_either($path, $x2,$y2, $x2s,$y2s, $n_limit)) {
    ### x2 y2 to s ...
    return 1;
  }
  if (! path_xyxy_is_traversed_either($path, $x1s,$y1s, $x2s,$y2s, $n_limit)) {
    ### s to s ...
    return 1;
  }
  ### not left boundary ...
  return 0;
}

sub path_xyxy_is_left_boundary {
  my ($path, $x1,$y1, $x2,$y2, $n_limit) = @_;
  ### path_xyxy_is_left_boundary(): "$x1,$y1, $x2,$y2, $n_limit"
  if (! path_xyxy_is_traversed_either($path, $x1,$y1, $x2,$y2, $n_limit)) {
    ### x1 y1 to x2 y2 not a segment ...
    return 0;
  }
  my $dx = $x2 - $x1;
  my $dy = $y2 - $y1;
  ($dx,$dy) = xy_rotate_plus90($dx,$dy);  # to left side
  my $x1s = $x1 + $dx;
  my $y1s = $y1 + $dy;
  my $x2s = $x2 + $dx;
  my $y2s = $y2 + $dy;
  if (! path_xyxy_is_traversed_either($path, $x1,$y1, $x1s,$y1s, $n_limit)) {
    ### x1 y1 to s untraversed ...
    return 1;
  }
  if (! path_xyxy_is_traversed_either($path, $x2,$y2, $x2s,$y2s, $n_limit)) {
    ### x2 y2 to s ...
    return 1;
  }
  if (! path_xyxy_is_traversed_either($path, $x1s,$y1s, $x2s,$y2s, $n_limit)) {
    ### s to s ...
    return 1;
  }
  ### not left boundary ...
  return 0;
}

sub path_xyxy_is_right_boundary {
  my ($path, $x1,$y1, $x2,$y2, $n_limit) = @_;
  ### path_xyxy_is_right_boundary(): "$x1,$y1, $x2,$y2, $n_limit"
  if (! path_xyxy_is_traversed_either($path, $x1,$y1, $x2,$y2, $n_limit)) {
    return 0;
  }
  my $dx = $x2 - $x1;
  my $dy = $y2 - $y1;
  ($dx,$dy) = xy_rotate_minus90($dx,$dy);  # to right side
  my $x1s = $x1 + $dx;
  my $y1s = $y1 + $dy;
  my $x2s = $x2 + $dx;
  my $y2s = $y2 + $dy;
  return (! path_xyxy_is_traversed_either($path, $x1,$y1, $x1s,$y1s, $n_limit)
          || ! path_xyxy_is_traversed_either($path, $x2,$y2, $x2s,$y2s, $n_limit)
          || ! path_xyxy_is_traversed_either($path, $x1s,$y1s, $x2s,$y2s, $n_limit));
}

sub path_xyxy_is_traversed_either {
  my ($path, $x1,$y1, $x2,$y2, $n_limit) = @_;
  ### xyxy_is_traversed_either(): "$x1,$y1 to $x2,$y2"
  my $arms = $path->arms_count;
  foreach my $n1 ($path->xy_to_n_list($x1,$y1)) {
    next if defined $n_limit && $n1 > $n_limit;
    foreach my $n2 ($path->xy_to_n_list($x2,$y2)) {
      next if defined $n_limit && $n2 > $n_limit;
      if (abs($n1-$n2) == $arms) {
        ### yes: "$n1 to $n2"
        return 1;
      }
    }
  }
  return 0;
}

END {
  foreach my $h (@ipc) {
    $h->finish;
  }
}

sub delta_n {
  my ($n, $k) = @_;
  my @bits = delta_bit_split_lowtohigh($n,$k);
  @bits = reverse @bits;
  return digit_join_lowtohigh (\@bits, 2);
}
sub delta_bit_split_lowtohigh {
  my ($n, $k) = @_;
  my @bits = bit_split_lowtohigh($n);
  while (@bits < $k) { push @bits, 0; } # pad
  # delta
  my $prev = 0;
  foreach my $i (reverse 0 .. $#bits) {  # high to low
    my $bit = $bits[$i];
    $bits[$i] = $bit ^ $prev;  # if different from bit above (before modified)
    $prev = $bit;
  }
  return @bits;
}

#------------------------------------------------------------------------------
# {
#   package Graph::CacheNeighbours;
#   use base 'Graph';
#   use Hash::Util::FieldHash;
#   Hash::Util::FieldHash::fieldhash my %cache_neighbours;
#   sub cache_neighbours {
#     my ($self) = @_;
#     my %neighbours;
#     foreach my $edge ($self->edges) {
#       push @{$neighbours{$edge->[0]}}, $edge->[1];
#       push @{$neighbours{$edge->[1]}}, $edge->[0];
#     }
#     $cache_neighbours{$self} = \%neighbours;
#   }
#   sub neighbours {
#     my ($self, $v) = @_;
#     if (my $href = $cache_neighbours{$self}) {
#       # print $href->{$v},"\n";
#       return @{$href->{$v} || []};
#     }
#     return $self->SUPER::neighbours;
#   }
# }
